#
/*
 *    Copyright (C)  2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	"am_processor.h"
#include	"gui.h"
#include	"pllC.h"
#include	"agchandler.h"
#

	amProcessor::amProcessor (RadioInterface	*ri,
	                          int32_t	rate) {
	myRadioInterface	= ri;
	WorkingRate		= rate;
	am_w1			= 1;
	cycleCount		= 0;
	amBandwidth		= 4500;
	am_pll			= new pllC (WorkingRate,
	                                    0, 
	                                    -500,
	                                    +500,
	                                    amBandwidth);
	agc			= new agcHandler (WorkingRate);
	currLock		= 0.5;
	prevLock		= 1.0;
	amDc			= 0;
	amSmooth		= 0;
	Volume			= 1;
	amDecoder		= SYNCHRONOUS_DETECTOR;

	connect (this,
	         SIGNAL (amStrength (float)),
	         myRadioInterface,
	         SLOT (set_amStrength (float)));
	connect (this,
	         SIGNAL (amShiftFreq (float)),
	         myRadioInterface,
	         SLOT (set_amShiftFreq (float)));
	connect (this,
	         SIGNAL (amshowAgc (float)),
	         myRadioInterface,
	         SLOT (set_amshowAgc (float)));
}

	amProcessor::~amProcessor (void) {
	delete	am_pll;
}
//
//	The parameter is a number between 1 .. 100
//	Bandwidth is here: in quadrant 1 and 2
int32_t	amProcessor::setBandwidth (int32_t g) {
return g;
	delete	am_pll;
	am_pll			= new pllC (WorkingRate,
	                                    0,
	                                    - 500,
	                                    + 500,
	                                    g);
	return g;
}

void	amProcessor::setAgc (uint8_t M) {
	agc	-> setMode (M);
}

void	amProcessor::setAgcLevel (int16_t l) {
	agc	-> setLevel (l);
}

void	amProcessor::setAgcThresholdSlider (int16_t l) {
	agc	-> setThreshold (l);
}

void	amProcessor::setDecoder (amDecoderType d) {
	amDecoder	= d;
}

#define	DC_ALPHA	0.75

void	amProcessor::processAM (DSPCOMPLEX *v, DSPCOMPLEX *o) {
int32_t		i;
DSPFLOAT	res;
DSPFLOAT	am_w0;
DSPFLOAT	agcGain	= agc	-> doAgc (*v);
	*v = cmul (*v, agcGain);

	if (++cycleCount > WorkingRate / 5) {
	   cycleCount = 0;
	   amStrength (get_db (abs (*v)));
	   if (amDecoder == SYNCHRONOUS_DETECTOR) 
	      amShiftFreq (currLock);
	   else
	      amShiftFreq (0);
	   amshowAgc (agcGain);
	}

	if (amDecoder == ENVELOPE_DETECTOR_1) {
	   currLock	= abs (*v);
	   amDc		= 0.9999 * amDc + 0.0001 * currLock;
	   amSmooth	= 0.5 * amSmooth + 0.5 * (currLock - amDc);
	   res		= 0.1 * Volume * amSmooth;
	   *o		= DSPCOMPLEX (res, res);
	}
	else
	if (amDecoder == ENVELOPE_DETECTOR_2) {
	   am_w0	= abs (*v) + (am_w1 * DC_ALPHA);
	   res		= 0.1 * Volume * (am_w0 - am_w1);
	   *o		= DSPCOMPLEX (res, res);
	   am_w1	= am_w0;
	}
	else
	if (amDecoder == SYNCHRONOUS_DETECTOR) {
	   am_pll	-> do_pll (*v);
	   currLock	= 0.999 * currLock +
	                  0.001 * fabs (imag (am_pll -> getDelay ()));
	   prevLock	= currLock;
	   amDc		= 0.9999 * amDc + 0.0001 * real (am_pll -> getDelay ());
	   amSmooth	= 0.5 * amSmooth + 
	                               0.5 * (real (am_pll -> getDelay ()) - amDc);
	   res		= 0.1 * Volume * amSmooth;
	   *o 		= DSPCOMPLEX (res, res);
	}
}

void	amProcessor::setVolume (int32_t V) {
	Volume = 0.1 * V;
}

