#
/*
 *
 *    Copyright (C) 2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	"fm_processor.h"
#include	"gui.h"
#include	"fm_demodulator.h"
#include	"sincos.h"
#include	"fm-constants.h"

#define	AUDIO_FREQ_DEV_PROPORTION 0.85f
#define	PILOT_FREQUENCY		19000
#define	RDS_FREQUENCY		(3 * PILOT_FREQUENCY)
#define	OMEGA_DEMOD		(1.0 / Rate_in) * (2 * M_PI)
#define	OMEGA_PILOT	((DSPFLOAT (PILOT_FREQUENCY)) / Rate_in) * (2 * M_PI)
#define	OMEGA_RDS	((DSPFLOAT) RDS_FREQUENCY / Rate_in) * (2 * M_PI)
//
//	The fm processor  takes a sample and returns 3 values
//	a. one to show in a spectrum to the world (selectable)
//	b. one containing the audio in two channels encoded in a complex
//	c. one containg the raw data for interpretation by the rds processor
//	Note that no decimation done as yet.
	fmProcessor::fmProcessor (RadioInterface	*RI,
	                          int32_t		Rate_in,
	                          SinCos		*mySinCos) {
	this	-> myRadioInterface	= RI;
	this	-> Rate_in		= Rate_in;
	this	-> mySinCos		= mySinCos;
	this	-> omega_demod		= 2 * M_PI / Rate_in;
/*
 *	default values, will be set through the user interface
 *	to their appropriate values
 */
	this	-> fmModus		= FM_STEREO;
	this	-> selector		= S_STEREO;
	this	-> balance		= 0;
	this	-> leftChannel		= - (balance - 50.0) / 100.0;
	this	-> rightChannel		= (balance + 50.0) / 100.0;
	this	-> Volume		= 0.1;
	this	-> viewSelector		= 1;
/*
 *	averagePeakLevel and audioGain are set
 *	prior to calling the processFM method
 */
	this	-> peakLevel		= -100;
	this	-> peakLevelcnt		= 0;
	this	-> max_freq_deviation	= 0.95 * (0.5 * Rate_in);
	this	-> norm_freq_deviation	= 0.8 * max_freq_deviation;
	this	-> audioGain		= 1.0;
//
	this	-> fm_Levels		= new fmLevels (LEVEL_SIZE,
	                                                Rate_in, LEVEL_FREQ);
//
//	for isolating the pilot signal, we need a reasonable
//	filter. The filtered signal is beautified by a pll
//
	fprintf (stderr, "width of transitionband = %d\n", 
		              (int)(4.0 / PILOTFILTER_SIZE * Rate_in));
	pilotBandFilter		= new fftFilter (FFT_SIZE, PILOTFILTER_SIZE);
	pilotBandFilter		-> setBand (PILOT_FREQUENCY - PILOT_WIDTH / 2,
		                            PILOT_FREQUENCY + PILOT_WIDTH / 2,
		                            Rate_in);
	pilotRecover		= new pilotRecovery (Rate_in,
	                                             OMEGA_PILOT,
	                                             25 * omega_demod,
	                                             mySinCos);
	pilotDelay	= (FFT_SIZE / 2 - PILOTFILTER_SIZE) * OMEGA_PILOT;

	rdsLowPassFilter	= new fftFilter (FFT_SIZE, RDSLOWPASS_SIZE);
	rdsLowPassFilter	-> setLowPass (RDS_WIDTH, Rate_in);
//
//	the constant K_FM is still subject to many questions
	DSPFLOAT	F_G	= 65000;	// highest freq in message
	DSPFLOAT	Delta_F	= 75000;	//
	DSPFLOAT	B_FM	= 2 * (Delta_F + F_G);
	K_FM			= 2 * M_PI / 1.0 * (B_FM / 2 - F_G);
	K_FM			= K_FM * 1.0 / Rate_in;

	TheDemodulator		= new fm_Demodulator (Rate_in,
	                                              mySinCos, K_FM);
	fmAudioFilter		= NULL;
//
//	In the case of mono we do not assume a pilot
//	to be available. We borrow the approach from CuteSDR
	rdsHilbertFilter	= new HilbertFilter (HILBERT_SIZE,
	                                     (DSPFLOAT)RDS_FREQUENCY / Rate_in,
	                                             Rate_in);
	rdsBandFilter		= new fftFilter (FFT_SIZE,
	                                         RDSBANDFILTER_SIZE);
	rdsBandFilter		-> setSimple (RDS_FREQUENCY - RDS_WIDTH / 2,
	                                      RDS_FREQUENCY + RDS_WIDTH / 2,
	                                      Rate_in);
	rds_plldecoder		= new pllC (Rate_in,
	                                    RDS_FREQUENCY,
	                                    RDS_FREQUENCY - 50,
	                                    RDS_FREQUENCY + 50,
	                                    200,
	                                    mySinCos);

//	for the deemphasis we use an in-line filter with
	xkm1			= 0;
	ykm1			= 0;
	alpha			= 1.0 / (Rate_in / (1000000.0 / 50.0 + 1));
	viewSelector		= 1;
}

	fmProcessor::~fmProcessor (void) {
	delete	TheDemodulator;
	delete	rds_plldecoder;
	delete	pilotRecover;
	delete	rdsHilbertFilter;
	delete	rdsBandFilter;
	delete	pilotBandFilter;
	delete	fm_Levels;
	if (fmAudioFilter != NULL)
	   delete fmAudioFilter;
}

DSPFLOAT	fmProcessor::getPilotStrength	(void) {
	return fm_Levels	-> getPilotStrength ();
}

DSPFLOAT	fmProcessor::getRdsStrength	(void) {
	return fm_Levels	-> getRdsStrength ();
}

DSPFLOAT	fmProcessor::getNoiseStrength	(void) {
	return fm_Levels	-> getNoiseStrength ();
}

DSPFLOAT	fmProcessor::get_DcComponent	(void) {
	return TheDemodulator	-> get_DcComponent ();
}

const char  *fmProcessor::nameofDecoder	(void) {
	return TheDemodulator -> nameofDecoder ();
}

void	fmProcessor::setfmMode (bool b) {
	fmModus	= b ? FM_STEREO : FM_MONO;
}

void	fmProcessor::setFMdecoder (int8_t d) {
	TheDemodulator	-> setDecoder (d);
}

void	fmProcessor::setView (int16_t d) {
	viewSelector	= d;
}

void	fmProcessor::setSoundMode (uint8_t selector) {
	this	-> selector = selector;
}

void	fmProcessor::setSoundBalance (int16_t balance) {
	this	-> balance = balance;
	leftChannel	= - (balance - 50.0) / 100.0;
	rightChannel	= (balance + 50.0) / 100.0;
}
//	Deemphasis	= 50 usec (3183 Hz, Europe)
//	Deemphasis	= 75 usec (2122 Hz US)
//	tau		= 2 * M_PI * Freq = 1000000 / time
void	fmProcessor::setDeemphasis	(int16_t v) {
DSPFLOAT	Tau;

	switch (v) {
	   default:
	      v	= 1;
	   case	1:
	   case 50:
	   case 75:
//	pass the Tau
	      Tau	= 1000000.0 / v;
	      alpha	= 1.0 / (DSPFLOAT (Rate_in) / Tau + 1.0);
	}
}

void	fmProcessor::setVolume (int16_t Vol) {
	Volume = 0.1 * Vol;
}

DSPCOMPLEX	fmProcessor::audioGainCorrection (DSPCOMPLEX z) {
	return cmul (z, audioGain * 0.1 * Volume);
}
/*
 *	process a single sample (high rate)
 */
void	fmProcessor::processFM (DSPCOMPLEX *v,
	                        DSPCOMPLEX *out, DSPFLOAT *rdsData) {
DSPCOMPLEX	result;
//
//	keep track of the peaklevel, we take segments
//	of 1024 samples
	if (abs (*v) > peakLevel)
	   peakLevel = abs (*v);
	if (++peakLevelcnt >= 1024) {
	   DSPFLOAT	ratio	= max_freq_deviation / norm_freq_deviation;
	   if (peakLevel > 0)
	      this -> audioGain	=
	                  (ratio / peakLevel) / AUDIO_FREQ_DEV_PROPORTION;
	   if (audioGain == 0)
	      audioGain = 1.0;
	   peakLevelcnt	= 0;
	   peakLevel	= -100;
	}

	if (fmModus == FM_MONO) {
	   mono (v, &result, rdsData);
	   result = audioGainCorrection (result);
	   if (fmAudioFilter != NULL)
	      result = fmAudioFilter -> Pass (result);
	   *out  = DSPCOMPLEX (leftChannel * real (result),
	                       rightChannel * imag (result));
	   if (viewSelector == 8)
	      *v = *out;
	}
	else {
	   stereo (v, &result, rdsData);
	   switch (selector) {
	      default:
	      case S_STEREO:
	         result = DSPCOMPLEX (real (result) + imag (result),
	                              - (- real (result) + imag (result)));
	         break;

	      case S_LEFT:
	         result = DSPCOMPLEX (real (result) + imag (result), 
	                              real (result) + imag (result));
	         break;

	      case S_RIGHT:
	         result = DSPCOMPLEX (- (imag (result) - real (result)),
	                              - (imag (result) - real (result)));
	         break;

	      case S_LEFTplusRIGHT:
	         result = DSPCOMPLEX (real (result),  real (result));
	         break;

	      case S_LEFTminusRIGHT:
	         result = DSPCOMPLEX (imag (result), imag (result));
	         break;
	   }

	   result = audioGainCorrection (result);
	   if (fmAudioFilter != NULL)
	      result = fmAudioFilter -> Pass (result);
	   *out =  DSPCOMPLEX (leftChannel * real (result),
	                       rightChannel * imag (result));
	   if (viewSelector == 8)
	      *v = *out;
	}
}

void	fmProcessor::mono (DSPCOMPLEX	*in,
	                   DSPCOMPLEX	*audioOut,
	                   DSPFLOAT	*rdsValue) {
DSPFLOAT	Re, Im;
DSPCOMPLEX	rdsBase;
DSPFLOAT	demod	= TheDemodulator	-> demodulate (*in);

	fm_Levels	-> addItem (demod);
	if (viewSelector == 1)
	   *in	= demod;
//
//	deemphasize
	Re	= xkm1 = (demod - xkm1) * alpha + xkm1;
	Im	= ykm1 = (demod - ykm1) * alpha + ykm1;
	*audioOut	= DSPCOMPLEX (Re, Im);

	if (viewSelector == 2 || viewSelector == 3)
	   *in = *audioOut;
//
//	fully inspired by cuteSDR, we try to decode the rds stream
//	by simply am decoding it (after creating a decent complex
//	signal by Hilbert filtering)
	rdsBase	= DSPCOMPLEX (5 * demod, 5 * demod);
	rdsBase = rdsHilbertFilter -> Pass (rdsBandFilter -> Pass (rdsBase));
	if (viewSelector == 4)
	   *in = rdsBase;
	rds_plldecoder -> do_pll (rdsBase);
	if (viewSelector == 5)
	   *in = mySinCos -> getComplex (rds_plldecoder -> getNco ());
	DSPFLOAT rdsDelay = imag (rds_plldecoder -> getDelay ());
	if (viewSelector == 6)
	   *in = rdsDelay;
	*rdsValue = rdsLowPassFilter -> Pass (5 * rdsDelay);
	if (viewSelector == 7)
	   *in =  *rdsValue;
}

void	fmProcessor::stereo (DSPCOMPLEX	*in,
	                     DSPCOMPLEX	*audioOut,
	                     DSPFLOAT	*rdsValue) {

DSPFLOAT	LRPlus	= 0;
DSPFLOAT	LRDiff	= 0;
DSPFLOAT	pilot	= 0;
DSPFLOAT	currentPilotPhase;
DSPFLOAT	PhaseforLRDiff	= 0;
DSPFLOAT	PhaseforRds	= 0;
/*
 */
	DSPFLOAT demod	= TheDemodulator  -> demodulate (*in);

	fm_Levels	-> addItem (demod);
	if (viewSelector == 1)
	   *in = demod;
	LRPlus = LRDiff = pilot	= demod;
/*
 *	get the phase for the "carrier to be inserted" right
 */
	pilot		= pilotBandFilter -> Pass (5 * pilot);
	if (viewSelector == 2)
	   *in = pilot;
	currentPilotPhase = pilotRecover -> getPilotPhase (5 * pilot);
	if (viewSelector == 3)
	   *in = mySinCos	-> getComplex (currentPilotPhase);
/*
 *	Now we have the right - i.e. synchronized - signal to work with
 */
	PhaseforLRDiff	= 2 * (currentPilotPhase + pilotDelay);
	PhaseforRds	= 3 * (currentPilotPhase + pilotDelay);

	LRDiff	= 2 * mySinCos	-> getCos (PhaseforLRDiff) * LRDiff;
	if (viewSelector == 4)
	   *in = LRDiff;
//
	DSPCOMPLEX  MixerValue = mySinCos -> getComplex (PhaseforRds);
	if (viewSelector == 5)
	   *in = MixerValue;

	*rdsValue = 5 * rdsLowPassFilter -> Pass (real (MixerValue) * demod);
	if (viewSelector == 6 || viewSelector == 7)
	   *in =  *rdsValue;

//	apply deemphasis
	LRPlus		= xkm1	= (LRPlus - xkm1) * alpha + xkm1;
	LRDiff		= ykm1	= (LRDiff - ykm1) * alpha + ykm1;
        *audioOut		= DSPCOMPLEX (LRPlus, LRDiff);
	if (viewSelector == 8)
	   *in = *audioOut;
}

void	fmProcessor::setLFcutoff (int32_t Hz) {
	if (fmAudioFilter != NULL)
	   delete	fmAudioFilter;
	fmAudioFilter	= NULL;
	if (Hz > 0)
	   fmAudioFilter	= new LowPassIIR (10,
	                                          Hz,
	                                          Rate_in,
	                                          S_BUTTERWORTH);
}

bool	fmProcessor::isLocked (void) {
	return ((fmModus == FM_STEREO) && (pilotRecover -> isLocked ()));
}

