#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef	__FM_PROCESSOR
#define	__FM_PROCESSOR

#include	"jff-include.h"
#include	"iir-filters.h"
#include	"fft-filters.h"
#include	"sincos.h"
#include	<QObject>
#include	"utilities.h"
#include	"pllC.h"
#include	"pllR.h"
#include	"fm_levels.h"

class		RadioInterface;
class		Matched_Filter;
class		fft_scope;
class		fm_Demodulator;

class	fmProcessor:public QObject {
Q_OBJECT
public:
			fmProcessor (RadioInterface *,
	                             int32_t,
	                             SinCos *);
	        	~fmProcessor (void);
	void		setfmMode	(bool);
	void		setFMdecoder	(int8_t);
	void		setView		(int16_t);
	void		setSoundMode	(uint8_t);
	void		setSoundBalance	(int16_t);
	void		setDeemphasis	(int16_t);
	void		setVolume	(int16_t);
	void		setLFcutoff	(int32_t);
	void		processFM	(DSPCOMPLEX *, DSPCOMPLEX *, DSPFLOAT *);

	DSPFLOAT	getPilotStrength	(void);
	DSPFLOAT	getRdsStrength		(void);
	DSPFLOAT	getNoiseStrength	(void);
	DSPFLOAT	get_DcComponent		(void);
	bool		isLocked		(void);

	const char *	nameofDecoder	(void);

	enum Channels {
	   S_STEREO	= 0,
	   S_LEFT	= 1,
	   S_RIGHT	= 2,
	   S_LEFTplusRIGHT	= 0103,
	   S_LEFTminusRIGHT	= 0104
	};

private:
	enum Mode {
	   FM_STEREO	= 0,
	   FM_MONO	= 1
	};

	void		stereo	(DSPCOMPLEX *, DSPCOMPLEX *, DSPFLOAT *);
	void		mono	(DSPCOMPLEX *, DSPCOMPLEX *, DSPFLOAT *);
	RadioInterface	*myRadioInterface;
	int32_t		Rate_in;
	SinCos		*mySinCos;
	fftFilter	*pilotBandFilter;
	fftFilter	*rdsBandFilter;
	fftFilter	*rdsLowPassFilter;
	HilbertFilter	*rdsHilbertFilter;

	fmLevels	*fm_Levels;
	DSPFLOAT	pilotDelay;
	DSPCOMPLEX	audioGainCorrection	(DSPCOMPLEX);
	int16_t		Volume;
	DSPFLOAT	audioGain;
	int32_t		max_freq_deviation;
	int32_t		norm_freq_deviation;
	DSPFLOAT	omega_demod;
	LowPassIIR	*fmAudioFilter;

	int16_t		balance;
	DSPFLOAT	leftChannel;
	DSPFLOAT	rightChannel;
	uint8_t		fmModus;
	uint8_t		selector;
	DSPFLOAT	peakLevel;
	int32_t		peakLevelcnt;
	fm_Demodulator	*TheDemodulator;

	int8_t		viewSelector;
	pllC		*rds_plldecoder;
	DSPFLOAT	K_FM;

	DSPFLOAT	xkm1;
	DSPFLOAT	ykm1;
	DSPFLOAT	alpha;
	class	pilotRecovery {
	   private:
	      int32_t	Rate_in;
	      DSPFLOAT	pilot_OscillatorPhase;
	      DSPFLOAT	pilot_oldValue;
	      DSPFLOAT	omega;
	      DSPFLOAT	gain;
	      SinCos	*mySinCos;
	      DSPFLOAT	pilot_Lock;
	      bool	pll_isLocked;
	      DSPFLOAT	quadRef;
	      DSPFLOAT	accumulator;
	      int32_t	count;
	   public:
	      pilotRecovery (int32_t	Rate_in,
	                     DSPFLOAT	omega,
	                     DSPFLOAT	gain,
	                     SinCos	*mySinCos) {
	         this	-> Rate_in	= Rate_in;
	         this	-> omega	= omega;
	         this	-> gain		= gain;
	         this	-> mySinCos	= mySinCos;
	         pll_isLocked		= false;
	         pilot_Lock		= 0;
	         pilot_oldValue		= 0;
	         pilot_OscillatorPhase	= 0;
	      }

	      ~pilotRecovery (void) {
	      }

	      bool	isLocked (void) {
	         return pll_isLocked;
	      }

	      DSPFLOAT	getPilotPhase	(DSPFLOAT pilot) {
	      DSPFLOAT	OscillatorValue =
	                  mySinCos -> getCos (pilot_OscillatorPhase);
	      DSPFLOAT	PhaseError	= pilot * OscillatorValue;
	      DSPFLOAT	currentPhase;
	         pilot_OscillatorPhase += PhaseError * gain;
	         currentPhase		= PI_Constrain (pilot_OscillatorPhase);

	         pilot_OscillatorPhase =
	                   PI_Constrain (pilot_OscillatorPhase + omega);
	         
	         quadRef	= (OscillatorValue - pilot_oldValue) / omega;
//	         quadRef	= PI_Constrain (quadRef);
	         pilot_oldValue	= OscillatorValue;
	         pilot_Lock	= decayingAverage (pilot_Lock,
	                                            -quadRef * pilot, 30);
	         pll_isLocked	= pilot_Lock > 0.1;
	         return currentPhase;
	      }
	};
	      
	pilotRecovery	*pilotRecover;

signals:
	void		setPLLisLocked		(bool);

};

#endif

