#
/*
 *    Copyright (C) 2010,2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the  JSDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	<QSettings>
#include	<QMessageBox>
#include	<QFileDialog>
#include	<QDebug>
#include	<qwt_color_map.h>
#include	<qwt_plot_spectrogram.h>
#include	<qwt_scale_widget.h>
#include	<qwt_scale_draw.h>
#include	<qwt_plot_zoomer.h>
#include	<qwt_plot_panner.h>
#include	<qwt_plot_layout.h>
#include	<QDateTime>
#include	<QFile>
#include	"no_rig.h"
#include	"pmsdr.h"
#include	"replay.h"
#include	"dll-handler.h"
#include	"dabstick.h"
#include	"dabstick-dll.h"
#include	"gui.h"
#include	"utilities.h"
#include	"sound_processor.h"
#include	"fm_demodulator.h"
#include	"fm_processor.h"
#include	"rds_decoder.h"
#include	"decimator.h"
#include	"am_processor.h"
#include	"fft-scope.h"
#include	"scope.h"
#include	"oscillator.h"
#include	"fm-constants.h"

#ifdef __MINGW32__
#include	<iostream>
#include	<windows.h>
#endif
//
//	Radio devices supported directly:
#define	NO_RIG		0077
//#define	ELEKTOR		0100
#define	PMSDR		0101
#define	REPLAY		0102
#define	DABSTICK	0103
#define	DABSTICK_DLL	0104
#define	EXTIO		0105

//
//	Processing modes
#define	IDLE		0100
#define	PAUSED		0101
#define	RUNNING		0102

#define	NTI_OFFSET	Mhz(80)

static
//int16_t	delayTable [] = {15, 13, 11, 10, 9, 8, 7, 5, 3, 2, 1};
int16_t	delayTable [] = {1, 3, 5, 7, 9, 10, 15};
#define delayTableSize	((int)(sizeof (delayTable) / sizeof (int16_t)))
/*
 *	We use the creation function merely to set up the
 *	user interface and make the connections between the
 *	gui elements and the handling agents. All real action
 *	is embedded in actions, initiated by gui buttons
 */
	RadioInterface::RadioInterface (QSettings	*Si,
	                                bool		doInit,
	                                int32_t		CardRate,
	                                int32_t		WorkingRate,
	                                int32_t		OutputRate,
					int16_t		displaySize,
	                                int16_t		spectrumSize,
	                                int16_t		repeatRate,
	                                int16_t		rastersize,
	                                int16_t		multiplier,
	                                DSPCOMPLEX	*Table,
	                                QWidget		*parent): QDialog (parent) {
bool	soundSeemsOk;
int16_t	i;
// 	the setup for the generated part of the ui
	setupUi (this);

	ISettings			= Si;
	this		-> CardRate	= CardRate;
	this		-> WorkingRate	= WorkingRate;
	this		-> OutputRate	= OutputRate;
	this		-> displaySize	= displaySize;
	this		-> spectrumSize	= spectrumSize;
	this		-> repeatRate	= repeatRate;
	this		-> rastersize	= rastersize;
	this		-> doInit	= doInit;
	this		-> multiplier	= multiplier;
	this		-> omega	= 1.0f/CardRate * (2 * M_PI);

	this		-> mySinCos	= new SinCos (Table, CardRate);
	localOscillator		= new Oscillator (Table, CardRate);
//
//	we need (very) high latency
	MySoundcardOut		= new paWriter (OutputRate,
	                                        VERYHIGHLATENCY);
	outTable		= new int16_t
	                           [MySoundcardOut -> numberofDevices () + 10];
	for (i = 0; i < MySoundcardOut -> numberofDevices (); i ++)
	   outTable [i] = -1;
	soundSeemsOk		= setupSoundOut (streamOutSelector,
	                                         MySoundcardOut,
	                                         OutputRate, outTable);
	MySoundcardIn		= new paReader (CardRate);
	inTable			= new int16_t
	                            [MySoundcardIn -> numberofDevices () + 10];
	for (i = 0; i < MySoundcardIn -> numberofDevices (); i ++)
	   inTable [i] = -1;
	(void)setupSoundIn  (streamInSelector,
	                     MySoundcardIn,
	                     CardRate, inTable);
	runMode			= IDLE;
	IandQ_switch		= false;
//
//	for upscaling the samplerate (i.e. the 'M' option), we need
//	a light lowpass filter
	upFilter		= new LowPassFIR (11,
	                                          WorkingRate / 2, 
	                                          OutputRate);
/*
 *	we also setup the basis for the two displays, overlapped
 *	with the spectrograms, and the separate IQ display
 */
	HFScope			= new fft_scope (hfscope,
	                                         displaySize,
	                                         1000,
	                                         rastersize,
	                                         spectrumSize,
	                                         CardRate,
	                                         repeatRate);
	HFScope		-> SelectView (SPECTRUM_MODE);
	HFAverager	= false;
	HFScope		-> setAverager	(HFAverager);
	connect (HFScope,
	         SIGNAL (clickedwithLeft (int)),
	         this,
	         SLOT (AdjustFrequency (int)));
	LFScope			= new fft_scope (lfscope,
	                                         displaySize,
	                                         1,
	                                         rastersize,
	                                         spectrumSize,
	                                         CardRate,
	                                         repeatRate);
	LFScope		-> SelectView (SPECTRUM_MODE);
	LFAverager	= false;
	LFScope		-> setAverager	(LFAverager);
	connect (LFScope,
	         SIGNAL (clickedwithLeft (int)),
	         this,
	         SLOT (Zoom_in (int)));
	connect (LFScope,
	         SIGNAL (clickedwithRight (int)),
	         this,
	         SLOT (Zoom_out (int)));
	ClearPanel		();
	FrequencyChangeAllowed	= true;
/*
 *	he does the connections from the gui buttons, sliders etc
 */
	localConnects		();
//
//	The filters with initial - useless - values
	amFilter		= new BandPassFIR (51,
	                                           CardRate / 4 - 1000,
	                                           CardRate / 4 + 1000,
	                                           CardRate);
	fmFilter		= new LowPassFIR (21,
	                                          0.95 * CardRate / 2,
	                                          CardRate);

	inputFilename		= new char [1024];
	dumpBuffersize		= 1024;
	dumpBuffer		= new float [dumpBuffersize];
	dumpBufferpointer	= 0;
	DumpisOn		= FALSE;
	MyRig			= new no_rig ();
	CurrentRig		= NO_RIG;
	rigSelect		-> setCurrentIndex (0);
	rigAnnex		-> setCurrentIndex (1);
	inputNeeded		-> setCurrentIndex (0);
//
//	Create a timer for dealing with a non acting (or reacting) user
//	on the keyboard
	lcd_timer		= new QTimer ();
	lcd_timer		-> setSingleShot (TRUE);	// single shot
	lcd_timer		-> setInterval (5000);
	connect (lcd_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (lcd_timeout ()));
//
//	Handling dll's requires polling!!!, so we need a
//	timer
	dll_timer		= new QTimer ();
	dll_timer		-> setInterval (50);	// msec
	connect (dll_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (handleDLLRequest ()));
//
//	a timer for autoincrement/decrement of the tuning
	autoIncrement_timer	= new QTimer ();
	autoIncrement_timer	-> setSingleShot (TRUE);
	autoIncrement_timer	-> setInterval (5000);
	connect (autoIncrement_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (autoIncrement_timeout ()));

	log_timer		= new QTimer ();
	log_timer		-> setSingleShot (true);
	log_timer		-> setInterval (5000);
	connect (log_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (log_timeout ()));
/*
 *	just as a gadget, we show the actual time
 */
	display_timer		= new QTimer ();
	display_timer		-> setInterval (1000);
	connect (display_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (updateTimeDisplay ()));
//
//	Note that setting parameters for the decoders has
//	to wait until the decoders are instantiated!!!
	audioDecimator		= new downDecimator (CardRate, WorkingRate);
	myFMprocessor		= new fmProcessor (this,
	                                           CardRate,
	                                           mySinCos);
	myRdsDecoder		= new rdsDecoder (this,
	                                          WorkingRate / 2,
	                                          mySinCos);
	rdsDecimator		= new downDecimator (CardRate, WorkingRate / 2);
	myAMprocessor		= new amProcessor (this, WorkingRate);
	mySoundprocessor	= new soundProcessor (CardRate, 2000, 1500);

	lcd_InputRate		-> display ((int)CardRate);
	lcd_OutputRate		-> display ((int)OutputRate);
	char	*s = (char *)alloca (256);
	sprintf (s, "sdrJ-FM-V%s", CURRENT_VERSION);
	systemindicator		-> setText (QString (s));
	display_timer		-> start (1000);
//
	currentPIcode		= 0;
	frequencyforPICode	= 0;
//	all elements seem to exist: set the control state to
//	its default values
	setInitialControlState	();
//	if we have a ".ini" file, we overrule the initial control state
	if ((ISettings != NULL) && doInit)
	   setControlState	(ISettings);
//
//	ready to go, one last check:

	if (soundSeemsOk && CardRate >= 150000)
	   qDebug ("GUI is set up and working\n"); 
	else 
	if (!soundSeemsOk) {
	   qDebug ("No Channels/streams\n");
	   accept ();
	   exit (21);	
	}
	else {
	   qDebug ("Sorry, we only run at speed of 150K or more\n");
	   accept ();
	   exit (22);
	}
}

	RadioInterface::~RadioInterface () {
fprintf (stderr, " going to destruct\n");
	delete		HFScope;
	delete		LFScope;
	delete[]	dumpBuffer;
	delete[]	inputFilename;

	delete		myFMprocessor;
	delete		myRdsDecoder;
	delete		audioDecimator;
	delete		rdsDecimator;
	delete		mySoundprocessor;
	delete		myAMprocessor;
	delete[]	inTable;
	delete[]	outTable;
	delete		lcd_timer;
	delete		autoIncrement_timer;
	delete		dll_timer;
	delete		log_timer;
	delete		display_timer;
	delete		MySoundcardIn;
	delete		MySoundcardOut;
	delete		MyRig;
fprintf (stderr, "end of destruction\n");
}
//
//	Eventually, we want that each individual element
//	of the control state can be overruled. We start
//	with showing this state through default settings
//	This can be overruled when an inifile is
//	provided.
void	RadioInterface::setInitialControlState	(void) {

	logFile			= NULL;

	pauseButton		-> setText (QString ("Pause"));

	dumpON			-> setText ("dumpOn");
	DumpisOn		= false;

	incrementingFlag ->
	                 setStyleSheet ("QLabel {background-color:blue}");
	incrementingFlag -> setText (" ");
	IncrementIndex		= 0;

	inputDevice		= MySoundcardIn		-> invalidDevice ();
	outputDevice		= MySoundcardOut	-> invalidDevice ();

	set_converter		(converter		-> currentText ());
	set_am_increment	(am_increment		-> value ());
	set_fm_increment	(fm_increment		-> value ());
	setAmplification	(spectrumAmplitudeSlider -> value ());
	setAttenuation		(attenuationSlider	 -> value ());
	setIQBalance		(IQbalanceSlider	-> value ());
	
	setInputMode		(inputModeSelect	-> currentText ());
	setVolume		(VolumeSlider		-> value ());
//
//	now setting the parameters for the fm decoder
	setfmBandwidth		(fmFilterSelect		-> currentText ());
	setfmBandwidth		(fmFilterDegree		-> value ());
	setfmMode		(fmMode			-> currentText ());
	setfmRdsSelector	(fmRdsSelector		-> currentText ());
	setfmDecoder		(fmDecoder		-> currentText ());
	setfmView		(fmView			-> currentText ());
	setfmChannelSelector	(fmChannelSelect	-> currentText ());
	setfmDeemphasis		(fmDeemphasisSelector	-> currentText ());
	setfmStereoSlider	(fmStereoSlider		-> value ());
	setfmLFcutoff		(fmLFcutoff		-> currentText ());
	setLogging		(logging		-> currentText ());

	setamDecoder		(amDecoderSelector	-> currentText ());
	setamBandwidth		(amBandwidthSlider	-> value ());
	set_amagc		(am_agc			-> currentText ());
	set_agcThresholdSlider	(agcThresholdSlider	-> value ());

	amButton		-> setChecked (false);
	audioTestButton		-> setChecked (false);
	fmButton		-> setChecked (true);	// gives a signal

	set_rigSelect		(rigSelect		-> currentText ());
	PMSDR_setmute		(pmsdr_mute		-> currentText ());

	am_min_loop_frequency	= AM_MIN_LOOP_FREQUENCY;
	am_max_loop_frequency	= AM_MAX_LOOP_FREQUENCY;
	fm_min_loop_frequency	= FM_MIN_LOOP_FREQUENCY;
	fm_max_loop_frequency	= FM_MAX_LOOP_FREQUENCY;
}
//
//	We have the possibility of creating a ".ini" file
void	RadioInterface::dumpControlState	(QSettings *s) {
	if (s == NULL)
	   return;

	s	-> setValue ("converter",
	                               converter -> currentText ());
	s	-> setValue ("am_increment",
	                               (double)(am_increment -> value ()));
	s	-> setValue ("fm_increment",
	                               fm_increment -> value ());
	s	-> setValue ("spectrumAmplitudeSlider",
	                               spectrumAmplitudeSlider -> value ());
	s	-> setValue ("attenuationSlider",
	                               attenuationSlider -> value ());
	s	-> setValue ("IQbalanceSlider",
	                               IQbalanceSlider	-> value ());
	s	-> setValue ("inputModeSelect",
	                               inputModeSelect	-> currentText ());
	s	-> setValue ("VolumeSlider",
	                               VolumeSlider	-> value ());
//
//	now setting the parameters for the fm decoder
	s	-> setValue ("fmFilterSelect", 
	                               fmFilterSelect	-> currentText ());
	s	-> setValue ("fmFilterDegree",
	                               fmFilterDegree	-> value ());

	s	-> setValue ("fmMode",
	                               fmMode 		-> currentText ());
	s	-> setValue ("fmDecoder",
	                               fmDecoder	-> currentText ());
	s	-> setValue ("fmRdsSelector",
	                               fmRdsSelector	-> currentText ());
	s	-> setValue ("fmView",
	                               fmView		-> currentText ());
	s	-> setValue ("fmChannelSelect",
	                               fmChannelSelect	-> currentText ());
	s	-> setValue ("fmDeemphasisSelector",
	                               fmDeemphasisSelector -> currentText ());
	s	-> setValue ("fmStereoSlider",
	                               fmStereoSlider	-> value ());
	s	-> setValue ("logging",
	                               logging		-> currentText ());
	s	-> setValue ("amBandwidthSlider",
	                               amBandwidthSlider -> value ());
	s	-> setValue ("am_agc",
	                               am_agc		-> currentText ());

	s	-> setValue ("amDecoderSelector",
	                               amDecoderSelector -> currentText ());
	s	-> setValue ("agcThresholdSlider",
	                               agcThresholdSlider -> value ());
	s	-> setValue ("amButton",
	                               amButton -> isChecked () ? 1 : 0);
	s	-> setValue ("audioTestButton",
	                               audioTestButton -> isChecked () ? 1 : 0);
	s	-> setValue ("fmButton",
	                               fmButton -> isChecked () ? 1 : 0);
	s	-> setValue ("rigSelect", 
	                               rigSelect -> currentText ());
//
//	in the ini file, we specify frequency in KHz
	s	-> setValue ("frequency",
	                               (int)CurrentFreq / Khz (1));
	s	-> setValue ("pmsdr_mute",
	                               pmsdr_mute -> currentText ());
	s	-> setValue ("streamInSelector",
	                               streamInSelector -> currentText ());
	s	-> setValue ("streamOutSelector",
	                               streamOutSelector -> currentText ());
	s	-> setValue ("IncrementIndex",
	                               IncrementIndex);

	s	-> setValue ("am_min_loop_frequency",
	                               am_min_loop_frequency);
	s	-> setValue ("am_max_loop_frequency",
	                               am_max_loop_frequency);
	s	-> setValue ("fm_min_loop_frequency",
	                               fm_min_loop_frequency);
	s	-> setValue ("fm_max_loop_frequency",
	                               fm_max_loop_frequency);
}
//
//	This does the trick: if an ".ini" file is specified,
//	the specified settings are restored. Note that
//	the ini file does not need to provide all settings,
void	RadioInterface::setControlState	(QSettings *s) {
QString	h;
int	k;
double	f;

	if (s == NULL)
	   return;

	h	= s -> value ("converter", "direct"). toString ();
	k	= converter	-> findText (h);
	if (k != -1) {
	   converter -> setCurrentIndex (k);
	   set_converter (h);
	}

	f	= s -> value ("am_increment", 4.50f). toDouble ();
	am_increment		-> setValue (f);	// will signal

	k	= s -> value ("fm_increment", 100). toInt ();
	fm_increment		-> setValue (k);	// will signal

	k	= s -> value ("spectrumAmplitudeSlider", 50). toInt ();
	spectrumAmplitudeSlider	-> setValue (k);

	k	= s -> value ("attenuationSlider", 50). toInt ();
	attenuationSlider	-> setValue (k);

	k	= s -> value ("IQbalanceSlider", 0). toInt ();
	IQbalanceSlider		-> setValue (k);	// will signal

	h	= s -> value ("inputModeSelect", "I and Q"). toString ();
	k	= inputModeSelect -> findText (h);
	if (k != -1) {
	   inputModeSelect -> setCurrentIndex (k);
	   setInputMode (h);
	}

	k	= s -> value ("VolumeSlider", 10). toInt ();
	VolumeSlider		-> setValue (k);	// will trigger

	h	= s -> value ("fmFilterSelect", "---"). toString ();
	k	= fmFilterSelect -> findText (h);
	if (k != -1) {
	   fmFilterSelect -> setCurrentIndex (k);
	   setfmBandwidth (h);
	}

	k	= s -> value ("fmFilterDegree", 21). toInt ();
	setfmBandwidth (k);
	fmFilterDegree		-> setValue (k);

	h	= s -> value ("fmMode", "stereo"). toString ();
	k	= fmMode -> findText (h);
	if (k != - 1) {
	   fmMode -> setCurrentIndex (k);
	   setfmMode (h);
	}

	h	= s -> value ("fmDecoder", "fm1decoder"). toString ();
	k	= fmDecoder -> findText (h);
	if (k != -1) {
	   fmDecoder -> setCurrentIndex (k);
	   setfmDecoder (h);
	}

	h	= s -> value ("fmRdsSelector", "rds1"). toString ();
	k	= fmRdsSelector -> findText (h);
	if (k != -1) {
	   fmRdsSelector -> setCurrentIndex (k);
	   setfmRdsSelector (h);
	}

	h	= s -> value ("fmView", "view 1"). toString ();
	k	= fmView -> findText (h);
	if (k != -1) {
	   fmView -> setCurrentIndex (k);
	   setfmView (h);
	}

	h	= s -> value ("fmChannelSelect", "stereo"). toString ();
	k	= fmChannelSelect -> findText (h);
	if (k != -1) {
	   fmChannelSelect -> setCurrentIndex (k);
	   setfmChannelSelector (h);
	}

	h	= s -> value ("fmDeemphasisSelector", "none"). toString ();
	k	= fmDeemphasisSelector -> findText (h);
	if (k != -1) {
	   fmDeemphasisSelector -> setCurrentIndex (k);
	   setfmDeemphasis (h);
	}

	k	= s -> value ("fmStereoSlider", 0). toInt ();
	fmStereoSlider	-> setValue (k);

	h	= s -> value ("fmLFcutoff", "off"). toString ();
	k	= fmLFcutoff -> findText (h);
	if (k != -1) {
	   fmLFcutoff -> setCurrentIndex (k);
	   setfmLFcutoff (h);
	}

	h	= s -> value ("logging", "log off"). toString ();
	k	= logging -> findText (h);
	if (k != -1) {
	   logging -> setCurrentIndex (k);
	   setLogging (h);
	}

	k	= s -> value ("amBandwidthSlider", 50). toInt ();
	amBandwidthSlider	-> setValue (k);	// will trigger

	h	= s -> value ("am_agc", "agc off"). toString ();
	k	= am_agc -> findText (h);
	if (k != -1) {
	   am_agc -> setCurrentIndex (k);
	   set_amagc (h);
	}

	k	= s -> value ("agcThresholdSlider", 50). toInt ();
	agcThresholdSlider	-> setValue (k);

	h	= s -> value ("amDecoderSelector", "am2"). toString ();
	k	= amDecoderSelector -> findText (h);
	if (k != -1) {
	   amDecoderSelector -> setCurrentIndex (k);
	   setamDecoder (h);
	}

	k	= s -> value ("amButton", 0). toInt ();
	amButton	-> setChecked (k != 0);
	
	k	= s -> value ("audioTestButton", 0). toInt ();
	audioTestButton	-> setChecked (k != 0);

	k	= s -> value ("fmButton", 0). toInt ();
	fmButton	-> setChecked (k != 0);

	h	= s -> value ("rigSelect", "no device"). toString ();
	k	= rigSelect	-> findText (h);
	if (k != -1) {
	   rigSelect -> setCurrentIndex (k);
	   set_rigSelect (h);
	}

	h	= s -> value ("pmsdr_mute", "no mute"). toString ();
	k	= pmsdr_mute -> findText (h);
	if (k != - 1) {
	   pmsdr_mute -> setCurrentIndex (k);
	   PMSDR_setmute (h);
	}
//
//	we specify frequency in Khz
	k	= s -> value ("frequency", 89300). toInt ();
	Set_Tuner (Khz (k), &CurrentFreq);
	Display (CurrentFreq, demodulatorModus);

	h	= s -> value ("streamInSelector", "select input"). toString ();
	k	= streamInSelector -> findText (h);
	if (k != -1) {
	   streamInSelector -> setCurrentIndex (k);
	   setStreamInSelector (k);
	}

	h	= s -> value ("streamOutSelector", "select output"). toString ();
	k	= streamOutSelector -> findText (h);
	if (k != -1) {
	   streamOutSelector -> setCurrentIndex (k);
	   setStreamOutSelector (k);
	}

	IncrementIndex	= s -> value ("IncrementIndex", 0). toInt ();
	am_min_loop_frequency	=
	 s -> value ("am_min_loop_frequency", AM_MIN_LOOP_FREQUENCY). toInt ();
	am_max_loop_frequency	=
	 s -> value ("am_max_loop_frequency", AM_MAX_LOOP_FREQUENCY). toInt ();
	fm_min_loop_frequency	=
	 s -> value ("fm_min_loop_frequency", FM_MIN_LOOP_FREQUENCY). toInt ();
	fm_max_loop_frequency	=
	 s -> value ("fm_max_loop_frequency", FM_MAX_LOOP_FREQUENCY). toInt ();
	if (IncrementIndex != 0)	// restart the incrementtimer if needed
	   autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
}

//
//	On start, we ensure that the streams are stopped so
//	that they can be restarted again.
void	RadioInterface::setStart	(void) {
bool	r = 0;

	if (runMode == RUNNING)
	   return;
//
//	always ensure that there is no datastream
	if (MyRig -> RigprovidesSamples ())
	   MyRig	-> stopReader ();
	else
	   MySoundcardIn -> stopReader ();
//
//	to prevent underflow in the output, we stop the
//	output writer
	MySoundcardOut -> stopWriter ();

	if (MyRig	-> RigprovidesSamples ())
	   disconnect (MyRig,
	               SIGNAL (samplesAvailable ()),
	               this,
	               SLOT (JansRadioThroughSignal ()));
	else
	   disconnect (MySoundcardIn,
	               SIGNAL (samplesAvailable ()),
	               this,
	               SLOT (JansRadioThroughSignal ()));
//
//
	if (MyRig -> RigprovidesSamples ())	
	   r = MyRig		-> restartReader ();
	else
	   r = MySoundcardIn	-> restartReader ();

	qDebug ("Starting %d\n", r);
	if (!r) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Opening  input stream failed\n"));
	   return;
	}

	r = MySoundcardOut	-> restartWriter ();
	if (!r) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Opening  output stream failed\n"));
	   if (MyRig -> RigprovidesSamples ())
	      MyRig	-> stopReader ();
	   else
	      MySoundcardIn -> stopReader ();
	   return;
	}
//	and we are on the move, so connect
	if (MyRig -> RigprovidesSamples ())
	   connect (MyRig,
	            SIGNAL (samplesAvailable ()),
	            this,
	            SLOT (JansRadioThroughSignal ()));
	else
	   connect (MySoundcardIn,
	            SIGNAL (samplesAvailable ()),
	            this,
	            SLOT (JansRadioThroughSignal ()));

//
//	and finally: recall that starting overrules pausing
	pauseButton -> setText (QString ("Pause"));
	runMode	= RUNNING;
	Set_Tuner (CurrentFreq, &CurrentFreq);	// for the rtlsdr plugin
}

void	RadioInterface::TerminateProcess () {
	if (DumpisOn)
	   setDumpisOff ();

	if (ISettings != NULL)
	   dumpControlState	(ISettings);
	set_rigSelect ("no rig");
	stop_lcdTimer		();
	stopIncrementing	();
//
//	The easiest way for a controlled going down is to
//	ensure that a "no rig" is the selected rig
	accept ();
	qDebug () <<  "Termination started";
}

void	RadioInterface::abortSystem (int d) {
	qDebug ("aborting for reason %d\n", d);
	accept ();
}
//
void	RadioInterface::set_rigSelect (const QString &s) {
int16_t	newRig;
bool	success;
QString	file;
//
//	first, find out the rig to be selected
	newRig =  s == "pmsdr" ? PMSDR :
	          s == "replay" ? REPLAY:
	          s == "dabstick" ? DABSTICK :
	          s == "dabstick (dll)" ? DABSTICK_DLL :
	          s == "extio" ? EXTIO : NO_RIG;

	if ((CurrentRig != NO_RIG) && (CurrentRig == newRig))
	   return;
//	
//	ensure that the previous rig goes down in a controlled way.
//	Stop some timers and register that we are IDLE
	runMode			= IDLE;
	stop_lcdTimer		();
	stopIncrementing	();
//
//	if the currentrig is driven by a timer, we stop
//	polling clock
	dll_timer	-> stop ();	// stop polling

	if (CurrentRig == EXTIO) {
//	   ((dll_handler *)MyRig)	-> HideGUI ();
	   disconnect (MyRig,
	               SIGNAL (setDLLRequest (int)),
	               this,
	               SLOT (setDLLRequest (int)));
	}
//
//	ok, we stop the samplestream in
//	disconnect the signal for the inputstream
	if (MyRig -> RigprovidesSamples ()) {
	   disconnect (MyRig,
	               SIGNAL (samplesAvailable ()),
	               this,
	               SLOT (JansRadioThroughSignal ()));
	   MyRig -> stopReader ();
	}
	else {
	   disconnect (MySoundcardIn,
	               SIGNAL (samplesAvailable ()),
	               this,
	               SLOT (JansRadioThroughSignal ()));
	   MySoundcardIn -> stopReader ();
	}
//	then the stream out 
	MySoundcardOut	-> stopWriter ();
//	and finally, delete the device
	usleep (100);
	delete MyRig;
fprintf (stderr, "MyRig is deleted\n");
//
//	OK, the old rig is shut down, set a default
	CurrentRig	= NO_RIG;		// default
	switch (newRig) {
	   case REPLAY: 
	      file = QFileDialog::getOpenFileName (this,
	                                           tr ("open file .."),
	                                           QDir::homePath (),
	                                           tr ("Sound (*.wav)"));
	      file = QDir::toNativeSeparators (file);
	      MyRig		= new replayRig (CardRate,
	                                         file. toAscii (). data (),
	                                         &success);
	      if (success) {
	         CurrentRig	= REPLAY;
	      }
	      else {
	         QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Could not open file"));
	         delete MyRig;
	         MyRig		= new no_rig ();
	         rigSelect		-> setCurrentIndex (0);
	      }
	      break;

	   case EXTIO:
#ifdef __MINGW32__
	      file = QFileDialog::getOpenFileName (this,
	                                           tr ("open file .."),
	                                           QDir::currentPath (),
	                                           tr ("dll (Extio*.dll)"));
#else
	      file = QFileDialog::getOpenFileName (this,
	                                           tr ("open file ..."),
	                                           QDir::currentPath (),
	                                           tr ("so (Extio*.so)"));
#endif
	      file	= QDir::toNativeSeparators (file);
	      MyRig	= new dll_handler (file. toAscii (). data (),
	                                   &success, CardRate);
	      if (success) {
	         CurrentRig = EXTIO;
	         dll_label -> setText (MyRig -> nameofRig ());

//	         if (MyRig -> isDONGLE ())
//	            ((dll_handler *)MyRig) -> ShowGUI ();
//	         else
	            ((dll_handler *)MyRig) -> HideGUI ();
	
	         connect (MyRig,
	                  SIGNAL (setDLLRequest (int)),
	                  this,
	                  SLOT (setDLLRequest (int)));
	         dll_timer	-> start (50);
	      } 
	      else {
	         QMessageBox::warning (this, tr ("sdr"),
	                                     tr ("error in opening"));
	         delete MyRig;
	         MyRig	= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	      }
	      break;

	   case DABSTICK:
	      MyRig	= new dabstick (CardRate,
	                                CardRate * multiplier, &success);
	      if (success) {
	         CurrentRig	= DABSTICK;
	         fprintf (stderr, "found the rig\n");
//
//	experiment: 
//	         dll_timer	-> start (50);
	      }
	      else {
	         QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Could not open dongle"));
	         delete (dabstick *)MyRig;
	         MyRig		= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	      }
	      break;

	   case DABSTICK_DLL:
	      MyRig	= new dabstick_dll (CardRate,
	                                    CardRate * multiplier, &success);
	      if (success) {
	         CurrentRig	= DABSTICK;
	         fprintf (stderr, "found the rig\n");
//
//	experiment: 
//	         dll_timer	-> start (50);
	      }
	      else {
	         QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Could not open dongle"));
	         delete (dabstick_dll *)MyRig;
	         MyRig		= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	      }
	      break;

	   case PMSDR:
	      MyRig		= new pmSDR (this, &success);
	      if (success) {
	         CurrentRig	= PMSDR;
	      }
	      else {
	         QMessageBox::warning(this, tr ("sdr"),
	                                    tr ("Could not open pmsdr"));
	         delete MyRig;
//	         CurrentRig	= NO_RIG;
	         MyRig		= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	         break;
	      }
	      break;
	      
	   default:			// should not happen
	   case NO_RIG:
	      CurrentRig	= NO_RIG;
	      MyRig		= new no_rig ();
	      rigSelect		-> setCurrentIndex (0);
	      break;
	}
//
//	if the rig itself provides samples, we do not need
//	the combobox for the input selection
	if (MyRig -> RigprovidesSamples ())
	   inputNeeded	-> setCurrentIndex (1);
	else
	   inputNeeded	-> setCurrentIndex (0);
//
//	we invented "RigAnnex" as name for the plane with specific
//	pmsdr buttons and inputNeeded for the plane with the input selector
//	setting some planes:
	switch (CurrentRig) {
	   case DABSTICK:
	   case DABSTICK_DLL:
	   default:
	   case REPLAY:
	   case NO_RIG:
	      rigAnnex		-> setCurrentIndex (1);
	      break;

	   case PMSDR:
	      rigAnnex		-> setCurrentIndex (0);
	      break;

	   case EXTIO:
	      rigAnnex		-> setCurrentIndex (2);
	      break;
	}

	Set_Tuner (MyRig -> defaultFrequency (), &CurrentFreq);
	Display (CurrentFreq, demodulatorModus);
}

void	RadioInterface::setHFAverager (void) {
	HFAverager	= !HFAverager;
	HFScope	-> setAverager (HFAverager);
}

void	RadioInterface::setLFAverager (void) {
	LFAverager	= !LFAverager;
	LFScope	-> setAverager (LFAverager);
}

void	RadioInterface::setAmplification (int a) {
	HFScope -> setAmplification (a);
	LFScope -> setAmplification (a);
}

void	RadioInterface::setInputMode (const QString &s) {
	if (s == "I and Q")
	   inputMode	= IandQ;
	else
	if (s == "Q and I")
	   inputMode	= QandI;
	else
	if (s == "I Only")
	   inputMode	= I_Only;
	else
	if (s == "Q Only")
	   inputMode	= Q_Only;
	else
	   inputMode	= IandQ;
}

void	RadioInterface::setfmChannelSelector (const QString &s) {
	if (s == "stereo")
	   channelSelector	= fmProcessor::S_STEREO;
	else
	if (s == "Left")
	   channelSelector	= fmProcessor::S_LEFT;
	else
	if (s == "Right")
	   channelSelector	= fmProcessor::S_RIGHT;
	else
	if (s == "Left+Right")
	   channelSelector	= fmProcessor::S_LEFTplusRIGHT;
	else
	if (s == "Left-Right")
	   channelSelector	= fmProcessor::S_LEFTminusRIGHT;
	else		// the default
	   channelSelector	= fmProcessor::S_LEFT;
	myFMprocessor	-> setSoundMode	(channelSelector);
}

void	RadioInterface::set_amDecoder (bool checked) {
	if (checked) {
	   demodulatorModus = AM_DECODER;
	   setDetectorScreen (AM_DECODER);
	   delete LFScope;
	   LFScope	= new fft_scope (lfscope,
	                                 displaySize,
	                                 1,
	                                 rastersize,
	                                 spectrumSize,
	                                 WorkingRate,
	                                 repeatRate);
	   LFScope	-> SelectView (SPECTRUM_MODE);

	   currentScope	= AM_DECODER;
	   setAmplification (spectrumAmplitudeSlider -> value ());
	   Set_Tuner (CurrentFreq, &CurrentFreq);
	   Display (CurrentFreq, AM_DECODER);
	}
}

void	RadioInterface::set_fmDecoder (bool checked) {
	if (checked) {
	   demodulatorModus = FM_DECODER;
	   setDetectorScreen (FM_DECODER);
	   delete LFScope;
	   LFScope	= new fft_scope (lfscope,
	                                 displaySize,
	                                 1,
	                                 rastersize,
	                                 spectrumSize,
	                                 CardRate,
	                                 repeatRate);
	   LFScope	-> SelectView (SPECTRUM_MODE);

	   connect (LFScope,
	            SIGNAL (clickedwithLeft (int)),
	            this,
	            SLOT (Zoom_in (int)));
	   connect (LFScope,
	            SIGNAL (clickedwithRight (int)),
	            this,
	            SLOT (Zoom_out (int)));
	   currentScope	= FM_DECODER;
	   setAmplification (spectrumAmplitudeSlider -> value ());
	   Set_Tuner (CurrentFreq, &CurrentFreq);
	   Display (CurrentFreq, FM_DECODER);
	}
}

void	RadioInterface::set_audioTest (bool checked) {
	if (checked) {
	   demodulatorModus = AUDIO_TEST;
	   setDetectorScreen (AUDIO_TEST);
	   delete LFScope;
	   LFScope	= new fft_scope (lfscope,
	                                 displaySize,
	                                 1,
	                                 rastersize,
	                                 spectrumSize,
	                                 CardRate,
	                                 repeatRate);
	   LFScope	-> SelectView (SPECTRUM_MODE);
	   currentScope	= AUDIO_TEST;
	   setAmplification (spectrumAmplitudeSlider -> value ());
	}
}

void	RadioInterface::setAttenuation (int n) {
int16_t	f	= IQBalanceDisplay	-> value ();
int16_t	bl, br;

	bl	= 100 - 2 * f;
	br	= 100 + 2 * f;
	attenuationLevelDisplay	-> display (n);
	currAttSliderValue	= 2 * n;
	attValueL	= currAttSliderValue * (float)bl / 100;
	attValueR	= currAttSliderValue * (float)br / 100;
	myAMprocessor	-> setAgcLevel ((int)(attValueL + attValueR) / 2);
}
/*
 * 	the balance slider runs between -30 .. 30
 */
void	RadioInterface::setIQBalance (int n) {
int16_t	bl, br;

	IQBalanceDisplay -> display (n);
	bl		= 100 - 2 * n;
	br		= 100 + 2 * n;
	attValueL	= currAttSliderValue * (float)bl / 100;
	attValueR	= currAttSliderValue * (float)br / 100;
}
/*
 * 	Handling the numeric keypad is boring, but inevitable
 */
void RadioInterface::addOne (void) {
	AddtoPanel (1);
}

void RadioInterface::addTwo (void) {
	AddtoPanel (2);
}

void RadioInterface::addThree (void) {
	AddtoPanel (3);
}

void RadioInterface::addFour (void) {
	AddtoPanel (4);
}

void RadioInterface::addFive (void) {
	AddtoPanel (5);
}

void RadioInterface::addSix (void) {
	AddtoPanel (6);
}

void RadioInterface::addSeven (void) {
	AddtoPanel (7);
}

void RadioInterface::addEight (void) {
	AddtoPanel (8);
}

void RadioInterface::addNine (void) {
	AddtoPanel (9);
}

void RadioInterface::addZero (void) {
	AddtoPanel (0);
}

void RadioInterface::addClear (void) {
	if (!lcd_timer -> isActive ())
	   return;

	stop_lcdTimer	();
//	we know that the increment timer is not running

	ClearPanel ();
	Display (CurrentFreq, demodulatorModus);
}
//
//	Note that in fm receiver, user commands are in KHz
//
void RadioInterface::AcceptFreqinKhz (void) {
int32_t	p;

	if (!lcd_timer -> isActive ())
	   return;

	stop_lcdTimer	();
//	we know that the increment timer is not running,
//	so there is no reason to stop it
	p = getPanel ();
	ClearPanel ();
	Set_Tuner (p * 1000, &CurrentFreq);
	Display (CurrentFreq, demodulatorModus);
}

void RadioInterface::addCorr (void) {
	if (!lcd_timer -> isActive ()) 
	   return;

	stop_lcdTimer	();
//	increment timer cannot run now so no reason
//	to stop it
	Panel = (Panel - (Panel % 10)) / 10;
	Display	(Panel, NO_DECODER);
	lcd_timer	-> start (5000);		// restart timer
}
//
//	In AddtoPanel, we might have any of the two
//	timers running, do not know which one though
void	RadioInterface::AddtoPanel (int16_t n) {
	stop_lcdTimer		();
	stopIncrementing	();

	Panel		= 10 * Panel + n;
	Display (Panel, NO_DECODER);
	lcd_timer	-> start (5000);		// restart timer
}

void	RadioInterface::ClearPanel (void) {
	Panel = 0;
}

int	RadioInterface::getPanel (void) {
	return Panel;
}

void RadioInterface::decT5 (void) {
	IncrementFrequency (mapIncrement (-1));
}

void RadioInterface::decT50 (void) {
	IncrementFrequency (mapIncrement (-10));
}

void RadioInterface::decT500 (void) {
	IncrementFrequency (mapIncrement (-100));
}

void RadioInterface::decT5000 (void) {
	IncrementFrequency (mapIncrement (-1000));
}

void RadioInterface::incT5 (void) {
	IncrementFrequency (mapIncrement (1));
}

void RadioInterface::incT50 (void) {
	IncrementFrequency (mapIncrement (10));
}

void RadioInterface::incT500 (void) {
	IncrementFrequency (mapIncrement (100));
}

void RadioInterface::incT5000 (void) {
	IncrementFrequency (mapIncrement (1000));
}
//
//	Increment frequency: with amount N, depending
//	on the mode of operation
//
int32_t 	RadioInterface::mapIncrement (int32_t n) {
	switch (demodulatorModus) {
	   case FM_DECODER:
	      return n * 1000;

	   default:
	   case AM_DECODER:
	      return n * 100;
	}
}
//
//	IncrementFrequency is called from the handlers
//	for the autoincrement, the increment and the
//	++, + etc knobs.
//	Deviations within 15K of the VFO are handled with an
//	offset "onscreen"
void	RadioInterface::IncrementFrequency (int32_t n) {
int32_t	vfoFreq;
	stop_lcdTimer	();
	stopIncrementing	();

	MyRig	-> getVFOFrequency (&vfoFreq);
	if (abs (vfoFreq - (CurrentFreq + n)) > KHz (15))
	   Set_Tuner (CurrentFreq + n, &CurrentFreq);
	else {
	   CurrentFreq += n;
	   LOFrequency	= CurrentFreq - vfoFreq;
	   HFScope -> setNeedle (LOFrequency);
	}
	Display (CurrentFreq, demodulatorModus);
}
//	AdjustFrequency is called whenever someone clicks
//	with the button on the screen. The amount
//	has to be multiplied with 1000
void	RadioInterface::AdjustFrequency (int n) {
	IncrementFrequency (n * 1000);
}
//
//	Whenever the mousewheel is changed, the frequency
//	is adapted
void	RadioInterface::wheelEvent (QWheelEvent *e) {
	if (e -> delta () > 0)
	   IncrementFrequency (KHz (1));
	else
	   IncrementFrequency (-KHz (1));
}

void	RadioInterface::Zoom_in (int n) {
	LFScope	-> setZoompoint (n);
}

bool	RadioInterface::RigwithThirdHarmonic (int16_t rig) {
	return MyRig -> isNO_RIG () || MyRig -> isPMSDR ();
}

void	RadioInterface::Zoom_out (int n) {
	(void)n;
	LFScope -> resetZoompoint ();
}
//
//	Set_Tuner accepts freq in Hz.
//	It handles the whole range of frequencies,
//	including the 3-th harmonic trick for the pmsdr
//
//	There are many cases to distinguish when tuning, i.e.
//	are we working with the 3-d harmonic? is
//	the NTI on?
//
void	RadioInterface::Set_TunerforNTI (int32_t n, int32_t *c) {
int32_t	TunedFrequency;
int32_t	VFOFrequency;
int32_t	vfo;
int16_t	filter;

	if (!MyRig -> legalFrequency (n)) 
	   n = MyRig -> defaultFrequency () + NTI_OFFSET;

	TunedFrequency		= n - NTI_OFFSET;
	VFOFrequency		= demodulatorModus == AM_DECODER ?
	                                TunedFrequency - CardRate / 4 :
	                                TunedFrequency;
	MyRig			-> setVFOFrequency (VFOFrequency);
	MyRig			-> getVFOFrequency (&vfo);
	MyRig			-> setFilter	(TunedFrequency, &filter);
	HFScope			-> setZero	(vfo + NTI_OFFSET);
	LOFrequency		= TunedFrequency - vfo;
	HFScope			-> setNeedle (TunedFrequency - vfo);
	HFScope			-> clearAverage	();
	LFScope			-> setZero (0);
	LFScope			-> clearAverage	();
	*c			= n;
	if (demodulatorModus == FM_DECODER)
	   myRdsDecoder	-> reset ();
}

void	RadioInterface::Set_TunerforThirdHarmonic (int32_t n, int32_t *c) {
int32_t	VFOFrequency;
int32_t	vfo;
int16_t	filter;

	VFOFrequency		= demodulatorModus == AM_DECODER ?
	                                n - CardRate / 4 :
	                                n;
	VFOFrequency 		/= 3;

	if (!MyRig -> legalFrequency (VFOFrequency)) {
	   Set_Tuner (3 * MyRig -> defaultFrequency (), c);
	   return;
	}

	MyRig		-> setVFOFrequency	(VFOFrequency);
	MyRig		-> getVFOFrequency	(&vfo);
	MyRig		-> setFilter		(50000000, &filter);

	HFScope		-> setZero (3 * vfo);
	LOFrequency	= n - 3 * vfo;
	HFScope		-> setNeedle (n - 3 * vfo);
	HFScope		-> clearAverage	();
	LFScope		-> setZero (0);
	LFScope		-> clearAverage	();
	if (demodulatorModus == FM_DECODER)
	   myRdsDecoder	-> reset ();
	*c		= n;
}
//
//	The generic Set_Tuner. When it is too difficult
//	a specialized set_tuner is dispatched
void	RadioInterface::Set_Tuner (int32_t n, int32_t *c) {
int32_t	vfo;
int32_t	VFOfrequency;
int16_t	filter;
//
	if (!FrequencyChangeAllowed) {	// only when extio
	   *c	= CurrentFreq;
	   return;
	}
//
	IandQ_switch = false;
	if (NTI_FMC && CurrentRig == PMSDR) {
	   Set_TunerforNTI (n, c);
	   return;
	}

	if (RigwithThirdHarmonic (CurrentRig) && n > MHz (55)) {
	   Set_TunerforThirdHarmonic (n, c);
	   IandQ_switch = true;
	   return;
	}

	VFOfrequency		= demodulatorModus == AM_DECODER ?
	                                n - CardRate / 4 :
	                                n;

	if (!MyRig -> legalFrequency (VFOfrequency)) {
	   Set_Tuner (MyRig -> defaultFrequency (), c);
	   return;
	}

	MyRig		-> setVFOFrequency	(VFOfrequency);
	MyRig		-> getVFOFrequency	(&vfo);
	MyRig		-> setFilter		(VFOfrequency, &filter);
	HFScope		-> setZero (vfo);
	LOFrequency	= n - vfo;
	HFScope		-> setNeedle (n - vfo);
	HFScope		-> clearAverage	();
	LFScope		-> setZero (0);
	LFScope		-> clearAverage	();
	if (demodulatorModus == FM_DECODER)
	   myRdsDecoder	-> reset ();
	*c		= n;
}
//
//===== code for auto increment/decrement
//	lots of code for something simple,

static inline
bool	frequencyInBounds (int32_t f, int32_t l, int32_t u) {
	return l <= f && f <= u;
}

int32_t	RadioInterface::IncrementInterval (int16_t index) {
	if (index < 0)
	   index = - index;

	if (index == 0)
	   index = 1;
	if (index >= delayTableSize)
	   index = delayTableSize;

	return 1000 * delayTable [index - 1];
}

void	RadioInterface::set_incrementFlag (int16_t incr) {
char temp [128];

	if (incr == 0) {
	   incrementingFlag ->
	                 setStyleSheet ("QLabel {background-color:blue}");
	   incrementingFlag -> setText (" ");
	   return;
	}
	if (incr < 0)
	   sprintf (temp, " << %d", IncrementInterval (incr) / 1000);
	else 
	   sprintf (temp, "%d >> ", IncrementInterval (incr) / 1000);
	incrementingFlag ->
	                 setStyleSheet ("QLabel {background-color:green}");
	incrementingFlag -> setText (temp);
}
//
//	just a quick hack
int32_t RadioInterface:: min_loop_frequency_for (int16_t mode) {
	if (mode == FM_DECODER)
	   return Khz (fm_min_loop_frequency);
	else
	if (mode == AM_DECODER)
	   return Khz (am_min_loop_frequency);
	else
	   return Khz (100);
}

int32_t RadioInterface::max_loop_frequency_for (int16_t mode) {
	if (mode == FM_DECODER)
	   return Khz (fm_max_loop_frequency);
	else
	if (mode == AM_DECODER)
	   return Khz (am_max_loop_frequency);
	else
	   return Mhz (165);
}

void	RadioInterface::autoIncrement_timeout (void) {
int32_t	amount;
int32_t	frequency	= CurrentFreq;
int32_t	low, high;

	if (demodulatorModus == AM_DECODER)
	   amount = amIncrement;
	else
	if (demodulatorModus == FM_DECODER)
	   amount =  fmIncrement;
	else
	   amount = 0;

	if (IncrementIndex < 0)
	   amount = - amount;
//
//	Very stupid code, but remember that our user might have
//	changed the modus from am to fm or vv
	frequency	+= amount;
	low	= min_loop_frequency_for (demodulatorModus);
	high	= max_loop_frequency_for (demodulatorModus);
	                       
	if ((IncrementIndex < 0) &&
	   !frequencyInBounds (frequency, low, high))
	   frequency = max_loop_frequency_for (demodulatorModus);

	if ((IncrementIndex > 0) &&
	   !frequencyInBounds (frequency, low, high))
	   frequency = min_loop_frequency_for (demodulatorModus);

	Set_Tuner (frequency, &CurrentFreq);
	Display (CurrentFreq, demodulatorModus);
	autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
}
//
//	stopIncrementing is called from various places to
//	just interrupt the autoincrementing
void	RadioInterface::stopIncrementing (void) {
	set_incrementFlag (0);

	if (autoIncrement_timer	-> isActive ())
	   autoIncrement_timer -> stop ();

	IncrementIndex = 0;
}

void	RadioInterface::autoIncrementButton (void) {
	stop_lcdTimer ();

	if (autoIncrement_timer	-> isActive ())
	   autoIncrement_timer -> stop ();

	if (++IncrementIndex > delayTableSize)
	   IncrementIndex = delayTableSize;

	if (IncrementIndex == 0) {
	   set_incrementFlag (0);
	   return;
	}
//
	autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
	set_incrementFlag (IncrementIndex);
}

void	RadioInterface::autoDecrementButton (void) {
	stop_lcdTimer ();

	if (autoIncrement_timer	-> isActive ())
	   autoIncrement_timer -> stop ();

	if (--IncrementIndex < - delayTableSize)
	   IncrementIndex = - delayTableSize;

	if (IncrementIndex == 0) {
	   set_incrementFlag (0);
	   return;
	}
//
	autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
	set_incrementFlag (IncrementIndex);
}

void	RadioInterface::set_am_increment (double v) {
	amIncrement	= (int32_t) (v * 1000);
}

void	RadioInterface::set_fm_increment (int v) {
	fmIncrement	= (int32_t) (v * 1000);
}

int32_t	RadioInterface::get_Increment_for (int16_t) {
	return demodulatorModus == AM_DECODER ? amIncrement :
	       demodulatorModus == FM_DECODER ? fmIncrement :
	       0;
}

void	RadioInterface::IncrementButton (void) {
	stopIncrementing ();
	Set_Tuner (CurrentFreq +
	                get_Increment_for (demodulatorModus), &CurrentFreq);
	Display (CurrentFreq, demodulatorModus);
}

void	RadioInterface::DecrementButton (void) {
	stopIncrementing ();
	Set_Tuner (CurrentFreq -
	                 get_Increment_for (demodulatorModus), &CurrentFreq);
	Display (CurrentFreq, demodulatorModus);
}
//
//=======================end of incrementing code
void	RadioInterface::stop_lcdTimer (void) {
	if (lcd_timer -> isActive ())
	   lcd_timer -> stop ();
}

void	RadioInterface::updateTimeDisplay (void) {
QDateTime	currentTime = QDateTime::currentDateTime ();

	timeDisplay	-> setText (currentTime.
	                            toString (QString ("dd.MM.yy:hh:mm:ss")));

	if (demodulatorModus == FM_DECODER)
	   showStrength ();
}

void	RadioInterface::lcd_timeout (void) {
	Panel		= 0;			// throw away anything
	Display (CurrentFreq, demodulatorModus);
}


void	RadioInterface::setStreamInSelector (int idx) {
	if (idx == 0)
	   return;

	inputDevice = inTable [idx];
//	if (!MySoundcardIn -> isValidDevice (inputDevice)) 
//	   return;
//
	MySoundcardIn		-> stopReader	();
	MySoundcardOut		-> stopWriter	();
fprintf (stderr, "Selecting input device %d (%d)\n", idx, inputDevice);
	if (!MySoundcardIn -> selectDevice (inputDevice)) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Selecting  input stream failed\n"));
	   return;
	}
	qWarning () << "selected input device " <<idx << inputDevice;
}

void	RadioInterface::setStreamOutSelector (int idx) {
	if (idx == 0)
	   return;

	outputDevice = outTable [idx];
	if (!MySoundcardOut -> isValidDevice (outputDevice)) 
	   return;

	MySoundcardOut	-> stopWriter	();
	if (!MySoundcardOut -> selectDevice (outputDevice)) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Selecting  output stream failed\n"));
	   return;
	}

	qWarning () << "selected output device " << idx << outputDevice;
}

void	RadioInterface::setDumpisOn (void) {
SF_INFO	* sf_info	= (SF_INFO *)alloca (sizeof (SF_INFO));

	if (DumpisOn)
	   return;

	QString file = QFileDialog::getSaveFileName (this,
	                                        tr ("open file .."),
	                                        QDir::homePath (),
	                                        tr ("Sound (*.wav)"));
	sf_info		-> samplerate	= CardRate;
	sf_info		-> channels	= 2;
	sf_info		-> format	= SF_FORMAT_WAV | SF_FORMAT_PCM_24;

	DumpfilePointer	= sf_open (file. toAscii (). data (),
	                                   SFM_WRITE, sf_info);
	if (DumpfilePointer == NULL) {
	   qDebug () << "Cannot open " << file. toAscii (). data ();
	   return;
	}

	dumpON		-> setText ("WRITING");
	DumpisOn	= TRUE;
}

void	RadioInterface::setDumpisOff (void) {
	if (!DumpisOn)
	   return;
	if (dumpBufferpointer > 0)
	      sf_writef_float (DumpfilePointer, dumpBuffer, dumpBufferpointer / 2);
	sf_close (DumpfilePointer);
	DumpisOn	= FALSE;
	dumpON		-> setText ("dumpOn");
}

void	RadioInterface::dump (DSPCOMPLEX *v, int amount) {
int32_t i;

	for (i = 0; i < amount; i ++) {
	   dumpBuffer [dumpBufferpointer ++] 	= real (v [i]);
	   dumpBuffer [dumpBufferpointer ++]	= imag (v [i]);
	   if (dumpBufferpointer >= dumpBuffersize) {
	      sf_writef_float (DumpfilePointer, dumpBuffer, dumpBufferpointer / 2);
	      dumpBufferpointer = 0;
	   }
	}
}
/*
 *	there is a tremendous amount of signal/slot connections
 *	The local connects, knobs, sliders and displays,
 *	are connected here.
 */
void	RadioInterface::localConnects (void) {
	connect (startButton, SIGNAL (clicked ()),
	              this, SLOT (setStart ()));
	connect (streamInSelector, SIGNAL (activated (int)),
	              this, SLOT (setStreamInSelector (int)));
	connect (streamOutSelector, SIGNAL (activated (int)),
	              this, SLOT (setStreamOutSelector (int)));
	connect (QuitButton, SIGNAL (clicked ()),
	              this, SLOT (TerminateProcess (void)));
	connect (HFAverageButton, SIGNAL (clicked (void)),
	              this, SLOT (setHFAverager (void)));
	connect (LFAverageButton, SIGNAL (clicked (void)),
	              this, SLOT (setLFAverager (void)));
	connect (spectrumAmplitudeSlider, SIGNAL (valueChanged (int)),
	         this,  SLOT (setAmplification (int)));
	connect (rigSelect, SIGNAL (activated (const QString &)),
	              this, SLOT (set_rigSelect (const QString &)));

	connect (pmsdr_mute, SIGNAL (activated (const QString &)),
	              this, SLOT (PMSDR_setmute (const QString &)));
	connect (dumpON, SIGNAL (clicked (void)),
	              this, SLOT (setDumpisOn (void)));
	connect (dumpOFF, SIGNAL (clicked (void)),
	              this, SLOT (setDumpisOff (void)));

	connect (fc_plus, SIGNAL (clicked (void)),
	              this, SLOT (autoIncrementButton (void)));
	connect (fc_minus, SIGNAL (clicked (void)),
	              this, SLOT (autoDecrementButton (void)));
	connect (f_plus, SIGNAL (clicked (void)),
	              this, SLOT (IncrementButton (void)));
	connect (f_minus, SIGNAL (clicked (void)),
	              this, SLOT (DecrementButton (void)));
	connect (am_increment, SIGNAL (valueChanged (double)),
	              this, SLOT (set_am_increment (double)));
	connect (fm_increment, SIGNAL (valueChanged (int)),
	              this, SLOT (set_fm_increment (int)));

	connect (logging, SIGNAL (activated (const QString &)),
	              this, SLOT (setLogging (const QString &)));
	connect (logSaving, SIGNAL (clicked (void)),
	              this, SLOT (setLogsaving (void)));

	connect (attenuationSlider, SIGNAL (valueChanged (int) ),
		      this, SLOT (setAttenuation (int) ) );
	connect (IQbalanceSlider, SIGNAL (valueChanged (int) ),
		      this, SLOT (setIQBalance (int) ) );
/*
 *	Mode setters
 */
	connect (inputModeSelect, SIGNAL (activated(const QString&) ),
	              this, SLOT (setInputMode (const QString&) ) );
/*
 */
	connect (fmChannelSelect, SIGNAL (activated (const QString &)),
	               this, SLOT (setfmChannelSelector (const QString &)));
	connect	(pauseButton,
	         SIGNAL (clicked (void)),
	         this,
	         SLOT (clickPause (void)));

	connect (createIniFile, SIGNAL (clicked (void)),
	         this, SLOT (createIniFileNow (void)));
/*
 *	connections for the console, first the digits
 */
	connect (add_one,   SIGNAL (clicked() ), this, SLOT (addOne() ) );
	connect (add_two,   SIGNAL (clicked() ), this, SLOT (addTwo() ) );
	connect (add_three, SIGNAL (clicked() ), this, SLOT (addThree() ) );
	connect (add_four,  SIGNAL (clicked() ), this, SLOT (addFour() ) );
	connect (add_five,  SIGNAL (clicked() ), this, SLOT (addFive() ) );
	connect (add_six,   SIGNAL (clicked() ), this, SLOT (addSix() ) );
	connect (add_seven, SIGNAL (clicked() ), this, SLOT (addSeven() ) );
	connect (add_eight, SIGNAL (clicked() ), this, SLOT (addEight() ) );
	connect (add_nine,  SIGNAL (clicked() ), this, SLOT (addNine() ) );
	connect (add_zero,  SIGNAL (clicked() ), this, SLOT (addZero() ) );
/*
 *	function buttons
 */
	connect (khzSelector, SIGNAL (clicked ()),
	                      this, SLOT (AcceptFreqinKhz ()));
	connect (add_correct, SIGNAL (clicked() ), this, SLOT (addCorr() ) );
	connect (add_clear,   SIGNAL (clicked() ), this, SLOT (addClear() ) );
/*
 *	The increment and decrement buttons
 */
	connect (dec_5,	   SIGNAL (clicked() ), this, SLOT (decT5() ) );
	connect (dec_50,   SIGNAL (clicked() ), this, SLOT (decT50() ) );
	connect (dec_500,  SIGNAL (clicked() ), this, SLOT (decT500() ) );
	connect (inc_5,	   SIGNAL (clicked() ), this, SLOT (incT5() ) );
	connect (inc_50,   SIGNAL (clicked() ), this, SLOT (incT50() ) );
	connect (inc_500,  SIGNAL (clicked() ), this, SLOT (incT500() ) );
/*
 *	detector selection
 */
	connect (amButton, SIGNAL (toggled (bool)),
	              this, SLOT (set_amDecoder (bool)));
	connect (fmButton, SIGNAL (toggled (bool)),
	              this, SLOT (set_fmDecoder (bool)));
	connect (audioTestButton, SIGNAL (toggled (bool)),
	              this, SLOT (set_audioTest (bool)));

	connect (fmFilterSelect, SIGNAL (activated (const QString &)),
	         this, SLOT (setfmBandwidth (const QString &)));
	connect (fmFilterDegree, SIGNAL (valueChanged (int)),
	         this, SLOT (setfmBandwidth (int)));
	connect (fmMode, SIGNAL (activated (const QString &)),
	               this, SLOT (setfmMode (const QString &)));
	connect (fmRdsSelector, SIGNAL (activated (const QString &)),
	               this, SLOT (setfmRdsSelector (const QString &)));
	connect (fmDecoder, SIGNAL (activated (const QString &)),
	               this, SLOT (setfmDecoder (const QString &)));
	connect (fmView, SIGNAL (activated (const QString &)),
	               this, SLOT (setfmView (const QString &)));
	connect	(VolumeSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (setVolume (int)));
	connect (fmStereoSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (setfmStereoSlider (int)));
	connect (fmDeemphasisSelector, SIGNAL (activated (const QString &)),
	              this, SLOT (setfmDeemphasis (const QString &)));
	connect (fmLFcutoff, SIGNAL (activated (const QString &)),
	              this, SLOT (setfmLFcutoff (const QString &)));
	connect (converter, SIGNAL (activated (const QString &)),
	              this, SLOT (set_converter (const QString &)));
	connect (amBandwidthSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (setamBandwidth (int)));
	connect (am_agc, SIGNAL (activated (const QString &)),
	              this, SLOT (set_amagc (const QString &)));
	connect (agcThresholdSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (set_agcThresholdSlider (int)));
	connect (amDecoderSelector, SIGNAL (activated (const QString &)),
	              this, SLOT (setamDecoder (const QString &)));
}

void	RadioInterface::setfmStereoSlider	(int n) {
	myFMprocessor	-> setSoundBalance (n);
	balanceDisplay	-> display (n);
}
//	Deemphasis	= 50 usec (3183 Hz, Europe)
//	Deemphasis	= 75 usec (2122 Hz US)
void	RadioInterface::setfmDeemphasis	(const QString& s) {
	if (s == "50")
	   myFMprocessor	-> setDeemphasis (50);
	else
	if (s == "75")
	   myFMprocessor	-> setDeemphasis (75);
	else
	   myFMprocessor	-> setDeemphasis (1);
}

void	RadioInterface::setVolume (int v) {
	Volume	= (int16_t)v;
	myFMprocessor	-> setVolume (Volume);
	myAMprocessor	-> setVolume (Volume);
}
//
//	In case we are signalled that there are samples:
//
#define	HANDLE_SIZE	1024
void	RadioInterface::JansRadioThroughSignal (void) {
int32_t	amount;
DSPCOMPLEX *v = (DSPCOMPLEX *)alloca (HANDLE_SIZE * sizeof (DSPCOMPLEX));
int32_t	actualAmount;

	if (MyRig -> RigprovidesSamples ()) {
	   amount	= MyRig -> Samples ();
	   while (amount >= HANDLE_SIZE) {
	      actualAmount = MyRig	-> getSamples (v, HANDLE_SIZE, inputMode);
	      HandleSamples (v, actualAmount);
	      amount -= actualAmount;
	   }
	}
	else {	// soundcard is providing samples
	   amount	= MySoundcardIn	-> Samples ();
	   while (amount >= HANDLE_SIZE) {
	      (void) MySoundcardIn -> getSamples (v, HANDLE_SIZE, inputMode);
	      HandleSamples (v, HANDLE_SIZE);
	      amount -= HANDLE_SIZE;
	   }
	}
}
/*
 *	The actual processor
 */
void	RadioInterface::HandleSamples (DSPCOMPLEX *v, int32_t amount) {
DSPCOMPLEX	audioOut;
DSPFLOAT	rdsData;
int32_t	i;
/*
 *	if requested for, dump the input signals to a file
 */
	if (amount <= 0)		// should not happen
	   return;

	if (DumpisOn)
	   dump (v, amount);

	for (i = 0; i < amount; i ++)  {
	   if (runMode == PAUSED) {
	      if (audioDecimator -> doDecimate (v [i], &v [i])) 
	         sendSampletoOutput (v [i]);
	      continue;
	   }

	   if (IandQ_switch) 
	      v [i] = DSPCOMPLEX (imag (v [i]) * attValueL,
	                          real (v [i]) * attValueR);
	   else
	      v [i] = DSPCOMPLEX (real (v [i]) * attValueL,
	                          imag (v [i]) * attValueR);

	   HFScope -> addElement (v [i]);

	   switch (demodulatorModus) {
	      case AUDIO_TEST:
	         mySoundprocessor -> makeSound (v [i], &audioOut);
	         if (audioDecimator -> doDecimate (audioOut, &audioOut)) 
	            sendSampletoOutput (audioOut);

	         if (currentScope == AUDIO_TEST)	// assert
	            LFScope -> addElement (v [i]);
	         break;

	      case AM_DECODER:
	         v [i]	= amFilter		-> Pass (v [i]);
	         v [i] *= localOscillator	-> nextValue (LOFrequency);

	         if (audioDecimator -> doDecimate (v [i], &v [i])) {
	            myAMprocessor	-> processAM (&v [i], &audioOut);
	            sendSampletoOutput (audioOut);

	            if (currentScope == AM_DECODER)	// assert
	               LFScope -> addElement (v [i]);
	         }
	         break;

	      case FM_DECODER:
	         if (fmBandwidth < 0.94 * CardRate)
	            v [i]	= fmFilter	-> Pass (v [i]);
	         v [i] *= localOscillator	-> nextValue (LOFrequency);

	         myFMprocessor	-> processFM (&v [i],
	                                          &audioOut, &rdsData);
	         if (audioDecimator -> doDecimate (audioOut, &audioOut)) 
	            sendSampletoOutput (audioOut);

	         if ((viewKeuze != 8) || (rdsModus == rdsDecoder::NO_RDS))
	            LFScope -> addElement (cmul (v [i], 3));

	         if ((rdsModus != rdsDecoder::NO_RDS) &&
	              rdsDecimator -> doDecimate (rdsData, &rdsData)) {
	            static int count = 0;
	            DSPFLOAT mag;
	            myRdsDecoder -> doDecode (rdsData, &mag,
	                                      (rdsDecoder::RdsMode)rdsModus);
//
//	just a hack. We want to show the samples of the RDS signal
//	in the time domain, we do not need all the samples (1 out of 4 here)
//	and we only show a segment once every 0.25 second
	            if (viewKeuze == 8) {
	               if (++count < 4 * displaySize) 
	                  LFScope -> addElement (10 * mag, 4);
	               else
	                 if (count > WorkingRate / 4)
	                    count = 0;
	            }
	         }
	         break;

	      case NO_DECODER:
	         audioOut = 0;
	         if (audioDecimator -> doDecimate (audioOut, &audioOut))
	            sendSampletoOutput (audioOut);
	         break;
	   }
	}
}

void	RadioInterface::sendSampletoOutput (DSPCOMPLEX s) {
int16_t i;

	if (OutputRate == WorkingRate)
	   MySoundcardOut	-> putSample (s);
	else {
	   MySoundcardOut	-> putSample (upFilter -> Pass (s));
	   for (i = 1; i < OutputRate / WorkingRate; i ++)
	      MySoundcardOut	-> putSample (DSPCOMPLEX (0, 0));
	}
}

void	RadioInterface::setCRCErrors	(int n) {
	crcErrors	-> display (n);
}

void	RadioInterface::setSyncErrors	(int n) {
	syncErrors	-> display (n);
}

void	RadioInterface::setbitErrorRate	(double v) {
	bitErrorRate	-> display (v);
}

void	RadioInterface::setGroup	(int n) {
	(void)n;
//	rdsGroupDisplay	-> display (n);
}

void	RadioInterface::setPTYCode	(int n) {
	(void)n;
//	rdsPTYDisplay	-> display (n);
}

void	RadioInterface::setAFDisplay	(int n) {
	rdsAFDisplay	-> display (n);
}

void	RadioInterface::setPiCode	(int n) {
int32_t	t	= CurrentFreq;

	if ((frequencyforPICode != t) || (n != 0)) {
	   currentPIcode	= n;
	   frequencyforPICode = t;
	}

	rdsPiDisplay	-> display (n);
}

void	RadioInterface::clearStationLabel (void) {
	StationLabel = QString ("");
	stationLabelTextBox	-> setText (StationLabel);
}
//
//	Note: although s is a char * type, its value does not
//	end with a zero, so it is not a C string
void	RadioInterface::setStationLabel (char *s, int size) {
uint16_t	i;

	StationLabel = QString ("");
	for (i = 0; i < size; i ++)
	   StationLabel. append (QString (QChar (s [i])));
	stationLabelTextBox	-> setText (StationLabel);
}

void	RadioInterface::setMusicSpeechFlag (int n) {
	if (n != 0)
	   speechLabel -> setText (QString ("music"));
	else
	   speechLabel -> setText (QString ("speech"));
}

void	RadioInterface::clearMusicSpeechFlag (void) {
	speechLabel	-> setText (QString (""));
}

void	RadioInterface::clearRadioText (void) {
	RadioText = QString ("");
	radioTextBox	-> setText (RadioText);
}
//
//	Note that, although s is a "char *", it is not a C string,
//	(no zero at the end)
void	RadioInterface::setRadioText (char *s, int size) {
uint16_t	i;
	RadioText	= QString ("");
	for (i = 0; i < size; i ++) {
	   RadioText. append (QString (QChar (s [i])));
	}
	radioTextBox	-> setText (RadioText);
}

void	RadioInterface::setRDSisSynchronized (bool syn) {
	if (!syn)
	   rdsSyncLabel -> setStyleSheet ("QLabel {background-color:red}");
	else
	   rdsSyncLabel -> setStyleSheet ("QLabel {background-color:green}");
}
//
//	we have within a stacked widget, a page for each of
//	the decoders
void	RadioInterface::setDetectorScreen (int16_t n) {
int16_t	index	= 0;

	switch (n) {
	   default:
	   case FM_DECODER:
	      index = 0;
	      break;

	   case AM_DECODER:
	      index = 1;
	      break;

	   case AUDIO_TEST:
	      index = 4;
	      break;
	}

	decoderWidget -> setCurrentIndex (index);
}

void	RadioInterface::setfmMode (const QString &s) {
	myFMprocessor	-> setfmMode (s == "stereo");
}

void	RadioInterface::setfmRdsSelector (const QString &s) {
	rdsModus = (s == "rds 1" ? rdsDecoder::RDS1 :
	            s == "rds 2" ? rdsDecoder::RDS2 : 
	            rdsDecoder::NO_RDS);
}

void	RadioInterface::setfmView (const QString &s) {
int16_t	d = 1;

	if (s == "view 1")
	   d = 1;
	if (s == "view 2")
	   d = 2;
	if (s == "view 3")
	   d = 3;
	if (s == "view 4")
	   d = 4;
	if (s == "view 5")
	   d = 5;
	if (s == "view 6")
	   d = 6;
	if (s == "view 7")
	   d = 7;
	if (s == "view 8")
	   d = 8;

	viewKeuze	= d;
	myFMprocessor	-> setView (d);
}

void	RadioInterface::setfmDecoder (const QString &s) {
int8_t	decoder	= 0;

	if (s == "fm1decoder")
	   decoder = fm_Demodulator::FM1DECODER;
	else
	if (s == "fm2decoder")
	   decoder = fm_Demodulator::FM2DECODER;
	else
	if (s == "fm3decoder")
	   decoder = fm_Demodulator::FM3DECODER;
	else
	if (s == "fm4decoder")
	   decoder = fm_Demodulator::FM4DECODER;
	else
	   decoder = fm_Demodulator::FM5DECODER;

	myFMprocessor	-> setFMdecoder (decoder);
	decoderDisplay	-> setText (QString (myFMprocessor -> nameofDecoder ()));
}
//
//	do not forget that icnt and ocnt start with 1, due
//	to Qt list conventions
bool	RadioInterface::setupSoundOut (QComboBox	*streamOutSelector,
	                               paWriter		*MySoundcardOut,
	                               int32_t		cardRate,
	                               int16_t		*table) {
uint16_t	ocnt	= 1;
uint16_t	i;

	for (i = 0; i < MySoundcardOut -> numberofDevices (); i ++) {
	   const char *so =
	             MySoundcardOut -> outputChannelwithRate (i, cardRate);
	   qDebug ("Investigating Device %d\n", i);

	   if (so != NULL) {
	      streamOutSelector -> insertItem (ocnt, so, QVariant (i));
	      table [ocnt] = i;
	      qDebug (" (output):item %d wordt stream %d\n", ocnt , i);
	      ocnt ++;
	   }
	}

	qDebug () << "added items to combobox";
	return ocnt > 1;
}

bool	RadioInterface::setupSoundIn (QComboBox	*streamInSelector,
	                              paReader	*MySoundcardIn,
	                              int32_t	cardRate,
	                              int16_t	*table) {
int16_t		icnt	= 1;
int16_t		i;

	for (i = 0; i < MySoundcardIn ->  numberofDevices (); i ++) {
	   qDebug ("Investigating Device %d\n", i);
	   const char *si =
	             MySoundcardIn -> inputChannelwithRate (i, cardRate);

	   if (si != NULL) {
	      streamInSelector -> insertItem (icnt, si, QVariant (i));	
	      table [icnt] = i;
	      qDebug (" (input): item %d wordt stream %d (%s) \n", 
	                        icnt, i, si);
	      icnt ++;
	   }
	}

	qDebug () << "added items to combobox";
	return icnt > 1;
}

void	RadioInterface::PMSDR_setmute (const QString &s) {
bool	b	= s == "mute";
	if (PMSDR_mute == b)
	   return;
	PMSDR_mute = b;
	if (CurrentRig == PMSDR)
	   ((pmSDR *)MyRig) -> setMute (PMSDR_mute);
}

void	RadioInterface::setfmLFcutoff (const QString &s) {
	if (s == "off")
	   myFMprocessor	-> setLFcutoff (-1);
	else
	   myFMprocessor	-> setLFcutoff ((int32_t)(s. toInt ()));
}
//
//	misery here, since we have to change the scale for
//	display
void	RadioInterface::set_converter (const QString &s) {
bool	tmp = s != "direct";
	if (NTI_FMC != tmp) {
	   NTI_FMC = tmp;
	   Set_Tuner (CurrentFreq, &CurrentFreq);
	   Display (CurrentFreq, demodulatorModus);
	}
}

void	RadioInterface::HandleDeviceError	(int e) {
	(void)e;
	QMessageBox::warning (this, tr ("sdr"),
	                            tr ("PMSDR is not connected anymore")); 
	
	delete MyRig;
	stop_lcdTimer		();
	stopIncrementing	();
	
	CurrentRig	= NO_RIG;
	MyRig		= new no_rig ();
	rigSelect 	-> setCurrentIndex (0);
	rigAnnex 	-> setCurrentIndex (1);
	inputNeeded	-> setCurrentIndex (0);
	Display (CurrentFreq, demodulatorModus);
}

static inline
int32_t numberofDigits (int32_t f) {

	if (f < 100000)
	   return 6;
	if (f < 100000000)
	   return 8;
	if (f < 1000000000)
	   return 9;
	return 10;
}

void	RadioInterface::Display (int32_t freq, int16_t Mode) {
int32_t nd	= numberofDigits (freq);
 
	if (Mode == AM_DECODER) {
	   lcd_Frequency	-> setNumDigits (nd);
	   lcd_Frequency	-> display ((int)freq);
	   MyRig		-> displayVFO ();
	}
	else
	if (Mode == FM_DECODER) {
	   lcd_Frequency	-> setNumDigits (6);
	   lcd_Frequency	-> display ((int)freq / 1000);
	   MyRig		-> displayVFO ();
	}
	else {
	   lcd_Frequency	-> setNumDigits (9);
	   lcd_Frequency	-> display ((int)freq);
	}
}

void	RadioInterface::set_amStrength	(float s) {
	amStrength	-> display (s);
}

void	RadioInterface::set_amShiftFreq (float s) {
	amShiftFreq	-> display (s);
}

void	RadioInterface::set_amshowAgc (float s) {
	amshowAgc	-> display (s);
}

void	RadioInterface::setamDecoder	(const QString &s) {
amProcessor::amDecoderType t = amProcessor::ENVELOPE_DETECTOR_1;

	if (s == "am2")
	   t = amProcessor::ENVELOPE_DETECTOR_2;
	else
	if (s == "am3")
	   t = amProcessor::SYNCHRONOUS_DETECTOR;
	else
	   t = amProcessor::ENVELOPE_DETECTOR_1;

	myAMprocessor	-> setDecoder (t);
}

void	RadioInterface::set_amagc	(const QString &s) {
agcHandler::Mode t;

	if (s == "agc fast")
	   t = agcHandler::AGC_FAST;
	else
	if (s == "agc slow")
	   t = agcHandler::AGC_SLOW;
	else
	   t = agcHandler::AGC_OFF;

	myAMprocessor	-> setAgc (t);
}

void	RadioInterface::set_agcThresholdSlider (int v) {
	myAMprocessor	-> setAgcThresholdSlider (v);
	agcThresholdDisplay	-> display (v);
}

void	RadioInterface::setamBandwidth	(int s) {
	amBandwidth = ((DSPFLOAT)s / 100 * WorkingRate);
	myAMprocessor	-> setBandwidth (amBandwidth);
	amBandwidthDisplay -> display (amBandwidth);
	amFilter	-> newKernel (CardRate / 4 - amBandwidth / 2,
	                              CardRate / 4 + amBandwidth / 2);
}

void	RadioInterface::setfmBandwidth (const QString &s) {

	if (s == "---") {
	   fmBandwidth	= 0.95 * CardRate;
	   return;
	}

	fmBandwidth = Khz (s. toInt ());
	fmFilter	-> newKernel (fmBandwidth / 2);
}
//
//	setting the degree of the fm bandwidth filter

void	RadioInterface::setfmBandwidth (int d) {
	delete fmFilter;
	fmFilter	= new LowPassFIR (d,
	                                  fmBandwidth / 2,
	                                  CardRate);
}

void	RadioInterface::showStrength (void) {
	pilotStrength	->
	           display ((int)(myFMprocessor -> getPilotStrength ()));
	noiseStrength	->
	           display ((int)(myFMprocessor -> getNoiseStrength ()));
	rdsStrength	->
	           display ((int)(myFMprocessor -> getRdsStrength ()));
	if (!myFMprocessor -> isLocked ())
	   pll_isLocked -> setStyleSheet ("QLabel {background-color:red}");
	else
	   pll_isLocked -> setStyleSheet ("QLabel {background-color:green}");
	dc_component	-> display (myFMprocessor -> get_DcComponent ());
}

void	RadioInterface::setLogging (const QString &s) {
	if (demodulatorModus != FM_DECODER)
	   return;

	if (s == "log off") {
	   if (logTime != 0)
	      stopLogging	();
	   return;
	}

	if (log_timer	-> isActive ())
	   log_timer -> stop ();

	if (s == "log 1 sec")
	   startLogging (1);
	else
	if (s == "log 2 sec")
	   startLogging (2);
	else
	if (s == "log 3 sec")
	   startLogging (3);
	else
	if (s == "log 4 sec")
	   startLogging (4);
	else
	   startLogging (5);
}

void	RadioInterface::startLogging (int16_t t) {

	if (demodulatorModus != FM_DECODER)
	   return;

	if (log_timer	-> isActive ())
	   log_timer -> stop ();
	logTime		= t;
	log_timer	-> start (t * 1000);
}

void	RadioInterface::stopLogging	(void) {
	if (logTime == 0)	// just return, logging was not active
	   return;

	logTime		= 0;
	if (log_timer	-> isActive ())
	   log_timer -> stop ();
}

void	RadioInterface::setLogsaving (void) {

	if (logFile != NULL) {		// just stop it
	   fclose (logFile);
	   logFile		= NULL;
	   logSaving		-> setText ("save");
	   return;
	}
	else {
           QString file	= QFileDialog::getSaveFileName (this,
	                                        tr ("open file .."),
	                                        QDir::homePath (),
	                                        tr ("text (*.txt)"));
	   logFile		= fopen (file.toAscii (). data (), "w");
	   if (logFile == NULL) {
	      QMessageBox::warning (this, tr ("sdr"),
	                                  tr ("/dev/null is used"));
	
//	      logFile	= NULL;		// remains zero
	   }
	   else {	// logFile != NULL
	      logSaving		-> setText ("halt");
	      fprintf (logFile, "\nlogging starting\n\n\n");
	   }
	}
}

void	RadioInterface::log_timeout (void) {
QDateTime	currentTime = QDateTime::currentDateTime ();
DSPFLOAT	signal, pilot, rds;
char	s [1024];

	if (demodulatorModus != FM_DECODER) {
	   stopLogging	();
	   return;
	}

	signal		= myFMprocessor -> getNoiseStrength	();
	pilot		= myFMprocessor	-> getPilotStrength	();
	rds		= myFMprocessor	-> getRdsStrength	();

	sprintf (s,
        "%s : Freq = %d,\n PI code = %4X, noise = %2d, pilot = %2d,  rds = %2d %s\n",
	currentTime.toString (QString ("dd.MM.yy:hh:mm:ss")). toStdString (). c_str (),
	(int)CurrentFreq,
	currentPIcode,
	(int)signal,
	(int)pilot,
	(int)rds,
	myFMprocessor -> isLocked () ? "pilot" : "no pilot");

	fputs (s, stderr);
//	and into the logfile
	if (logFile != NULL)
	   fputs (s, logFile);

	log_timer	-> start (logTime * 1000);
}

void	RadioInterface::clickPause (void) {

	if (runMode == IDLE)
	   return;

	if (runMode == RUNNING) {
	   if (autoIncrement_timer	-> isActive ())
	      autoIncrement_timer 	-> stop ();

	   if (MyRig -> RigprovidesSamples ())
	      MyRig		-> stopReader	();
	   else
	      MySoundcardIn	-> stopReader	();
	   MySoundcardOut	-> stopWriter	();
	   pauseButton -> setText (QString ("Continue"));
	   runMode = PAUSED;
	}
	else
	if (runMode == PAUSED) {
	   if (IncrementIndex != 0)	// restart the incrementtimer if needed
	      autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
	   if (MyRig -> RigprovidesSamples ())
	      MyRig		-> restartReader ();
	   else
	      MySoundcardIn	-> restartReader ();
	   MySoundcardOut	-> restartWriter ();
	   pauseButton -> setText (QString ("Pause"));
	   runMode = RUNNING;
	}
}

void	RadioInterface::createIniFileNow (void) {
QSettings	*s;

        QString file	= QFileDialog::getSaveFileName (this,
	                                     tr ("open file .."),
	                                     QDir::homePath (),
	                                     tr ("ini file (*.ini)"));
	s	= new QSettings (file, QSettings::IniFormat);
	if (s == NULL) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("could not open ini file"));
	   return;
	}

	dumpControlState (s);
	s	-> ~QSettings ();
}
//
//	These functions currently only make sense when
//	a dll is controling a device
//	we queue the requests and once every 10 msec we try to
//	process the requests
//	Just having a dll thread issue a signal does not work
//	so we have to poll the request queue.
//
void	RadioInterface::setDLLRequest	(int r) {
	requestQueue. enqueue (r);
}
//
void	RadioInterface::handleDLLRequest (void) {
int32_t	myRequest;
dll_handler	*myHandler = (dll_handler *)MyRig;

	if (runMode == RUNNING)
	   JansRadioThroughSignal ();
	while (!requestQueue. isEmpty ()) {
	   myRequest = requestQueue. dequeue ();
	   switch (myRequest) {
	      case 100:			// sampling frequency changed
	      {  int32_t newRate = myHandler -> GetHWSR ();
	         fprintf (stderr, "External rate set to %d\n", newRate);
	         myHandler	-> setExternalRate (newRate);
	      }
	         break;

	      case 101:			// change LO
	         break;			// ignore

	      case 102:			// block changes in frequency
	         FrequencyChangeAllowed	= false;
	         break;

	      case 103:			// unblock
	         FrequencyChangeAllowed = true;
	         break;

	      case 104:			// not implemented
	         break;

	      case 105:			// Tuned frequency changed
	         break;			// just ignore

	      case 106:			// changed demodulation mode
	         break;			// just ignore

	      case 107:			// behave as if pressed start
	         break;

	      case 108:			// behave as if pressed stop
	         break;

	      case 109:			// change passband limits
	      {  int32_t lo, hi, pitch;
	         myHandler -> GetFilters (lo, hi, pitch);
//	         fprintf (stderr, "filter %d -> %d, %d\n", lo, hi, pitch);
//	         set_bandWidthSelect ((hi - lo) * 1000 / _WorkingRate);
//	         set_bandWidthOffset (pitch * 1000 / _WorkingRate);
//	         bandwidthSlider -> setValue ((hi - lo) * 1000 / _WorkingRate);
//	         bandoffsetSlider -> setValue (pitch * 1000 / _WorkingRate);
	      }
	      break;  

	      default:
	         break;
	   }
	}
}

