#
/*
 *
 *    Copyright (C)  2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _GUI
#define _GUI

#include	"jff-include.h"
#include	<QDialog>
#include	"ui_sdrgui.h"
#include	<qwt.h>
#include	<qwt_slider.h>
#include	<qwt_plot.h>
#include	<qwt_plot_curve.h>
#include	<qwt_plot_marker.h>
#include	<qwt_plot_grid.h>
#include	<qwt_dial.h>
#include	<qwt_dial_needle.h>
#include	<qwt_plot_spectrogram.h>
#include	<QTimer>
#include	<QQueue>
#include	<QWheelEvent>
#include	"scope.h"
#include	"iqdisplay.h"
#include	"virtualrig.h"
#include	"sound-defines.h"
#include	"pa_writer.h"
#include	"pa_reader.h"
#include	"oscillator.h"
#include	"sincos.h"
#include	"fir-filters.h"
#include	<sndfile.h>

class	QSettings;
class	soundProcessor;
class	fmProcessor;
class	rdsDecoder;
class	downDecimator;
class	amProcessor;
class	fft_scope;
class	Oscillator;

#define	HFSPECTRUM	0200
#define	LFSPECTRUM	0201

/*
 *	GThe main gui object. It inherits from
 *	QDialog and the generated form
 */
class RadioInterface: public QDialog,
		      private Ui_elektorSDR {
Q_OBJECT
public:
		RadioInterface		(QSettings	*,
	                                 bool,
	                                 int32_t,		// card-in rate
	                                 int32_t,		// working rate
	                                 int32_t,		// output rate
	                                 int16_t,		// displaysize
	                                 int16_t,		// spectrumSize
	                                 int16_t,		// repeatRate
	                                 int16_t,		// rastersize
	                                 int16_t,		// multiplier
	                                 DSPCOMPLEX	*,	// sine table
	                                 QWidget *parent = NULL);
		~RadioInterface		();

private:
	enum Decoder {
	   NO_DECODER	= 0,
	   AM_DECODER	= 1,
	   FM_DECODER	= 2,
	   AUDIO_TEST	= 3,
	};

	bool		doInit;
	int16_t		multiplier;
	int16_t		inputDevice;
	int16_t		outputDevice;
	void		localConnects		(void);
	bool		setupSoundOut		(QComboBox *,
	                                         paWriter	*,
	                                         int32_t, int16_t *);
	bool		setupSoundIn		(QComboBox	*,
	                                         paReader	*,
	                                         int32_t, int16_t *);
	void		setInitialControlState	(void);
	void		dumpControlState	(QSettings *);
	void		setControlState		(QSettings *);
	LowPassFIR	*upFilter;
	void		sendSampletoOutput	(DSPCOMPLEX);

	fft_scope	*HFScope;
	fft_scope	*LFScope;
	bool		HFAverager;
	bool		LFAverager;

	QSettings	*ISettings;
	paWriter	*MySoundcardOut;
	paReader	*MySoundcardIn;
	int32_t		CardRate;
	int32_t		WorkingRate;
	int32_t		OutputRate;
	int16_t		displaySize;
	int16_t		spectrumSize;
	int16_t		repeatRate;
	int16_t		rastersize;

	int16_t		Volume;
	int8_t		channelSelector;
	SinCos		*mySinCos;
	virtualRig	*MyRig;
	bool		NTI_FMC;
	int16_t		*inTable;
	int16_t		*outTable;
	int16_t		numberofDevices;

	uint8_t		inputMode;
	int16_t		currAttSliderValue;
	DSPFLOAT	attValueL;
	DSPFLOAT	attValueR;

	int32_t		amBandwidth;
	int32_t		fmBandwidth;
	Oscillator	*LocalOscillator;
	int32_t		LOFrequency;
	int32_t		CurrentFreq;
	LowPassFIR	*fmFilter;
	BandPassFIR	*amFilter;

	int32_t		am_min_loop_frequency;
	int32_t		am_max_loop_frequency;
	int32_t		fm_min_loop_frequency;
	int32_t		fm_max_loop_frequency;
	void		ClearPanel		();
	void		AddtoPanel		(int16_t);
	int		getPanel		();
	void		CorrectPanel		();

	void		setDetectorScreen	(int16_t);

	bool		IandQ_switch;
	Oscillator	*localOscillator;
	DSPFLOAT	omega;

	int32_t		mapIncrement		(int32_t);
	int32_t		IncrementInterval	(int16_t);
	bool		RigwithThirdHarmonic	(int16_t);
	void		Set_Tuner		(int32_t, int32_t *);
	void		Set_TunerforNTI		(int32_t, int32_t *);
	void		Set_TunerforThirdHarmonic	(int32_t, int32_t *);
	void		Display			(int32_t, int16_t);
	bool		PMSDR_mute;
	QTimer		*lcd_timer;
	QTimer		*autoIncrement_timer;
	int16_t		IncrementIndex;
	int32_t		autoIncrement_amount;
	int32_t		amIncrement;
	int32_t		fmIncrement;
	
	void		set_incrementFlag	(int16_t);
	void		stopIncrementing	(void);
	int32_t		get_Increment_for	(int16_t);
	int32_t		min_loop_frequency_for	(int16_t);
	int32_t		max_loop_frequency_for	(int16_t);

	void		stop_lcdTimer		(void);
	int32_t		Panel;
	QTimer		*dll_timer;
	int16_t		CurrentRig;
	QTimer		*display_timer;
	QQueue<int>	requestQueue;
	bool		FrequencyChangeAllowed;
/*
 *	dumping
 */
	uint8_t		DumpisOn;
	void		dump		(DSPCOMPLEX *, int);
	int		dumpBufferpointer;
	int		dumpBuffersize;
	float		*dumpBuffer;
	SNDFILE		*DumpfilePointer;
	char		*inputFilename;

/*
 *	The UI acts as a kind of server, the public slots are the entries
 *	for the radio elements, signals (below) are the triggers
 *	for the radio elements.
 */
	int8_t		demodulatorModus;
	int8_t		currentScope;
	fmProcessor	*myFMprocessor;
	rdsDecoder	*myRdsDecoder;
	downDecimator	*audioDecimator;
	downDecimator	*rdsDecimator;
	int8_t		rdsModus;
	int8_t		viewKeuze;
	soundProcessor	*mySoundprocessor;
	amProcessor	*myAMprocessor;
	DSPCOMPLEX	*HighRateTable;

	void		HandleSamples		(DSPCOMPLEX *, int32_t);
	QString		RadioText;
	QString		StationLabel;
	void		IncrementFrequency	(int32_t);

	DSPFLOAT	currentPilotStrength;
	DSPFLOAT	currentSignalStrength;
	int32_t		currentPIcode;
	int32_t		frequencyforPICode;
	int16_t		logTime;
	void		startLogging		(int16_t);
	QTimer		*log_timer;
	void		stopLogging		(void);
	FILE		*logFile;

	int8_t		runMode;
	void		showStrength		(void);
/*
 */
private slots:
	void	setDLLRequest		(int);
	void	handleDLLRequest	(void);
	void	setStart		(void);
	void	lcd_timeout		(void);
	void	updateTimeDisplay	(void);
	void	clickPause		(void);

	void	createIniFileNow	(void);
	void	setInputMode		(const QString &);
	void	addOne			(void);
	void	addTwo			(void);
	void	addThree		(void);
	void	addFour			(void);
	void	addFive			(void);
	void	addSix			(void);
	void	addSeven		(void);
	void	addEight		(void);
	void	addNine			(void);
	void	addZero			(void);
	void	addClear		(void);
	void	AcceptFreqinKhz		(void);
	void	addCorr			(void);

	void	setAttenuation		(int);
	void	setIQBalance		(int);

	void	setHFAverager		(void);
	void	setLFAverager		(void);

	void	setAmplification	(int);
	void	decT5			(void);
	void	decT50			(void);
	void	decT500			(void);
	void	decT5000		(void);
	void	incT5			(void);
	void	incT50			(void);
	void	incT500			(void);
	void	incT5000		(void);

	void	set_fmDecoder		(bool);
	void	set_amDecoder		(bool);
	void	set_audioTest		(bool);

	void	setStreamInSelector	(int);
	void	setStreamOutSelector	(int);
	void	abortSystem		(int);
	void	TerminateProcess	(void);
	void	set_rigSelect		(const QString &);
	void	set_converter		(const QString &);
	void	PMSDR_setmute		(const QString &);
	void	setDumpisOn		(void);
	void	setVolume		(int);
	void	setDumpisOff		(void);

	void	setfmBandwidth		(const QString &);
	void	setfmBandwidth		(int);
	void	setfmMode		(const QString &);
	void	setfmRdsSelector	(const QString &);
	void	setfmView		(const QString &);
	void	setfmDecoder		(const QString &);
	void	setfmChannelSelector	(const QString &);
	void	setfmDeemphasis		(const QString &);
	void	setfmStereoSlider	(int);
	void	setfmLFcutoff		(const QString &);

	void	autoIncrement_timeout	(void);
	void	autoIncrementButton	(void);
	void	autoDecrementButton	(void);
	void	set_am_increment	(double);
	void	set_fm_increment	(int);
	void	IncrementButton		(void);
	void	DecrementButton		(void);

	void	setamBandwidth		(int);
	void	setamDecoder		(const QString &);
	void	set_amagc		(const QString &);
	void	set_agcThresholdSlider	(int);
public slots:
	void	wheelEvent		(QWheelEvent *);
	void	setLogging		(const QString &);
	void	setLogsaving		(void);
	void	log_timeout		(void);
	void	HandleDeviceError	(int);
	void	JansRadioThroughSignal	(void);
	void	AdjustFrequency		(int);
	void	Zoom_in			(int);
	void	Zoom_out		(int);
	void	setCRCErrors		(int);
	void	setSyncErrors		(int);
	void	setbitErrorRate		(double);
	void	setGroup		(int);
	void	setPTYCode		(int);
	void	setPiCode		(int);
	void	clearStationLabel	(void);
	void	setStationLabel		(char *, int);
	void	clearRadioText		(void);
	void	setRadioText		(char *, int);
	void	setAFDisplay		(int);
	void	setRDSisSynchronized	(bool);
	void	setMusicSpeechFlag	(int);
	void	clearMusicSpeechFlag	(void);

	void	set_amStrength		(float);
	void	set_amShiftFreq		(float);
	void	set_amshowAgc		(float);
};

#endif

