#
/*
 *
 *    Copyright (C) 2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ESDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	Main program
 */
#include	<QApplication>
#include	<QSettings>
#include	"utilities.h"
#include	"jff-include.h"
#include	"gui.h"
#include	"fm-constants.h"
#include	<QDir>

void	fullPathfor (const char *v, char *out) {
int16_t	i;
QString	homeDir;

	if (v == NULL) {
	   sprintf (out, "%s", MISERY);
	   return;	// should not happen
	}

	if (v [0] == '/') {		// full path specified
	   sprintf (out, "%s", v);
	   return;
	}

	homeDir = QDir::homePath ();
	homeDir. append ("/");
	homeDir. append (v);
	homeDir	= QDir::toNativeSeparators (homeDir);
	sprintf (out, "%s", homeDir. toAscii (). data ());
	fprintf (stderr, "ini file = %s\n", out);

	for (i = 0; out [i] != 0; i ++);
	if (out [i - 4] != '.' ||
	    out [i - 3] != 'i' ||
	    out [i - 2] != 'n' ||
	    out [i - 1] != 'i') {
	    out [i] = '.';
	    out [i + 1] = 'i';
	    out [i + 2] = 'n';
	    out [i + 3] = 'i';
	    out [i + 4] = 0;
	}
}

bool	fileExists (char *v) {
FILE *f;

	f = fopen (v, "r");
	if (f == NULL)
	   return false;
	fclose (f);
	return true;
}

int	main (int argc, char **argv) {
int32_t		i;
int32_t		tmp;
int32_t		opt;
/*
 *	The default values
 */
int16_t		displaySize		= DISPLAYSIZE;
int16_t		spectrumSize		= SPECTRUMSIZE;
int16_t		repeatRate		= REPEAT_RATE;
int16_t		rastersize		= RASTER;
QSettings	*ISettings;		/* .ini file	*/
bool		setoutputRate	= false;
int16_t		multiplier	= -1;
int16_t		factor		= - 1;
int32_t		CardRate	= RATE_IN;
int32_t		WorkingRate	= 48000;
int32_t		OutputRate	= WorkingRate;
RadioInterface	*MyRadioInterface;
DSPCOMPLEX	*Table;
bool		doInit		= true;
char		*initFilename	= NULL;
char		*defaultInit	= (char *)alloca (512 * sizeof (char));

	fullPathfor (DEFAULT_INI, defaultInit);

	while ((opt = getopt (argc, argv, "A:B:m:i:o:C:T:dIFEMSG")) != -1) {
	   switch (opt) {
	      case 'A':	tmp  = atoi (optarg);
	                spectrumSize = tmp * displaySize;
	                break;

	      case 'B': repeatRate = atoi (optarg);
	                break;

	      case 'C':	rastersize = atoi (optarg);
	                break;

	      case 'm':	multiplier = atoi (optarg);
	                break;

	      case 'E': factor		= 5;
	                break;

	      case 'F':	factor		= 6;
	                break;

	      case 'T':	factor		= atoi (optarg);
	                break;

	      case 'M': setoutputRate	= true;
	                break;

	      case 'i':	initFilename	=
	                            (char *) alloca (512 * sizeof (char));
	                fullPathfor (optarg, initFilename);
	                break;

	      case 'd':	doInit = false;
	                break;

	      default:
	                break;
	      }
	}

/*
 *	... and the settings of the "environment"
 */
	if (initFilename == NULL)
	   initFilename = defaultInit;
	ISettings	= new QSettings (initFilename, QSettings::IniFormat);

	if (factor != -1) {
	   CardRate = factor * WorkingRate;
	}

	OutputRate	= WorkingRate;
	if (setoutputRate) {
	   CardRate	= 192000;
	   OutputRate	= 192000;
	}

	Table		= new DSPCOMPLEX [CardRate];
	for (i = 0; i < CardRate; i ++)
	   Table [i] = DSPCOMPLEX (cos (2 * M_PI * i / CardRate),
	                           sin (2 * M_PI * i / CardRate));

	if (multiplier == -1) 
	   multiplier = 5;
	else
	if (multiplier * CardRate < 960000) {
	   multiplier = 9600000 / CardRate;
	}
	if (multiplier * CardRate > 3200000) {
	   multiplier = 3000000 / CardRate;
	}

/*
 *	Before we connect control to the gui, we have to
 *	instantiate
 */
	QApplication a (argc, argv);
	MyRadioInterface = new RadioInterface (ISettings,
	                                       doInit,
	                                       CardRate,
	                                       WorkingRate,
	                                       OutputRate,
			                       displaySize,
	                                       spectrumSize,
	                                       repeatRate,
	                                       rastersize,
	                                       multiplier,
	                                       Table);

	MyRadioInterface -> show ();
	a. exec ();
/*
 *	done:
 */
	fflush (stdout);
	fflush (stderr);
	qDebug ("It is done\n");
//	MyRadioInterface	-> ~RadioInterface ();
	ISettings	-> ~QSettings ();
	exit (1);
}

