#
/*
 *    Copyright (C) 2010,2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the  JSDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	<QSettings>
#include	<QMessageBox>
#include	<QFileDialog>
#include	<QDebug>
#include	<qwt_color_map.h>
#include	<qwt_scale_widget.h>
#include	<qwt_scale_draw.h>
#include	<QDateTime>
#include	<QFile>
#include	"dabstick-dll.h"
#include	"dabstick.h"
#include	"gui.h"
#include	"utilities.h"
#include	"fm_demodulator.h"
#include	"fm_processor.h"
#include	"rds_decoder.h"
#include	"decimator.h"
#include	"oscillator.h"
#include	"fm-constants.h"

#ifdef __MINGW32__
#include	<iostream>
#include	<windows.h>
#endif
//
static
//int16_t	delayTable [] = {15, 13, 11, 10, 9, 8, 7, 5, 3, 2, 1};
int16_t	delayTable [] = {1, 2, 4, 6, 8, 10, 12, 15};
#define delayTableSize	((int)(sizeof (delayTable) / sizeof (int16_t)))
/*
 *	We use the creation function merely to set up the
 *	user interface and make the connections between the
 *	gui elements and the handling agents. All real action
 *	is embedded in actions, initiated by gui buttons
 */
	RadioInterface::RadioInterface (QSettings	*Si,
	                                int32_t		CardRate,
	                                int32_t		OutputRate,
	                                int16_t		multiplier,
	                                QWidget		*parent): QDialog (parent) {
bool	soundSeemsOk;
int16_t	i;
bool	success;
int32_t	k;
// 	the setup for the generated part of the ui
//
	setupUi (this);

	this		-> ISettings	= Si;
	this		-> CardRate	= CardRate;
	this		-> OutputRate	= OutputRate;
	this		-> multiplier	= multiplier;
	this		-> mySinCos	= new SinCos (CardRate);
//
//	we need (very) high latency in the output
	MySoundcardOut		= new paWriter (OutputRate,
	                                        VERYHIGHLATENCY);
	outTable		= new int16_t
	                           [MySoundcardOut -> numberofDevices () + 10];
	for (i = 0; i < MySoundcardOut -> numberofDevices (); i ++)
	   outTable [i] = -1;

	startingImmediate	= false;
//	might be altered by
	soundSeemsOk		= setupSoundOut (streamOutSelector,
	                                         MySoundcardOut,
	                                         OutputRate, outTable);
	if (!soundSeemsOk) {
	   fprintf (stderr, "No channels\n");
	   exit (2);
	}
//
	connect (startButton, SIGNAL (clicked (void)),
	              this, SLOT (setStart (void)));
	connect (QuitButton, SIGNAL (clicked (void)),
	              this, SLOT (setQuit (void)));
	connect (streamOutSelector, SIGNAL (activated (int)),
	              this, SLOT (setStreamOutSelector (int)));
	connect (VolumeSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (setVolume (int)));
	connect (frequencySetter, SIGNAL (valueChanged (int)),
	              this, SLOT (setTuner (int)));
	connect (fc_minus, SIGNAL (clicked ()),
	              this, SLOT (startDecrementing (void)));
	connect (fc_plus, SIGNAL (clicked ()),
	              this, SLOT (startIncrementing (void)));
//
	audioDecimator		= new downDecimator (CardRate, OutputRate);
	myFMprocessor		= new fmProcessor (this,
	                                           CardRate,
	                                           mySinCos);
	myRdsDecoder		= new rdsDecoder (this,
	                                          OutputRate / 2,
	                                          mySinCos);
	rdsDecimator		= new downDecimator (CardRate, OutputRate / 2);
//
//	see whether the device is connected
	dabstickSelect	= true;
	if (ISettings != NULL) {
	   QString s;
	   s = ISettings -> value ("dabstick", "dabstick-dll"). toString ();
	   dabstickSelect = s == "dabstick-dll";
	}
	if (dabstickSelect)
	   MyRig	= new dabstick_dll (CardRate,
	                            CardRate * multiplier, &success);
	else
	   MyRig	= new dabstick (CardRate,
	                            CardRate * multiplier, &success);
	if (!success) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Could not open dongle"));
	   delete MyRig;
	   exit (101);
	}
	k	= ISettings -> value ("frequency", 893). toInt ();
	frequencySetter	-> setValue (k);
	connect (MyRig,
	         SIGNAL (samplesAvailable ()),
	         this,
	         SLOT (JansRadioThroughSignal ()));
	k	= ISettings -> value ("Volume", 10). toInt ();
	VolumeSlider	-> setValue (k);

//	a timer for autoincrement/decrement of the tuning
	myFMprocessor	-> setfmMode ("stereo");
	myFMprocessor	-> setFMdecoder (fm_Demodulator::FM1DECODER);
	rdsModus		= rdsDecoder::RDS1;
	runMode			= false;
	currentPICode		= 0;
	frequencyforPICode	= 0;
	lastFrequency		= 947;
	IncrementIndex		= 0;

	autoIncrement_timer	= new QTimer ();
	autoIncrement_timer	-> setSingleShot (TRUE);
	autoIncrement_timer	-> setInterval (5000);
	connect (autoIncrement_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (autoIncrement_timeout ()));
	if (startingImmediate)
	   setStart ();
}

	RadioInterface::~RadioInterface () {
	delete		myFMprocessor;
	delete		myRdsDecoder;
	delete		audioDecimator;
	delete		rdsDecimator;
	delete[]	outTable;
	delete		MySoundcardOut;
	delete		MyRig;
}

void	RadioInterface::setControlState (QSettings *s) {
QString	h;
int	k;

	if (s == NULL)
	   return;
}

void	RadioInterface::dumpControlState (QSettings *s) {
	if (s == NULL)
	   return;
	s	-> setValue ("dabstick",
	                  dabstickSelect ? "dabstick-dll" : "dabstick");
	s	-> setValue ("frequency", lastFrequency);
	s	-> setValue ("volume", VolumeSlider -> value ());
}
	                         
void	RadioInterface::setStart (void) {
int	r;

	if (runMode)
	   return;
//	ok, lets move
	r = MyRig		-> restartReader ();
	if (!r) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Opening inputstream failed\n"));
	   goto err;
	}
	r = MySoundcardOut	-> restartWriter ();
	if (!r) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Opening  output stream failed\n"));
	   goto err;
	}

	runMode	= true;
	return;

err:
	MyRig	-> stopReader ();
	accept ();
}

void	RadioInterface::setQuit (void) {
	MyRig	-> stopReader ();
	if (ISettings != NULL)
	   dumpControlState (ISettings);
	accept ();
}

//	Note the frequency is now in  units of 100 KHz
void	RadioInterface::setTuner (int n) {
	setFrequency ((int32_t)n);
}

void	RadioInterface::setFrequency (int32_t n) {
int32_t	VFOfrequency;
//
	VFOfrequency		= 100 * KHz (n);
	if (!MyRig -> legalFrequency (VFOfrequency)) {
	   setFrequency (MyRig -> defaultFrequency () / (100 * KHz (1)));
	   return;
	}

	MyRig		-> setVFOFrequency	(VFOfrequency);
	myRdsDecoder	-> reset ();
	lastFrequency	= n;
}

void	RadioInterface::setVolume (int v) {
	myFMprocessor	-> setVolume ((int16_t)v);
}

//	Deemphasis	= 50 usec (3183 Hz, Europe)
//	Deemphasis	= 75 usec (2122 Hz US)
//void	RadioInterface::setfmDeemphasis	(const QString& s) {
//	if (s == "50")
//	   myFMprocessor	-> setDeemphasis (50);
//	else
//	if (s == "75")
//	   myFMprocessor	-> setDeemphasis (75);
//	else
//	   myFMprocessor	-> setDeemphasis (1);
//}
//
//	In case we are signalled that there are samples:
//
#define	HANDLE_SIZE	1024
void	RadioInterface::JansRadioThroughSignal (void) {
int32_t	amount;
DSPCOMPLEX *v = (DSPCOMPLEX *)alloca (HANDLE_SIZE * sizeof (DSPCOMPLEX));
int32_t	actualAmount;

	amount	= MyRig -> Samples ();
	while (amount >= HANDLE_SIZE) {
	   actualAmount = MyRig	-> getSamples (v, HANDLE_SIZE, IandQ);
	   HandleSamples (v, actualAmount);
	   amount -= actualAmount;
	}
}
/*
 *	The actual processor
 */
void	RadioInterface::HandleSamples (DSPCOMPLEX *v, int32_t amount) {
DSPCOMPLEX	audioOut;
DSPFLOAT	rdsData;
int32_t	i;
/*
 *	if requested for, dump the input signals to a file
 */
	if (amount <= 0)		// should not happen
	   return;

	for (i = 0; i < amount; i ++) {
	   myFMprocessor	-> processFM (&v [i],
	                                      &audioOut, &rdsData);
	   if (audioDecimator -> doDecimate (audioOut, &audioOut)) 
	      MySoundcardOut	-> putSample (audioOut);
	   if (rdsDecimator -> doDecimate (rdsData, &rdsData)) {
	      DSPFLOAT mag;
	      myRdsDecoder -> doDecode (rdsData, &mag,
	                                      (rdsDecoder::RdsMode)rdsModus);
	   }
	}
}

void	RadioInterface::setPTYCode	(int n) {
	(void)n;
//	rdsPTYDisplay	-> display (n);
}

void	RadioInterface::setAFDisplay	(int n) {
	rdsAFDisplay	-> display (n);
}

void	RadioInterface::setPiCode	(int n) {
int32_t	t	= lastFrequency;

	if ((frequencyforPICode != t) || (n != 0)) {
	   currentPICode	= n;
	   frequencyforPICode = t;
	}

	rdsPiDisplay	-> display (n);
}

void	RadioInterface::clearStationLabel (void) {
	StationLabel = QString ("");
	stationLabelTextBox	-> setText (StationLabel);
}
//
//	Note: although s is a char * type, its value does not
//	end with a zero, so it is not a C string
void	RadioInterface::setStationLabel (char *s, int size) {
uint16_t	i;

	StationLabel = QString ("");
	for (i = 0; i < size; i ++)
	   StationLabel. append (QString (QChar (s [i])));
	stationLabelTextBox	-> setText (StationLabel);
}

void	RadioInterface::setMusicSpeechFlag (int n) {
	if (n != 0)
	   speechLabel -> setText (QString ("music"));
	else
	   speechLabel -> setText (QString ("speech"));
}

void	RadioInterface::clearMusicSpeechFlag (void) {
	speechLabel	-> setText (QString (""));
}

void	RadioInterface::clearRadioText (void) {
	RadioText = QString ("");
	radioTextBox	-> setText (RadioText);
}
//
//	Note that, although s is a "char *", it is not a C string,
//	(no zero at the end)
void	RadioInterface::setRadioText (char *s, int size) {
uint16_t	i;
	RadioText	= QString ("");
	for (i = 0; i < size; i ++) {
	   RadioText. append (QString (QChar (s [i])));
	}
	radioTextBox	-> setText (RadioText);
}

void	RadioInterface::setRDSisSynchronized (bool syn) {
	if (!syn)
	   rdsSyncLabel -> setStyleSheet ("QLabel {background-color:red}");
	else
	   rdsSyncLabel -> setStyleSheet ("QLabel {background-color:green}");
}
//
//	do not forget that icnt and ocnt start with 1, due
//	to Qt list conventions
bool	RadioInterface::setupSoundOut (QComboBox	*streamOutSelector,
	                               paWriter		*MySoundcardOut,
	                               int32_t		cardRate,
	                               int16_t		*table) {
uint16_t	ocnt	= 1;
uint16_t	i;

	for (i = 0; i < MySoundcardOut -> numberofDevices (); i ++) {
	   const char *so =
	             MySoundcardOut -> outputChannelwithRate (i, cardRate);
	   qDebug ("Investigating Device %d\n", i);

	   if (so != NULL) {
	      streamOutSelector -> insertItem (ocnt, so, QVariant (i));
	      table [ocnt] = i;
	      qDebug (" (output):item %d wordt stream %d\n", ocnt , i);
	      ocnt ++;
	   }
	}

	qDebug () << "added items to combobox";
	if (MySoundcardOut -> selectDefaultDevice ())
	   startingImmediate = true;
	return ocnt > 1;
}

void	RadioInterface::setStreamOutSelector (int idx) {
	if (idx == 0)
	   return;

	outputDevice = outTable [idx];
	if (!MySoundcardOut -> isValidDevice (outputDevice)) 
	   return;

//	MySoundcardOut	-> stopWriter	();

	if (!MySoundcardOut -> selectDevice (outputDevice)) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Selecting  output stream failed\n"));
	   return;
	}

	qWarning () << "selected output device " << idx << outputDevice;
}

void	RadioInterface::setCRCErrors	(int a) {
	(void)a;
}

void	RadioInterface::setSyncErrors	(int a) {
	(void)a;
}

void	RadioInterface::setGroup	(int a) {
	(void)a;
}

void	RadioInterface::setbitErrorRate	(double a) {
	(void)a;
}

//=====================================================================
//
//	Stupid code for autoincrementing and decrementing
static	inline
int32_t IncrementInterval (int16_t incr) {

	if (incr < 0)
	   incr = - incr;

	if (incr == 0)
	   incr = 1;
	if (incr >= delayTableSize)
	   incr = delayTableSize;

	return 1000 * delayTable [incr - 1];
}

void	RadioInterface::set_incrementFlag (int16_t incr) {
	if (incr != 0) {
	   char temp [128];
	   if (incr < 0)
	      sprintf (temp, " << %d", IncrementInterval (incr) / 1000);
	   else
	      sprintf (temp, "%d >> ", IncrementInterval (incr) / 1000);
	   incrementingFlag ->
	                 setStyleSheet ("QLabel {background-color:green}");
	   incrementingFlag -> setText (temp);
	}
	else {
	   incrementingFlag ->
	                 setStyleSheet ("QLabel {background-color:blue}");
	   incrementingFlag -> setText (" ");
	}
}

void	RadioInterface::autoIncrement_timeout (void) {
int32_t	frequency;

	MyRig	-> getVFOFrequency (&frequency);
	frequency	/= 100 * KHz (1);
//
//	Very stupid code, but remember that our user might have
//	changed the modus from am to fm or vv
	frequency	+= IncrementIndex < 0 ? -1 : 1;

	if ((IncrementIndex < 0) &&
	         (frequency < 865))
	   frequency = 1100;

	if ((IncrementIndex > 0) &&
	         (frequency > 1100))
	   frequency = 865;
//
//	Note that setting the box will cause a signal
//	to be raised!!!
	frequencySetter	-> setValue (frequency);
	autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
}

void	RadioInterface::stopIncrementing (void) {
	set_incrementFlag (0);

	if (autoIncrement_timer	-> isActive ())
	   autoIncrement_timer -> stop ();

	IncrementIndex = 0;
}
//
//	response on pressing the + button
void	RadioInterface::startIncrementing (void) {
	if (autoIncrement_timer	-> isActive ())
	   autoIncrement_timer -> stop ();
//
	if ( ++IncrementIndex == 0) {	// it was decrementing
	   set_incrementFlag (0);
	   return;
	}
//
	if (IncrementIndex >= delayTableSize)
	   IncrementIndex = delayTableSize;
//
//	(re)start the timer
	autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
	set_incrementFlag (IncrementIndex);
}
//
//	response on pressing the - button
void	RadioInterface::startDecrementing (void) {
	if (autoIncrement_timer	-> isActive ())
	   autoIncrement_timer -> stop ();
//
	if ( --IncrementIndex == 0) {
	   set_incrementFlag (0);
	   return;
	}
	
	if (IncrementIndex <= - delayTableSize)
	   IncrementIndex = - delayTableSize; 
//
//	(re)start the timer
	autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
	set_incrementFlag (IncrementIndex);
}

