#
/*
 *
 *    Copyright (C)  2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _GUI
#define _GUI

#include	"jff-include.h"
#include	<QDialog>
#include	"ui_sdrgui.h"
#include	<qwt.h>
#include	<qwt_slider.h>
#include	<QTimer>
#include	"virtualrig.h"
#include	"sound-defines.h"
#include	"pa_writer.h"
#include	"oscillator.h"
#include	"sincos.h"
#include	"fir-filters.h"
#include	<sndfile.h>

class	QSettings;
class	fmProcessor;
class	rdsDecoder;
class	downDecimator;

/*
 *	GThe main gui object. It inherits from
 *	QDialog and the generated form
 */
class RadioInterface: public QDialog,
		      private Ui_elektorSDR {
Q_OBJECT
public:
		RadioInterface		(QSettings	*,
	                                 int32_t,		// card-in rate
	                                 int32_t,		// output rate
	                                 int16_t,		// multiplier
	                                 QWidget *parent = NULL);
		~RadioInterface		(void);
private:
	QSettings	*ISettings;
	int32_t		CardRate;
	int32_t		OutputRate;
	int16_t		multiplier;

	void		dumpControlState	(QSettings *);
	void		setControlState		(QSettings *);
	int16_t		outputDevice;
	bool		setupSoundOut		(QComboBox *,
	                                         paWriter	*,
	                                         int32_t, int16_t *);
	paWriter	*MySoundcardOut;

	bool		dabstickSelect;
	int16_t		Volume;
	SinCos		*mySinCos;
	virtualRig	*MyRig;
	int16_t		*outTable;
	int16_t		numberofDevices;
	Oscillator	*localOscillator;
	int32_t		LOFrequency;
	int32_t		lastFrequency;

	int32_t		fm_min_loop_frequency;
	int32_t		fm_max_loop_frequency;
	QTimer		*autoIncrement_timer;
	int16_t		IncrementIndex;
	
	void		setFrequency		(int);
	void		set_incrementFlag	(int16_t);
	void		stopIncrementing	(void);
/*
 *	The UI acts as a kind of server, the public slots are the entries
 *	for the radio elements, signals (below) are the triggers
 *	for the radio elements.
 */
	fmProcessor	*myFMprocessor;
	rdsDecoder	*myRdsDecoder;
	downDecimator	*audioDecimator;
	downDecimator	*rdsDecimator;
	int8_t		rdsModus;
	void		HandleSamples		(DSPCOMPLEX *, int32_t);
	QString		RadioText;
	QString		StationLabel;
	void		IncrementFrequency	(int32_t);
	int32_t		currentPICode;
	int32_t		frequencyforPICode;
	bool		runMode;
	bool		startingImmediate;
/*
 */
private slots:
	void	setStart		(void);
	void	setQuit			(void);
	void	setTuner		(int);
	void	setStreamOutSelector	(int);
	void	setVolume		(int);

	void	autoIncrement_timeout	(void);
	void	startIncrementing	(void);
	void	startDecrementing	(void);

public slots:
	void	JansRadioThroughSignal	(void);
	void	setPTYCode		(int);
	void	setPiCode		(int);
	void	clearStationLabel	(void);
	void	setStationLabel		(char *, int);
	void	clearRadioText		(void);
	void	setRadioText		(char *, int);
	void	setAFDisplay		(int);
	void	setRDSisSynchronized	(bool);
	void	setMusicSpeechFlag	(int);
	void	clearMusicSpeechFlag	(void);

	void	setbitErrorRate		(double);
	void	setGroup		(int);
//
//	dummies
	void	setCRCErrors		(int);
	void	setSyncErrors		(int);

};

#endif

