#
/*
 *
 *    Copyright (C) 2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ESDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	Main program
 */
#include	<QApplication>
#include	<QSettings>
#include	<QDir>
#include	"utilities.h"
#include	"jff-include.h"
#include	"gui.h"
#include	"fm-constants.h"

void	fullPathfor (const char *v, char *out) {
int16_t	i;
QString	homeDir;

	if (v == NULL) {
	   sprintf (out, "%s", MISERY);
	   return;	// should not happen
	}

	if (v [0] == '/') {		// full path specified
	   sprintf (out, "%s", v);
	   return;
	}

	homeDir	= QDir::homePath ();
	homeDir. append ("/");
	homeDir. append (v);
	homeDir = QDir::toNativeSeparators (homeDir);
	sprintf (out, "%s", homeDir. toAscii (). data ());

	for (i = 0; out [i] != 0; i ++);
	if (out [i - 4] != '.' ||
	    out [i - 3] != 'i' ||
	    out [i - 2] != 'n' ||
	    out [i - 1] != 'i') {
	    out [i] = '.';
	    out [i + 1] = 'i';
	    out [i + 2] = 'n';
	    out [i + 3] = 'i';
	    out [i + 4] = 0;
	}
}

int	main (int argc, char **argv) {
int32_t		opt;
/*
 *	The default values
 */
QSettings	*ISettings;
int16_t		multiplier	= -1;
int16_t		factor		= -1;
int32_t		CardRate	= 192000;
int32_t		OutputRate	= 48000;
RadioInterface	*MyRadioInterface;
char		*defaultInit	= (char *)alloca (512 * sizeof (char));

	fullPathfor (".jsdr-mini.ini", defaultInit);
	while ((opt = getopt (argc, argv, "m:T:F:f")) != -1) {
	   switch (opt) {
	      case 'm':	multiplier = atoi (optarg);
	                break;

	      case 'F':	factor		= 6;
	                break;

	      case 'T':	factor		= atoi (optarg);
	                break;
	      default:
	                break;
	      }
	}

	if (factor != -1) {
	   CardRate = factor * OutputRate;
	}

	if (multiplier == -1) 
	   multiplier = 5;
	else
	if (multiplier * CardRate < 960000) {
	   multiplier = 9600000 / CardRate;
	}
	if (multiplier * CardRate > 3200000) {
	   multiplier = 3000000 / CardRate;
	}
//
//	create an ini file is it does not exist
	if (fopen (defaultInit, "a") == NULL) {
	   fprintf (stderr, "Cannot open files, fatal !!!\n");
	   exit (2);
	}
	ISettings	= new QSettings (defaultInit, QSettings::IniFormat);

	QApplication a (argc, argv);
	MyRadioInterface = new RadioInterface (ISettings,
	                                       CardRate,
	                                       OutputRate,
	                                       multiplier);

	MyRadioInterface -> show ();
	a. exec ();
/*
 *	done:
 */
	fflush (stdout);
	fflush (stderr);
	if (ISettings != NULL)
	   ISettings	-> ~QSettings ();
//	MyRadioInterface	-> ~RadioInterface ();

	qDebug ("It is done\n");
	exit (1);
}

