#ifndef __TUNER_E4000_H
#define __TUNER_E4000_H

// Definition (implemented for E4000)
#define E4000_1_SUCCESS			1
#define E4000_1_FAIL			0
#define E4000_I2C_SUCCESS		1
#define E4000_I2C_FAIL			0

#define E4K_I2C_ADDR		0xc8
#define E4K_CHECK_ADDR		0x02
#define E4K_CHECK_VAL		0x40

//
//	we packed the tuner functions into a class
#include	"virtual_tuner.h"

class	RTL2832U;

class	TUNER_E4000: public virtual_tuner {
public:
	TUNER_E4000	(RTL2832U *);
	~TUNER_E4000	(void);

int32_t	init		(void);
int32_t	exit		(void);
int32_t	tune		(uint32_t);
int32_t	set_bw		(int32_t);
int32_t	set_gain	(int32_t);
bool	legalFrequency	(uint32_t);

private:
RTL2832U	*env_class;
int32_t	I2CReadByte	(uint8_t NoUse, uint8_t RegAddr, uint8_t *pReadingByte);
int32_t	I2CWriteByte	(uint8_t NoUse, uint8_t RegAddr, uint8_t WritingByte);

int32_t	I2CWriteArray	(uint8_t NoUse, uint8_t RegStartAddr,
	                                uint8_t ByteNum,
	                                uint8_t *pWritingBytes);
// Functions (from E4000 source code)
int32_t tunerreset	(void);
int32_t Tunerclock	(void);
int32_t Qpeak		(void);
int32_t DCoffloop	(void);
int32_t GainControlinit (void);

int32_t	Gainmanual	(void);
int32_t E4000_gain_freq	(int32_t frequency);
int32_t PLL		(int32_t Ref_clk, int32_t Freq);
int32_t LNAfilter	(int32_t Freq);
int32_t IFfilter	(int32_t bandwidth, int32_t Ref_clk);
int32_t freqband	(int32_t Freq);
int32_t DCoffLUT	(void);
int32_t GainControlauto	(void);

int32_t E4000_sensitivity(int32_t Freq, int32_t bandwidth);
int32_t E4000_linearity	(int32_t  Freq, int32_t bandwidth);
int32_t E4000_high_linearity (void);
int32_t E4000_nominal	(int32_t Freq, int32_t bandwidth);

// The following context is E4000 tuner API source code
// Definitions

// Bandwidth in Hz
enum E4000_BANDWIDTH_HZ {
	E4000_BANDWIDTH_6000000HZ = 6000000,
	E4000_BANDWIDTH_7000000HZ = 7000000,
	E4000_BANDWIDTH_8000000HZ = 8000000,
};


// Manipulaing functions
void	e4000_GetTunerType (int32_t *pTunerType);
void	e4000_GetDeviceAddr (uint8_t *pDeviceAddr);
int32_t	e4000_Initialize (void);
int32_t e4000_SetRfFreqHz (unsigned long RfFreqHz);
int32_t	e4000_GetRfFreqHz (unsigned long *pRfFreqHz);

// Extra manipulaing functions
int32_t e4000_GetRegByte (uint8_t RegAddr, uint8_t *pReadingByte);
int32_t e4000_SetBandwidthHz (unsigned long BandwidthHz);
int32_t e4000_GetBandwidthHz (unsigned long *pBandwidthHz);
};

#endif
