#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#ifndef __TUNER_FC0013_H
#define __TUNER_FC0013_H

/**

@file

@brief   FC0013 tuner module declaration

One can manipulate FC0013 tuner through FC0013 module.
FC0013 module is derived from tuner module.

**/

#define FC0013_I2C_ADDR		0xc6
#define FC0013_CHECK_ADDR	0x00
#define FC0013_CHECK_VAL	0xa3

#include	"virtual_tuner.h"

class	RTL2832U;

class	TUNER_FC0013 : public virtual_tuner {
public:

	TUNER_FC0013 (RTL2832U *);
	~TUNER_FC0013 (void);

	int32_t		init		(void);
	int32_t		exit		(void);
	int32_t		tune		(uint32_t);
	int32_t		set_bw		(int);
	bool		legalFrequency	(uint32_t);
	bool		setGain		(int32_t);

enum FC0013_I2C_STATUS {
	FC0013_I2C_SUCCESS,
	FC0013_I2C_ERROR,
};


enum FC0013_FUNCTION_STATUS {
	FC0013_FUNCTION_SUCCESS,
	FC0013_FUNCTION_ERROR,
};

// Bandwidth mode
enum FC0013_BANDWIDTH_MODE {
	FC0013_BANDWIDTH_6000000HZ = 6,
	FC0013_BANDWIDTH_7000000HZ = 7,
	FC0013_BANDWIDTH_8000000HZ = 8,
};

private:
// Functions
//	two helpers:
	int32_t	FC0013_Write	(uint8_t RegAddr, uint8_t Byte);
	int32_t FC0013_Read 	(uint8_t RegAddr, uint8_t *pByte);
	bool	FC0013_update	(uint8_t RegAddr,
	                         uint8_t e, uint8_t f);
//
//	the workers:
	int32_t	FC0013_Open	();
	int32_t FC0013_SetFrequency (uint32_t Frequency,
	                             uint32_t Bandwidth);

// Set VHF Track depends on input frequency
	int32_t	FC0013_SetVhfTrack	(uint32_t Frequency);

// Default for initialing
#define FC0013_RF_FREQ_HZ_DEFAULT		50000000
#define FC0013_BANDWIDTH_MODE_DEFAULT		FC0013_BANDWIDTH_8000000HZ
	RTL2832U	*env_class;

// Tuner LNA
enum FC0013_LNA_GAIN_VALUE {
	FC0013_LNA_GAIN_LOW     = 0x00,	// -6.3dB
	FC0013_LNA_GAIN_MIDDLE  = 0x08,	//  7.1dB
	FC0013_LNA_GAIN_HIGH_17 = 0x11,	// 19.1dB
	FC0013_LNA_GAIN_HIGH_19 = 0x10,	// 19.7dB
};

//
//	functions, reminiscent of the original code, somewhere
//	in the gnuradio environment
	int	fc0013_Initialize (void *);
	int	fc0013_SetRfFreqHz (void *, unsigned long);
	int	fc0013_GetRfFreqHz (void *, unsigned long *);
// Extra manipulaing functions
	int	fc0013_SetBandwidthMode (void *, int);
	int	fc0013_GetBandwidthMode (void *, int *);
	int	fc0013_RcCalReset	(void *);
	int	fc0013_RcCalAdd		(void *, int);
};

#endif

