#
/*
 *
 *    Copyright (C)  2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	"noxon.h"
#include	"jff-include.h"
#include	"rtl2832u.h"
#include	"dongles.h"
#include	"oscillator.h"
//
static
bool	rtlsdr_callback (uint8_t	*buf,
	                 uint32_t	len,
	                 void		*ctx) {
RingBuffer<uint8_t>	*_ibuf;
int32_t	t;
	if (ctx) {
	   _ibuf	= ((noxonRig *)ctx) -> _I_Buffer;
	   _ibuf -> putDataIntoBuffer (len, buf);
	   ((noxonRig *)ctx) -> sampleCounter += len / 2;
//	   if (_ibuf -> ReadSpace () > 
//	              ((noxonRig *)ctx) -> oversampling * 1024)
//	      ((noxonRig *)ctx) -> raiseSignal ();
	   return ((noxonRig *)ctx) -> readerRunning;
	}
	return false;
}
//
//	Ok, it seems a little tricky what is being done here.
//	What we do is we get samples in with rate "sampleRate",
//	which is pretty high (10 - 15 times the desired rate).
//	We apply a bandpass filter such that we end up
//	with a band 0 .. rateOut with a samplerate of 2 * rateOut
//	We then shift the samples 0.5 * rateOut and we end up
//	with the band we want.
	noxonRig::noxonRig	(int32_t rateOut,
	                         int32_t tuningShift,
	                         int16_t downscale, bool *s) {
int16_t	i;
int16_t	devCount;
int16_t	devIndex	= 0;

	*s			= false;
	rig_is_ok		= false;
	sampleRate		= downscale * rateOut;
	oversampling		= downscale;
	this -> tuningShift	= tuningShift;
	sampleCounter		= 0;
	if (tuningShift == 0)
	   lowpass		= new DecimatingFIR (DECIMATION_FIR_SIZE,
	                                             rateOut / 2,
	                                             sampleRate, downscale);
	else
	   lowpass		= new DecimatingFIR (DECIMATION_FIR_SIZE,
	                                             0, rateOut,
	                                             sampleRate, downscale / 2);
	shifter			= new Oscillator (2 * rateOut);
	decimator		= false;

	devCount		= deviceCount	();
	if (devCount == 0) {
	   fprintf(stderr, "No supported devices found.\n");
	   return;
	}

	fprintf(stderr, "Found %d device(s):\n", devCount);
	for (i = 0; i < devCount; i++)
	   fprintf (stderr, "  %d:  %s\n", i, deviceName (i));
	fprintf (stderr, "\n");

	fprintf (stderr, "Using device %d: %s\n",
		devIndex,
		deviceName (devIndex));

	_I_Buffer		= new RingBuffer<uint8_t> (16 * 1024 * 1024);
	readerRunning		= false;
	usbInterface		= new RTL2832U	(devIndex,
	                                         rtlsdr_callback,
	                                         this,
	                                         s);
	if (!(*s)) {
	   fprintf (stderr, "Cannot open the usb device\n");
	   delete _I_Buffer;
	   return;
	}
	 
	usbInterface	-> set_sample_rate	(sampleRate);
	usbInterface	-> set_tuner_gain	(0);
	usbInterface	-> set_center_freq	(94700000 - tuningShift);
	lastFrequency	= 14000000;
	*s		= true;
	rig_is_ok	= true;
}

	noxonRig::~noxonRig	(void) {
	delete	lowpass;
	delete	shifter;
	if (rig_is_ok) {
//	we have to kill the task
	   usbInterface	-> stopStream ();
	   delete	usbInterface;
	   delete	_I_Buffer;
	}
}

void	noxonRig::setVFOFrequency	(int32_t f) {
	if (!rig_is_ok)
	   return;
	usbInterface	-> set_center_freq ((uint32_t)f - tuningShift);
	lastFrequency = f;
}

void	noxonRig::getVFOFrequency	(int32_t *f) {
	*f = lastFrequency; 
}

bool	noxonRig::legalFrequency	(int32_t f) {
	if (usbInterface == NULL)
	   return false;
	return usbInterface	-> legalFrequency (f/Khz (1));
}

int32_t	noxonRig::defaultFrequency	(void) {
	return Khz (94700);
}

const char	*noxonRig::nameofRig	(void) {
	return "noxon";
}

int32_t noxonRig::deviceCount (void) {
int i;
libusb_context *ctx;
libusb_device **list;
uint32_t device_count = 0;
struct libusb_device_descriptor dd;
ssize_t cnt;

	libusb_init (&ctx);
	cnt = libusb_get_device_list (ctx, &list);

	for (i = 0; i < cnt; i++) {
	   libusb_get_device_descriptor (list[i], &dd);

	   if (known_device (dd.idVendor, dd.idProduct))
	      device_count++;
	}

	libusb_free_device_list (list, 0);
	libusb_exit (ctx);

	return device_count;
}

const char *noxonRig::deviceName (int32_t index) {
int16_t	 i;
libusb_context *ctx;
libusb_device **list;
struct libusb_device_descriptor dd;
rtlsdr_device_t *device = NULL;
uint32_t device_count = 0;
ssize_t cnt;

	libusb_init (&ctx);
	cnt = libusb_get_device_list (ctx, &list);

	for (i = 0; i < cnt; i++) {
	   libusb_get_device_descriptor (list[i], &dd);

	   device = known_device (dd.idVendor, dd.idProduct);

	   if (device != NULL) {
	      device_count++;

	      if (index == (int)(device_count - 1))
	         break;
	   }
	}

	libusb_free_device_list (list, 0);
	libusb_exit (ctx);
	if (device != NULL)
	   return device -> name;
	else
	   return "";
}

bool	noxonRig::restartReader	(void) {
	if (readerRunning)
	   return true;
	(void)usbInterface	-> startStream ();
	readerRunning	= true;
	return true;
}

void	noxonRig::stopReader	(void) {
	if (!readerRunning)
	   return;
	usbInterface -> stopStream ();
	readerRunning = false;
}

//
//	just written to help in creating a
//	spectrum analyzer.
//	The function allows reading n samples (i.e 2 * n bytes)
//	and moving the reader pointer up segmentSize sample positions
int32_t	noxonRig::getSamplesandSkip	(DSPCOMPLEX *v,
	                                 int32_t n,
	                                 int32_t segmentSize) {
uint8_t *buf = (uint8_t *)alloca (2 * n * oversampling * sizeof (uint8_t));
int32_t i;
int32_t	skipSize	= 2 * (segmentSize - n);

	 _I_Buffer	-> getDataFromBuffer (2 * n, buf);
	 _I_Buffer	-> skipDataInBuffer  (skipSize);
	
	for (i = 0; i < n; i ++) 
	   v [i] = DSPCOMPLEX ((buf [2 * i] - 127.0) / 128.0,
	                       (buf [2 * i + 1] - 127.0) / 128.0);

	return n;
}

uint32_t	noxonRig::getSampleCount (void) {
	return sampleCounter;
}
//
//	The brave old getSamples. For the noxon rig, we get
//	a highrate sample stream the buffer in, and we deliver
//	a N-fold decimation. So, filtering is unavoidable.
int32_t	noxonRig::getSamples		(DSPCOMPLEX *V,
	                                 int32_t n, uint8_t Mode) {
uint8_t	*buf = (uint8_t *)alloca (2 * n * oversampling * sizeof (uint8_t));
int32_t	i;
int32_t s = _I_Buffer -> ReadSpace ();
int32_t	am;

	if (!readerRunning)
	   return 0;
//	
//	we have ensured that there is sufficient data in the buffer
	(void) _I_Buffer	-> getDataFromBuffer (2 * n * oversampling, buf);
//
//	Ok, data is in
	for (i = 0; i < n * oversampling; i ++) {
	   DSPCOMPLEX tmp = DSPCOMPLEX ((buf [2 * i] - 127.0) / 128.0,
	                                (buf [2 * i + 1] - 127.0 ) / 128.0);
//
//	The decimating filter returns a samplestream
//	with double speed, so on the fly we decimate again
	   if (lowpass -> Pass (tmp, &tmp)) {
//
//	The values we get are (0, 1), (1, 0), (0, -1), (-1, 0)
//	we skip half of the values, so there is no real need
//	to compute these
//	      tmp = tmp * shifter  -> nextValue (tuningShift);
	      if (decimator || (tuningShift == 0)) {
	         if (tuningShift != 0)
	            tmp = tmp * shifter  -> nextValue (2 * tuningShift);
	         decimator = false;
	         switch (Mode) {
	            default:
	            case IandQ:
	               V [i / oversampling] = tmp;
	               break;

	            case QandI:
	               V [i / oversampling] =
	                              DSPCOMPLEX (imag (tmp), real (tmp));
	               break;

	            case I_Only:
	               V [i / oversampling] =
	                              DSPCOMPLEX (real (tmp), 0.0);
	               break;

	            case Q_Only:
	               V [i / oversampling] =
	                              DSPCOMPLEX (imag (tmp), 0.0);
	               break;
	         }
	      }
	      else
	         decimator = true;
	   }
	}

	return n;
}

int32_t	noxonRig::Samples	(void) {
	return _I_Buffer	-> ReadSpace () / (2 * oversampling);
}

bool	noxonRig::RigprovidesSamples	(void) {
	return true;
}

