
//
//	This rtl2832u code is an adaptation of the code
//	as presented in the osmocom rtlsdr code
//	all rights are acknowledged
//

#include	"rtl2832u.h"
#include	"dongles.h"
#include	"j-tuner_fc0012.h"
#include	"j-tuner_fc0013.h"
#include	"j-tuner_e4000.h"
#include	"noxon.h"
#include	<QThread>
#include	<virtualrig.h>

#define DEFAULT_BUF_NUMBER	32
#define DEFAULT_BUF_LENGTH	(16 * 16384)

#ifndef	LIBUSB_CALL
#define	LIBUSB_CALL
#endif

//	Thread for handling events and callbacks for the
//	libusb
//	callback: through xfer we can reach the device structure, through
//	which we can access the user data passed through the higher level
//	caller (noxon)
static	
void	LIBUSB_CALL _libusb_callback (struct libusb_transfer *xfer) {
rtlsdrDev_t	*dev	= (rtlsdrDev_t *) xfer -> user_data;

	if (dev -> stop) {	// stop requested
	   dev -> async_status = RTLSDR_INACTIVE;
	   return;
	}
	
	if ((xfer -> status == LIBUSB_TRANSFER_COMPLETED)  &&
	    (dev -> cb != NULL)) {
	      (dev -> cb)(xfer -> buffer,
	                  xfer -> actual_length,
	                  dev  -> userData);
	   libusb_submit_transfer (xfer);
	   return;
	}
//	something wrong, just defaulting to!!!
	dev -> async_status = RTLSDR_CANCELING;
}
//
//	The sole reason of existence of the transfer task is
//	to create a control thread where we can handle the
//	libusb bulk transfer
//	Keep the task simple and
//	a. when doing a start read, we create and run the task
//	b, when doing a stop read, we cancel the requests and
//	   let the task die

class transferTask : public QThread {
private:
	rtlsdrDev_t	*TheDevice;
public:
	transferTask (rtlsdrDev_t *TheDevice) {
	this	-> TheDevice	= TheDevice;
	start ();
}

	~transferTask () {
}

void	run (void) {
int16_t	i;
struct timeval tv	= {1, 0};

	fprintf (stderr, "The task is running (again)\n");
//	start the bulk transfers
	TheDevice -> async_status = RTLSDR_RUNNING;
	for (i = 0; i < (int)(TheDevice -> xfer_buf_num); i ++) {
	   libusb_fill_bulk_transfer (TheDevice -> xfer [i],
	                              TheDevice -> devh,
	                              0x81,
	                              TheDevice -> xfer_buf [i],
	                              TheDevice -> xfer_buf_len,
	                              _libusb_callback,
	                              (void *)TheDevice,
	                              BULK_TIMEOUT);
	   libusb_submit_transfer (TheDevice -> xfer [i]);
//	   fprintf (stderr, "submitting transfer %d\n", i);
	}

//
//	proper handling of time outs
	while (TheDevice -> async_status != RTLSDR_INACTIVE) {
	   libusb_handle_events_timeout (TheDevice -> ctx, &tv);

	   int32_t d = ((noxonRig *)(TheDevice -> userData)) ->
	                       _I_Buffer -> ReadSpace ();
	   int32_t o = ((noxonRig *)(TheDevice -> userData)) -> oversampling;

	   ((noxonRig *)(TheDevice -> userData)) -> raiseSignal ();
//
//	If we are asked to stop (async_status == CANCELING) then
//	we set the status to INACTIVE, unless we find a transfer
//	that is still running
	   if (TheDevice -> async_status == RTLSDR_CANCELING) {
	      TheDevice -> async_status = RTLSDR_INACTIVE;

	      if (TheDevice ->xfer == NULL)
	         break;

	      for (i = 0; i < (int)(TheDevice -> xfer_buf_num); i ++) {
	         if (TheDevice -> xfer [i] == NULL)
	            continue;

	         if (TheDevice -> xfer [i] -> status ==
	                              LIBUSB_TRANSFER_COMPLETED) {
	            libusb_cancel_transfer (TheDevice -> xfer [i]);
	            TheDevice -> async_status = RTLSDR_CANCELING;
	          }
	      }

	   }
	}
//
//	when we are here, we know that the async_status
//	of the device is INACTIVE
fprintf (stderr, "leaving the task\n");
}
};

//
//	The class RTL2832U is a wrapper around a simple
//	thread-class that is capable of handling usb events
//	Most code is straightforward, having nothing
//	to do with concurrency at all
	RTL2832U::RTL2832U (int32_t di,
	                    rtlsdr_read_async_cb_t cb,	// user callback
	                    void *udata,		// calling env
	                    bool *s) {
int32_t index		= di;
libusb_device **list;
libusb_device *device	= NULL;
int32_t device_count	= 0;
struct libusb_device_descriptor dd;
struct libusb_device_descriptor devDesc;
uint8_t reg;
ssize_t cnt;
int32_t r, i;
uint8_t	szManufacturer [256];
uint8_t szProduct [256];

	*s		= false;	// unless proven otherwise
	TheDevice	= new rtlsdrDev_t;
	if (TheDevice == NULL) {	// should not happen
	   return;
	}

	tuner		= NULL;
	memset (TheDevice, 0, sizeof (rtlsdrDev_t));

	libusb_init (&TheDevice -> ctx);
	cnt = libusb_get_device_list (TheDevice -> ctx, &list);
	device		= NULL;
	for (i = 0; i < cnt; i++) {
	   device = list [i];
	   fprintf (stderr, "Bus %03d Device %03d ",
	                    libusb_get_bus_number (device),
	                    libusb_get_device_address (device) );
	   
	   if (libusb_get_device_descriptor (list [i], &dd) == 0) {
	      fprintf(stderr, "%d: %04x:%04x (bus %d, device %d)", i,
	                       dd.idVendor, dd.idProduct,
	                       libusb_get_bus_number (device),
	                       libusb_get_device_address (device));

	      if (known_device (dd.idVendor, dd.idProduct) != NULL) {
	         device_count++;
	         devDesc = dd;
	         fprintf (stderr, " <<***\n");
	      }
	      else
	         fprintf (stderr, "\n");

	      if (index == (int)(device_count - 1))
	         break;
	      device = NULL;
	   }
	}

	if (device == NULL) {
	   r = -1;
	   goto err;
	}
fprintf (stderr, "Opening the device:");
	r = libusb_open (device, &TheDevice -> devh);
	if (r < 0) {
	   libusb_free_device_list (list, 0);
	   fprintf(stderr, "failed %d\n", r);
	   *s = false;
	   return;
	}
fprintf (stderr, "succeeded\n");
//	if (libusb_get_string_descriptor_ascii (TheDevice -> devh,
//	                                        devDesc.iManufacturer,
//	                                        szManufacturer,
//	                                        sizeof szManufacturer) >= 0 &&
//	    libusb_get_string_descriptor_ascii (TheDevice -> devh,
//	                                        devDesc.iProduct,
//	                                        szProduct,
//	                                        sizeof szProduct) >= 0) 
//	     fprintf (stderr, "Found %s %s\n", szManufacturer, szProduct);

	libusb_free_device_list (list, 0);
	r = libusb_claim_interface (TheDevice -> devh, 0);
	if (r < 0) {
	   fprintf (stderr, "with error %d\n", r);
	   goto err;
	}
	init_baseband ();
fprintf (stderr, "going to probe tuners\n");
//	probe tuners, 
	set_i2c_repeater (true);
	reg = i2cRead_reg (E4K_I2C_ADDR, E4K_CHECK_ADDR);
	if (reg == E4K_CHECK_VAL) {
	   fprintf (stderr, "Found Elonics E4000 tuner\n");
	   tuner	= new TUNER_E4000 (this);
	   goto found;
	}

	reg = i2cRead_reg (FC0013_I2C_ADDR, FC0013_CHECK_ADDR);
	if (reg == FC0013_CHECK_VAL) {
	   fprintf (stderr, "Found Fitipower FC0013 tuner \n");
	   tuner	= new TUNER_FC0013 (this);
	   goto found;
	}

	/* initialise GPIOs */
	set_gpio_output (5);
	/* reset tuner before probing */
	set_gpio_bit (5, 1);
	set_gpio_bit (5, 0);

//	reg = i2cRead_reg (FC2580_I2C_ADDR, FC2580_CHECK_ADDR);
//	if ((reg & 0x7f) == FC2580_CHECK_VAL) {
//	   fprintf (stderr, "Found FCI 2580 tuner\n");
//	   TheDevice -> tuner = &tuners[RTLSDR_TUNER_FC2580];
//	   goto found;
//	}
//
	reg = i2cRead_reg (FC0012_I2C_ADDR, FC0012_CHECK_ADDR);
	if (reg == FC0012_CHECK_VAL) {
	   fprintf (stderr, "Found Fitipower FC0012 tuner\n");
	   set_gpio_output (6);
	   tuner	= new TUNER_FC0012 (this);
	   goto found;
	}

	fprintf (stderr, "No tuners found\n");	// fall through
err:
	if (TheDevice != NULL) {
	   if (TheDevice -> ctx != NULL)
	      libusb_exit (TheDevice -> ctx);

	   free (TheDevice);
	}

	set_i2c_repeater (false);
	*s		= false;
	return;

found:
	if (tuner != NULL)
	   (void)(tuner	-> init ());

	set_i2c_repeater (false);
	*s				= true;
	TheDevice	-> cb		= cb;
	TheDevice	-> userData	= udata;
	TheDevice	-> xfer_buf_num	= DEFAULT_BUF_NUMBER;
	TheDevice	-> xfer_buf_len	= DEFAULT_BUF_LENGTH;

	(void)alloc_async_buffers ();
//
//	all preparations made
	ourDriver	= NULL;
	TheDevice	-> async_status	= RTLSDR_INACTIVE;
}

	RTL2832U::~RTL2832U (void) {
int	r;
	if (TheDevice == NULL)
	   return;

	stopStream	();
	deinit_baseband ();
	
	free_async_buffers	();
	r = libusb_release_interface (TheDevice -> devh, 0);
if (r != 0)
	fprintf (stderr, "release gave an error %d\n", r);
	libusb_close		(TheDevice -> devh);
	libusb_exit		(TheDevice -> ctx);
	if (tuner != NULL) {
	   tuner -> exit ();
	   delete tuner;
	}

	if (TheDevice != NULL)
	   delete TheDevice;
fprintf (stderr, "OK, the rtl device is closed\n");
}
//
//	we start with low level communications

int32_t	RTL2832U::readArray (uint8_t	block,
	                     uint16_t	addr, 
	                     uint8_t	*array,
	                     uint8_t	len) {
int32_t	r;
uint16_t	index	= (block << 8);
	r	= libusb_control_transfer (TheDevice	-> devh,
	                                   CTRL_IN, 0,
	                                   addr, index, array,
	                                   len, CTRL_TIMEOUT);
	return r;
}

int32_t	RTL2832U::writeArray (uint8_t	block,
	                      uint16_t	addr, 
	                      uint8_t	*array,
	                      uint8_t	len) {

int32_t		r;
uint16_t	index = (block << 8) | 0x10;
	r	= libusb_control_transfer (TheDevice	-> devh,
	                                   CTRL_OUT, 0,
	                                   addr, index, array,
	                                   len, CTRL_TIMEOUT);
	return r;
}

uint16_t RTL2832U::readReg (uint8_t block,
	                    uint16_t addr, uint8_t len) {
int r;
unsigned char data[2];
uint16_t index = (block << 8);
uint16_t reg;

	r = libusb_control_transfer (TheDevice -> devh,
	                             CTRL_IN, 0,
	                             addr, index, data, len, CTRL_TIMEOUT);
	if (r < 0)
	   fprintf(stderr, "%s failed with %d\n", __FUNCTION__, r);

	reg = (data[1] << 8) | data[0];
	return reg;
}

void	RTL2832U::writeReg (uint8_t block,
	                    uint16_t addr, uint16_t val, uint8_t len) {
int r;
unsigned char data[2];
uint16_t index = (block << 8) | 0x10;

	if (len == 1)
	   data [0] = val & 0xff;
	else
	   data [0] = val >> 8;
	data[1] = val & 0xff;

	r = libusb_control_transfer (TheDevice -> devh,
	                             CTRL_OUT, 0,
	                             addr, index, data, len, CTRL_TIMEOUT);
	if (r < 0)
	   fprintf(stderr, "%s failed with %d\n", __FUNCTION__, r);
}

uint16_t RTL2832U::demodRead_reg (uint8_t page,
	                          uint16_t addr, uint8_t len) {
int	r;
unsigned char data[2];
uint16_t index = page;
uint16_t reg;

	addr = (addr << 8) | 0x20;

	r = libusb_control_transfer (TheDevice -> devh,
	                             CTRL_IN, 0,
	                             addr, index, data, len, CTRL_TIMEOUT);
	if (r < 0)
	   fprintf(stderr, "%s failed with %d\n", __FUNCTION__, r);

	reg = (data[1] << 8) | data[0];
	return reg;
}

void	RTL2832U::demodWrite_reg (uint8_t page,
	                          uint16_t addr,
	                          uint16_t val, uint8_t len) {
int r;
unsigned char data[2];
uint16_t index = 0x10 | page;

	addr = (addr << 8) | 0x20;
	if (len == 1)
	   data [0] = val & 0xff;
	else
	   data [0] = val >> 8;

	data [1] = val & 0xff;

	r = libusb_control_transfer (TheDevice -> devh,
	                             CTRL_OUT, 0,
	                             addr, index, data, len, CTRL_TIMEOUT);

	if (r < 0)
	   fprintf(stderr, "%s failed with %d\n", __FUNCTION__, r);

	demodRead_reg (0x0a, 0x01, 1);
}
//
//	i2c routines, using the lower levels
//
int32_t	RTL2832U::i2cWrite_reg (uint8_t i2c_addr,
	                        uint8_t reg, uint8_t val) {
uint16_t addr = i2c_addr;
uint8_t	data[2];

	data[0] = reg;
	data[1] = val;
	return writeArray (IICB, addr, (uint8_t *)&data, 2);
}

uint8_t RTL2832U::i2cRead_reg (uint8_t i2c_addr, uint8_t reg) {
uint16_t addr = i2c_addr;
uint8_t data;

	writeArray	(IICB, addr, &reg, 1);
	readArray	(IICB, addr, &data, 1);
	return data;
}

int32_t	RTL2832U::i2cWrite (uint8_t i2c_addr,
	                    uint8_t *buffer, int32_t len) {
uint16_t addr = i2c_addr;

	if (TheDevice == NULL)
	   return -1;

	return writeArray (IICB, addr, buffer, len);
}

int32_t	RTL2832U::i2cRead (uint8_t i2c_addr,
	                   uint8_t *buffer, int32_t len) {
uint16_t addr = i2c_addr;

	if (TheDevice == NULL)
	   return -1;

	return readArray (IICB, addr, buffer, len);
}
//
//
void	RTL2832U::init_baseband (void) {
uint16_t i;

/* default FIR coefficients used for DAB/FM by the Windows driver,
 * the DVB driver uses different ones */
uint8_t fir_coeff[] = {
	0xca, 0xdc, 0xd7, 0xd8, 0xe0, 0xf2, 0x0e, 0x35, 0x06, 0x50,
	0x9c, 0x0d, 0x71, 0x11, 0x14, 0x71, 0x74, 0x19, 0x41, 0xa5,
};

	/* initialize USB */
	writeReg (USBB, USB_SYSCTL, 0x09, 1);
	writeReg (USBB, USB_EPA_MAXPKT, 0x0002, 2);
	writeReg (USBB, USB_EPA_CTL, 0x1002, 2);

	/* poweron demod */
	writeReg (SYSB, DEMOD_CTL_1, 0x22, 1);
	writeReg (SYSB, DEMOD_CTL, 0xe8, 1);

	/* reset demod (bit 3, soft_rst) */
	demodWrite_reg (1, 0x01, 0x14, 1);
	demodWrite_reg (1, 0x01, 0x10, 1);

	/* disable spectrum inversion and adjacent channel rejection */
	demodWrite_reg (1, 0x15, 0x00, 1);
	demodWrite_reg (1, 0x16, 0x0000, 2);

	/* set IF-frequency to 0 Hz */
	demodWrite_reg (1, 0x19, 0x0000, 2);

	/* set FIR coefficients */
	for (i = 0; i < sizeof (fir_coeff); i++)
	   demodWrite_reg (1, 0x1c + i, fir_coeff[i], 1);

	demodWrite_reg (0, 0x19, 0x25, 1);

	/* init FSM state-holding register */
	demodWrite_reg (1, 0x93, 0xf0, 1);

	/* disable AGC (en_dagc, bit 0) */
	demodWrite_reg (1, 0x11, 0x00, 1);

	/* disable PID filter (enable_PID = 0) */
	demodWrite_reg (0, 0x61, 0x60, 1);

	/* opt_adc_iq = 0, default ADC_I/ADC_Q datapath */
	demodWrite_reg (0, 0x06, 0x80, 1);

	/* Enable Zero-IF mode (en_bbin bit), DC cancellation (en_dc_est),
	 * IQ estimation/compensation (en_iq_comp, en_iq_est) */
	demodWrite_reg (1, 0xb1, 0x1b, 1);
}

void	RTL2832U::deinit_baseband (void) {
	if (TheDevice == NULL)
	   return;

	/* deinitialize tuner */
	set_i2c_repeater (true);
	tuner	-> exit ();
	set_i2c_repeater (false);

	/* poweroff demodulator and ADCs */
	writeReg (SYSB, DEMOD_CTL, 0x20, 1);
}

void	RTL2832U::set_i2c_repeater (bool on) {
	demodWrite_reg (1, 0x01, on ? 0x18 : 0x10, 1);
}

void	RTL2832U::set_gpio_bit (uint8_t gpio, int32_t val) {
uint8_t r;

	gpio = 1 << gpio;
	r = readReg (SYSB, GPO, 1);
	r = val ? (r | gpio) : (r & ~gpio);
	writeReg (SYSB, GPO, r, 1);
}

void	RTL2832U::set_gpio_output (uint8_t gpio) {
int	r;

	gpio = 1 << gpio;
	r = readReg (SYSB, GPD, 1);
	writeReg (SYSB, GPO, r & ~gpio, 1);
	r = readReg (SYSB, GPOE, 1);
	writeReg (SYSB, GPOE, r | gpio, 1);
}

int32_t	RTL2832U::set_center_freq (uint32_t freq) {
int32_t r;
double f = (double) freq;

	if (tuner == NULL)
	   return -1;

	f *= 1.0 + tuner -> get_correction () /1e6;
	set_i2c_repeater (true);
	r = tuner -> tune ((uint32_t)f);
	set_i2c_repeater (false);
	return r;
}

int32_t	RTL2832U::get_center_freq (void) {

	if (tuner == NULL)
	   return -1;

	return tuner -> get_frequency ();
}

int32_t	RTL2832U::set_freq_correction (int32_t ppm) {
int32_t r;

	if (tuner == NULL)
	   return -1;

	if (tuner -> get_correction () == ppm)
	   return -1;
	set_i2c_repeater (true);
	tuner	-> set_correction (ppm);
	r = set_center_freq (tuner -> get_frequency ());
	set_i2c_repeater (false);
	return r;
}

int32_t	RTL2832U::get_freq_correction (void) {
	if (tuner == NULL)
	   return -1;

	return tuner	-> get_correction ();
}

int32_t	RTL2832U::set_tuner_gain (int32_t gain) {
int32_t r;

	if (tuner == NULL)
	   return - 1;

	set_i2c_repeater (true);
	r = tuner -> set_gain (gain);
	set_i2c_repeater (false);
	return r;
}

int32_t	RTL2832U::get_tuner_gain (void) {

	if (tuner == NULL)
	   return -1;

	return tuner -> get_gain ();
}

bool	RTL2832U::set_sample_rate (uint32_t samp_rate) {
uint16_t tmp;
uint32_t rsamp_ratio;
double	real_rate;

	if (TheDevice == NULL)
	   return false;

	/* check for the maximum rate the resampler supports */
	if (samp_rate > MAX_SAMP_RATE)
	   samp_rate = MAX_SAMP_RATE;

	rsamp_ratio = (CRYSTAL_FREQ * pow (2, 22)) / samp_rate;
	rsamp_ratio &= ~3;

	real_rate = (CRYSTAL_FREQ * pow (2, 22)) / rsamp_ratio;

	if ( ((double)samp_rate) != real_rate )
	   fprintf(stderr, "Exact sample rate: %f Hz\n", real_rate);

	if (tuner != NULL) {
	   set_i2c_repeater (true);
	   tuner -> set_bw (real_rate);
	   set_i2c_repeater (false);
	}

	TheDevice -> rate = samp_rate;

	tmp = (rsamp_ratio >> 16);
	demodWrite_reg	(1, 0x9f, tmp, 2);

	tmp = rsamp_ratio & 0xffff;
	demodWrite_reg	(1, 0xa1, tmp, 2);

	/* reset demod (bit 3, soft_rst) */
	demodWrite_reg (1, 0x01, 0x14, 1);
	demodWrite_reg (1, 0x01, 0x10, 1);

	return true;
}

int32_t	RTL2832U::get_sample_rate (void) {

	if (TheDevice == NULL)
	   return -1;

	return TheDevice -> rate;
}

bool	RTL2832U::resetBuffer (void) {
	if (TheDevice == NULL)
	   return false;

	writeReg (USBB, USB_EPA_CTL, 0x1002, 2);
	writeReg (USBB, USB_EPA_CTL, 0x0000, 2);
	return true;
}

//
//	These functions are made this way to be consistent
//	with the functionality provided by the portaudio interface
//	as the name suggests, startstream causes the stream to (re)start
bool	RTL2832U::startStream	(void) {

	if ((TheDevice == NULL) ||  (ourDriver != NULL))
	   return false;

	resetBuffer ();
	ourDriver	= new transferTask (TheDevice);
	return true;
}

void	RTL2832U::stopStream (void) {

	if ((TheDevice == NULL) || (ourDriver == NULL))
	   return;

	if (TheDevice -> async_status == RTLSDR_INACTIVE)
	   return;

	TheDevice -> async_status = RTLSDR_CANCELING;
	while (!ourDriver -> isFinished ())
	  usleep (100);

	delete ourDriver;
	ourDriver	= NULL;
}

bool	RTL2832U::alloc_async_buffers (void) {
int32_t	i;

	if (TheDevice == NULL)
	   return false;

	if (TheDevice -> xfer == NULL) {
	   TheDevice -> xfer =
	          new struct libusb_transfer * [TheDevice -> xfer_buf_num];

	   for (i = 0; i < (int)(TheDevice -> xfer_buf_num); i ++)
	      TheDevice -> xfer [i] = libusb_alloc_transfer (0);
	}

	if (TheDevice -> xfer_buf == NULL) {
	   TheDevice -> xfer_buf = new uint8_t *[TheDevice -> xfer_buf_num];

	   for (i = 0; i < (int)(TheDevice -> xfer_buf_num); i ++)
	      TheDevice -> xfer_buf [i] =
	                      new uint8_t [TheDevice -> xfer_buf_len];
	}

	return true;
}

bool	RTL2832U::free_async_buffers (void) {
int32_t i;

	if (TheDevice == NULL)
	   return false;

	if (TheDevice -> xfer != NULL) {
	   for (i = 0; i < (int)(TheDevice -> xfer_buf_num); i++) {
	      if (TheDevice -> xfer [i] != NULL) {
	         libusb_free_transfer (TheDevice -> xfer [i]);
	      }
	   }

	   delete[]  TheDevice -> xfer;
	   TheDevice -> xfer = NULL;
	}

	if (TheDevice -> xfer_buf != NULL) {
	   for (i = 0; i < (int)(TheDevice -> xfer_buf_num); i ++) {
	      if (TheDevice -> xfer_buf [i] != NULL)
	         delete []  TheDevice -> xfer_buf [i];
	   }

	   delete [] TheDevice -> xfer_buf;
	   TheDevice -> xfer_buf = NULL;
	}

	return true;
}
//
//	legalFrequency is in Khz
bool	RTL2832U::legalFrequency	(int32_t f) {
	if (tuner == NULL)
	   return false;

	return tuner -> legalFrequency (f);
}

