#
/*
 *
 *    Copyright (C) 2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __RTL2832U
#define	__RTL2832U

#include	<signal.h>
#include	<string.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<QThread>
#include	<QObject>
#include	"libusb-1.0/libusb.h"
#include	"virtual_tuner.h"

class		transferTask;
//
//	This is a hack of the osmocom software for the rtl2832u
//	It was hacked this way to have it create a separate thread/class
//	for managing the usb interface to the rtl2832u
//	All rights are acknowledged.

class	RTL2832U;

typedef bool (*rtlsdr_read_async_cb_t)(uint8_t *, uint32_t, void *);

typedef struct rtlsdr_device {
	uint16_t vid;
	uint16_t pid;
	const char *name;
} rtlsdr_device_t;


enum rtlsdr_async_status {
	RTLSDR_INACTIVE = 0,
	RTLSDR_CANCELING,
	RTLSDR_RUNNING
};

//
typedef struct rtlsdr_dev {
	libusb_context		*ctx;
	struct libusb_device_handle *devh;
	uint32_t		xfer_buf_num;
	uint32_t		xfer_buf_len;
	struct libusb_transfer	**xfer;
	unsigned char		**xfer_buf;
	rtlsdr_read_async_cb_t	cb;
	void			*cb_ctx;
	uint32_t		rtl_xtal;
	enum rtlsdr_async_status async_status;
	bool			stop;
	int			rate; /* Hz */
	void			*userData;		// just something I need
} rtlsdrDev_t;

#define CRYSTAL_FREQ	28800000
#define MAX_SAMP_RATE	5000000

#define CTRL_IN		(LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_IN)
#define CTRL_OUT	(LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_OUT)
#define CTRL_TIMEOUT	300
#define BULK_TIMEOUT	0

enum usb_reg {
	USB_SYSCTL		= 0x2000,
	USB_CTRL		= 0x2010,
	USB_STAT		= 0x2014,
	USB_EPA_CFG		= 0x2144,
	USB_EPA_CTL		= 0x2148,
	USB_EPA_MAXPKT		= 0x2158,
	USB_EPA_MAXPKT_2	= 0x215a,
	USB_EPA_FIFO_CFG	= 0x2160,
};

enum sys_reg {
	DEMOD_CTL		= 0x3000,
	GPO			= 0x3001,
	GPI			= 0x3002,
	GPOE			= 0x3003,
	GPD			= 0x3004,
	SYSINTE			= 0x3005,
	SYSINTS			= 0x3006,
	GP_CFG0			= 0x3007,
	GP_CFG1			= 0x3008,
	SYSINTE_1		= 0x3009,
	SYSINTS_1		= 0x300a,
	DEMOD_CTL_1		= 0x300b,
	IR_SUSPEND		= 0x300c,
};

enum blocks {
	DEMODB			= 0,
	USBB			= 1,
	SYSB			= 2,
	TUNB			= 3,
	ROMB			= 4,
	IRB			= 5,
	IICB			= 6,
};

class	RTL2832U {
public:
		RTL2832U	(int32_t,
	                         rtlsdr_read_async_cb_t,
	                         void *,
	                         bool *);
		~RTL2832U	(void);

	int32_t		set_center_freq		(uint32_t);
	int32_t		get_center_freq		(void);
	int32_t		set_freq_correction	(int32_t);
	int32_t		get_freq_correction	(void);
	int32_t		set_tuner_gain		(int32_t);
	int32_t		get_tuner_gain		(void);
	bool		set_sample_rate		(uint32_t);
	int32_t		get_sample_rate		(void);
	bool		startStream		(void);
	void		stopStream		(void);
	void		set_stop		(void);
	bool		stillRunning		(void);
	int32_t		i2cWrite		(uint8_t, uint8_t *, int32_t);
	int32_t		i2cRead			(uint8_t, uint8_t *, int32_t);
	bool		legalFrequency		(int32_t);	// khz
	void		set_gpio_bit		(uint8_t, int32_t);
	rtlsdrDev_t	*TheDevice;
private:
	bool		resetBuffer	(void);
	void		run		(void);
	int32_t		readArray	(uint8_t, uint16_t, uint8_t *, uint8_t);
	int32_t		writeArray	(uint8_t,
	                                 uint16_t, uint8_t *, uint8_t);
	uint16_t	readReg		(uint8_t, uint16_t, uint8_t);
	void		writeReg	(uint8_t, uint16_t, uint16_t, uint8_t);
	uint16_t	demodRead_reg	(uint8_t, uint16_t, uint8_t);
	void		demodWrite_reg	(uint8_t, uint16_t, uint16_t, uint8_t);
	int32_t		i2cWrite_reg	(uint8_t, uint8_t, uint8_t);
	uint8_t		i2cRead_reg	(uint8_t, uint8_t);
	void		init_baseband	(void);
	void		deinit_baseband	(void);
	void		set_i2c_repeater	(bool);
	void		set_gpio_output	(uint8_t);
	bool		alloc_async_buffers	(void);
	bool		free_async_buffers	(void);
	int32_t		devIndex;
	transferTask	*ourDriver;
	virtual_tuner	*tuner;
};

#endif

