#
/*
 *
 *    Copyright (C) 2008, 2009, 2010, 2011
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * 	This particular driver is a very simple wrapper around the
 * 	librtlsdr. The librtlsdr is osmocom software and all rights
 * 	are greatly acknowledged
 *
 * 	To keep things simple, we just include the code
 * 	for the librtlsdr as for the tuners
 */


#define READLEN_DEFAULT 8192

#include	<QThread>
#include	"librtlsdr.c"
//	do not mess with this order, we are a user of
//	librtlsdr
#include	"dabstick.h"

#include	"tuner_e4k.c"
#include	"tuner_fc0012.c"
#include	"tuner_fc0013.c"
#include	"tuner_fc2580.c"
#include	"tuner_r820t.c"
//
//
//	For the callback, we do need some environment which
//	is not provides by the librtlsdr. That is why we just
//	put the "context", i.e. the (sole) instance of the class
//	dabstick, into a static variable	
static	dabstick	*DAB	= NULL;
//
//	This is the user-side call back function
static
void	RTLSDRCallBack (unsigned char *buf, uint32_t len, void *ctx) {
	if ((len != READLEN_DEFAULT) || (DAB == NULL))
	   return;
//	Onze context staat in DAB
	DAB	-> sampleCounter	+= len / 2;
	DAB	-> _I_Buffer -> putDataIntoBuffer (buf, len);
	if (DAB -> Samples () > DAB -> rateOut / 20)
	   DAB	-> raiseSignal ();
}
//
//	for handling the events in libusb, we need a controlthread
//	whose sole purpose is to process the rtlsdr_read_async function
//	from the lib.
class	Driver : public QThread {
private:
	dabstick	*theStick;
public:

	Driver (dabstick *d) {
	theStick	= d;
	start ();
	}
	~Driver (void) {
	}
private:
virtual void	run (void) {
	rtlsdr_read_async (theStick -> device,
	                   (rtlsdr_read_async_cb_t)&RTLSDRCallBack,
	                   NULL,
	                   0,
	                   READLEN_DEFAULT);
	}
};

//
//	Our wrapper is a simple classs
	dabstick::dabstick (int32_t rateOut,
	                    int32_t rateIn, bool *success) {
int16_t	deviceCount;
int32_t	r;
int	error;

	*success	= false;	// just the default
	open		= false;
	_I_Buffer	= NULL;
	lastFrequency	= KHz(94700);	// just a dummy
	this	-> rateOut	= rateOut;
	this	-> rateIn	= rateIn;
	this	-> realScale	= (double)rateIn / rateOut;
	this	-> decimatingScale	= (int)floor (realScale);
	this	-> integerDecimation	=
	                         abs (realScale / decimatingScale - 1) < 0.001;
	this	-> sampleCounter	= 0;
	this	-> lowpass		= NULL;

	deviceCount = rtlsdr_get_device_count ();
	if (deviceCount == 0) {
	   fprintf (stderr, "No devices found\n");
	   return;
	}
//
//	OK, now open the hardware
	r	= rtlsdr_open (&device, 0);
	if (r < 0) {
	   fprintf (stderr, "Opening dabstick failed\n");
	   *success = false;
	   return;
	}
	open	= true;
	r	= rtlsdr_set_sample_rate (device, rateIn);
	if (r < 0) {
	   fprintf (stderr, "Setting samplerate failed\n");
	   *success = false;
	   return;
	}

	rtlsdr_set_tuner_gain_mode (device, 1);
	rtlsdr_set_tuner_gain (device, 190);
	_I_Buffer	= new RingBuffer<uint8_t>(1024 * 1024);
	DAB		= this;
	workerHandle	= NULL;
	converter	= src_new (SRC_SINC_BEST_QUALITY, 2, &error);
	if (converter == NULL) {
	   fprintf (stderr, " Fatal error creating a converter\n");
	   return;
	}
	lowpass		= new DecimatingFIR (decimatingScale * 5,
	                                     rateOut / 2,
	                                     rateIn,
	                                     decimatingScale);
	src_data	= new SRC_DATA;
	*success = true;
}

	dabstick::~dabstick	(void) {
//fprintf (stderr, "closing dabstick workinghandle = %d\n", workerHandle != NULL);
	if (open)
	   rtlsdr_close (device);
	if (_I_Buffer != NULL)
	   delete _I_Buffer;
	if (lowpass != NULL) {
	   delete lowpass;
	   src_delete (converter);
	   delete	src_data;
	}

	lowpass = NULL;
	open = false;
}

void	dabstick::setVFOFrequency	(int32_t f) {
	lastFrequency	= f;
	(void)rtlsdr_set_center_freq (device, f + vfoOffset);
}

void	dabstick::getVFOFrequency	(int32_t *f) {
	*f = (int32_t)rtlsdr_get_center_freq (device) - vfoOffset;
}

int32_t	dabstick::defaultFrequency	(void) {
	return KHz (94700);
}

const char	*dabstick::nameofRig		(void) {
	return "dabstick";
}
//
//
bool	dabstick::restartReader	(void) {
int32_t	r;

	if (workerHandle != NULL)
	   return true;
	r = rtlsdr_reset_buffer (device);
	if (r < 0)
	   return false;

	rtlsdr_set_center_freq (device, lastFrequency + vfoOffset);
	workerHandle	= new Driver (this);
	return true;
}

void	dabstick::stopReader		(void) {
	if (workerHandle == NULL)
	   return;

	rtlsdr_cancel_async (device);
	if (workerHandle != NULL) {
	   while (!workerHandle -> isFinished ()) 
	      usleep (100);

	   delete	workerHandle;
	}
	workerHandle	= NULL;
}

int32_t	dabstick::setExternalRate	(int32_t newRate) {
int32_t	r;

	if (newRate < 900000) return rateIn;
	if (newRate > 3200000) return rateIn;

	realScale	= (double) newRate / rateOut;
	decimatingScale	= (int)floor (realScale);
	this	-> integerDecimation =
	            abs (realScale / decimatingScale - 1) < 0.001;

	if (workerHandle == NULL) {
	   r	= rtlsdr_set_sample_rate (device, newRate);
	   if (r < 0) {
	      rtlsdr_set_sample_rate (device, rateIn);
	   }

	   delete lowpass;
	   lowpass	= new DecimatingFIR (decimatingScale * 5,	// order
	                                     rateOut / 2,	// cut off freq
	                                     newRate, // samplefreq
	                                     decimatingScale);	// going down
	   r	= rtlsdr_get_sample_rate (device);
	}
	else {	
//	we stop the transmission
	   stopReader ();
	   r	= rtlsdr_set_sample_rate (device, newRate);
	   if (r < 0) {
	      rtlsdr_set_sample_rate (device, rateIn);
	   }

	   delete lowpass;
	   lowpass		= new DecimatingFIR (decimatingScale * 5,
	                                             rateOut / 2,
	                                             newRate, 
	                                             decimatingScale);	
	   r	= rtlsdr_get_sample_rate (device);
//	ok all set, continue
	   restartReader ();
	}

	rateIn	= newRate;
	return r;
}

static
int gains[] = {-10, 15, 40, 65, 90, 115, 140, 165, 190,
	           215, 240, 290, 340, 420, 430, 450, 470, 490};
int32_t	dabstick::setExternalGain	(int32_t gain) {
static int	oldGain	= 0;

	if (gain == oldGain)
	   return gains [oldGain];
	if ((gain < 0) || (gain > 17))
	   return gains [oldGain];	

	oldGain	= gain;
	rtlsdr_set_tuner_gain (device, gains [gain]);
	return rtlsdr_get_tuner_gain (device);
}

//
//	The brave old getSamples. For the dab stick, we get
//	a highrate sample stream the buffer in, and we deliver
//	a R-fold decimation. So, filtering is unavoidable.
//
int32_t	dabstick::getSamples (DSPCOMPLEX *V,
	                      int32_t n, uint8_t Mode) {
	return integerDecimation ?
	          getSamples_I (V, n, Mode) :
	       getSamples_C (V, n, Mode);
}

int32_t	dabstick::getSamples_I (DSPCOMPLEX *V,
	                        int32_t n, uint8_t Mode) {
int32_t	size	= n * decimatingScale;
uint8_t	*buf	= (uint8_t *)alloca (2 * size * sizeof (uint8_t));
int32_t	i, index;
//
//	first step: get data from the buffer
	(void) _I_Buffer	-> getDataFromBuffer (buf, 2 * size);

	for (i = 0, index = 0; i < size; i ++)  {
	   DSPCOMPLEX tmp = DSPCOMPLEX ((buf [2 * i] - 127.0) / 128.0,
	                                (buf [2 * i + 1] - 127.0) / 128.0);
	   if (lowpass -> Pass (tmp, &tmp)) {
	      switch (Mode) {
	         default:
	         case IandQ:
	            V [index ++] = tmp;
	            break;

	         case QandI:
	            V [index ++] = DSPCOMPLEX (imag (tmp), real (tmp));
	            break;

	         case Q_Only:
	            V [index ++] = DSPCOMPLEX (real (tmp), 0.0);
	            break;

	         case I_Only:
	            V [index ++] = DSPCOMPLEX (imag (tmp), 0.0);
	            break;
	      }
	   }
	}

	return index;
}

int32_t	dabstick::getSamples_C (DSPCOMPLEX *V,
	                        int32_t n, uint8_t Mode) {
int32_t	IBuf	= 2 * ceil (realScale * n);
uint8_t	*buf	= (uint8_t *)alloca (IBuf * sizeof (uint8_t));
float	*FBuf	= (float *)alloca (2 * decimatingScale * n * sizeof (float));
float	*OBuf	= (float *)alloca (2 * n * sizeof (float));
int32_t	i, index;
int	res;

	src_data -> data_in		= FBuf;
	src_data -> data_out		= OBuf;
	src_data -> end_of_input	= 0;
	src_data -> src_ratio		= realScale / decimatingScale;
	src_data -> output_frames	= n;
	src_data -> input_frames	= 2 * src_data -> src_ratio * n;
//
//	first step: get data from the buffer
	(void) _I_Buffer -> getDataFromBuffer (buf, decimatingScale *
	                                            src_data -> input_frames);

//	Second step: decimate and filter an integer amount
	for (i = 0, index = 0; i < IBuf / 2; i ++)  {
	   DSPCOMPLEX tmp = DSPCOMPLEX ((buf [2 * i] - 127.0) / 128.0,
	                                (buf [2 * i + 1] - 127.0) / 128.0);
	   if (lowpass -> Pass (tmp, &tmp)) {
	      FBuf [2 * index]		= real (tmp);
	      FBuf [2 * index + 1]	= imag (tmp);
	      index ++;
	   }
	}
//
//	third step: resample realScale / decimatingScale
//
	res		= src_process (converter, src_data);
	if (res != 0) {
	   fprintf (stderr, "error %s\n", src_strerror (res));
	   return 0;
	}

	for (i = 0; i < src_data -> output_frames_gen; i ++) {
	   switch (Mode) {
	      default:
	      case IandQ:
	         V [i] = DSPCOMPLEX (FBuf [2 * i],
	                             FBuf [2 * i + 1]);
	         break;

	      case QandI:
	         V [i] = DSPCOMPLEX (FBuf [2 * i + 1],
	                             FBuf [2 * i]);
	         break;

	      case Q_Only:
	         V [i] = DSPCOMPLEX (FBuf [2 * i + 1], 0.0);
	         break;

	      case I_Only:
	         V [i] = DSPCOMPLEX (FBuf [2 * i], 0.0);
	         break;
	   }
	}

	return src_data -> output_frames_gen; 
}

int32_t	dabstick::Samples	(void) {
	return _I_Buffer	-> GetRingBufferReadAvailable () / ((int)(2 * realScale));
}
//
//	local redefine
bool	dabstick::RigprovidesSamples	(void) {
	return	true;
}
//	local redefine
bool	dabstick::legalFrequency	(int32_t f) {
	return true;
	return MHz(20) <= f && f <= MHz (1500);
}
//
//	and especially for our beloved spectrum viewer we provide
int32_t	dabstick::getSamplesandSkip 	(DSPCOMPLEX *v,
	                                 int32_t n,
	                                 int32_t segmentSize) {
uint8_t *buf		= (uint8_t *)alloca (2 * n);
int32_t i;
int32_t skipSize	= 2 * (segmentSize - n);

	_I_Buffer	-> getDataFromBuffer (buf, 2 * n);
	_I_Buffer	-> skipDataInBuffer (skipSize);

	for (i = 0; i < n; i ++) 
	   v [i] = DSPCOMPLEX ((buf [2 * i] - 127.0) / 128.0,
	                       (buf [2 * i + 1] - 127.0) / 128.0);
	return n;
}

int32_t	dabstick::getSampleCount	(void) {
	return sampleCounter;
}

