#
/*
 *
 *    Copyright (C) 2008, 2009, 2010, 2011
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * 	This particular driver is a very simple wrapper around the
 * 	librtlsdr. The librtlsdr is osmocom software and all rights
 * 	are greatly acknowledged
 *
 *	To keep things simple, we just load the librtlsdr library
 *	(for Linux we created a shared lib ourselves)
 */

#define READLEN_DEFAULT 8192

#include	<QThread>
#include	"dabstick-dll.h"
#include	<stdio.h>
//
//	The naming of functions for accessing shared libraries
//	differ between Linux and Windows
#ifdef __MINGW32__
#define	GETPROCADDRESS	GetProcAddress
#else
#define	GETPROCADDRESS	dlsym
#endif
//	For the callback, we do need some environment which
//	is not provides by the librtlsdr. That is why we just
//	put the "context", i.e. the (sole) instance of the class
//	dabstick, into a static variable	
static	dabstick_dll	*DAB	= NULL;
//
//	This is the user-side call back function
static
void	RTLSDRCallBack (unsigned char *buf, uint32_t len, void *ctx) {
	(void)ctx;
	if ((len != READLEN_DEFAULT) || (DAB == NULL))
	   return;
//	Onze context staat in DAB
	DAB	-> sampleCounter	+= len / 2;
	DAB	-> _I_Buffer -> putDataIntoBuffer (buf, len);
	if (DAB -> Samples () > DAB -> rateOut / 20) 
	   DAB	-> raiseSignal ();
}
//
//	for handling the events in libusb, we need a controlthread
//	whose sole purpose is to process the rtlsdr_read_async function
//	from the lib.
class	dll_driver : public QThread {
private:
	dabstick_dll	*theStick;
public:

	dll_driver (dabstick_dll *d) {
	theStick	= d;
	start ();
	}
	~dll_driver (void) {
	}
private:
virtual void	run (void) {
	(theStick -> rtlsdr_read_async) (theStick -> device,
	                          (rtlsdr_read_async_cb_t)&RTLSDRCallBack,
	                          NULL,
	                          0,
	                          READLEN_DEFAULT);
	}
};
//
//	Our wrapper is a simple classs
//	The difference with the dabstick in dabstick is
//	that first we load here the function pointers
//
//
	dabstick_dll::dabstick_dll (int32_t rateOut,
	                            int32_t rateIn, bool *success) {
int	r;
int	error;
//
	*success	= false;	// default
	libraryLoaded	= false;
	workerHandle	= NULL;
	d_filter	= NULL;
	DAB		= NULL;
	_I_Buffer	= NULL;
	open		= false;

#ifdef	__MINGW32__
	Handle		= LoadLibrary ((wchar_t *)L"rtlsdr.dll");
#else
	Handle		= dlopen ("./librtlsdr.so", RTLD_NOW);
#endif
	if (Handle == NULL) {
	   fprintf (stderr, "Failed to open rtlsdr.dll\n");
	   return;
	}

	libraryLoaded	= true;
	fprintf (stderr, "rtlsdr is loaded\n");

	if (!load_rtlFunctions	())
	   goto err;
//
//	OK, from here we have the code of the "other" dabstick
//	vfoFrequency is the VFO frequency
	vfoFrequency		= KHz(94700);	// just a dummy
	localOffset		= 0;
	this	-> rateOut	= rateOut;
	this	-> rateIn	= rateIn;
	this	-> realScale	= (double)rateIn / rateOut;	// a starter
	this	-> decimatingScale	= (int)floor (realScale);
	this	-> integerDecimation	=
	                  abs (realScale / decimatingScale - 1) < 0.001;
	this	-> d_filter	= NULL;
	this	-> sampleCounter	= 0;
	this	-> localOscillator	= new Oscillator (rateIn);
	DAB			= this;
	deviceCount 		= (this -> rtlsdr_get_device_count) ();
	if (deviceCount == 0) {
	   fprintf (stderr, "No devices found\n");
	   goto err;
	}

	fprintf (stderr, "%d devices found\n", deviceCount);
//
//	OK, now open the hardware
	r	= (this -> rtlsdr_open) (&device, 0);
	if (r < 0) {
	   fprintf (stderr, "Opening dabstick failed\n");
	   goto err;
	}

	open	= true;
	r	= (this -> rtlsdr_set_sample_rate) (device, rateIn);
	if (r < 0) {
	   fprintf (stderr, "Setting samplerate failed\n");
	   goto err;
	}

	fprintf (stderr, "samplerate is set to %d\n", 
	        rtlsdr_get_sample_rate (device));
	(this -> rtlsdr_set_tuner_gain_mode) (device, 1);
	(this -> rtlsdr_set_tuner_gain) (device, 190);
	_I_Buffer	= new RingBuffer<uint8_t>(1024 * 1024);
	workerHandle	= NULL;
	converter	= src_new (SRC_SINC_BEST_QUALITY, 2, &error);
	if (converter == NULL) {
	   fprintf (stderr, " Fatal error creating a converter\n");
	   return;
	}
	d_filter	= new DecimatingFIR (decimatingScale * 5 - 1,
	                                     - rateOut / 2,
	                                     rateOut / 2,
	                                     rateIn,
	                                     decimatingScale);
	src_data	= new SRC_DATA;
	*success = true;
	return;

err:
	if (open)
	   rtlsdr_close (device);
#ifdef __MINGW32__
	FreeLibrary (Handle);
#else
	dlclose (Handle);
#endif
	libraryLoaded	= false;
	open		= false;
	*success	= false;
	return;
}

	dabstick_dll::~dabstick_dll	(void) {

	if (open) 
	   rtlsdr_close (device);

	if (libraryLoaded)
#ifdef __MINGW32__
	   FreeLibrary (Handle);
#else
	   dlclose (Handle);
#endif
	if (_I_Buffer != NULL)
	   delete _I_Buffer;
	if (d_filter != NULL)
	   delete d_filter;
	d_filter	= NULL;
}
//
//	If the newly selected frequency fits within the current
//	window, we only change the localOffset and the bandpass filter
//	otherwise, we set_center_freq
void	dabstick_dll::setVFOFrequency	(int32_t f) {
	if ((f + rateOut / 2 < vfoFrequency + 0.9 * rateIn / 2) &&
	    (f - rateOut / 2 > vfoFrequency - 0.9 * rateIn / 2)) 
	   localOffset	= f - vfoFrequency;
	else {
	   vfoFrequency	= f;
	   localOffset		= 0;
	   if (workerHandle == NULL) 
	      (void)rtlsdr_set_center_freq (device, f + vfoOffset);
	   else {
	      stopReader ();	//
	      (void)rtlsdr_set_center_freq (device, f + vfoOffset);
	      workerHandle	= new dll_driver (this);
	   }
	}

//	fprintf (stderr, "localOffset = %d\n", localOffset);
}
//
//	Careful, from the perspective of the user, the frequency
//	on which the stick is tuned is
//	centerfreq - vfoOffset + localoffset!!!!
void	dabstick_dll::getVFOFrequency	(int32_t *f) {
	*f = (int32_t)rtlsdr_get_center_freq (device) - vfoOffset + localOffset;
}

int32_t	dabstick_dll::defaultFrequency	(void) {
	return KHz (94700);
}

const char	*dabstick_dll::nameofRig		(void) {
	return "dabstick_dll";
}
//
//
bool	dabstick_dll::restartReader	(void) {
int32_t	r;

	if (workerHandle != NULL)	// running already
	   return true;

	r = rtlsdr_reset_buffer (device);
	if (r < 0)
	   return false;

	rtlsdr_set_center_freq (device, vfoFrequency + vfoOffset + localOffset);
	workerHandle	= new dll_driver (this);
	return true;
}

void	dabstick_dll::stopReader	(void) {
	if (workerHandle == NULL)
	   return;

	rtlsdr_cancel_async (device);
	if (workerHandle != NULL) {
	   while (!workerHandle -> isFinished ()) 
	      usleep (100);

	   delete	workerHandle;
	}
	workerHandle	= NULL;
}

int32_t	dabstick_dll::setExternalRate	(int32_t newRate) {
int32_t	r;

	if (newRate == rateIn)
	   return rateIn;

	if (newRate < 900000) return rateIn;
	if (newRate > 3200000) return rateIn;

	realScale	        = (double) newRate / rateOut;
	decimatingScale         = (int)floor (realScale);
	this	-> integerDecimation	=
	                  abs (realScale / decimatingScale - 1) < 0.001;

	if (workerHandle == NULL) {
	   r	= rtlsdr_set_sample_rate (device, newRate);
	   if (r < 0) {
	      rtlsdr_set_sample_rate (device, rateIn);
	   }
	   delete d_filter;
	   d_filter	= new DecimatingFIR (decimatingScale * 4,
	                                     - rateOut / 2,
	                                     rateOut / 2,
	                                     rateIn,
	                                     decimatingScale);
	   r	= rtlsdr_get_sample_rate (device);
	   vfoFrequency	= vfoFrequency +  localOffset;
	   (void)rtlsdr_set_center_freq (device, vfoFrequency + vfoOffset);
	}
	else {	
//	we stop the transmission!!!
	   stopReader ();
	   r	= rtlsdr_set_sample_rate (device, newRate);
	   if (r < 0) {
	      rtlsdr_set_sample_rate (device, rateIn);
	   }

	   delete d_filter;
	   d_filter	= new DecimatingFIR (decimatingScale * 4,
	                                     - rateOut / 2,
	                                     rateOut / 2,
	                                     rateIn,
	                                     decimatingScale);
	   r	= rtlsdr_get_sample_rate (device);
	   vfoFrequency	= vfoFrequency + localOffset;
	   (void)rtlsdr_set_center_freq (device, vfoFrequency + vfoOffset);
//	ok all set, continue
	   restartReader ();
	}

	rateIn	= newRate;
	return r;
}

static
int gains[] = {-10, 15, 40, 65, 90, 115, 140, 165, 190,
	           215, 240, 290, 340, 420, 430, 450, 470, 490};
int32_t	dabstick_dll::setExternalGain	(int32_t gain) {
static int	oldGain	= 0;

	if (gain == oldGain)
	   return gains [oldGain];
	if ((gain < 0) || (gain > 17))
	   return gains [oldGain];	

	oldGain	= gain;
	rtlsdr_set_tuner_gain (device, gains [gain]);
	return rtlsdr_get_tuner_gain (device);
}

bool	dabstick_dll::load_rtlFunctions (void) {
//
//	link the required procedures
	rtlsdr_open	= (pfnrtlsdr_open)
	                       GETPROCADDRESS (Handle, "rtlsdr_open");
	if (rtlsdr_open == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_open\n");
	   return false;
	}
	rtlsdr_close	= (pfnrtlsdr_close)
	                     GETPROCADDRESS (Handle, "rtlsdr_close");
	if (rtlsdr_close == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_close\n");
	   return false;
	}

	rtlsdr_set_sample_rate =
	    (pfnrtlsdr_set_sample_rate)GETPROCADDRESS (Handle, "rtlsdr_set_sample_rate");
	if (rtlsdr_set_sample_rate == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_set_sample_rate\n");
	   return false;
	}

	rtlsdr_get_sample_rate	=
	    (pfnrtlsdr_get_sample_rate)GETPROCADDRESS (Handle, "rtlsdr_get_sample_rate");
	if (rtlsdr_get_sample_rate == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_get_sample_rate\n");
	   return false;
	}

	rtlsdr_set_tuner_gain_mode	= (pfnrtlsdr_set_tuner_gain_mode)
	                     GETPROCADDRESS (Handle, "rtlsdr_set_tuner_gain_mode");
	if (rtlsdr_set_tuner_gain_mode == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_set_tuner_gain_mode\n");
	   return false;
	}

	rtlsdr_set_tuner_gain	= (pfnrtlsdr_set_tuner_gain)
	                     GETPROCADDRESS (Handle, "rtlsdr_set_tuner_gain");
	if (rtlsdr_set_tuner_gain == NULL) {
	   fprintf (stderr, "Cound not find rtlsdr_set_tuner_gain\n");
	   return false;
	}

	rtlsdr_get_tuner_gain	= (pfnrtlsdr_get_tuner_gain)
	                     GETPROCADDRESS (Handle, "rtlsdr_get_tuner_gain");
	if (rtlsdr_get_tuner_gain == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_get_tuner_gain\n");
	   return false;
	}
	rtlsdr_set_center_freq	= (pfnrtlsdr_set_center_freq)
	                     GETPROCADDRESS (Handle, "rtlsdr_set_center_freq");
	if (rtlsdr_set_center_freq == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_set_center_freq\n");
	   return false;
	}

	rtlsdr_get_center_freq	= (pfnrtlsdr_get_center_freq)
	                     GETPROCADDRESS (Handle, "rtlsdr_get_center_freq");
	if (rtlsdr_get_center_freq == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_get_center_freq\n");
	   return false;
	}

	rtlsdr_reset_buffer	= (pfnrtlsdr_reset_buffer)
	                     GETPROCADDRESS (Handle, "rtlsdr_reset_buffer");
	if (rtlsdr_reset_buffer == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_reset_buffer\n");
	   return false;
	}

	rtlsdr_read_async	= (pfnrtlsdr_read_async)
	                     GETPROCADDRESS (Handle, "rtlsdr_read_async");
	if (rtlsdr_read_async == NULL) {
	   fprintf (stderr, "Cound not find rtlsdr_read_async\n");
	   return false;
	}

	rtlsdr_get_device_count	= (pfnrtlsdr_get_device_count)
	                     GETPROCADDRESS (Handle, "rtlsdr_get_device_count");
	if (rtlsdr_get_device_count == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_get_device_count\n");
	   return false;
	}

	rtlsdr_cancel_async	= (pfnrtlsdr_cancel_async)
	                     GETPROCADDRESS (Handle, "rtlsdr_cancel_async");
	if (rtlsdr_cancel_async == NULL) {
	   fprintf (stderr, "Could not find rtlsdr_cancel_async\n");
	   return false;
	}


	fprintf (stderr, "OK, functions seem to be loaded\n");
	return true;
}
//
//	Good old getSamples
//
int32_t	dabstick_dll::getSamples (DSPCOMPLEX *V,
	                                 int32_t n, uint8_t Mode) {
	if (integerDecimation) 
	   return getSamples_I (V, n, Mode);
	else
	   return getSamples_C (V, n, Mode);
}

int32_t	dabstick_dll::getSamples_I (DSPCOMPLEX *V,
	                            int32_t n, uint8_t Mode) {
int32_t	size	= n * decimatingScale;
uint8_t	*buf	= (uint8_t *)alloca (2 * size * sizeof (uint8_t));
int32_t	i, index;
//
//	first step: get data from the buffer
	(void) _I_Buffer	-> getDataFromBuffer (buf, 2 * size);

	for (i = 0, index = 0; i < size; i ++)  {
	   DSPCOMPLEX tmp = DSPCOMPLEX ((buf [2 * i] - 127.0) / 128.0,
	                                (buf [2 * i + 1] - 127.0) / 128.0);
	   tmp *= localOscillator -> nextValue (localOffset);
	   if (d_filter -> Pass (tmp, &tmp)) {
	      switch (Mode) {
	         default:
	         case IandQ:
	            V [index ++] = tmp;
	            break;

	         case QandI:
	            V [index ++] = DSPCOMPLEX (imag (tmp), real (tmp));
	            break;

	         case Q_Only:
	            V [index ++] = DSPCOMPLEX (real (tmp), 0.0);
	            break;

	         case I_Only:
	            V [index ++] = DSPCOMPLEX (imag (tmp), 0.0);
	            break;
	      }
	   }
	}

	return index;
}

int32_t	dabstick_dll::getSamples_C (DSPCOMPLEX *V,
	                            int32_t n, uint8_t Mode) {
int32_t	IBuf	= 2 * ceil (realScale * n);
uint8_t	*buf	= (uint8_t *)alloca (IBuf * sizeof (uint8_t));
float	*FBuf	= (float *)alloca (2 * decimatingScale * n * sizeof (float));
float	*OBuf	= (float *)alloca (2 * n * sizeof (float));
int32_t	i, index;
int	res;

	src_data -> data_in		= FBuf;
	src_data -> data_out		= OBuf;
	src_data -> end_of_input	= 0;
	src_data -> src_ratio		= realScale / decimatingScale;
	src_data -> output_frames	= n;
	src_data -> input_frames	= 2 * src_data -> src_ratio * n;
//
//	first step: get data from the buffer
	(void) _I_Buffer -> getDataFromBuffer (buf, decimatingScale *
	                                            src_data -> input_frames);

//	Second step: decimate and filter an integer amount
	for (i = 0, index = 0; i < IBuf / 2; i ++)  {
	   DSPCOMPLEX tmp = DSPCOMPLEX ((buf [2 * i] - 127.0) / 128.0,
	                                (buf [2 * i + 1] - 127.0) / 128.0);	
	   tmp *= localOscillator -> nextValue (localOffset);
	   if (d_filter -> Pass (tmp, &tmp)) {
	      FBuf [2 * index]		= real (tmp);
	      FBuf [2 * index + 1]	= imag (tmp);
	      index ++;
	   }
	}
//
//	third step: resample realScale / decimatingScale
//
	res		= src_process (converter, src_data);
	if (res != 0) {
	   fprintf (stderr, "error %s\n", src_strerror (res));
	   return 0;
	}

	for (i = 0; i < src_data -> output_frames_gen; i ++) {
	   switch (Mode) {
	      default:
	      case IandQ:
	         V [i] = DSPCOMPLEX (FBuf [2 * i],
	                             FBuf [2 * i + 1]);
	         break;

	      case QandI:
	         V [i] = DSPCOMPLEX (FBuf [2 * i + 1],
	                             FBuf [2 * i]);
	         break;

	      case Q_Only:
	         V [i] = DSPCOMPLEX (FBuf [2 * i + 1], 0.0);
	         break;

	      case I_Only:
	         V [i] = DSPCOMPLEX (FBuf [2 * i], 0.0);
	         break;
	   }
	}

	return src_data -> output_frames_gen; 
}

int32_t	dabstick_dll::Samples	(void) {
	return _I_Buffer -> GetRingBufferReadAvailable () / ((int)(2 * realScale));
}
//
//	local redefine
bool	dabstick_dll::RigprovidesSamples	(void) {
	return	true;
}
//	local redefine
bool	dabstick_dll::legalFrequency	(int32_t f) {
//	return MHz(22) <= f && f <= MHz (2200);
	return true;
}
//
//	and especially for our beloved spectrum viewer we provide
int32_t	dabstick_dll::getSamplesandSkip 	(DSPCOMPLEX *v,
	                                 int32_t n,
	                                 int32_t segmentSize) {
uint8_t *buf		= (uint8_t *)alloca (2 * n);
int32_t i;
int32_t skipSize	= 2 * (segmentSize - n);

	_I_Buffer	-> getDataFromBuffer (buf, 2 * n);
	_I_Buffer	-> skipDataInBuffer (skipSize);

	for (i = 0; i < n; i ++) 
	   v [i] = DSPCOMPLEX ((buf [2 * i] - 127.0) / 128.0,
	                       (buf [2 * i + 1] - 127.0) / 128.0);
	return n;
}

int32_t	dabstick_dll::getSampleCount	(void) {
	return sampleCounter;
}

