
#
/*
 *
 *    Copyright (C) 2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in ESDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _DABSTICK_DLL
#define	_DABSTICK_DLL

#include	<stdlib.h>
#include	<stdio.h>
#include	<math.h>
#include	<string.h>
#include	<unistd.h>
#include	<stdint.h>
#include	<QObject>
#include	"jff-include.h"
#include	"ringbuffer.h"
#include	"virtualrig.h"
#include	"fir-filters.h"
#include	"samplerate.h"
#include	"oscillator.h"
class	dll_driver;
//
//	create typedefs for the library functions
typedef	struct rtlsdr_dev rtlsdr_dev_t;

typedef	void (*rtlsdr_read_async_cb_t) (uint8_t *buf, uint32_t len, void *ctx);
typedef	 int (*  pfnrtlsdr_open )(rtlsdr_dev_t **, uint32_t);
typedef	int (*  pfnrtlsdr_close) (rtlsdr_dev_t *);
typedef	int (*  pfnrtlsdr_set_center_freq) (rtlsdr_dev_t *, uint32_t);
typedef uint32_t (*  pfnrtlsdr_get_center_freq) (rtlsdr_dev_t *);
typedef	int (*  pfnrtlsdr_set_tuner_gain_mode) (rtlsdr_dev_t *, int);
typedef	int (*  pfnrtlsdr_set_sample_rate) (rtlsdr_dev_t *, uint32_t);
typedef	int (*  pfnrtlsdr_get_sample_rate) (rtlsdr_dev_t *);
typedef	int (*  pfnrtlsdr_set_tuner_gain) (rtlsdr_dev_t *, int);
typedef	int (*  pfnrtlsdr_get_tuner_gain) (rtlsdr_dev_t *);
typedef int (*  pfnrtlsdr_reset_buffer) (rtlsdr_dev_t *);
typedef	int (*  pfnrtlsdr_read_async) (rtlsdr_dev_t *,
	                               rtlsdr_read_async_cb_t,
	                               void *,
	                               uint32_t,
	                               uint32_t);
typedef int (*  pfnrtlsdr_cancel_async) (rtlsdr_dev_t *);
typedef uint32_t (*  pfnrtlsdr_get_device_count) (void);

class	dabstick_dll: public virtualRig {
public:
			dabstick_dll	(int32_t, int32_t, bool *);
			~dabstick_dll	(void);
	void		setVFOFrequency	(int32_t);
	void		getVFOFrequency	(int32_t *);
	bool		legalFrequency	(int32_t);
	int32_t		defaultFrequency	(void);
	const	char	*nameofRig	(void);
	int32_t		setExternalRate	(int32_t);
	int32_t		setExternalGain	(int32_t);
//
//	interface to the reader
	bool		restartReader	(void);
	void		stopReader	(void);
	int32_t		getSamples	(DSPCOMPLEX *, int32_t, uint8_t);
	int32_t		getSamplesandSkip (DSPCOMPLEX *, int32_t, int32_t);
	int32_t		getSampleCount	(void);
	int32_t		Samples		(void);
	bool		RigprovidesSamples	(void);
//
//	we need the following items to be visible
	RingBuffer<uint8_t>	*_I_Buffer;
	int32_t		sampleCounter;
	int32_t		rateOut;
	pfnrtlsdr_read_async rtlsdr_read_async;
	struct rtlsdr_dev	*device;
private:
	int32_t		getSamples_I	(DSPCOMPLEX *, int32_t, uint8_t);
	int32_t		getSamples_C	(DSPCOMPLEX *, int32_t, uint8_t);
	HINSTANCE	Handle;
	int32_t		deviceCount;
	dll_driver	*workerHandle;
	bool		libraryLoaded;
	int32_t		rateIn;
	int32_t		vfoFrequency;
	int32_t		localOffset;
	Oscillator	*localOscillator;
	bool		open;
	bool		integerDecimation;
	double		realScale;
	int32_t		decimatingScale;
	DecimatingFIR	*d_filter;
	SRC_STATE	*converter;
	SRC_DATA	*src_data;
//
//	here we need to load functions from the dll
	bool		load_rtlFunctions	(void);
	pfnrtlsdr_open	rtlsdr_open;
	pfnrtlsdr_close	rtlsdr_close;
	pfnrtlsdr_set_center_freq rtlsdr_set_center_freq;
	pfnrtlsdr_get_center_freq rtlsdr_get_center_freq;
	pfnrtlsdr_set_tuner_gain_mode rtlsdr_set_tuner_gain_mode;
	pfnrtlsdr_set_sample_rate rtlsdr_set_sample_rate;
	pfnrtlsdr_get_sample_rate rtlsdr_get_sample_rate;
	pfnrtlsdr_set_tuner_gain rtlsdr_set_tuner_gain;
	pfnrtlsdr_get_tuner_gain rtlsdr_get_tuner_gain;
	pfnrtlsdr_reset_buffer rtlsdr_reset_buffer;
	pfnrtlsdr_cancel_async rtlsdr_cancel_async;
	pfnrtlsdr_get_device_count rtlsdr_get_device_count;
};
#endif

