#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * 	This version depends on ftdi, both under windows and linux
 */
#ifndef _ELEKTOR507
#define	_ELEKTOR507

#include	"virtualrig.h"
#include	"frequencytables.h"
#include	"pa_reader.h"
#include	<ftdi.h>

#define	ANT_AUTO		0200
#define	ANT_EXT			0201
#define	ANT_BAND4		0202
#define	ANT_BAND5		0203
#define	ANT_BAND6		0204
#define	ANT_BAND7		0205
#define	ANT_TEST		0210
#define	ANT_ERROR		0211
#define	NO_ANT			0222

#define	OSCFREQ			10000	/* in Hz		*/
#define	XTAL_CAL		128
#define	FT_OutBuffer_MAX	1024

#define	CY_I2C_RAM_ADR		210
#define	CY_I2C_EEPROM_ADR	208
#define	CLKOE_REG		0x09
#define	DIV1_REG		0x0c
#define	DIV2_REG		0x47
#define	XTALCTL_REG		0x12
#define	CAPLOAD_REG		0x13
#define	PUMPCOUNTERS_REG	0x40
#define	CLKSRC_REG		0x44

#define	PCF8951_ADDRESS		0x90
#define DAC_ENABLE		0x40

struct priv_data {
	unsigned	xtal_cal;
	unsigned	osc_freq;
	unsigned	ant;
	int		voltage;
	int		P;
	int		Q;
	int		Div1N;
	uint8_t		FT_port;
	int		Buf_address;
	uint8_t		FT_OutBuffer [FT_OutBuffer_MAX];
};

/*
 */
class	Elektor507: public virtualRig {
public:
		Elektor507	(const char *f, paReader *, bool *);
		~Elektor507	(void);
	bool	isWorking	(void);
	void	setVFOFrequency	(int32_t);
	void	setFrequency	(int32_t, int32_t);
	void	getVFOFrequency	(int32_t *);
	void	setLevel	(int16_t);
	bool	getLevel	(int16_t *);
	bool	setAntenna	(int16_t);
	bool	getAntenna	(int16_t *);
	bool	setXtalCal	(int16_t);
	bool	preSelect	(int32_t, int16_t *, int16_t *);
	bool	setVoltage	(int16_t);
	bool	legalFrequency	(int32_t);
	const char	*nameofRig	(void);
//	reader
	bool		restartReader	(void);
	void		stopReader	(void);
	int32_t		getSamples	(DSPCOMPLEX *, int32_t, uint8_t);
	int32_t		Samples		(void);
	bool		RigprovidesSamples	(void);
private:
	frequencytables		*ElektorFrequencies;
	paReader	*Reader;
	const	char	*preselection;
	struct ftdi_context	ftdic;
	bool	cy_update_pll	(uint8_t);
	bool	i2c_write_reg	(uint8_t, uint8_t, uint8_t);
	bool	i2c_write_regs	(uint8_t, int, uint8_t, uint8_t, uint8_t, uint8_t);
	void	ftdi_I2C_Write_Byte (uint8_t);
	void	ftdi_I2C_Init	(void);
	void	ftdi_I2C_Start	(void);
	void	ftdi_I2C_Stop	(void);
	void	ftdi_SDA	(int8_t);
	void	ftdi_SCL	(int8_t);
	void	find_P_Q	(struct priv_data *, int32_t);
	void	find_P_Q_DIV1N	(struct priv_data *, int32_t);
	int16_t	getPump		(int16_t);
	bool	WriteData	(void *, uint32_t);

	uint8_t	rig_is_working;
	priv_data	*priv;
};

#endif

