
#
#include	"dll-handler.h"
#include	"vreader.h"
#ifndef	__MINGW32__
#include	"dlfcn.h"
#endif
using namespace std;

#ifdef __MINGW32__
#define	GETPROCADDRESS	GetProcAddress
#define	FREELIBRARY	FreeLibrary
#else
#define	GETPROCADDRESS	dlsym
#define	FREELIBRARY	dlclose
#endif

//
//	Note that we need to go dirty: since the callback has no
//	options to specify the right "environment", we are forced
//	to create such an environment. We therefore create a
//	global filled with the address of the instance of the dll_handler
//	class

static	dll_handler *callbackContext;
//
//	Note that calling complex Qt functions from dll-threads does
//	not work. So, the callback will merely queue a request in the main
//	gui functions and the main gui will poll the status.
//
static	int extioCallback (int cnt, int status, float IQoffs, void *IQData) {
	if (cnt < 0)  
	   callbackContext -> dllRequest (status);
	else  {
	   callbackContext -> TheReader ->
	          addtoBuffer ((int16_t *)IQData,
	                   callbackContext -> bufsize);
	}
	return 1;
}
//
//	the seemingly additional complexity is caused by the fact
//	that naming of files in windows is in MultiBytechars
	dll_handler::dll_handler (const char *f, bool *s, int32_t rate) {
	TheReader	= NULL;
#ifdef	__MINGW32__
char	temp [256];
int wchars_num = MultiByteToWideChar (CP_UTF8, 0, f, -1, NULL, 0);
wchar_t	*windowsName = new wchar_t [wchars_num];
	MultiByteToWideChar (CP_UTF8, 0, f, -1, windowsName, wchars_num);
	wcstombs (temp, windowsName, 128);
	Handle		= LoadLibrary (windowsName);
#else
	Handle		= dlopen (f, RTLD_NOW);
#endif
	if (Handle == NULL) {
	   fprintf (stderr, "Failed to load  %s\n", f);
	   dll_open	= false;
	   *s	= false;
	   return;
	}

	this		-> rateOut = rate;
	TheReader	= NULL;
//	apparently, the library is open, so record that
	dll_open	= true;

//	start binding addresses
	InitHW		= (pfnInitHW)GETPROCADDRESS (Handle, "InitHW");
	if (InitHW == NULL) {
	   fprintf (stderr, "Failed to load InitHW\n");
	   goto err;
	}

	OpenHW		= (pfnOpenHW)GETPROCADDRESS (Handle, "OpenHW");
	if (OpenHW == NULL) {
	   fprintf (stderr, "Failed to load OpenHW\n");
	   goto err;
	}

	StartHW		= (pfnStartHW)GETPROCADDRESS (Handle, "StartHW");
	if (StartHW == NULL) {
	   fprintf (stderr, "Failed to load StartHW\n");
	   goto err;
	}

	StopHW		= (pfnStopHW)GETPROCADDRESS (Handle, "StopHW");
	if (StopHW == NULL) {
	   fprintf (stderr, "Failed to load StopHW\n");
	   goto err;
	}

	CloseHW		= (pfnCloseHW)GETPROCADDRESS (Handle, "CloseHW");
	if (CloseHW == NULL) {
	   fprintf (stderr, "Failed to load CloseHW\n");
	   goto err;
	}

	SetHWLO		= (pfnSetHWLO)GETPROCADDRESS (Handle, "SetHWLO");
	if (SetHWLO == NULL) {
	   fprintf (stderr, "Failed to load SetHWLO\n");
	   goto err;
	}

	GetStatus	= (pfnGetStatus)GETPROCADDRESS (Handle, "GetStatus");
	if (GetStatus == NULL) {
	   fprintf (stderr, "Failed to load GetStatus\n");
	   goto err;
	}

	SetCallback	= (pfnSetCallback)
	                             GETPROCADDRESS (Handle, "SetCallback");
	if (SetCallback == NULL) {
	   fprintf (stderr, "Failed to load SetCallback\n");
	   goto err;
	}
//
//	These functions might be available. In the interface they
//	are enveloped in a contained function
	L_GetHWLO	= (pfnGetHWLO)GETPROCADDRESS (Handle, "GetHWLO");
	L_GetTune	= (pfnGetTune)GETPROCADDRESS (Handle, "GetTune");
	L_GetFilters	= (pfnGetFilters)GETPROCADDRESS (Handle, "GetFilters");
	L_GetMode	= (pfnGetMode)GETPROCADDRESS (Handle, "GetMode");
	L_ModeChanged	= (pfnModeChanged)GETPROCADDRESS (Handle, "ModeChanged");
	L_ShowGUI	= (pfnShowGUI)GETPROCADDRESS (Handle, "ShowGUI");
	L_HideGUI	= (pfnHideGUI)GETPROCADDRESS (Handle, "HideGUI");
	L_IfLimitsChanged
	                = (pfnIfLimitsChanged)GETPROCADDRESS (Handle,
	                                                   "IfLimitsChanged");
	L_TuneChanged	= (pfnTuneChanged)GETPROCADDRESS (Handle, "TuneChanged");
	L_FiltersChanged
	                = (pfnFiltersChanged)GETPROCADDRESS (Handle,
	                                                   "FiltersChanged");
	L_RawDataReady	= (pfnRawDataReady)GETPROCADDRESS (Handle,
	                                                   "RawDataReady");
	L_GetHWSR	= (pfnGetHWSR)GETPROCADDRESS (Handle, "GetHWSR");

//	and start the rig
	rigprovidesSamples	= false;
	rigName		= new char [128];
	rigModel	= new char [128];
	if (!(*InitHW) (rigName, rigModel, hardwareType)) {
	   fprintf (stderr, "Sorry cannot init hardware\n");
	   goto err;
	}

	fprintf (stderr, "rigName = %s, rigModel = %s, hardwareType = %d\n",
	          rigName, rigModel, hardwareType);

	switch (hardwareType) {
	   case exthwNone:
	   case exthwSDR14:
	   case exthwSDRX:
	   case exthwUSBdata24:
	   case exthwUSBdata32:
	   case exthwUSBfloat32:
	   case exthwHPSDR:
	   default:
	      fprintf (stderr, "Sorry, mode %d not (yet) supported\n",
	                                                       hardwareType);
	      goto err;
	      break;

	   case exthwSCdata:
	      TheReader = new VReader ();
	      break;

	   case exthwUSBdata16:
	      fprintf (stderr, "Dongle support is experimental\n");
	      fprintf (stderr, "Currently only rate of 2048000 is supported\n");
	      fprintf (stderr, "You might better use dabstick (dll)\n");
	      TheReader = new dongleReader (rate);
	      break;
	}

	callbackContext	= this;		// global static variable, carefull

	SetCallback (extioCallback);

	if ((*OpenHW)())
	   fprintf (stderr, "OpenHW successfull\n");
	else {
	   fprintf (stderr, "OpenHW not successful");
	   goto err;
	}

	if (hardwareType == exthwUSBdata16) {	// suspect dongle
	   int32_t r = GetHWSR ();
	   fprintf (stderr, "Rate of dongle set to %d\n", r);
	   TheReader	-> setInputRate (r);
	   minFreq	= 20000000;
	   maxFreq	= 1500000000;
	   *s = true;
	   return;
	}
	else
	if (hardwareType == exthwSCdata) {
	   if (isPMSDR ()) {
	      minFreq = 100000;
	      maxFreq = 55000000;
	   }
	   else {
	      minFreq = 100000;
	      maxFreq = 29999999;
	   }

	   fprintf (stderr, "Hardware %s (%s) %d %d\n", 
	                       rigName, rigModel, minFreq, maxFreq);
	   *s = true;
	   return;
	}

err:
	FREELIBRARY (Handle);
	dll_open = false;
	*s = false;
	return;
}

	dll_handler::~dll_handler (void) {
	if (dll_open) {
	   StopHW ();
	   CloseHW ();
	   FREELIBRARY (Handle);
	}
	fprintf (stderr, "Library is freed\n");
	if (TheReader != NULL)
	   delete TheReader;
}

bool	dll_handler::check (const char *n, const char *ref) {
std::string help (n);
size_t found = help. find (ref);
	return found != string::npos;
}

bool	dll_handler::isPMSDR	(void) {
	return check (rigName, "pmsdr") || check (rigName, "PMSDR");
}

bool	dll_handler::isDONGLE	(void) {
	return check (rigName, "RTLSDR") || 
	       hardwareType ==  exthwUSBdata16;
}

//
//	we overlad some of the virtual functions
bool	dll_handler::isWorking (void) {
	return dll_open;
}

void	dll_handler::setVFOFrequency (int32_t f) {
	lastFrequency = f;
	if (isDONGLE ())
	   (void) SetHWLO ((int)f + vfoOffset);
	else
	   (void) SetHWLO ((int)f);
}

void	dll_handler::getVFOFrequency (int32_t *f) {
	*f = lastFrequency;
}

void	dll_handler::setExternalRate	(int32_t f) {
	TheReader	-> setInputRate (f);
}

const char	*dll_handler::nameofRig (void) {
	return rigName;
}

bool	dll_handler::legalFrequency	(int32_t f) {
	return minFreq <= f && f <= maxFreq;
}

int32_t	dll_handler::defaultFrequency	(void) {
	if (hardwareType == exthwUSBdata16)
	   return 947000000;
	else
	   return 14070000;
}
//
//	The envelopes for functions that might (or might not)
//	be available

long	dll_handler::GetHWLO		(void) {
	if (L_GetHWLO != NULL)
	   return L_GetHWLO ();
	return 0;
}

long	dll_handler::GetTune		(void) {
	if (L_GetTune != NULL)
	   return L_GetTune ();
	return 0;
}

void	dll_handler::GetFilters	(int& lo, int& hi, int& pitch) {
	if (L_GetFilters != NULL)
	   L_GetFilters (lo, hi, pitch);
}

uint8_t	dll_handler::GetMode		(void) {
	if (L_GetMode != NULL)
	   return L_GetMode ();
	return 0;
}

void	dll_handler::ModeChanged	(uint8_t c) {
	if (L_ModeChanged != NULL)
	   L_ModeChanged (c);
}

void	dll_handler::ShowGUI		(void) {
	if (L_ShowGUI != NULL)
	   L_ShowGUI ();
}

void	dll_handler::HideGUI		(void) {
	if (L_HideGUI != NULL)
	   L_HideGUI ();
}

void	dll_handler::IfLimitsChanged	(long lo, long hi) {
	if (L_IfLimitsChanged != NULL)
	   L_IfLimitsChanged (lo, hi);
}

void	dll_handler::TuneChanged	(long freq) {
	if (L_TuneChanged != NULL)
	   L_TuneChanged (freq);
}

void	dll_handler::FiltersChanged	(int lo, int hi, int pitch, bool mute) {
	if (L_FiltersChanged != NULL)
	   L_FiltersChanged (lo, hi, pitch, mute);
}

void	dll_handler::RawDataReady	(long sr, int *ld, int *rd, int num) {
	if (L_RawDataReady != NULL)
	   L_RawDataReady (sr, ld, rd, num);
}

long	dll_handler::GetHWSR		(void) {
	if (L_GetHWSR != NULL)
	   return L_GetHWSR ();
	return 0;
}

bool	dll_handler::RigprovidesSamples	(void) {
	return TheReader -> RigprovidesSamples ();
}

int32_t	dll_handler::Samples		(void) {
	return TheReader -> Samples ();
}

int32_t	dll_handler::getSamples		(DSPCOMPLEX *buffer,
	                                 int32_t number, uint8_t mode) {
	return TheReader -> getSamples (buffer, number, mode);
}

void	dll_handler::stopReader		(void) {
	StopHW ();
}

bool	dll_handler::restartReader	(void) {
	bufsize = StartHW (100000000);
	return bufsize > 512;
}

