#
#include	"vreader.h"
#include	"stdio.h"

//
//	To handle polymorphic reader functions, we
//	again create a reader to abstract away from
//	the specifics of how to read samples

	VReader::VReader (void) {
}

	VReader::~VReader (void) {
}

int32_t	VReader::getSamples (DSPCOMPLEX *b, int32_t c, uint8_t m) {
	(void)b;
	(void)c;
	(void)m;
	return 0;
}

int32_t	VReader::Samples (void) {
	return 0;
}

void	VReader::restart (int32_t buf){
	(void)buf;
}

void	VReader::stop (void) {
}

int32_t	VReader::addtoBuffer	(int16_t *buffer, int32_t amount) {
	(void)buffer;
	(void) amount;
	return 0;
}

void	VReader::setInputRate	(int32_t s) {
	(void)s;
}

bool	VReader::RigprovidesSamples	(void) {
	return false;
}

//
//##################################################################
// The real one is now for the 16 bits datastream
//
	dongleReader::dongleReader (int32_t rate) {
int	error;

	outputRate	= rate;
	inputRate	= 2048000;	// will change
	realScale	=  (double)inputRate / outputRate;
	decimatingScale	= (int)floor (realScale);

	_I_Buffer	= new RingBuffer<int16_t>(1024 * 1024);
	converter	= src_new (SRC_SINC_BEST_QUALITY, 2, &error);
	if (converter == NULL) {
	   fprintf (stderr, "Fatal: error creating a converter\n");
	}

	lowpass		= new DecimatingFIR (decimatingScale * 6,
	                                     outputRate / 2,
	                                     inputRate,
	                                     decimatingScale);
	src_data	= new SRC_DATA;
	fprintf (stderr, "dongleReader with outputRate = %d, inputRate = %d\n",
	                                          outputRate, inputRate);
}

	dongleReader::~dongleReader (void) {
	delete	lowpass;
	delete	_I_Buffer;
}

int32_t	dongleReader::addtoBuffer (int16_t *b, int32_t a) {
	return _I_Buffer -> putDataIntoBuffer (b, 2 * a);
}
//
//	for the dongle we keep things a little oversimplistic
//	by assuming that the inputrate = N * rate (where N is integer)
int32_t	dongleReader::getSamples (DSPCOMPLEX *V,
	                          int32_t n, uint8_t Mode) {
int32_t	IBuf	= 2 * decimatingScale * n;
int16_t	*buf	= (int16_t *)alloca (IBuf * sizeof (int16_t));
int32_t	i, index;
//
//	first step: get data from the buffer
	(void) _I_Buffer	-> getDataFromBuffer (buf, IBuf);
//
//	Second step: decimate and filter an integer amount
	for (i = 0, index = 0; i < IBuf / 2; i ++)  {
	   DSPCOMPLEX tmp = DSPCOMPLEX (buf [2 * i], buf [2 * i + 1]);
	   if (lowpass -> Pass (tmp, &tmp)) {
//	      FBuf [2 * index]		= real (tmp);
//	      FBuf [2 * index + 1]	= imag (tmp);
	      V [index]		= tmp;
	      index ++;
	   }
	}
	return n;
}

int32_t	dongleReader::Samples (void) {
	return _I_Buffer -> GetRingBufferReadAvailable () / (int)(2 * realScale);
}

bool	dongleReader::RigprovidesSamples (void) {
	return true;
}

void	dongleReader::setInputRate (int32_t newRate) {
DecimatingFIR	*newF;

	if (newRate == inputRate)
	   return;

	realScale		= (double)newRate / outputRate;
	decimatingScale		= (int)floor (realScale);

	newF			= new DecimatingFIR (decimatingScale * 5,
	                                             outputRate / 2,
	                                             newRate,
	                                             decimatingScale);
	delete	lowpass;
	lowpass		= newF;
	inputRate	= newRate;
}


