#
/*
 *
 *	This is a rewrite of the code for
 *	Library for
 *	MSDR command line manager (experimental)
 *	control a PMSDR hardware on Linux
 *	Modified to comply to firmware version 2.1.x
 *
 *	Copyright (C) 2008,2009,2010  Andrea Montefusco IW0HDV,
 *	Martin Pernter IW3AUT
 *	his program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef	__PMSDR
#define	__PMSDR

#include	"jff-include.h"
#include	<stdio.h>
#include	<string.h>
#include	<stdbool.h>
#include	<assert.h>
#include	<stdlib.h>
#include	"pmsdr_comm.h"
#include	"virtualrig.h"
//
//	This is layer 2 of the pmsdr control software. This layer
//	concentrates on the functionality
//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------

class	RadioInterface;

//these must be floating point number especially 2^28 so that 
//there is enough memory to use them in the calculation 
#define POW_2_16         65536.0 
#define POW_2_24         16777216.0 
#define POW_2_28         268435456.0 
#define FOUT_START_UP	 56.320       //MHz 
#define FXTAL_DEVICE	 114.27902015
#define	PPM		 3500

#ifndef I2C_SLA_SI570
#define I2C_SLA_SI570     0x55
#endif
 
#ifndef STARTUP_FREF
#define STARTUP_FREF   56320000UL
#endif

static const float FDCO_MAX = 5670; //MHz 
static const float FDCO_MIN = 4850; //MHz 
static const unsigned char HS_DIV[6] = {11, 9, 7, 6, 5, 4};
static const float fout0 = FOUT_START_UP;
static const float SI570_FOUT_MIN = 10; //MHz

const static QsdBiasFilterIfGain qsdBfigDefault = {
    512,  // bias at half scale
    0,    // mute off
    0,    // no filter
    1     // 20 dB gain
};

typedef
struct si570data_ {
        unsigned char	n1;
        unsigned char	hsdiv;
        uint32_t	frac_bits;
	double		currentFreq;
        double		rfreq;
        double		fxtal;
} si570data;

class	pmSDR: public virtualRig {
public:
		pmSDR	(RadioInterface *, bool *);
		~pmSDR	(void);
	bool	isWorking		(void);
	void	frequencyBounds		(int32_t *, int32_t *);
	bool	legalFrequency		(int32_t);
	void	displayKhz		(int32_t);
	void	displayMhz		(int32_t);
	void	getVFOFrequency		(int32_t *);
	void	setVFOFrequency		(int32_t);
	int32_t	defaultFrequency	(void);
const	char	*nameofRig		(void);
	void	setBias			(int16_t);
	int16_t	getBias			(void);
	void	setGain			(int16_t);
	int16_t	getGain			(void);
	void	setMute			(bool);
	bool	getMute			(void);
	void	setFilter		(int16_t);
	int16_t	getFilter		(void);
	void	setFilter		(int32_t, int16_t *);
const	char	*getFilterRange		(int16_t);

	bool	restartReader	(void);
	void	stopReader	(void);
	int32_t	getSamples	(DSPCOMPLEX *, int32_t, uint8_t);
	int32_t	Samples		(void);
	bool	RigprovidesSamples	(void);
	bool	isPMSDR		(void);
private:
	QsdBiasFilterIfGain qsdBfig;
	RadioInterface	*myRadioInterface;
	bool		radioOK;
	pmsdr_comm	*pmsdrDevice;
	int16_t		usbFirmwareSub;
	int16_t		usbFirmwareMajor;
	int16_t		usbFirmwareMinor;
	int32_t		LOfreq;
	uint8_t		fLcd;
	
	int16_t		currentFilter;
	uint8_t		postdivider;
	uint8_t		si570_reg[8];
	si570data	si570pmsdr;

	bool		ReadandConvertRegisters	(int8_t);
	void		calc_si570registers	(uint8_t *, si570data *);
	uint8_t		SetBits			(uint8_t, uint8_t, uint8_t);
	bool		ThirdHarmonic;
	void		ComputeandSetSi570	(double);
	void		convertFloattoHex	(si570data *, uint16_t *);
	bool		computeDividers		(double, uint8_t *, uint8_t *);
	bool		setFrequencySmallChange	(si570data *,
	                                         double, uint16_t *);
	bool		setFrequencyLargeChange	(si570data *,
	                                         double, uint16_t *);
	bool		store_hwlo		(int32_t);
	bool		read_hwlo		(int32_t *);
};

#endif

