#
/*
 *
 *    Copyright (C) 2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include	"replay.h"
#include	"jff-include.h"
#include	"utilities.h"

#define	INPUT_FRAMEBUFFERSIZE	64000

	replayRig::replayRig	(int32_t rate, char *f, bool *success) {
SF_INFO	*sf_info;

	lastFrequency = 14000000;
	fileName	= f;
	readerOK	= false;
	this	-> rate	= rate;
	_I_Buffer	= new RingBuffer<float>(INPUT_FRAMEBUFFERSIZE);

	sf_info		= (SF_INFO *)alloca (sizeof (SF_INFO));
	sf_info		-> format	= 0;
	readerOK	= false;
	filePointer	= sf_open (fileName, SFM_READ, sf_info);
	if (filePointer == NULL) {
	   fprintf (stderr, "file %s no legitimate sound file\n", f);
	   *success	= false;
	   return;
	}

	samplesinFile	= sf_info	-> frames;
	sampleRate	= sf_info	-> samplerate;
	numofChannels	= sf_info	-> channels;
	readerOK	= true;
	readerPausing	= true;
	
	*success	= true;
	currPos		= 0;
	fprintf (stderr, "file %s geopend\n", f);
	start ();
}

	replayRig::~replayRig	(void) {
	ExitCondition = TRUE;
	if (readerOK) {		//
	   while (!ThreadFinished)
	      usleep (100);
	   sf_close (filePointer);
	fprintf (stderr, "File closed\n");
	}
	delete	_I_Buffer;
}

bool	replayRig::isWorking	(void) {
	return readerOK;
}

void	replayRig::setVFOFrequency	(int32_t f) {
	lastFrequency = f;
}

void	replayRig::getVFOFrequency	(int32_t *f) {
	*f = lastFrequency; 
}

bool	replayRig::legalFrequency	(int32_t f) {
	(void)f;
	return true;
}

const char	*replayRig::nameofRig	(void) {
	return "replay";
}

bool	replayRig::restartReader	(void) {
	if (readerOK)
	   readerPausing = false;
	return readerOK;
}

void	replayRig::stopReader	(void) {
	if (readerOK)
	   readerPausing = true;
}

int32_t	replayRig::getSamples	(DSPCOMPLEX *V, int32_t n, uint8_t Mode) {
float	*buf = (float *)alloca (2 * n * sizeof (float));
int32_t	i;

	while (_I_Buffer -> GetRingBufferReadAvailable () < 2 * n) {
	   msleep (1);
	}

	 _I_Buffer	-> getDataFromBuffer (buf, 2 * n);
	for (i = 0; i < n; i ++) {
	   switch (Mode) {
	      default:
	      case IandQ:
	         V [i] = DSPCOMPLEX (buf [2 * i], buf [2 * i + 1]);
	         break;

	      case QandI:
	         V [i] = DSPCOMPLEX (buf [2 * i + 1], buf [2 * i]);
	         break;

	      case I_Only:
	         V [i]= DSPCOMPLEX (buf [2 * i], 0.0);
	         break;

	      case Q_Only:
	         V [i]	= DSPCOMPLEX (buf [2 * i + 1], 0.0);
	         break;
	   }
	}

	return n;
}

int32_t	replayRig::Samples		(void) {
	return _I_Buffer -> GetRingBufferReadAvailable () / 2;
}

bool	replayRig::RigprovidesSamples	(void) {
	return true;
}
/*
 *	length is number of floats that we read.
 *	In case of mono, we add zeros to the right channel
 *	and therefore only read length / 2 floats
 *	Notice that sf_readf_xxx reads frames (samples * numofchannels)
 */
int32_t	replayRig::readBuffer (float *data, int32_t length) {
int32_t	n, i;
float	*tempBuffer;

	if (numofChannels == 2) {
	   n = sf_readf_float (filePointer, data, length / 2);
	}
	else {
//	apparently numofChannels == 1
	   tempBuffer	= (float *)alloca (length / 2 * sizeof (float));
	   n		= sf_readf_float (filePointer, tempBuffer, length / 2);
	   for (i = 0; i < n; i ++) {
	      data [2 * i] = tempBuffer [i];
	      data [2 * i + 1] = 0;
	   }
	}

	currPos		+= n;
	if (n < length / 2)
	   sf_seek (filePointer, 0, SEEK_SET);
	return	2 * n;
}

/*
 *	for processing the file-io we start a separate thread
 *	In this thread, we also apply samplerate conversions
 *	- if any
 *	Note we want buffers of app the size of BUFSIZE to
 *	get out from here (which is not the same as setting
 *	the buffersize for reading files to BUFSIZE
 */
void	replayRig::run () {
int32_t	t, i;
float	*bi;
int32_t	bufferSize;
int32_t	period;
int64_t	nextStop;

	if (!readerOK)
	   return;

	ExitCondition = FALSE;
	ThreadFinished	= FALSE;

	if (rate == sampleRate) {	
	   bufferSize	= 2 * rate / 10;
	   period	= 100000;	// microseconds
	   bi		= new float [bufferSize];
	   nextStop	= getMyTime ();
	   while (!ExitCondition) {
	      if (readerPausing) {
	         usleep (1000);
	         nextStop = getMyTime ();
	         continue;
	      }

	      while (_I_Buffer -> WriteSpace () < bufferSize + 10) {
	         if (ExitCondition)
	            break;
	         usleep (100);
	      }

	      nextStop += period;
	      t = readBuffer (bi, bufferSize);
	      if (t <= 0) {
	         for (i = 0; i < bufferSize; i ++)
	             bi [i] = 0;
	         t = bufferSize;
	      }
	       _I_Buffer -> putDataIntoBuffer (bi, t);
	      raiseSignal ();
	      if (nextStop - getMyTime () > 0)
	         usleep (nextStop - getMyTime ());
	   }
	}
	else {
	   int32_t	outputLimit	= 2 * rate / 10;
	   double	ratio	= (double)sampleRate / rate;
	   bufferSize		= ceil (outputLimit * ratio);
	   bi			= new float [bufferSize];
	   int		error;
	   float	*bo	=
	                    (float *)alloca (outputLimit  * sizeof (float));
	   SRC_STATE	*converter	= src_new (SRC_SINC_BEST_QUALITY, 2, &error);
	   SRC_DATA	*src_data	= new SRC_DATA;
	   period		= 90000;

	   if (converter == 0) {
	      fprintf (stderr, "error creating a converter %d\n", error);
	      return;
	   }

	   fprintf (stderr,
	               "Starting converter with ratio %f (in %d, out %d)\n",
	                                       ratio, sampleRate, rate);
	   fprintf (stderr, 
	                "bufferSize = %d, outputLimit = %d\n", 
	                          bufferSize, outputLimit);
	   src_data -> data_in		= bi;
	   src_data -> data_out		= bo;
	   src_data -> src_ratio	= ratio;
	   src_data -> end_of_input	= 0;
	   nextStop			= getMyTime ();
	   while (!ExitCondition) {
	      int res;
	      if (readerPausing) {
	         msleep (1);
	         nextStop	= getMyTime ();
	         continue;
	      }
	      while (_I_Buffer -> WriteSpace () < outputLimit + 10) {
	         if (ExitCondition)
	            break;
	         usleep (100);
	      }

	      nextStop	+= period;
	      t = readBuffer (bi, bufferSize);
	      if (t <= 0) {
	         for (i = 0; i < bufferSize; i ++)
	            bi [i] = 0;
	         t = bufferSize;
	      }
	      src_data -> input_frames	= t / 2;
	      src_data -> output_frames	= outputLimit / 2;
// we are reading continously, after eof we just start all over again,
// therefore end_of_input flag is not altered
	      res	= src_process	(converter, src_data);
	      if (res != 0) {
	         fprintf (stderr, "error %s\n", src_strerror (t));
	      }
	      else {
	         int numofOutputs = src_data -> output_frames_gen * 2;
	         _I_Buffer -> putDataIntoBuffer (src_data -> data_out,
                                                 numofOutputs);
	      }
	      raiseSignal ();
	      if (nextStop - getMyTime () > 0)
	         usleep (nextStop - getMyTime ());
	   }

	   delete	bi;
	   src_delete (converter);
	   delete src_data;
	}
	fprintf (stderr, "taak voor replay eindigt hier\n");
	ThreadFinished	= TRUE;
}


