#
/*
 *
 *    Copyright (C) 2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in ESDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ESDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	"ringbuffer.h"
#include	"pa_reader.h"
/*
 *	The class describes the reading of
 *	data from the soundcard, storing it into the
 *	ringbuffer and making samples available for the user
 */
	paReader::paReader 	(int32_t cardRate) {
int16_t	i;

	CardRate	= cardRate;
	_I_Buffer	= new RingBuffer<float>(INPUT_FRAMEBUFFERSIZE);
	portAudio	= false;
	readerRunning	= false;
	if (Pa_Initialize () != paNoError) {
	   fprintf (stderr, "PaInit failed for reader\n");
	   return;
	}
	portAudio	= true;

	qDebug ("Hostapis: %d\n", Pa_GetHostApiCount ());
	for (i = 0; i < Pa_GetHostApiCount (); i ++)
	   qDebug ("Api %d is %s\n", i, Pa_GetHostApiInfo (i) -> name);

	numofDevices	= Pa_GetDeviceCount ();
	istream		= NULL;
}

	paReader::~paReader	(void) {
	if ((istream != NULL) &&!Pa_IsStreamStopped (istream)) {
	   paCallbackReturn = paAbort;
	   (void) Pa_AbortStream (istream);
	   while (!Pa_IsStreamStopped (istream))
	      Pa_Sleep (1);
	   readerRunning = false;
	}

	if (istream != NULL)
	   Pa_CloseStream (istream);
	   

	if (portAudio)
	   Pa_Terminate ();

	delete	_I_Buffer;
}

//
//	for setting up the combobox we need
int16_t	paReader::numberofDevices	(void) {
	return numofDevices;
}

const char	*paReader::inputChannelwithRate (int16_t ch, int32_t rate) {
const	PaDeviceInfo *deviceInfo;
const	char	*name	= NULL;

	if ((ch < 0) || (ch >= numofDevices))
	   return name;

	deviceInfo = Pa_GetDeviceInfo (ch);
	if (deviceInfo == NULL)
	   return name;

	if (deviceInfo -> maxInputChannels <= 0)
	   return name;

	if (InputrateIsSupported (ch, rate))
	   name = deviceInfo -> name;
	return name;
}

int16_t	paReader::invalidDevice	(void) {
	return numofDevices + 128;
}

bool	paReader::isValidDevice	(int16_t dev) {
	if (dev < 0)
	   return FALSE;
	return (dev < numofDevices);
}

bool	paReader::selectDevice	(int16_t idev) {
PaError	err;

//	fprintf (stderr, "selecting device %d\n", idev);
	if (!isValidDevice (idev))
	   return false;

//	fprintf (stderr, "device %d is valid\n", idev);
	if ((istream != NULL) &&!Pa_IsStreamStopped (istream)) {
	   paCallbackReturn = paAbort;
	   (void) Pa_AbortStream (istream);
	   while (!Pa_IsStreamStopped (istream))
	      Pa_Sleep (1);
	   readerRunning = false;
	}

	if (istream != NULL)
	   Pa_CloseStream (istream);

        inputParameters. device = idev;
	inputParameters. channelCount		= 2;	/* I and Q	*/
	inputParameters. sampleFormat		= paFloat32;
	inputParameters. suggestedLatency	= 
	                 Pa_GetDeviceInfo (idev) -> 
	                       defaultLowInputLatency;
	inputParameters. hostApiSpecificStreamInfo = NULL;

	bufSize	= (int)((float)inputParameters. suggestedLatency * 
	                                        (float)CardRate);
	fprintf (stderr, "suggested bufsize for reader = %d\n", bufSize);
	err = Pa_OpenStream (
	             &istream,
	             &inputParameters,
	             NULL,
	             CardRate,
	             bufSize,
	             0,
	             this	-> paCallback_i,
	             this
	      );
//	fprintf (stderr, " inputstream gives %d\n", err);
	if (err != paNoError) {
	   qDebug ("Open istream error\n");
	   return false;
	}
	return true;
}

bool	paReader::restartReader (void) {
PaError	err;

	if (!Pa_IsStreamStopped (istream))
	   return true;

	paCallbackReturn = paContinue;
	err = Pa_StartStream (istream);
	if (err == paNoError)
	   readerRunning	= true;

	return readerRunning;
}

void	paReader::stopReader	(void) {
	if (Pa_IsStreamStopped (istream))
	   return;

	paCallbackReturn = paAbort;
	(void) Pa_StopStream (istream);
	while (!Pa_IsStreamStopped (istream))
	   Pa_Sleep (1);
	readerRunning		= false;
}
/*
 * 	Notice that, in order to be consistent, outside the reader
 * 	area, we mean with samples full IQ pairs, while in reader/writer
 * 	we mean single values
 */
int32_t	paReader::Samples () {
	return _I_Buffer -> GetRingBufferReadAvailable () / 2;
}

int32_t	paReader::getSamples	(DSPCOMPLEX *V,
	                         int32_t n, int8_t Mode) {
float	*buf = (float *)alloca (2 * n * sizeof (float));
int32_t	i;

	if (!portAudio)
	   return 0;

	while (_I_Buffer -> GetRingBufferReadAvailable () < 2 * n) {
	   if (!readerRunning)
	      return 0;
	   Pa_Sleep (1);
	}

	 _I_Buffer	-> getDataFromBuffer (buf, 2 * n);

	for (i = 0; i < n; i ++) {
	   switch (Mode) {
	      default:
	      case IandQ:
	         V [i] = DSPCOMPLEX (buf [2 * i], buf [2 * i + 1]);
	         break;

	      case QandI:
	         V [i] = DSPCOMPLEX (buf [2 * i + 1], buf [2 * i]);
	         break;

	      case I_Only:
	         V [i]= DSPCOMPLEX (buf [2 * i], 0.0);
	         break;

	      case Q_Only:
	         V [i]	= DSPCOMPLEX (buf [2 * i + 1], 0.0);
	         break;
	   }
	}

	return n;
}
//
//	helper
bool	paReader::InputrateIsSupported (int16_t device, int32_t Rate) {
PaStreamParameters *inputParameters =
	           (PaStreamParameters *)alloca (sizeof (PaStreamParameters)); 

	inputParameters	-> device		= device;
	inputParameters -> channelCount		= 2;	/* I and Q	*/
	inputParameters -> sampleFormat		= paFloat32;
	inputParameters -> suggestedLatency	= 0;
	inputParameters -> hostApiSpecificStreamInfo = NULL;

	return Pa_IsFormatSupported (inputParameters, NULL, Rate) ==
	                                          paFormatIsSupported;
}

int	paReader::paCallback_i (
		const void*			inputBuffer,
                void*				outputBuffer,
		unsigned long			framesPerBuffer,
		const PaStreamCallbackTimeInfo	*timeInfo,
	        PaStreamCallbackFlags		statusFlags,
	        void				*userData) {
RingBuffer<float>	*inB;
float *inp	= (float *)inputBuffer;
paReader	*ud	= reinterpret_cast <paReader *>(userData);

	(void) outputBuffer;
	(void) timeInfo;
	(void) statusFlags;

	if (inputBuffer != NULL) {
	   if (ud -> paCallbackReturn == paContinue) {
	      inB =  ud -> _I_Buffer;
	      (void) ((inB) -> putDataIntoBuffer (inp, 2 * framesPerBuffer));
	   }

	   emit ud -> samplesAvailable ();
	}

	return ud -> paCallbackReturn;
}


