#
/*
 *    Copyright (C) 2010,2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the  JSDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	<QSettings>
#include	<QMessageBox>
#include	<QFileDialog>
#include	<QDebug>
#include	<qwt_color_map.h>
#include	<qwt_plot_spectrogram.h>
#include	<qwt_scale_widget.h>
#include	<qwt_scale_draw.h>
#include	<qwt_plot_zoomer.h>
#include	<qwt_plot_panner.h>
#include	<qwt_plot_layout.h>
#include	<QDateTime>
#include	<QFile>
#include	"dabstick-dll.h"
#include	"dabstick.h"
#include	"gui.h"
#include	"utilities.h"
#include	"fft-scope.h"
#include	"scope.h"
#include	"spectrum-constants.h"

#ifdef __MINGW32__
#include	<iostream>
#endif

static
//int16_t	delayTable [] = {15, 13, 11, 10, 9, 8, 7, 5, 3, 2, 1};
int16_t	delayTable [] = {1, 3, 5, 7, 9, 10, 15};
#define delayTableSize	((int)(sizeof (delayTable) / sizeof (int16_t)))
/*
 *	We use the creation function merely to set up the
 *	user interface and make the connections between the
 *	gui elements and the handling agents. All real action
 *	is embedded in actions, initiated by gui buttons
 */
	RadioInterface::RadioInterface (QSettings	*Si,
	                                bool		doInit,
	                                int32_t		CardRate,
					int16_t		displaysize,
	                                int16_t		rastersize,
	                                int32_t		spectrumWidth,
	                                QWidget		*parent): QDialog (parent) {
bool	success;
int16_t	i;

// 	the setup for the generated part of the ui
	setupUi (this);

	ISettings			= Si;
	this		-> doInit	= doInit;
	this		-> CardRate	= CardRate;
	this		-> displaysize	= displaysize;
	this		-> rastersize	= rastersize;
	this		-> spectrumWidth	= spectrumWidth;
/*
 */
	HFScope		= new fft_scope (hfscope,
	                                 displaysize,
	                                 1000,
	                                 rastersize,
	                                 spectrumWidth,
	                                 CardRate,
	                                 REPEAT_RATE);
	HFViewmode	= SPECTRUM_MODE;
	HFScope		-> SelectView (SPECTRUM_MODE);
	HFAverager	= false;
	HFScope		-> setAverager (false);
	connect (HFScope,
	         SIGNAL (clickedwithLeft (int)),
	         this,
	         SLOT (AdjustFrequency (int)));
	ClearPanel		();
	Window		= NULL;
/*
 *	he does the connections from the gui buttons, sliders etc
 */
	localConnects		();

	dabstickSelect		= true;		// default
	if (ISettings != NULL) {
	   QString s;
	   s = ISettings -> value ("dabstick", "dabstick-dll"). toString ();
	   dabstickSelect = s == "dabstick-dll";
	}
	if (dabstickSelect)
	   MyRig	= new dabstick_dll (CardRate,
	                                    CardRate, &success);
	else
	   MyRig	= new dabstick (CardRate,
	                                CardRate, &success);
	if (!success) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Could not open dongle"));
	   delete MyRig;
	   exit (101);
	}

	fprintf (stderr, "found the rig\n");
//
//	Create a timer for dealing with a non acting (or reacting) user
//	on the keyboard
	lcd_timer		= new QTimer ();
	lcd_timer		-> setSingleShot (TRUE);	// single shot
	lcd_timer		-> setInterval (5000);
	connect (lcd_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (lcd_timeout ()));
//
//	a timer for autoincrement/decrement of the tuning
	autoIncrement_timer	= new QTimer ();
	autoIncrement_timer	-> setSingleShot (TRUE);
	autoIncrement_timer	-> setInterval (5000);
	connect (autoIncrement_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (autoIncrement_timeout ()));
/*
 *	just as a gadget, we show the actual time
 */
	display_timer		= new QTimer ();
	display_timer		-> setInterval (1000);
	connect (display_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (updateTimeDisplay ()));
/*
 *	for the purpose of driving the processor when reading
 *	and processing a file we have a timer
 */
	run_timer		= new QTimer ();
	run_timer		-> setInterval (MSECFORTIMER);
	run_timer		-> start (MSECFORTIMER);

	lcd_InputRate		-> display ((int)CardRate);
	char	*s = (char *)alloca (256);
	sprintf (s, "sdrJ-SA-V%s", CURRENT_VERSION);
	systemindicator		-> setText (QString (s));
	display_timer		-> start (1000);
//
//	all elements seem to exist: set the control state to
//	its default values
	setInitialControlState	();
//	if we have a ".ini" file, we overrule the initial control state
	if ((ISettings != NULL) && doInit)
	   setControlState	(ISettings);
//
//	We window the vector that is send to the display
	Window		= new DSPFLOAT [spectrumWidth / 2];
	for (i = 0; i < spectrumWidth / 2; i ++)  
	   Window [i]	= 0.42 - 0.5 * cos ((2.0 * M_PI * i) / (spectrumWidth / 2)) +
	                        0.08 * cos ((4.0 * M_PI * i) / (spectrumWidth / 2));


	timeCount		= 0;
	previousCount		= 0;
	realAverage		= CardRate;
	sampleCounter		= 0;
}

	RadioInterface::~RadioInterface (void) {
	delete		lcd_timer;
	delete		autoIncrement_timer;
	delete		MyRig;
	delete		HFScope;
	delete		display_timer;
	delete		run_timer;
	if (Window != NULL)
	   delete[]	Window;
}
//
//	Eventually, we want that each individual element
//	of the control state can be overruled. We start
//	with showing this state through default settings
void	RadioInterface::setInitialControlState	(void) {

	Paused			= false;
	pauseButton		-> setText (QString ("Pause"));
	incrementingFlag ->
	                 setStyleSheet ("QLabel {background-color:blue}");
	incrementingFlag -> setText (" ");
	IncrementIndex		= 0;

	set_spectrum_increment	(spectrum_increment	-> value ());
	setAmplification	(spectrumAmplitudeSlider -> value ());
	setAttenuation		(attenuationSlider	 -> value ());
	setIQBalance		(IQbalanceSlider	-> value ());

	spectrum_min_loop_frequency	= 25000;
	spectrum_max_loop_frequency	= 2200000;
}
//
//	We have the possibility of creating a ".ini" file
void	RadioInterface::dumpControlState	(QSettings *s) {
	if (s == NULL)
	   return;

	s	-> setValue ("dabstick", 
	                      dabstickSelect ? "dabstick-dll" : "dabstick");
	s	-> setValue ("spectrum_increment",
	                               spectrum_increment -> value ());
	s	-> setValue ("spectrumAmplitudeSlider",
	                               spectrumAmplitudeSlider -> value ());
	s	-> setValue ("attenuationSlider",
	                               attenuationSlider -> value ());
	s	-> setValue ("IQbalanceSlider",
	                               IQbalanceSlider	-> value ());
//
//	now setting the parameters for the fm decoder
	s	-> setValue ("frequency",
	                               (int)get_UserFrequency () / Khz (1));
	s	-> setValue ("PauseButton",
	                               Paused ? 1 : 0);

	s	-> setValue ("IncrementIndex",
	                               IncrementIndex);

	s	-> setValue ("spectrum_min_loop_frequency",
	                               spectrum_min_loop_frequency);
	s	-> setValue ("spectrum_max_loop_frequency",
	                               spectrum_max_loop_frequency);
}
//
//	This does the trick: if an ".ini" file is specified,
//	the specified settings are restored
void	RadioInterface::setControlState	(QSettings *s) {
QString	h;
int	k;

	if (s == NULL)
	   return;

	k	= s -> value ("spectrum_increment", 500). toInt ();
	spectrum_increment	-> setValue (k);	// will signal

	k	= s -> value ("spectrumAmplitudeSlider", 50). toInt ();
	spectrumAmplitudeSlider	-> setValue (k);

	k	= s -> value ("attenuationSlider", 50). toInt ();
	attenuationSlider	-> setValue (k);

	k	= s -> value ("IQbalanceSlider", 0). toInt ();
	IQbalanceSlider		-> setValue (k);	// will signal

//	we specify frequency in Khz
	k	= s -> value ("frequency", 89300). toInt ();
	Set_Tuner (Khz (k));
	Display (get_UserFrequency (), 0);

	k	= s -> value ("PauseButton", 0). toInt ();
	Paused	= (k != 0);
	if (Paused) 
	   pauseButton -> setText (QString ("Continue"));
	else
	   pauseButton -> setText (QString ("Pause"));

	IncrementIndex	= s -> value ("IncrementIndex", 0). toInt ();
	   
	spectrum_min_loop_frequency	=
	s -> value ("spectrum_min_loop_frequency", 
	                       SPECTRUM_MIN_LOOP_FREQUENCY). toInt ();
	spectrum_max_loop_frequency	=
	s -> value ("spectrum_max_loop_frequency",
	                       SPECTRUM_MAX_LOOP_FREQUENCY). toInt ();
	if (IncrementIndex != 0)
	   doIncrementStep	();

}
//
//	On start, we ensure that the streams are stopped so
//	that they can be restarted again.
void	RadioInterface::setStart	(void) {
bool	r = 0;

	MyRig		-> stopReader ();	
	disconnect (MyRig,
	            SIGNAL (samplesAvailable (void)),
	            this,
	            SLOT (JansRadioThroughSignal (void)));
//
//	
	r = MyRig -> restartReader ();
	qDebug ("Starting %d\n", r);
	if (!r) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Opening  input stream failed\n"));
	   return;
	}

//	and we are on the move, so let the timer run
	connect (MyRig,
	         SIGNAL (samplesAvailable (void)),
	         this,
	         SLOT (JansRadioThroughSignal ()));
}

void	RadioInterface::TerminateProcess () {
	if (ISettings != NULL)
	   dumpControlState	(ISettings);

	MyRig -> stopReader ();
//
	disconnect (autoIncrement_timer,
	            SIGNAL (timeout ()),
	            this,
	            SLOT (autoIncrement_timeout ()));
	accept ();
	qDebug () <<  "End of termination procedure";
}

void	RadioInterface::abortSystem (int d) {
	qDebug ("aborting for reason %d\n", d);
	exit (11);
	accept ();
	
}

void	RadioInterface::setAmplification (int a) {
	HFScope -> setAmplification (a);
}

void	RadioInterface::setAttenuation (int n) {
int16_t	f	= IQBalanceDisplay	-> value ();
int16_t	bl, br;

	bl	= 100 - 2 * f;
	br	= 100 + 2 * f;
	attenuationLevelDisplay	-> display (n);
	currAttSliderValue	= 2 * n;
	attValueL	= currAttSliderValue * (float)bl / 100;
	attValueR	= currAttSliderValue * (float)br / 100;
}
/*
 * 	the balance slider runs between -30 .. 30
 */
void	RadioInterface::setIQBalance (int n) {
int16_t	bl, br;

	IQBalanceDisplay -> display (n);
	bl		= 100 - 2 * n;
	br		= 100 + 2 * n;
	attValueL	= currAttSliderValue * (float)bl / 100;
	attValueR	= currAttSliderValue * (float)br / 100;
}
/*
 * 	Handling the numeric keypad is boring, but needed
 */
void RadioInterface::addOne (void) {
	AddtoPanel (1);
}

void RadioInterface::addTwo (void) {
	AddtoPanel (2);
}

void RadioInterface::addThree (void) {
	AddtoPanel (3);
}

void RadioInterface::addFour (void) {
	AddtoPanel (4);
}

void RadioInterface::addFive (void) {
	AddtoPanel (5);
}

void RadioInterface::addSix (void) {
	AddtoPanel (6);
}

void RadioInterface::addSeven (void) {
	AddtoPanel (7);
}

void RadioInterface::addEight (void) {
	AddtoPanel (8);
}

void RadioInterface::addNine (void) {
	AddtoPanel (9);
}

void RadioInterface::addZero (void) {
	AddtoPanel (0);
}
//
//	If we are keying, the lcd timer is on, so
//	if it is no on, we are not keying and clear does not
//	mean anything
void RadioInterface::addClear (void) {
	if (!lcd_timer -> isActive ())
	   return;

	stop_lcdTimer	();
//	we know that the increment timer is not running

	ClearPanel ();
	Display (get_UserFrequency (), 0);
}
//
//	Note that in fm receiver, user commands are in KHz
//
void RadioInterface::AcceptFreqinKhz (void) {
int32_t	p;

	if (!lcd_timer -> isActive ())
	   return;

	stop_lcdTimer	();
//	we know that the increment timer is not running,
//	so there is no reason to stop it
	p = getPanel ();
	ClearPanel ();
	Set_Tuner (p * 1000);
	Display (get_UserFrequency (), 0);
}

void RadioInterface::addCorr (void) {
	if (!lcd_timer -> isActive ()) 
	   return;

	stop_lcdTimer	();
//	increment timer cannot run now so no reason
//	to stop it
	Panel = (Panel - (Panel % 10)) / 10;
	Display	(Panel, 1);
	lcd_timer	-> start (5000);		// restart timer
}
//
//	In AddtoPanel, we might have any of the two
//	timers running, do not know which one though
void	RadioInterface::AddtoPanel (int16_t n) {
	stop_lcdTimer	();
	stopIncrementing ();

	Panel		= 10 * Panel + n;
	Display (Panel, 1);
	lcd_timer	-> start (5000);		// restart timer
}

void	RadioInterface::ClearPanel (void) {
	Panel = 0;
}

int	RadioInterface::getPanel (void) {
	return Panel;
}

void RadioInterface::decT5 (void) {
	IncrementFrequency (mapIncrement (-1));
}

void RadioInterface::decT50 (void) {
	IncrementFrequency (mapIncrement (-10));
}

void RadioInterface::decT500 (void) {
	IncrementFrequency (mapIncrement (-100));
}

void RadioInterface::decT5000 (void) {
	IncrementFrequency (mapIncrement (-1000));
}

void RadioInterface::incT5 (void) {
	IncrementFrequency (mapIncrement (1));
}

void RadioInterface::incT50 (void) {
	IncrementFrequency (mapIncrement (10));
}

void RadioInterface::incT500 (void) {
	IncrementFrequency (mapIncrement (100));
}

void RadioInterface::incT5000 (void) {
	IncrementFrequency (mapIncrement (1000));
}
//
//	Increment frequency: with amount N
//
int32_t 	RadioInterface::mapIncrement (int32_t n) {
      return n * 1000;
}
//
//	IncrementFrequency is called from the handlers
//	for the autoincrement, the increment and the
//	++, + etc knobs.
void	RadioInterface::IncrementFrequency (int32_t n) {
	stop_lcdTimer	();
	stopIncrementing	();

	Set_Tuner (get_UserFrequency () + n);
	Display (get_UserFrequency (), 0);
}
//	AdjustFrequency is called whenever someone clicks
//	with the button on the screen. The amount
//	has to be multiplied with 1000
void	RadioInterface::AdjustFrequency (int n) {
	IncrementFrequency (n * 1000);
}
//
//	Set_Tuner accepts freq in Hz.
//
//	We distinguish among
//	- VFO frequency, i.e. the frequency the device is set to
//	- the tuned frequency, i.e. the frequency the software tuner is set to
//	- the user frequency, i.e. the frequency the user thinks he is
//	  tuning to
//
void	RadioInterface::Set_Tuner (int32_t n) {
int32_t	vfo;
int32_t	VFOfrequency;
int32_t	TunedFrequency;
int32_t	userFrequency	= n;
uint8_t	flag;
int16_t	filter;

	TunedFrequency	= userFrequency;
	if (!MyRig -> legalFrequency (TunedFrequency))
	   return;

	VFOfrequency		= TunedFrequency;
	MyRig	-> setVFOFrequency	(VFOfrequency);
	MyRig	-> getVFOFrequency	(&vfo);
	MyRig	-> setFilter		(TunedFrequency, &filter);
	LOFrequency	= TunedFrequency - vfo;
	HFScope		-> setZero (VFOfrequency);
	HFScope		-> setNeedle (LOFrequency);
	HFScope		-> clearAverage	();
}

int32_t	RadioInterface::get_TunedFrequency (void) {
int32_t vfo;
uint8_t	flag;

	MyRig -> getVFOFrequency (&vfo);
	return (vfo + LOFrequency);
}

int32_t	RadioInterface::get_UserFrequency (void) {
	return get_TunedFrequency ();
}
//
//	Stupid code for autoincrementing and decrementing
static	inline
int32_t IncrementInterval (int16_t incr) {

	if (incr < 0)
	   incr = - incr;

	if (incr == 0)
	   incr = 1;
	if (incr >= delayTableSize)
	   incr = delayTableSize;

	return 1000 * delayTable [incr - 1];
}

void	RadioInterface::set_incrementFlag (int16_t incr) {

	if (incr != 0) {
	   char temp [128];
	   if (incr > 0)
	      sprintf (temp, "%d >>>", IncrementInterval (incr) / 1000);
	   else
	      sprintf (temp, "<<< %d", IncrementInterval (incr) / 1000);
	   incrementingFlag ->
	                 setStyleSheet ("QLabel {background-color:green}");
	   incrementingFlag -> setText (temp);
	}
	else {
	   incrementingFlag ->
	                 setStyleSheet ("QLabel {background-color:blue}");
	   incrementingFlag -> setText (" ");
	}
}
//
//	just a quick hack
int32_t RadioInterface:: min_loop_frequency_for (int16_t mode) {
	return (spectrum_min_loop_frequency);
}

int32_t RadioInterface::max_loop_frequency_for (int16_t mode) {
	return (spectrum_max_loop_frequency);
}

static	inline
bool	frequencyInBounds (int32_t f, int32_t l, int32_t u) {
	return l <= f && f <= u;
}

void	RadioInterface::autoIncrement_timeout (void) {
int32_t	frequency	= get_UserFrequency () / KHz (1);

	frequency	+= IncrementIndex < 0 ? - fmIncrement : fmIncrement;
//
	if ((IncrementIndex < 0) &&
	         !frequencyInBounds (frequency,
	                              min_loop_frequency_for (0),
	                              max_loop_frequency_for (0)))
	   frequency = max_loop_frequency_for (0);

	if ((IncrementIndex > 0) &&
	         !frequencyInBounds (frequency,
	                              min_loop_frequency_for (0),
	                              max_loop_frequency_for (0)))
	   frequency = min_loop_frequency_for (0);

	Set_Tuner (KHz (frequency));
	Display (get_UserFrequency (), 0);
	autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
}

void	RadioInterface::stopIncrementing (void) {
	set_incrementFlag (0);

	if (autoIncrement_timer	-> isActive ())
	   autoIncrement_timer -> stop ();

	IncrementIndex = 0;
}

void	RadioInterface::autoIncrementButton (void) {
	if (++IncrementIndex > delayTableSize)
	   IncrementIndex = delayTableSize;
	doIncrementStep ();
}

void	RadioInterface::autoDecrementButton (void) {
	if (--IncrementIndex < -delayTableSize)
	   IncrementIndex = -delayTableSize;
	doIncrementStep ();
}

void	RadioInterface::doIncrementStep  (void) {
	stop_lcdTimer ();

	if (autoIncrement_timer	-> isActive ())
	   autoIncrement_timer -> stop ();

	if (IncrementIndex == 0) {
	   set_incrementFlag (0);
	   return;
	}
//
	autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
	set_incrementFlag (IncrementIndex);
}

void	RadioInterface::set_spectrum_increment (int v) {
	fmIncrement	= (int32_t) v;
}

int32_t	RadioInterface::get_Increment_for (int16_t) {
	return fmIncrement;
}

void	RadioInterface::IncrementButton (void) {
	stopIncrementing ();
	Set_Tuner (get_UserFrequency () +
	                get_Increment_for (0) * KHz (1));
	Display (get_UserFrequency (), 0);
}

void	RadioInterface::DecrementButton (void) {
	stopIncrementing ();
	Set_Tuner (get_UserFrequency () -
	                 get_Increment_for (0) * KHz (1));
	Display (get_UserFrequency (), 0);
}

void	RadioInterface::stop_lcdTimer (void) {
	if (lcd_timer -> isActive ())
	   lcd_timer -> stop ();
}

void	RadioInterface::updateTimeDisplay (void) {
QDateTime	currentTime = QDateTime::currentDateTime ();

	timeDisplay	-> setText (currentTime.
	                            toString (QString ("dd.MM.yy:hh:mm:ss")));
}

void	RadioInterface::lcd_timeout (void) {
	Panel		= 0;			// throw away anything
	Display (get_UserFrequency (), 0);
}

/*
 *	there is a tremendous amount of signal/slot connections
 *	The local connects, knobs, sliders and displays,
 *	are connected here.
 */
void	RadioInterface::localConnects (void) {
	connect (startButton, SIGNAL (clicked ()),
	              this, SLOT (setStart ()));
	connect (QuitButton, SIGNAL (clicked ()),
	              this, SLOT (TerminateProcess ()));
	connect (averageButton, SIGNAL (clicked (void)),
	              this, SLOT (setAverager (void)));
	connect (viewmodeButton, SIGNAL (clicked (void)),
	              this, SLOT (setViewmode (void)));
	connect (spectrumAmplitudeSlider, SIGNAL (valueChanged (int)),
	         this,  SLOT (setAmplification (int)));

	connect (fc_plus,
	         SIGNAL (clicked ()),
	         this,
	         SLOT (autoIncrementButton ()));
	connect (fc_minus,
	         SIGNAL (clicked ()),
	         this,
	         SLOT (autoDecrementButton ()));
	connect (f_plus,
	         SIGNAL (clicked ()),
	         this,
	         SLOT (IncrementButton ()));
	connect (f_minus,
	         SIGNAL (clicked ()),
	         this,
	         SLOT (DecrementButton ()));
	connect (spectrum_increment,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (set_spectrum_increment (int)));

	connect (attenuationSlider, SIGNAL (valueChanged (int) ),
		      this, SLOT (setAttenuation (int) ) );
	connect (IQbalanceSlider, SIGNAL (valueChanged (int) ),
		      this, SLOT (setIQBalance (int) ) );
/*
 *	Mode setters
 */
	connect	(pauseButton,
	         SIGNAL (clicked (void)),
	         this,
	         SLOT (clickPause (void)));

/*
 *	connections for the console, first the digits
 */
	connect (add_one,   SIGNAL (clicked() ), this, SLOT (addOne() ) );
	connect (add_two,   SIGNAL (clicked() ), this, SLOT (addTwo() ) );
	connect (add_three, SIGNAL (clicked() ), this, SLOT (addThree() ) );
	connect (add_four,  SIGNAL (clicked() ), this, SLOT (addFour() ) );
	connect (add_five,  SIGNAL (clicked() ), this, SLOT (addFive() ) );
	connect (add_six,   SIGNAL (clicked() ), this, SLOT (addSix() ) );
	connect (add_seven, SIGNAL (clicked() ), this, SLOT (addSeven() ) );
	connect (add_eight, SIGNAL (clicked() ), this, SLOT (addEight() ) );
	connect (add_nine,  SIGNAL (clicked() ), this, SLOT (addNine() ) );
	connect (add_zero,  SIGNAL (clicked() ), this, SLOT (addZero() ) );
/*
 *	function buttons
 */
	connect (khzSelector, SIGNAL (clicked ()),
	                      this, SLOT (AcceptFreqinKhz ()));
	connect (add_correct, SIGNAL (clicked() ), this, SLOT (addCorr() ) );
	connect (add_clear,   SIGNAL (clicked() ), this, SLOT (addClear() ) );
/*
 *	The increment and decrement buttons
 */
	connect (dec_5,	   SIGNAL (clicked() ), this, SLOT (decT5() ) );
	connect (dec_50,   SIGNAL (clicked() ), this, SLOT (decT50() ) );
	connect (dec_500,  SIGNAL (clicked() ), this, SLOT (decT500() ) );
	connect (inc_5,	   SIGNAL (clicked() ), this, SLOT (incT5() ) );
	connect (inc_50,   SIGNAL (clicked() ), this, SLOT (incT50() ) );
	connect (inc_500,  SIGNAL (clicked() ), this, SLOT (incT500() ) );
}
/*
 *	HandleSamples is the real processor. It is called
 *	either through a signal issued by a timer when reading
 *	from a file, or through a signal from the soundsystem
 *	indicating that samples are waiting to be processed.
 *	For reading from a file, we have to limit the number
 *	of samples that will be processed per clocktick
 *	to CardRate / 1000 * MSECFORTIMER. To handle delays
 *	in the timer, we read a little more per timer tick.
 */
void	RadioInterface::addtoAverage (uint32_t count,
	                              int32_t msec, int32_t samples) {

	timeCount += msec;
	sampleCounter = 0.1 * samples + 0.9 * sampleCounter;
	if (timeCount > 1000) {
	   uint32_t average = count - previousCount;
	   timeCount = 0;
	   previousCount = count;
	   realAverage = 0.1 * average + 0.9 * realAverage;
	   lcd_usbAverage	-> display ((int)realAverage);
	   lcd_Samples		-> display (sampleCounter);
	}
}

void	RadioInterface::JansRadioThroughSignal (void) {
int32_t	amount	= spectrumWidth / 2;
DSPCOMPLEX	*v = (DSPCOMPLEX *)alloca (amount * sizeof (DSPCOMPLEX));
int32_t		i;
int32_t		currentAmount = MyRig -> Samples ();

	if ((currentAmount < CardRate / 20) || Paused)
	   return;

	if (dabstickSelect)
	   addtoAverage (((dabstick_dll *)MyRig) -> getSampleCount (),
	                                                100,
	                                                currentAmount);
	else
	   addtoAverage (((dabstick *)MyRig) -> getSampleCount (),
	                                                100,
	                                                currentAmount);

	if (dabstickSelect)
	   ((dabstick_dll *)MyRig) -> getSamplesandSkip (v,
	                                               amount, currentAmount);
	else
	   ((dabstick *)MyRig) -> getSamplesandSkip (v,
	                                             amount, currentAmount);
	for (i = 0; i < spectrumWidth / 2; i ++)  
	   v [i] = DSPCOMPLEX (real (v [i]) * attValueL * Window [i],
	                       imag (v [i]) * attValueR * Window [i]);
	HFScope	-> addElementsandShow (v, amount);
}

static inline
int32_t numberofDigits (int32_t f) {

	if (f < 100000)
	   return 6;
	if (f < 100000000)
	   return 8;
	if (f < 1000000000)
	   return 9;
	else
	   return 10;
}

void	RadioInterface::Display (int32_t freq, int16_t Mode) {
int32_t nd	= numberofDigits (freq);

	if (Mode == 0) {
	   lcd_Frequency	-> setNumDigits (nd);
	   lcd_Frequency	-> display ((int)freq / 1000);
	}
	else {
	   lcd_Frequency	-> setNumDigits (nd);
	   lcd_Frequency	-> display ((int)freq);
	}
}

void	RadioInterface::clickPause (void) {
	Paused	= !Paused;
	if (Paused) {
	   if (autoIncrement_timer	-> isActive ())
	      autoIncrement_timer 	-> stop ();

	   MyRig		-> stopReader	();
	   pauseButton -> setText (QString ("Continue"));
	}
	else {
	   if (IncrementIndex != 0)	// restart the incrementtimer if needed
	      autoIncrement_timer	-> start (IncrementInterval (IncrementIndex));
	   MyRig		-> restartReader ();
	   pauseButton -> setText (QString ("Pause"));
	}
}

void	RadioInterface::setAverager (void) {
	HFAverager	= !HFAverager;
	HFScope		-> setAverager (HFAverager);
}

void	RadioInterface::setViewmode (void) {
	if (HFViewmode == SPECTRUM_MODE) 
	   HFViewmode = WATERFALL_MODE;
	else 
	   HFViewmode = SPECTRUM_MODE;

	HFScope	-> SelectView (HFViewmode);
}

void	RadioInterface::wheelEvent (QWheelEvent *e) {
	stopIncrementing ();
	if (e -> delta () > 0)
	   Set_Tuner (get_UserFrequency () + 50000);
	else
	   Set_Tuner (get_UserFrequency () - 50000);
	Display (get_UserFrequency (), 0);
}
	 
