#
/*
 *
 *    Copyright (C)  2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _GUI
#define _GUI

#include	"jff-include.h"
#include	<QDialog>
#include	"ui_sdrgui.h"
#include	<qwt.h>
#include	<qwt_slider.h>
#include	<qwt_plot.h>
#include	<qwt_plot_curve.h>
#include	<qwt_plot_marker.h>
#include	<qwt_plot_grid.h>
#include	<qwt_dial.h>
#include	<qwt_dial_needle.h>
#include	<qwt_plot_spectrogram.h>
#include	<QTimer>
#include	"scope.h"
#include	"iqdisplay.h"
#include	"virtualrig.h"
#include	"fir-filters.h"
//#include	<sndfile.h>
#include	<QWheelEvent>
class	QSettings;
class	fft_scope;

#define	HFSPECTRUM	0200
#define	LFSPECTRUM	0201

/*
 *	GThe main gui object. It inherits from
 *	QDialog and the generated form
 */
class RadioInterface: public QDialog,
		      private Ui_elektorSDR {
Q_OBJECT
public:
		RadioInterface		(QSettings	*,
	                                 bool,
	                                 int32_t,		// card-in rate
	                                 int16_t,		// displaysize
	                                 int16_t,		//rastersize
	                                 int32_t,		// spectrum width
	                                 QWidget *parent = NULL);
		~RadioInterface		();

private:
	enum Decoder {
	   NO_DECODER	= 0,
	   AM_DECODER	= 1,
	   FM_DECODER	= 2,
	   AUDIO_TEST	= 3,
	};

	void		localConnects		(void);
	void		setInitialControlState	(void);
	void		dumpControlState	(QSettings *);
	void		setControlState		(QSettings *);

	bool		dabstickSelect;
	fft_scope	*HFScope;
	bool		HFAverager;
	int16_t		HFViewmode;
	DSPFLOAT	*Window;
	QSettings	*ISettings;
	bool		doInit;
	int32_t		CardRate;
	int16_t		displaysize;
	int16_t		rastersize;
	int32_t		spectrumWidth;
	virtualRig	*MyRig;

	int16_t		currAttSliderValue;
	DSPFLOAT	attValueL;
	DSPFLOAT	attValueR;

	int32_t		LOFrequency;

	int32_t		spectrum_min_loop_frequency;
	int32_t		spectrum_max_loop_frequency;
	void		ClearPanel		();
	void		AddtoPanel		(int16_t);
	int		getPanel		();
	void		CorrectPanel		();

	int32_t		get_TunedFrequency	(void);
	int32_t		get_UserFrequency	(void);
	int32_t		mapIncrement		(int32_t);
	void		Set_Tuner		(int32_t);
	void		Display			(int32_t, int16_t);
	QTimer		*lcd_timer;
	QTimer		*autoIncrement_timer;
	int16_t		IncrementIndex;
	int32_t		autoIncrement_amount;
	int32_t		fmIncrement;
	
	void		set_incrementFlag	(int16_t);
	void		stopIncrementing	(void);
	void		stop_lcdTimer		(void);
	int32_t		get_Increment_for	(int16_t);
	int32_t		min_loop_frequency_for	(int16_t);
	int32_t		max_loop_frequency_for	(int16_t);

	QTimer		*run_timer;
	int		Panel;
	int16_t		CurrentRig;
	QTimer		*display_timer;
/*
 *	The UI acts as a kind of server, the public slots are the entries
 *	for the radio elements, signals (below) are the triggers
 *	for the radio elements.
 */
	void		IncrementFrequency	(int32_t);
	bool		Paused;
//
//	for the statistics of the usb interface:
	uint32_t	previousCount;
	uint32_t	average;
	int32_t		timeCount;
	DSPFLOAT	realAverage;
	void		addtoAverage		(uint32_t, int32_t, int32_t);
	int32_t		sampleCounter;
/*
 */
private slots:
	void	setStart		(void);
	void	lcd_timeout		(void);
	void	updateTimeDisplay	(void);
	void	clickPause		(void);
	void	setAverager		(void);
	void	setViewmode		(void);

	void	addOne			(void);
	void	addTwo			(void);
	void	addThree		(void);
	void	addFour			(void);
	void	addFive			(void);
	void	addSix			(void);
	void	addSeven		(void);
	void	addEight		(void);
	void	addNine			(void);
	void	addZero			(void);
	void	addClear		(void);
	void	AcceptFreqinKhz		(void);
	void	addCorr			(void);

	void	set_spectrum_increment	(int);
	void	setAttenuation		(int);
	void	setIQBalance		(int);

	void	setAmplification	(int);
	void	decT5			(void);
	void	decT50			(void);
	void	decT500			(void);
	void	decT5000		(void);
	void	incT5			(void);
	void	incT50			(void);
	void	incT500			(void);
	void	incT5000		(void);

	void	abortSystem		(int);
	void	TerminateProcess	(void);

	void	autoIncrement_timeout	(void);
	void	autoIncrementButton	(void);
	void	autoDecrementButton	(void);
	void	IncrementButton		(void);
	void	DecrementButton		(void);
	void	doIncrementStep		(void);

public slots:
	void	JansRadioThroughSignal	(void);
	void	AdjustFrequency		(int);
	void	wheelEvent		(QWheelEvent *);
};

#endif

