#
/*
 *
 *    Copyright (C) 2010, 2011, 2012
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JSDR.
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ESDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	Main program
 */
#include	<QApplication>
#include	<QSettings>
#include	<QDir>
#include	"jff-include.h"
#include	"utilities.h"
#include	"gui.h"
#include	"spectrum-constants.h"

void	fullPathfor (const char *v, char *out) {
int16_t	i;
QString	homeDir;

	if (v == NULL) {
	   sprintf (out, "%s", MISERY);
	   return;	// should not happen
	}

	if (v [0] == '/') {		// full path specified
	   sprintf (out, "%s", v);
	   return;
	}

	homeDir	= QDir::homePath ();
	homeDir. append ("/");
	homeDir. append (v);
	homeDir = QDir::toNativeSeparators (homeDir);
	sprintf (out, "%s", homeDir. toAscii (). data ());

	for (i = 0; out [i] != 0; i ++);
	if (out [i - 4] != '.' ||
	    out [i - 3] != 'i' ||
	    out [i - 2] != 'n' ||
	    out [i - 1] != 'i') {
	    out [i] = '.';
	    out [i + 1] = 'i';
	    out [i + 2] = 'n';
	    out [i + 3] = 'i';
	    out [i + 4] = 0;
	}
}

int	main (int argc, char **argv) {
int32_t		i;
int32_t		tmp;
int32_t		opt;
/*
 *	The default values
 */
int16_t		displaysize		= FE_DISPLAYSIZE;
int16_t		rastersize		= RASTER;
int32_t		spectrumWidth		= SPECTRUM_WIDTH;

QSettings	*ISettings;		/* input .ini file	*/
int32_t		CardRate	= 2000000;
RadioInterface	*MyRadioInterface;
bool		doInit		= true;
char		*initFilename	= NULL;
char		*defaultInit	= (char *)alloca (512 * sizeof (char));

	fullPathfor (DEFAULT_INI, defaultInit);

	while ((opt = getopt (argc, argv, "A:B:C:i:o:dFEGS")) != -1) {
	   switch (opt) {
	      case 'A':	tmp  = atoi (optarg);
	                displaysize = nearestTwoPower (tmp);
	                break;

	      case 'B': tmp = atoi (optarg);
	                spectrumWidth	= nearestTwoPower (tmp);
	                break;

	      case 'C': tmp = atoi (optarg);
	                CardRate = tmp;
	                break;

	      case 'G':	CardRate	= 3200000;
	                break;

	      case 'F':	CardRate	= 3000000;
	                break;

	      case 'E':	CardRate	= 2000000;
	                break;

	      case 'S':	CardRate	= 1500000;
	                break;

	      case 'i':	initFilename	=
	                            (char *) alloca (512 * sizeof (char));
	                fullPathfor (optarg, initFilename);
	                i ++;
	                break;

	      case 'd':	doInit = false;
	                break;

	      default:
	                break;
	      }
	}

/*
 *	... and the settings of the "environment"
 */
	if (initFilename == NULL)
	   initFilename = defaultInit;
	ISettings	= new QSettings (initFilename, QSettings::IniFormat);
/*
 *	Before we connect control to the gui, we have to
 *	instantiate
 */
	QApplication a (argc, argv);
	MyRadioInterface = new RadioInterface (ISettings,
	                                       doInit,
	                                       CardRate,
			                       displaysize,
	                                       rastersize,
	                                       spectrumWidth);

	MyRadioInterface -> show ();
	a. exec ();
/*
 *	done:
 */
	fflush (stdout);
	fflush (stderr);
	qDebug ("It is done\n");
//	MyRadioInterface	-> ~RadioInterface ();
	if (ISettings != NULL)
	   ISettings 		-> ~QSettings ();
	exit (1);
}

