#
/*
 *
 *    Copyright (C) 2009, 2010, 2011
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in ESDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ESDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include		"filtering.h"
/*
 *	The delay caused by the filters is pretty large, but then
 *	the filters are pretty sharp
 */
	Filtering::Filtering     (int32_t  CardRate,
	                          int32_t WorkingRate,
	                          DSPCOMPLEX *Table) {

	_CardRate		= CardRate;
	_WorkingRate		= WorkingRate;
	LocalOscillator		= new Oscillator (Table, _CardRate);
/*
 * 	... and the defaults and dummies
 */
	TunedIF			= _CardRate / 4;
	filterDegree		= 511;
	S_attack		= 0;
	S_decay			= 0;
	delayCount		= 0;
	sampleSum		= 0;
//
//	BandPassFilter filters the selected frequency with as
//	bandwidth the width of WorkingRate
	BandPassFilter		= new fftFilter (4096, filterDegree);
	BandPassFilter		-> setBand (0, 2000, _CardRate);
	LowPassHighRate		= new fftFilter (4096, filterDegree);
	LowPassHighRate		-> setLowPass (2000, _CardRate);
	BandPassLowRate		= new fftFilter (2048, filterDegree);
}
/*
 */
	Filtering::~Filtering () {
	delete		LocalOscillator;
	delete		LowPassHighRate;
	delete		BandPassLowRate;
	delete		BandPassFilter;
}
/*
 * 	We filter in three stages:
 * 	- stage 1: we filter (running high rate) the IF bandwidth
 * 	- stage 2: following frequency shifting, we filter low pass
 * 	- stage 3; (after decimation) we apply a bandfilter for the
 * 	 specified subband
 *	In set filter we set the bandfilter first, and
 *	we set the filter to be applied AFTER the
 *	frequencyshift.
 */
void	Filtering::setSelectedFrequencyto (int32_t fot,
	                                   int32_t low, int32_t high) {
int32_t		l, h;

        (void)low; (void)high;
	TunedIF		= fot;
	l		= TunedIF - _WorkingRate / 2;
	h		= TunedIF + _WorkingRate / 2;
	if (l <  -_CardRate / 2)
	   l = - _CardRate / 2 + 500;
	if (h > _CardRate / 2)
	   h = _CardRate / 2 - 500;

	BandPassFilter	-> setBand (l, h, _CardRate);
}

void	Filtering::setSelectedBandto (int32_t fot, int32_t low, int32_t high) {
        (void)fot;

	if (low < - _WorkingRate / 2)
	   low = - _WorkingRate / 2 + 500;
	if (high > _WorkingRate / 2)
	   high = _WorkingRate / 2 - 500;

	BandPassLowRate	-> setBand (low, high, _WorkingRate);
}
/*
 */
void	Filtering::setWorkingRate (int32_t wr) {
int32_t l, h;

	if (_WorkingRate == wr)
	   return;

	_WorkingRate	= wr;
	l = TunedIF - _WorkingRate / 2;
	h = TunedIF + _WorkingRate / 2;
	if (l < - _CardRate / 2) l = - _CardRate / 2 + 500;
	if (h >  _CardRate / 2) h = _CardRate / 2 - 500;

	BandPassFilter	-> setBand (l, h, _CardRate);
	LowPassHighRate	-> setLowPass (_WorkingRate / 2, _CardRate);
	BandPassLowRate	-> setBand (- _WorkingRate / 2, 
	                             _WorkingRate / 2, _WorkingRate);
}
/*
 *	The actual filtering and frequencyshifting:
 */
DSPCOMPLEX	Filtering::HandleSampleHighSpeed (DSPCOMPLEX sample) {
	sample	= BandPassFilter   -> Pass (sample);
	sample	= LocalOscillator  -> nextValue (TunedIF) * sample;
	sample	= LowPassHighRate  -> Pass (sample);
	return sample;
}
//
//	bandpass for low speed, called by the main processing thread
//	and also updating the S meter
DSPCOMPLEX	Filtering::BandPassLowSpeed (DSPCOMPLEX sample) {
DSPCOMPLEX	result;

	result	= BandPassLowRate -> Pass (sample);
	if (++delayCount > _WorkingRate / 10) {
	   delayCount = 0;
	   showPeak	(0, FromdbtoS (S_Meter (result)));
	}
	else
	   (void)S_Meter (result);

	return result;
}

#define	ATTACK	(_WorkingRate / 100)
#define	DELAY	(_WorkingRate / 2)

DSPFLOAT	Filtering::S_Meter (DSPCOMPLEX sample) {
DSPFLOAT	mag	= 20 * abs (sample);
DSPFLOAT	S_average;

	S_attack	= decayingAverage (S_attack, mag, ATTACK);
	S_decay		= decayingAverage (S_decay, mag, DELAY);

	if (S_attack > S_decay) {		//increasing signal strength
	   S_average	= S_attack;
	   S_decay	= S_attack;
	}
	else 
	   S_average	= S_decay;

	return get_db (S_average) + 20;
}
/*
 */
int16_t	Filtering::FromdbtoS		(float db) {
	if (db > -73)
	   return 9;
	if (db > -79)
	   return 8;
	if (db > -85)
	   return 7;
	if (db > -91)
	   return 6;
	if (db > -97)
	   return 5;
	if (db > -103)
	   return 4;
	if (db > -109)
	   return 3;
	if (db > -115)
	   return 2;
	return 1;
}
