#
/*
 *
 *    Copyright (C)  2011, 2012, 2013
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are acknowledged and recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	<QSettings>
#include	<QMessageBox>
#include	<QFileDialog>
#include	<QDebug>
#include	<qwt_color_map.h>
#include	<qwt_plot_spectrogram.h>
#include	<qwt_scale_widget.h>
#include	<qwt_scale_draw.h>
#include	<qwt_plot_zoomer.h>
#include	<qwt_plot_panner.h>
#include	<qwt_plot_layout.h>
#include	<QDateTime>
#include	<QFile>
#include	"no_rig.h"
#include	"pmsdr.h"
#include	"dll-handler.h"
#include	"replay.h"
#include	"filtering.h"
#include	"agchandler.h"
#include	"gui.h"
#include	"fft-scope.h"
#include	"oscillator.h"
#include	"spectrummanager.h"
#include	"fir-filters.h"
#include	"iir-filters.h"
#include	"sdrdecoder.h"
#include	"analogdecoder.h"
#include	"rttydecoder.h"
#include	"cwdecoder.h"
#include	"pskdecoder.h"
#include	"mfskdecoder.h"
#include	"amtordecoder.h"
#include	"faxdecoder.h"
#include	"dominodecoder.h"
#include	"oliviadecoder.h"
#include	"scope.h"
#include	"decimator.h"
#include	"sw-constants.h"
#include	"dabstick.h"
#include	"dabstick-dll.h"


#define	NO_RIG		0077
#define	ELEKTOR		0100
#define	PMSDR		0101
#define	REPLAY		0102
#define	DABSTICK	0103
#define	DABSTICK_DLL	0104
#define	EXTIO		0105

#define	OFFSET		116000

#define	IDLE		0100
#define	PAUSED		0101
#define	RUNNING		0102
/*
 *	We use the creation function merely to set up the
 *	user interface and make the connections between the
 *	gui elements and the handling agents. All real action
 *	is embedded in actions, initiated by gui buttons
 */
	RadioInterface::RadioInterface (QSettings	*sI,
	                                bool		doInit,
	                                int32_t		ICardRate,
	                                int32_t		OCardRate,
					int16_t		displaywidth,
	                                int16_t		rastersize,
	                                DSPCOMPLEX	*Table,
	                                QWidget		*parent): QDialog (parent) {
bool	soundSeemsOk;
int16_t	i;
/*
 * 	the setup for the generated part of the ui
 */
	setupUi (this);

	this		-> ISettings	= sI;
	this		-> doInit	= doInit;
	this		-> _ICardRate	= ICardRate;
	this		-> _OCardRate	= OCardRate;
	this		-> displaywidth	= displaywidth;
	this		-> rastersize	= rastersize;
	this		-> Table	= Table;

	FrequencyChangeAllowed		= true;
//
//	Allocations and defaults

	_WorkingRate			= 9600;	 // value will be overruled
	upCount				= _OCardRate / _WorkingRate;
	MySoundcardOut			= new paWriter (_OCardRate,
	                                                HIGHLATENCY);
	outTable			= new int16_t
	                           [MySoundcardOut -> numberofDevices () + 10];
	for (i = 0; i < MySoundcardOut -> numberofDevices (); i ++)
	   outTable [i] = -1;
	soundSeemsOk			= setupSoundOut (streamOutSelector,
	                                                 MySoundcardOut,
	                                                 _WorkingRate,
	                                                 outTable);
	MySoundcardIn			= new paReader (_ICardRate);
	inTable				= new int16_t
	                            [MySoundcardIn -> numberofDevices () + 10];
	for (i = 0; i < MySoundcardIn -> numberofDevices (); i ++)
	   inTable [i] = -1;
	soundSeemsOk			= setupSoundIn  (streamInSelector,
	                                                 MySoundcardIn,
	                                                 ICardRate,
	                                                 inTable);
	runMode				= IDLE;
//
	BackendScope			= NULL;
	FrontEndoperation		= RADIO;	// default
	IQScope				= new IQDisplay (IQdisplay);

	mfsk_Scope			= new Scope (mfsk_scope,
	                                             4 * (32 + 16) , 100);
	mfsk_Scope		-> SelectView  (WATERFALL_MODE);
	mfsk_X			= new int [64];
	mfsk_Y			= new double [64];
	mfsk_count		= 0;

	LFScope			= new fft_scope (lfscope,
	                                         displaywidth,
	                                         1,
	                                         rastersize,
	                                         SPECTRUM_SIZE_BACK,
	                                         _WorkingRate,
	                                         REPEAT_RATE);
	connect (LFScope,
	         SIGNAL (clickedwithLeft (int)),
	         this,
	         SLOT (AdjustFrequency (int)));
	LFScope		-> setNeedle (0);

	HFScope			= new fft_scope (hfscope,
	                                         displaywidth,
	                                         1000,
	                                         rastersize,
	                                         SPECTRUM_SIZE_FRONT,
	                                         _ICardRate,
	                                         REPEAT_RATE);
	connect (HFScope,
	         SIGNAL (clickedwithLeft (int)),
	         this,
	         SLOT (AdjustFrequencywithKHz (int)));
	HFScope		-> setNeedle (_ICardRate / 4);

	createpskPhaseDisplay	();
	inputFilename		= new char [1024];
	dumpFilename		= new char [1024];
	dumpBuffersize		= 1024;
	dumpBuffer		= new float [dumpBuffersize];
	dumpBufferpointer	= 0;
	DumpisOn		= FALSE;
/*
 * he does the connections from the gui buttons, sliders etc
 */
	localConnects		();
	agc			= new agcHandler ((int)_WorkingRate);
	agc			-> setMode (agcHandler::AGC_OFF);

	MyFrontEnd		= new Filtering   (_ICardRate,
	                                           _WorkingRate,
	                                           Table);
	myDecimator		= new downDecimator (_ICardRate, _WorkingRate);
	connect (MyFrontEnd, SIGNAL (showPeak (int, double)),
	         this, SLOT (showPeak (int, double)));

	MyDetector		= new sdrDecoder (_WorkingRate, this);
	setDetectorScreen (SDR_DECODER);

	BackendShifter		= new Oscillator (_WorkingRate);
	BackendState		= 0;
	upFilter		= new LowPassIIR (10,
	                                          (int32_t)_WorkingRate / 2,
	                                          (int32_t)_OCardRate,
	                                          S_BUTTERWORTH);
	BackendOnlyFilter	= new LowPassFIR (11,
	                                          (int32_t)_ICardRate / 3,
	                                          (int32_t)_ICardRate);
	sampleSum		= 0;
	symbol_Leftshift	= 0;
	Circular		= new DSPCOMPLEX [16];
	cc			= 0;
//
//	just for FAXes
	QImage image		= QImage (M_PI * 288, 576,
	                                  QImage::Format_RGB32);
	QLabel	*imageLabel 	= new QLabel;
	imageLabel	-> setPixmap (QPixmap::fromImage (image));
	imageLabel	-> setMouseTracking (false);
/*
 * 	Timers:
 * 	a. display_timer deals with showing actual time
 * 	b. lcd_timer deals with frequency display lcd screen
 *	Create a timer for dealing with a non acting (or reacting) user
 *	on the keyboard
 *	dll_timer: when handling dll's, we need to poll the input
 */
	display_timer		= new QTimer ();
	display_timer		-> setInterval (1000);
	connect (display_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (updateTimeDisplay ()));

	lcd_timer		= new QTimer ();
	lcd_timer		-> setSingleShot (TRUE);	// single shot
	lcd_timer		-> setInterval (5000);
	connect (lcd_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (lcd_timeout ()));
	dll_timer		= new QTimer ();
	dll_timer		-> setInterval (100);
	connect (dll_timer,
	         SIGNAL (timeout ()),
	         this,
	         SLOT (handleDLLRequest ()));

	PMSDR_preSelectorMode		= MANUAL;
	PMSDR_preSelectorBand		= 5;		/* default	*/
	PMSDR_mute			= false;
	bandOffset			= 0;
	bandLowend			= - _WorkingRate / 2;
	bandHighend			= _WorkingRate / 2;

// just defaulting
	CurrentRig			= NO_RIG;
	MyRig				= new no_rig ();
	setrigControlScreen (NO_RIG);
	Set_Tuner			(MyRig -> defaultFrequency (),
	                                 &CurrentFreq, &TunedIF);
	lcd_Frequency			-> display (CurrentFreq);

	inputDevice			= MySoundcardIn	-> invalidDevice ();
	outputDevice			= MySoundcardOut-> invalidDevice ();
/*
 *	The version label, still to be replaced with
 *	something more intelligible
 */
	char	*s = (char *)alloca (256);
	sprintf (s, "sdrJ-SW-V%s", CURRENT_VERSION);
	systemindicator		-> setText (QString (s));
	lcd_SampleRate		-> display ((int)_ICardRate);

	ClearPanel		();
	faxContainer		-> setWidget (imageLabel);

	setInitialControlState	();
	if ((ISettings != NULL) && doInit)
	   setControlState	(ISettings);
	
	if (soundSeemsOk)
	   qDebug ("GUI is set up and working\n"); 
	else {
	   qDebug ("No channels, bailing out\n");
	   accept ();
	   exit (21);
	}
/*
 *	Finally: let the timers run
 */
	display_timer		-> start (1000);
}

	RadioInterface::~RadioInterface () {
	if (HFScope != NULL)
	   delete	HFScope;
	if (BackendScope != NULL)
	   delete	BackendScope;
	delete		LFScope;
	delete		IQScope;
	delete		lcd_timer;
	delete		display_timer;
	delete		MyRig;
	delete		myDecimator;
	delete		MyFrontEnd;
	delete		MyDetector;
	delete		agc;
	delete		upFilter;
	delete		dumpBuffer;
	delete[]	dumpFilename;
	delete[]	inputFilename;
	delete[]	Circular;
}
//
//	
void	RadioInterface::setInitialControlState (void) {
	pauseButton		-> setText (QString ("Pause"));
	dumpON			-> setText ("dumpOn");
	DumpisOn		= false;

	setBalanceSlider	(balanceSlider		-> value ());
	setAttenuator		(attenuationSlider	-> value ());
	set_HFplotterView	(HFplotterView		-> currentText ());
	set_LFplotterView	(LFplotterView		-> currentText ());
	setOperation		(operationSelect	-> currentText ());
	setInputMode		(inputModeSelect	-> currentText ());
	setAGCMode		(AGC_select		-> currentText ());
	setquickBand		(quickbandSelect	-> currentText ());
//	set_bandWidthSelect	(bandwidthSlider	-> value ());
//	set_bandWidthOffset	(bandoffsetSlider 	-> value ());
	set_LFspectrumAmplitude (LFspectrumamplitudeSlider -> value ());
	set_HFspectrumAmplitude (HFspectrumamplitudeSlider -> value ());
	IQDisplay_setAmpl	(iqslider		-> value ());
	setWorkingRate		(workingrateSelect	-> currentText ());
	set_symbolLeftshift	(symbol_shifter		-> value ());
	set_agcThresholdSlider	(agcThresholdSlider	-> value ());

	nodetectorButton	-> setChecked (true);
	analogDecoderButton	-> setChecked (false);
	rttyButton		-> setChecked (false);
	cwButton		-> setChecked (false);
	pskButton		-> setChecked (false);
	mfskButton		-> setChecked (false);
	amtorButton		-> setChecked (false);
	faxButton		-> setChecked (false);
	dominoButton		-> setChecked (false);
	oliviaButton		-> setChecked (false);

	set_rigSelect		(rigSelect  	-> currentText ());
	offsetSlider		-> setValue (OFFSET);
	Set_Tuner		(MyRig -> defaultFrequency (),
	                         &CurrentFreq, &TunedIF);
	setPreSelector		(CurrentFreq);
	lcd_Frequency		-> display (CurrentFreq);
	MyRig			-> displayVFO ();
}
//	
//	the function is called whenever there is an existing
//	ini file
void	RadioInterface::setControlState (QSettings *s) {
int	k;
QString	h;

	if (s == NULL)
	   return;

	k			= s -> value ("balanceSlider", 0). toInt ();
	balanceSlider		-> setValue (k);

	k			= s -> value ("attenuationSlider", 50). toInt ();
	attenuationSlider	-> setValue (k);

	h			= s -> value ("HFplotterView",
	                                        "spectrumView"). toString ();
	k			= HFplotterView -> findText (h);
	if (k != -1) {
	   HFplotterView -> setCurrentIndex (k);
	   set_HFplotterView	(h);
	}

	h			= s -> value ("LFplotterView",
	                                        "spectrumView"). toString ();
	k			= LFplotterView -> findText (h);
	if (k != -1) {
	   LFplotterView -> setCurrentIndex (k);
	   set_LFplotterView	(h);
	}

	h			= s -> value ("operationSelect", "Radio"). toString ();
	k		= operationSelect	-> findText (h);
	if (k != -1) {
	   operationSelect -> setCurrentIndex (k);
	   setOperation (h);
	}

	h		= s -> value ("inputModeSelect", "IandQ"). toString ();
	k		= inputModeSelect	-> findText (h);
	if (k != -1) {
	   inputModeSelect -> setCurrentIndex (k);
	   setInputMode (h);
	}

	h		= s -> value ("AGC_select", "AGC off"). toString ();
	k		= AGC_select		-> findText (h);
	if (k != -1) {
	   AGC_select	-> setCurrentIndex (k);
	   setAGCMode	(h);
	}

	h		= s -> value ("quickbandSelect", "usb"). toString ();
	k		= quickbandSelect 	-> findText (h);
	if (k != -1) {
	   quickbandSelect -> setCurrentIndex (k);
	   setquickBand (h);
	}


//	set_bandWidthSelect	(bandwidthSlider	-> value ());
//	set_bandWidthOffset	(bandoffsetSlider 	-> value ());

	k		= s -> value ("LFspectrumamplitudeSlider", 50). toInt ();
	LFspectrumamplitudeSlider	-> setValue (k);
	k		= s -> value ("HFspectrumamplitudeSlider", 50). toInt ();
	HFspectrumamplitudeSlider	-> setValue (k);

	k		= s -> value ("iqslider", 50). toInt ();
	iqslider	-> setValue (k);

	h		= s -> value ("workingrateSelect", 9600). toString ();
	k		= workingrateSelect -> findText (h);
	if (k != -1) {
	   workingrateSelect -> setCurrentIndex (k);
	   setWorkingRate (h);
	}

	k		= s -> value ("symbol_shifter", 0). toInt ();
	symbol_shifter		-> setValue (k);

	k		= s -> value ("agcThresholdSlider", -115). toInt ();
	agcThresholdSlider	-> setValue (k);

	h	= s -> value ("streamInSelector", "select input"). toString ();
	k	= streamInSelector -> findText (h);
	if (k != -1) {
	   streamInSelector -> setCurrentIndex (k);
	   setStreamInSelector (k);
	}

	h	= s -> value ("streamOutSelector", "select output"). toString ();
	k	= streamOutSelector -> findText (h);
	if (k != -1) {
	   streamOutSelector -> setCurrentIndex (k);
	   setStreamOutSelector (k);
	}
//
//	for each of the decoders, we set each slider to a value indicated
//	by the ini file, or to a suitable default. Note that
//	the decoders themselves are not instantiated, so take care that
//	we do not try to react on these settings
//	analog decoder:
	h		= s -> value ("analogDecoderSelect", "am"). toString ();
	k		= analogDecoderSelect -> findText (h);
	if (k != -1) 
	   analogDecoderSelect -> setCurrentIndex (k);

	k		= s -> value ("analogDecoderVolume", 10). toInt ();
	analogDecoderVolume	-> setValue (k);

	k		= s -> value ("analogDecoderLowPass", 100). toInt ();
	analogDecoderLowPass	-> setValue (k);

	k		= s -> value ("analogDecoderHighPass", 100). toInt ();
	analogDecoderHighPass	-> setValue (k);

	k		= s -> value ("squelchSlider", 0). toInt ();
	squelchSlider		-> setValue (k);

//	for the CW decoder
	k		= s -> value ("cwwpm", 32). toInt ();
	cwwpm			-> setValue (k);

	h		= s -> value ("cwTracker", "On Tracking"). toString ();
	k		= cwTracker	-> findText (h);
	if (k != -1)
	   cwTracker	-> setCurrentIndex (k);

	k		= s -> value ("cwFilterDegree", 10). toInt ();
	cwFilterDegree		-> setValue (k);

	k		= s -> value ("cwSquelchLevel", 5). toInt ();
	cwSquelchLevel		-> setValue (k);
//
//	psk decoder
	h		= s -> value ("pskAfcon", "afc off"). toString ();
	k		= pskAfcon	-> findText (h);
	if (k != -1)
	   pskAfcon	-> setCurrentIndex (k);

	h		= s -> value ("pskReverse", "normal"). toString ();
	k		= pskReverse	-> findText (h);
	if (k != -1)
	   pskReverse	-> setCurrentIndex (k);

	k		= s -> value ("pskSquelchLevel", 5). toInt ();
	pskSquelchLevel		-> setValue (k);

	h		= s -> value ("qpskSelector", "psk31"). toString ();
	k		= qpskSelector	-> findText (h);
	if (k != -1)
	   qpskSelector	-> setCurrentIndex (k);

	k		= s -> value ("pskFilterDegree", 12). toInt ();
	pskFilterDegree	-> setValue (k);

	h		= s -> value ("rigSelect", "no rig"). toString ();
	k		= rigSelect	-> findText (h);
	if (k != -1) {
	   rigSelect	-> setCurrentIndex (k);
	   set_rigSelect (h);
	}
//
//	rtty decoder
	h		= s -> value ("rttyBaudrateSelect", "45"). toString ();
	k		= rttyBaudrateSelect -> findText (h);
	if (k != -1)
	   rttyBaudrateSelect	-> setCurrentIndex (k);

	h		= s -> value ("rttyWidthSelect", "170"). toString ();
	k		= rttyWidthSelect -> findText (h);
	if (k != -1)
	   rttyWidthSelect	-> setCurrentIndex (k);

	h		= s -> value ("rttyParity", "170"). toString ();
	k		= rttyParity -> findText (h);
	if (k != -1)
	   rttyParity	-> setCurrentIndex (k);

	h		= s -> value ("rttyMsb", "msb true"). toString ();
	k		= rttyMsb -> findText (h);
	if (k != -1)
	   rttyMsb	-> setCurrentIndex (k);

	h		= s -> value ("rttyNbits", "5"). toString ();
	k		= rttyNbits -> findText (h);
	if (k != -1)
	   rttyNbits	-> setCurrentIndex (k);

	h		= s -> value ("rttyStopbits", "1.5"). toString ();
	k		= rttyStopbits -> findText (h);
	if (k != -1)
	   rttyStopbits	-> setCurrentIndex (k);

	h		= s -> value ("rttyReverse", "normal"). toString ();
	k		= rttyReverse -> findText (h);
	if (k != -1)
	   rttyReverse	-> setCurrentIndex (k);

	h		= s -> value ("rttyAfcon", "yes"). toString ();
	k		= rttyAfcon -> findText (h);
	if (k != -1)
	   rttyAfcon	-> setCurrentIndex (k);
//
//	amtor decoder
	h		= s -> value ("amtorAfcon", "afc on"). toString ();
	k		= amtorAfcon	-> findText (h);
	if (k != -1)
	   amtorAfcon	-> setCurrentIndex (k);
 
	h		= s -> value ("amtorReverse", "normal"). toString ();
	k		= amtorReverse	-> findText (h);
	if (k != -1)
	   amtorReverse	-> setCurrentIndex (k);
 
	h		= s -> value ("amtorFecError", "strict fec"). toString ();
	k		= amtorFecError	-> findText (h);
	if (k != -1)
	   amtorFecError	-> setCurrentIndex (k);

	h		= s -> value ("amtorMessage", "message"). toString ();
	k		= amtorMessage	-> findText (h);
	if (k != -1)
	   amtorMessage	-> setCurrentIndex (k);
//
//	domino decoder
	k		= s -> value ("dominoSquelch", 5). toInt ();
	dominoSquelch		-> setValue (k);

	h		= s -> value ("dominoFec", "fec"). toString ();
	k		= dominoFec	-> findText (h);
	if (k != -1)
	   dominoFec	-> setCurrentIndex (k);

	h		= s -> value ("dominoReverse", "normal"). toString ();
	k		= dominoReverse	-> findText (h);
	if (k != -1)
	   dominoReverse	-> setCurrentIndex (k);
//	
//	fax decoder
	h		= s -> value ("faxIOCSelector", "560"). toString ();
	k		= faxIOCSelector	-> findText (h);
	if (k != -1)
	   faxIOCSelector	-> setCurrentIndex (k);

	h		= s -> value ("faxModeSelector", "FM"). toString ();
	k		= faxModeSelector	-> findText (h);
	if (k != -1)
	   faxModeSelector	-> setCurrentIndex (k);

	k		= s -> value ("faxDeviationSelector", 400). toInt ();
	faxDeviationSelector	-> setValue (k);
	
	k		= s -> value ("LPMBox", 120). toInt ();
	LPMBox			-> setValue (k);

	h		= s -> value ("phaseSelector", "phase"). toString ();
	k		= phaseSelector	-> findText (h);
	if (k != -1)
	   phaseSelector	-> setCurrentIndex (k);

	h		= s -> value ("faxColorSelector", "BW"). toString ();
	k		= faxColorSelector	-> findText (h);
	if (k != -1)
	   faxColorSelector	-> setCurrentIndex (k);
	
	k		= s -> value ("faxstartFrequency", 300). toInt ();
	faxstartFrequency	-> setValue (k);

	k		= s -> value ("faxstopFrequency", 450). toInt ();
	faxstopFrequency	-> setValue (k);

	k		= s -> value ("faxCarrier", 0). toInt ();
	faxCarrier		-> setValue (k);

	h		= s -> value ("faxdecoderSelector", "fm1"). toString ();
	k		= faxdecoderSelector	-> findText (h);
	if (k != -1)
	   faxdecoderSelector	-> setCurrentIndex (k);
//
//	mfsk decoder
	h		= s -> value ("mfskMode", "mfsk 16"). toString ();
	k		= mfskMode -> findText (h);
	if (k != -1)
	   mfskMode	-> setCurrentIndex (k);

	h		= s -> value ("mfskReverse", "normal"). toString ();
	k		= mfskReverse -> findText (h);
	if (k != -1)
	   mfskReverse	-> setCurrentIndex (k);

	h		= s -> value ("mfskAfcon", "afc on"). toString ();
	k		= mfskAfcon -> findText (h);
	if (k != -1)
	   mfskAfcon	-> setCurrentIndex (k);

	k		= s -> value ("mfskSquelch", 5). toInt ();
	mfskSquelch	-> setValue (k);

	k		= s -> value ("mfskIFadjuster", 800). toInt ();
	mfskIFadjuster	-> setValue (k);
//
//	olivia decoder
	h		= s -> value ("oliviaTones", "16"). toString ();
	k		= oliviaTones	-> findText (h);
	if (k != -1)
	   oliviaTones	-> setCurrentIndex (k);

	h		= s -> value ("oliviaBW", "512"). toString ();
	k		= oliviaBW	-> findText (h);
	if (k != -1)
	   oliviaBW	-> setCurrentIndex (k);

	k		= s -> value ("oliviaSyncMargin", 10). toInt ();
	oliviaSyncMargin	-> setValue (k);

	k		= s -> value ("oliviaSyncIntegLen", 4). toInt ();
	oliviaSyncIntegLen	-> setValue (k);

	h		= s -> value ("oliviaSquelch", "squelch on"). toString ();
	k		= oliviaSquelch	-> findText (h);
	if (k != -1)
	   oliviaSquelch	-> setCurrentIndex (k);

	k		= s -> value ("oliviaSquelchvalue", 0). toInt ();
	oliviaSquelchvalue	-> setValue (k);

	h		= s -> value ("oliviaReverse", "normal"). toString ();
	k		= oliviaReverse	-> findText (h);
	if (k != -1)
	   oliviaReverse	-> setCurrentIndex (k);

	k		= s -> value ("nodetectorButton", 0). toInt ();
	nodetectorButton	-> setChecked (k != 0);

	k		= s -> value ("analogDecoderButton", 0). toInt ();
	analogDecoderButton	-> setChecked (k != 0);

	k		= s -> value ("rttyButton", 0). toInt ();
	rttyButton		-> setChecked (k != 0);

	k		= s -> value ("cwButton", 0). toInt ();
	cwButton		-> setChecked (k != 0);

	k		= s -> value ("pskButton", 0). toInt ();
	pskButton		-> setChecked (k != 0);

	k		= s -> value ("mfskButton", 0). toInt ();
	mfskButton		-> setChecked (k != 0);

	k		= s -> value ("amtorButton", 0). toInt ();
	amtorButton		-> setChecked (k != 0);

	k		= s -> value ("faxButton", 0). toInt ();
	faxButton		-> setChecked (k != 0);

	k		= s -> value ("dominoButton", 0). toInt ();
	dominoButton		-> setChecked (k != 0);

	k		= s -> value ("oliviaButton", 0). toInt ();
	oliviaButton		-> setChecked (k != 0);

	k		= s -> value ("frequencyOffset", 0). toInt ();
	offsetSlider	-> setValue (k);

	k		= s -> value ("dongleGain", 0). toInt ();
	dongleGain	-> setValue (k);

	h		= s -> value ("dongleRate", "960000"). toString ();
	k		= dongleRate	-> findText (h);
	if (k != -1)
	   dongleRate	-> setCurrentIndex (k);

	k		= s -> value ("tunedFrequency",
	                       (int)(MyRig -> defaultFrequency ()/ KHz (1))). toInt ();
	Set_Tuner	(KHz (k), &CurrentFreq, &TunedIF);
	setPreSelector	(CurrentFreq);
	lcd_Frequency	-> display (CurrentFreq);
	MyRig		-> displayVFO ();
}
//
//
//
void	RadioInterface::dumpControlState (QSettings *s) {
	if (s == NULL)
	   return;

	s	-> setValue ("balanceSlider",
	                          balanceSlider -> value ());
	s	-> setValue ("attenuationSlider",
	                          attenuationSlider -> value ());

	s	-> setValue ("HFplotterView",
	                          HFplotterView	-> currentText ());
	s	-> setValue ("LFplotterView",
	                          LFplotterView	-> currentText ());
	s	-> setValue ("operationSelect",
	                          operationSelect -> currentText ());
	s	-> setValue ("inputModeSelect",
	                          inputModeSelect -> currentText ());
	s	-> setValue ("AGC_select",
	                          AGC_select	-> currentText ());
	s	-> setValue ("quickbandSelect",
	                          quickbandSelect -> currentText ());

//	set_bandWidthSelect	(bandwidthSlider	-> value ());
//	set_bandWidthOffset	(bandoffsetSlider 	-> value ());

	s	-> setValue ("LFspectrumamplitudeSlider",
	                          LFspectrumamplitudeSlider -> value ());
	s	-> setValue ("HFspectrumamplitudeSlider",
	                          HFspectrumamplitudeSlider -> value ());
	s	-> setValue ("iqslider",
	                          iqslider	-> value ());
	s	-> setValue ("workingrateSelect",
	                          workingrateSelect -> currentText ());
	s	-> setValue ("symbol_shifter",
	                          symbol_shifter -> value ());

	s	-> setValue ("agcThresholdSlider",
	                          agcThresholdSlider -> value ());
	s	-> setValue ("streamInSelector",
	                          streamInSelector -> currentText ());
	s	-> setValue ("streamOutSelector",
	                          streamOutSelector -> currentText ());
	s	-> setValue ("rigSelect",
	                          rigSelect	-> currentText ());
	s	-> setValue ("tunedFrequency",
	                          (int)(CurrentFreq) / Khz (1));
	s	-> setValue ("frequencyOffset",
	                          offsetSlider -> value ());
	s	-> setValue ("dongleGain",
	                          dongleGain -> value ());

	s	-> setValue ("dongleRate",
	                          dongleRate -> currentText ());
//
//	we also record the settings for the decoders
//	Analog decoder:
	s	-> setValue ("analogDecoderSelect",
	                          analogDecoderSelect	-> currentText	());
	s	-> setValue ("analogDecoderVolume",
	                          analogDecoderVolume	-> value	());
	s	-> setValue ("analogDecoderLowPass",
	                          analogDecoderLowPass	-> value	());
	s	-> setValue ("analogDecoderHighPass",
	                          analogDecoderHighPass	-> value	());
//	for the CW decoder
	s	-> setValue ("cwwpm",
	                          cwwpm			-> value	());
	s	-> setValue ("cwTracker",
	                          cwTracker		-> currentText	());
	s	-> setValue ("cwFilterDegree",
	                          cwFilterDegree	-> value	());
	s	-> setValue ("cwSquelchLevel",
	                          cwSquelchLevel	-> value	());
//	for the psk decoder
	s	-> setValue ("pskAfcon",
	                          pskAfcon		-> currentText	());
	s	-> setValue ("pskReverse",
	                          pskReverse		-> currentText	());
	s	-> setValue ("pskSquelchLevel",
	                          pskSquelchLevel	-> value	());
	s	-> setValue ("qpskSelector",
	                          qpskSelector		-> currentText	());
	s	-> setValue ("pskFilterDegree",
	                          pskFilterDegree	-> value	());
	
	s	-> setValue ("rttyBaudrateSelect",
	                          rttyBaudrateSelect	-> currentText	());
	s	-> setValue ("rttyWidthSelect",
	                          rttyWidthSelect	-> currentText	());
	s	-> setValue ("rttyParity",
	                          rttyWidthSelect	-> currentText	());
	s	-> setValue ("rttyParity",
	                          rttyParity		-> currentText	());
	s	-> setValue ("rttyMsb",
	                          rttyMsb		-> currentText	());
	s	-> setValue ("rttyNbits",
	                          rttyNbits		-> currentText	());
	s	-> setValue ("rttyStopbits",
	                          rttyStopbits		-> currentText	());
	s	-> setValue ("rttyReverse",
	                          rttyReverse		-> currentText	());
	s	-> setValue ("rttyAfcon",
	                          rttyAfcon		-> currentText	());
	
	s	-> setValue ("amtorAfcon",
	                          amtorAfcon		-> currentText	());
	s	-> setValue ("amtorReverse",
	                          amtorReverse		-> currentText	());
	s	-> setValue ("amtorFecError",
	                          amtorFecError		-> currentText	());
	s	-> setValue ("amtorMessage",
	                          amtorMessage		-> currentText	());

	s	-> setValue ("dominoSquelch",
	                          dominoSquelch		-> value	());
	s	-> setValue ("dominoFec",
	                          dominoFec		-> currentText	());
	s	-> setValue ("dominoReverse",
	                          dominoReverse		-> currentText	());

	s	-> setValue ("faxIOCSelector",
	                          faxIOCSelector	-> currentText	());
	s	-> setValue ("faxModeSelector",
	                          faxModeSelector	-> currentText	());
	s	-> setValue ("faxDeviationSelector",
	                          faxDeviationSelector	-> value	());
	s	-> setValue ("LPMBox",
	                          LPMBox		-> value	());
	s	-> setValue ("phaseSelector",
	                          phaseSelector		-> currentText	());
	s	-> setValue ("faxColorSelector",
	                          faxColorSelector	-> currentText	());
	s	-> setValue ("faxstartFrequency",
	                          faxstartFrequency	-> value	());
	s	-> setValue ("faxstopFrequency",
	                          faxstopFrequency	-> value	());
	s	-> setValue ("faxCarrier",
	                          faxCarrier		-> value	());
	s	-> setValue ("faxdecoderDelector",
	                          faxdecoderSelector	-> currentText	());

	s	-> setValue ("mfskMode",
	                          mfskMode		-> currentText	());
	s	-> setValue ("mfskReverse",
	                          mfskReverse		-> currentText	());
	s	-> setValue ("mfskAfcon",
	                          mfskAfcon		-> currentText	());
	s	-> setValue ("mfskSquelch",
	                          mfskSquelch		-> value	());
	s	-> setValue ("mfskIFadjuster",
	                          mfskIFadjuster	-> value	());

	s	-> setValue ("oliviaTones",
	                          oliviaTones		-> currentText	());
	s	-> setValue ("oliviaBW",
	                          oliviaBW		-> currentText	());
	s	-> setValue ("oliviaSyncMargin",
	                          oliviaSyncMargin	-> value	());
	s	-> setValue ("oliviaSyncIntegLen",
	                          oliviaSyncIntegLen	-> value	());
	
	s	-> setValue ("oliviaSquelch",
	                          oliviaSquelch		-> currentText	());
	s	-> setValue ("oliviaSquelchvalue",
	                          oliviaSquelchvalue	-> value	());
	s	-> setValue ("oliviaReverse",
	                          oliviaReverse		-> currentText	());
//
//	and we write out the selected decoder
	s	-> setValue ("nodetectorButton",
	                          nodetectorButton -> isChecked () ? 1 : 0);
	s	-> setValue ("analogDecoderButton",
	                          analogDecoderButton -> isChecked () ? 1 : 0);
	s	-> setValue ("rttyButton",
	                          rttyButton -> isChecked () ? 1 : 0);
	s	-> setValue ("cwButton",
	                          cwButton -> isChecked () ? 1 : 0);
	s	-> setValue ("pskButton",
	                          pskButton -> isChecked () ? 1 : 0);
	s	-> setValue ("mfskButton",
	                          mfskButton -> isChecked () ? 1 : 0);
	s	-> setValue ("amtorButton",
	                          amtorButton -> isChecked () ? 1 : 0);
	s	-> setValue ("faxButton",
	                          faxButton -> isChecked () ? 1 : 0);
	s	-> setValue ("dominoButton",
	                          dominoButton -> isChecked () ? 1 : 0);
	s	-> setValue ("oliviaButton",
	                          oliviaButton -> isChecked () ? 1 : 0);
}
//

void	RadioInterface::clickPause (void) {

	if (runMode == IDLE)
	   return;

	if (runMode == RUNNING) {		// setting to Pause
	   if (MyRig 	-> RigprovidesSamples ())
	      MyRig	-> stopReader ();	
	   else
	      MySoundcardIn 	-> stopReader ();

	   MySoundcardOut	-> stopWriter ();
	   pauseButton -> setText (QString ("Continue"));
	   runMode = PAUSED;
	}
	else 
	if (runMode == PAUSED) {
	   if (MyRig		-> RigprovidesSamples ())
	      MyRig		-> restartReader ();
	   else
	      MySoundcardIn	-> restartReader ();
	   MySoundcardOut	-> restartWriter ();
	   pauseButton -> setText (QString ("Pause"));
	   runMode = RUNNING;
	}
}
/*
 *	In setStart we start (or restart) the samplestream
 */
void	RadioInterface::setStart	(void) {
bool	r = 0;
//
	if (runMode == RUNNING)
	   return;

//	always ensure that the datastream stops
	if (MyRig -> RigprovidesSamples ())
	   MyRig		-> stopReader ();	
	else
	   MySoundcardIn 	-> stopReader ();
	MySoundcardOut	-> stopWriter ();

	if (MyRig	-> RigprovidesSamples ())
	   disconnect (MyRig,
	               SIGNAL (samplesAvailable ()),
	               this,
	               SLOT (JansRadioThroughSignal ()));
	else
	   disconnect (MySoundcardIn,
	               SIGNAL (samplesAvailable ()),
	               this,
	               SLOT (JansRadioThroughSignal ()));
//
//	
	if (MyRig -> RigprovidesSamples ())
	   r = MyRig	-> restartReader ();
	else
	   r = MySoundcardIn -> restartReader ();

	qDebug ("Starting %d\n", r);
	if (!r) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Opening  input stream failed\n"));
	   return;
	}

	r = MySoundcardOut	-> restartWriter ();
	if (!r) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Opening  output stream failed\n"));
	   if (MyRig -> RigprovidesSamples ())
	      MyRig	-> stopReader ();
	   else
	      MySoundcardIn -> stopReader ();
	   return;
	}

	if (MyRig -> RigprovidesSamples ())
	   connect (MyRig,
	            SIGNAL (samplesAvailable ()),
	            this,
	            SLOT (JansRadioThroughSignal ()));
	else
	   connect (MySoundcardIn,
	            SIGNAL (samplesAvailable ()),
	            this,
	            SLOT (JansRadioThroughSignal ()));
//
//	and finally: recall that starting overrules pausing
	pauseButton -> setText (QString ("Pause"));
	runMode = RUNNING;
}

void	RadioInterface::TerminateProcess (void) {
	if (DumpisOn)
	   setDumpisOff ();
//
//	save some slider values

	if (ISettings != NULL)
	   dumpControlState (ISettings);

	MySoundcardOut	-> stopWriter ();
	if (MyRig	-> RigprovidesSamples ())
	   MyRig -> stopReader ();
	else
	   MySoundcardIn -> stopReader ();

	accept ();
	qDebug () <<  "End of termination procedure";
}

void	RadioInterface::abortSystem (int d) {
	qDebug ("aborting for reason %d\n", d);
	accept ();
}

void	RadioInterface::setWorkingRate (const QString &s) {
int32_t	newWorkingRate	= (int32_t)s. toInt ();

	if (newWorkingRate == _WorkingRate)
	   return;

	_WorkingRate		= newWorkingRate;

	delete	myDecimator;
	myDecimator		= new downDecimator
	                                         (_ICardRate, _WorkingRate);

	upCount			= (int32_t)_OCardRate / _WorkingRate;
	MyFrontEnd		-> setWorkingRate (_WorkingRate);
	agc			-> setSamplerate (_WorkingRate);

	disconnect (LFScope,
	            SIGNAL (clickedwithLeft (int)),
	            this,
	            SLOT (AdjustFrequency (int)));
	delete	LFScope;
	LFScope			= new fft_scope (lfscope,
	                                         displaywidth,
	                                         1,
	                                         rastersize,
	                                         SPECTRUM_SIZE_BACK,
	                                         _WorkingRate,
	                                         REPEAT_RATE);
	connect (LFScope,
	         SIGNAL (clickedwithLeft (int)),
	         this,
	         SLOT (AdjustFrequency (int)));
	LFScope	-> setNeedle (0);
	set_LFplotterView	(LFplotterView		-> currentText ());

	setquickBand		(quickbandSelect	-> currentText ());
	Set_Tuner		(CurrentFreq, &CurrentFreq, &TunedIF);
	delete BackendShifter;
	BackendShifter		= new Oscillator (_WorkingRate);

	MyDetector		-> setWorkingRate (_WorkingRate);
	delete		upFilter;
	upFilter		= new LowPassIIR (10,
	                                          (int32_t)_WorkingRate / 2,
	                                          (int32_t)_OCardRate,
	                                          S_BUTTERWORTH);
/*
 *	if we are running in backend mode, we also need an adapted
 *	top scope
 */
	if ((FrontEndoperation == BACKEND) &&
	    (BackendScope != NULL)) {
	   disconnect (BackendScope,
	               SIGNAL (clickedwithLeft (int)),
	               this,
	               SLOT (AdjustFrequency (int)));
	   delete BackendScope;
	   BackendScope		= new fft_scope (hfscope,
	                                         displaywidth,
	                                         1,
	                                         rastersize,
	                                         SPECTRUM_SIZE_BACK,
	                                         _WorkingRate,
	                                         REPEAT_RATE);
	   connect (BackendScope,
	            SIGNAL (clickedwithLeft (int)),
	            this,
	            SLOT (AdjustFrequency (int)));
	   BackendScope		-> setNeedle (0);
	   set_HFplotterView	(HFplotterView	-> currentText ());
	   TunedIF			= 0;
	}

}

void	RadioInterface::setDetectorMarker (int foi) {
	LFScope	-> setNeedle (foi);
}

void RadioInterface::setOperation (const QString &s) {
uint8_t	newOp = (s == "Radio") ? RADIO : BACKEND;

	if (newOp == FrontEndoperation)
	   return;

	FrontEndoperation = newOp;
	if (newOp == RADIO) {
	   int32_t vfo;
	   if (BackendScope != NULL) {
	      disconnect (BackendScope,
	                  SIGNAL (clickedwithLeft (int)),
	                  this,
	                  SLOT (AdjustFrequency (int)));
	      delete BackendScope;
	      BackendScope = NULL;
	   }
	   HFScope		= new fft_scope (hfscope,
	                                         displaywidth,
	                                         1000,
	                                         rastersize,
	                                         SPECTRUM_SIZE_FRONT,
	                                         _ICardRate,
	                                         REPEAT_RATE);
	   connect (HFScope,
	            SIGNAL (clickedwithLeft (int)),
	            this,
	            SLOT (AdjustFrequencywithKHz (int)));
	   MyRig	-> getVFOFrequency (&vfo);
	   HFScope	-> setZero (vfo);
	   TunedIF	= _ICardRate / 4;
	   CurrentFreq	= vfo + TunedIF;
	   HFScope	-> setNeedle (TunedIF);
	   lcd_Frequency	-> display (CurrentFreq);
	   setPreSelector	(CurrentFreq);
	}
	else  {		// FrontEndOperation == BACKEND
	   if (HFScope != NULL) {
	      disconnect (HFScope,
	                  SIGNAL (clickedwithLeft (int)),
	                  this,
	                  SLOT (AdjustFrequencywithKHz (int)));
	      delete HFScope;
	      HFScope = NULL;
	   }
	   BackendScope		= new fft_scope (hfscope,
	                                         displaywidth,
	                                         1,
	                                         rastersize,
	                                         SPECTRUM_SIZE_BACK,
	                                         _WorkingRate,
	                                         REPEAT_RATE);
	   connect (BackendScope,
	            SIGNAL (clickedwithLeft (int)),
	            this,
	            SLOT (AdjustFrequency (int)));
	   TunedIF		= 0;
	   BackendScope	-> setNeedle (TunedIF);
	   lcd_Frequency	-> display (TunedIF);
	}

	set_HFplotterView	(HFplotterView		-> currentText ());
}

void	RadioInterface::setInputMode (const QString &s) {
	if (s == "I and Q")
	   inputMode	= IandQ;
	else
	if (s == "Q and I")
	   inputMode	= QandI;
	else
	if (s == "I Only")
	   inputMode	= I_Only;
	else
	if (s == "Q Only")
	   inputMode	= Q_Only;
	else
	   inputMode	= IandQ;
}

void	RadioInterface::set_analyzer (bool checked) {
	if (!checked)
	   delete (sdrDecoder *)MyDetector;
	if (checked) {
	   MyDetector = new sdrDecoder (_WorkingRate, this);
	   setDetectorScreen (SDR_DECODER);
	}
}

void	RadioInterface::set_analogDecoder (bool checked) {
	if (!checked) {
	   unsetAnalogSliders	();
	   delete (analogDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new analogDecoder (_WorkingRate, this);
	   setDetectorScreen (ANALOG_DETECTOR);
	   setAnalogSliders	();
	}
}

void	RadioInterface::set_cw (bool checked) {
	if (!checked) {
	   unsetCwSliders	();
	   delete (cwDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new cwDecoder (_WorkingRate, this);
	   setDetectorScreen (CW_DETECTOR);
	   setCwSliders		();
	}
}

void	RadioInterface::set_rtty (bool checked) {
	if (!checked) {
	   unsetRttySliders	();
	   delete (rttyDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new rttyDecoder (_WorkingRate, this);
	   setDetectorScreen (RTTY_DETECTOR);
	   setRttySliders	();
	}
}

void	RadioInterface::set_psk (bool checked) {
	if (!checked) {
	   setPskSliders	();
	   delete (pskDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new pskDecoder (_WorkingRate, this);
	   setDetectorScreen (PSK_DETECTOR);
	   setPskSliders	();
	}
}

void	RadioInterface::set_mfsk (bool checked) {
	if (!checked) {
	   setMfskSliders	();
	   delete (mfskDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new mfskDecoder (_WorkingRate, this);
	   setDetectorScreen (MFSK_DETECTOR);
	   setMfskSliders	();
	}
}

void	RadioInterface::set_amtor (bool checked) {
	if (!checked) {
	   setAmtorSliders	();
	   delete (amtorDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new amtorDecoder (_WorkingRate, this);
	   setDetectorScreen (AMTOR_DETECTOR);
	   setAmtorSliders	();
	}
}

void	RadioInterface::set_fax (bool checked) {
	if (!checked) {
	   unsetFaxSliders	();
	   delete (faxDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new faxDecoder (_WorkingRate, this);
	   setDetectorScreen (FAX_DETECTOR);
	   setFaxSliders	();
	}
}

void	RadioInterface::set_domino (bool checked) {
	if (!checked) {
	   setDominoSliders	();
	   delete (dominoDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new dominoDecoder (_WorkingRate, this);
	   setDetectorScreen (DOMINO_DETECTOR);
	   setDominoSliders	();
	}
}

void	RadioInterface::set_olivia (bool checked) {
	if (!checked) {
	   setOliviaSliders	();
	   delete (oliviaDecoder *)MyDetector;
	}
	if (checked) {
	   MyDetector = new oliviaDecoder (_WorkingRate, this);
	   setDetectorScreen (OLIVIA_DETECTOR);
	   setOliviaSliders	();
	}
}

void	RadioInterface::setAGCMode (const QString &s) {
uint8_t	gainControl;

	if (s == "off AGC")
	   gainControl	= agcHandler::AGC_OFF;
	else
	if (s == "slow")
	   gainControl	= agcHandler::AGC_SLOW;
	else
	   gainControl	= agcHandler::AGC_FAST;
	agc	-> setMode (gainControl);
}

void	RadioInterface::set_agcThresholdSlider (int v) {
	agc	-> setThreshold (v);
}

void	RadioInterface::setAttenuator (int n) {
	attenuationLevelDisplay	-> display (n);
	currAttSliderValue	= 10 * n;
	attValueL	= currAttSliderValue * (float)Bl / 100;
	attValueR	= currAttSliderValue * (float)Br / 100;
	agc	-> setLevel ((int)(attValueL + attValueR) / 2);
}

void	RadioInterface::set_HFspectrumAmplitude (int n) {
	if (HFScope != NULL)
	   HFScope	-> setAmplification (n);
	if (BackendScope != NULL)
	   BackendScope -> setAmplification (n);
}

void	RadioInterface::set_LFspectrumAmplitude (int n) {
	LFScope	-> setAmplification (n);
}
/*
 * 	the balance slider runs between 0 .. 30
 */
void	RadioInterface::setBalanceSlider (int n) {
	IQBalanceDisplay -> display (n);
	Bl		= 100 - n;
	Br		= 100 + n;
	attValueL	= currAttSliderValue * (float)Bl / 100;
	attValueR	= currAttSliderValue * (float)Br / 100;
	agc	-> setLevel ((int)(attValueL + attValueR) / 2);
}
/*
 * 	Handling the numeric keypad is boring, but needed
 */
void RadioInterface::addOne() {
	AddtoPanel (1);
}

void RadioInterface::addTwo() {
	AddtoPanel (2);
}

void RadioInterface::addThree() {
	AddtoPanel (3);
}

void RadioInterface::addFour() {
	AddtoPanel (4);
}

void RadioInterface::addFive() {
	AddtoPanel (5);
}

void RadioInterface::addSix() {
	AddtoPanel (6);
}

void RadioInterface::addSeven() {
	AddtoPanel (7);
}

void RadioInterface::addEight() {
	AddtoPanel (8);
}

void RadioInterface::addNine() {
	AddtoPanel (9);
}

void RadioInterface::addZero() {
	AddtoPanel (0);
}

void RadioInterface::addClear() {
	ClearPanel ();
	if (lcd_timer	-> isActive ())
	   lcd_timer -> stop ();
	lcd_Frequency -> display (CurrentFreq);
}

void RadioInterface::AcceptFreqinHz() {
int32_t	p;

	if (!lcd_timer -> isActive ())
	   return;
	lcd_timer	-> stop ();
	p 		= getPanel ();
	ClearPanel	();
	Set_Tuner 	(p, &CurrentFreq, &TunedIF);
	setPreSelector	(CurrentFreq);
	lcd_Frequency	-> display (CurrentFreq);
	MyRig		-> displayVFO ();
}

void RadioInterface::AcceptFreqinKhz () {
int32_t	p;

	if (!lcd_timer -> isActive ())
	   return;
	lcd_timer -> stop ();
	p = getPanel () * 1000;
	ClearPanel ();
	Set_Tuner	(p, &CurrentFreq, &TunedIF);
	setPreSelector	(CurrentFreq);
	lcd_Frequency	-> display (CurrentFreq);
	MyRig		-> displayVFO ();
}

void RadioInterface::addCorr() {
	if (lcd_timer -> isActive ()) {
	   lcd_timer	-> stop ();
	   Panel = (Panel - (Panel % 10)) / 10;
	   lcd_Frequency -> display ((int)Panel);
	   lcd_timer	-> start (5000);		// restart timer
	}
}

void	RadioInterface::AddtoPanel (int n) {
	if (lcd_timer -> isActive ())
	   lcd_timer -> stop ();
	   
	Panel = 10 * Panel + n;
	lcd_Frequency	-> display ((int)Panel);
	lcd_timer	-> start (5000);		// restart timer
}

void	RadioInterface::ClearPanel () {
	Panel = 0;
}

int32_t	RadioInterface::getPanel () {
	return Panel;
}

void RadioInterface::decT5() {
	AdjustFrequency (-1);
}

void RadioInterface::decT50() {
	AdjustFrequency (-10);
}

void RadioInterface::decT500() {
	AdjustFrequency (-100);
}

void RadioInterface::decT5000() {
	AdjustFrequency (-1000);
}

void RadioInterface::incT5() {
	AdjustFrequency (1);
}

void RadioInterface::incT50() {
	AdjustFrequency (10);
}

void RadioInterface::incT500() {
	AdjustFrequency (100);
}

void RadioInterface::incT5000() {
	AdjustFrequency (1000);
}

void	RadioInterface::AdjustFrequency (int32_t n) {
	if (lcd_timer	-> isActive ()) {
	   ClearPanel ();
	   lcd_timer -> stop ();
	}

	if (FrontEndoperation == RADIO) {
	   Inc_Tuner (n);
	}
	else {
	   IncrementFrequencyforBackend (n);
	   lcd_Frequency	-> display ((int)TunedIF);
	}
}

void	RadioInterface::AdjustFrequencywithKHz (int n) {
	AdjustFrequency (n * 1000);
}

void	RadioInterface::Tuneto (int32_t freq,
	                        int32_t offset,
	                        int32_t low,
	                        int32_t high) {

	if (low < - (int32_t)_WorkingRate / 2)
	   low = -  (int32_t)_WorkingRate / 2;
	if (high >  (int32_t)_WorkingRate / 2)
	   high = (int32_t)_WorkingRate / 2;

	bandOffset	= offset;
	bandLowend	= low;
	bandHighend	= high;
	MyFrontEnd		-> setSelectedBandto (freq, low, high);
	bandoffsetDisplay	-> display (offset);
	bandwidthDisplay	-> display (high - low);
}

uint8_t	RadioInterface::FitsInWindow (int32_t n) {
	return -(int32_t) _ICardRate / 2  < n - (int32_t) _WorkingRate / 2  && 
	                n + (int32_t) _WorkingRate / 2 < (int32_t)_ICardRate / 2;
}

void	RadioInterface::setInMiddle (void) {

	Set_Tuner	(CurrentFreq, &CurrentFreq, &TunedIF);
	lcd_Frequency	-> display (CurrentFreq);
	MyRig		-> displayVFO ();
}
//
//	for handling frequencies using the third harmonic
//	of the pmsdr kit we have:
bool	RadioInterface::RigwithThirdHarmonic (virtualRig *rig) {
	return rig -> isNO_RIG () || rig -> isPMSDR ();
}
//
//
void	RadioInterface::Set_TunerThirdHarmonic (int32_t n,
	                                        int32_t *c, int32_t *f) {
int32_t h, vfo;

	if (!MyRig -> legalFrequency (n / 3)) {
	   *c	= CurrentFreq;
	   *f	= TunedIF;	
	}

	*f	= (int32_t)_ICardRate / 4;	// distance to VFO
	h		= n / 3 - *f / 3;
//
	MyRig		-> setVFOFrequency (h);
	MyRig		-> getVFOFrequency (&vfo);

//	h is the vfo we wanted, "vfo" the one we get
//	This means we have to adjust the TunedIF
//	But then we have to ensure that it fits in the window
	*f		= *f + (h - vfo);
	if (!FitsInWindow (*f + bandLowend) ||
	    !FitsInWindow (*f + bandHighend)) 
	   *f	= 0;
	HFScope		-> setZero (3 * vfo);
	HFScope		-> setNeedle (*f);
	MyFrontEnd	-> setSelectedFrequencyto (*f,
	                                           bandLowend, bandHighend);
	*c		= n;
}
//
//	OK, this is the N-th revision of Set_Tuner, 
//	The external operation is to set two variables,
//	the CurrentFreq and the TunedIF
//	Internally, we hope that the rig VFO will also be set
void	RadioInterface::Set_Tuner (int32_t n, int32_t *c, int32_t *f) {
int32_t	h;
int32_t	vfo;
/*
 *	we assume that the new frequency chosen is to be
 *	displayed in the middle of the right half window.
 *	The elektor pll approximates the frequencies, it sometimes
 *	deviates up to a few thousand Hz from the
 *	requested frequency. The si570 is more precise
 */
//
	if (!FrequencyChangeAllowed) {
	   *c	= CurrentFreq;
	   *f	= TunedIF;
	}

	if ((FrontEndoperation != RADIO) || (HFScope == NULL)) {
	   *c	= CurrentFreq;
	   *f	= TunedIF;
	}

	IandQ_switch = false;
	if (RigwithThirdHarmonic (MyRig) && n > Mhz (55)) {
	   IandQ_switch = true;
	   Set_TunerThirdHarmonic (n, c, f);
	   return;
	}

	if (!MyRig -> legalFrequency (n)) {
	   *c	= CurrentFreq;
	   *f	= TunedIF;
	}

	*f	= (int32_t)_ICardRate / 4;	// distance to VFO
	h		= n - *f;
//
	MyRig		-> setVFOFrequency (h);
	MyRig		-> getVFOFrequency (&vfo);
//	h is the vfo we wanted, "vfo" the one we get
//	This means we have to adjust the TunedIF
//	But then we have to ensure that it fits in the window
	*f		= *f + (h - vfo);
	if (!FitsInWindow (*f + bandLowend) ||
	    !FitsInWindow (*f + bandHighend)) 
	   *f	= 0;
	HFScope		-> setZero (vfo);
	HFScope		-> setNeedle (*f);
	MyFrontEnd	-> setSelectedFrequencyto (*f,
	                                           bandLowend, bandHighend);
	*c	= vfo + *f;
}
/*
 * 	Handle the requests for a slight frequency change, one
 * 	on the same screen.
 *	In case we cannot handle the change within the current window
 *	we  will set the tuner as well.
 */
void	RadioInterface::IncrementFrequencyforBackend (int32_t amount) {
	if ((FrontEndoperation != BACKEND) || (BackendScope == NULL))
	   return;

	TunedIF	+= amount;
	if ((TunedIF <= -_WorkingRate / 2) ||
	   (TunedIF >= _WorkingRate / 2)) {
	   TunedIF = 0;
	}

	BackendScope		-> setNeedle (TunedIF);
}

void	RadioInterface::Inc_Tuner (int32_t amount) {

	if ((FrontEndoperation != RADIO) || (HFScope == NULL))
	   return;
/*
 *	if the increment causes the freq to be outside the window,
 *	just set the tuner
 */
	if (!FitsInWindow (TunedIF + bandLowend + amount) ||
	    !FitsInWindow (TunedIF + bandHighend + amount)) {
	   Set_Tuner (CurrentFreq + amount, &CurrentFreq, &TunedIF);
	   setPreSelector	(CurrentFreq);
	   lcd_Frequency	-> display (CurrentFreq);
	   MyRig		-> displayVFO ();
	   return;
	}
/*
 *	we are  fitting within the current window,
 *	so the VFO does not change
 */
	TunedIF		+= amount;
	HFScope		-> setNeedle (TunedIF);
	CurrentFreq	+= amount;
	setPreSelector (CurrentFreq);
	lcd_Frequency	-> display (CurrentFreq);
	MyRig		-> displayVFO ();
	MyFrontEnd	->  setSelectedFrequencyto (TunedIF, bandLowend, bandHighend);
}

//
//	The offset slider is influencing the righandling,
//	hardly visible here
void	RadioInterface::set_offsetSlider (int s) {
	if (MyRig != NULL) {
	   MyRig -> setOffset (s);
	   Set_Tuner (CurrentFreq, &CurrentFreq, &TunedIF);
	}
}


void	RadioInterface::setPreSelector (int32_t freq) {
	if (CurrentRig != PMSDR)
	   return;

	if ((MyRig -> legalFrequency (freq)) &&
	    (PMSDR_preSelectorMode == AUTOMATIC)) {
	   int16_t	band;
	   ((pmSDR *)MyRig)	-> setFilter (freq, &band);
	   PMSDR_bandlabel -> setText (((pmSDR *)MyRig) -> getFilterRange (band));
	   return;
	}
/*
 *	The inevitable default
 */
	((pmSDR *)MyRig) -> setFilter (0);
	PMSDR_bandlabel -> setText (((pmSDR *)MyRig) -> getFilterRange (0));
}

void	RadioInterface::set_HFplotterView (const QString &s) {
	if (s == "waterfallView") {
	   HFViewMode = WATERFALL_MODE;
	}
	else {	// s == "spectrumView"
	   HFViewMode = SPECTRUM_MODE;
	}

	if (HFScope != NULL)
	   HFScope	-> SelectView (HFViewMode);
	else
	if (BackendScope != NULL)
	   BackendScope -> SelectView (HFViewMode);
}

void	RadioInterface::set_LFplotterView (const QString &s) {
	if (s == "waterfallView") {
	   LFViewMode = WATERFALL_MODE;
	}
	else {	// s == "spectrum"
	   LFViewMode = SPECTRUM_MODE;
	}

	LFScope	-> SelectView (LFViewMode);
}
/*
 *	... and now we manage the frequency select and the passband here.
 *	n is an integer range 0 .. 1000
 */
void	RadioInterface::set_bandWidthSelect (int n) {
int32_t	offset	= bandoffsetSlider -> value () * _WorkingRate / 1000;
int32_t	low	= offset - n * _WorkingRate / 1000 / 2;
int32_t	high	= offset + n * _WorkingRate / 1000 / 2;
	if (FrontEndoperation != RADIO)
	   return;
	bandwidthDisplay	-> display (high - low);
	bandoffsetDisplay	-> display (offset);
	Tuneto	(TunedIF, offset, low, high);
}
/*
 * 	n is an integer range -500 .. 500
 */
void	RadioInterface::set_bandWidthOffset (int n) {
int32_t	offset	= n * (int32_t)_WorkingRate / 1000;
int32_t	k	= bandwidthSlider	-> value ();
int32_t	low	= offset - k * (int32_t)_WorkingRate / 1000 / 2;
int32_t	high	= offset + k * (int32_t)_WorkingRate / 1000 / 2;
	if (FrontEndoperation != RADIO)
	   return;

	bandwidthDisplay	-> display (high - low);
	bandoffsetDisplay	-> display (offset);
	Tuneto (TunedIF, offset, low, high);
}

void	RadioInterface::setquickBand (const QString &s) {
	if (FrontEndoperation != RADIO)
	   return;

	if (s == "am") {
	   int n	= 9000 * 1000 / _WorkingRate;
	   bandwidthSlider		-> setValue (n);
	   set_bandWidthSelect		(n);
	   bandoffsetSlider		-> setValue (0 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(0 * 1000 / _WorkingRate);
	}
	else 
	if (s == "usb") {
	   int n	= 2500 * 1000 / _WorkingRate;
	   bandwidthSlider		-> setValue (n);
	   set_bandWidthSelect		(n);
	   bandoffsetSlider		-> setValue (1250 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(1250 * 1000 / _WorkingRate);
	}
	else
	if (s == "lsb") {		/* mode = lsb	*/
	   int	bw, bs;
	   bw = 2500 * 1000 / (int32_t)_WorkingRate;
	   bs = -1250 * 1000 / (int32_t)_WorkingRate;
	   bandwidthSlider		-> setValue (bw);
	   set_bandWidthSelect		(bw);
	   bandoffsetSlider		-> setValue (bs);
	   set_bandWidthOffset		(bs);
	}
	else
	if (s == "1000") {
	   bandwidthSlider		-> setValue (1000 * 1000 / _WorkingRate);
	   set_bandWidthSelect		(1000 * 1000 / _WorkingRate);
	   bandoffsetSlider		-> setValue (0 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(0 * 1000 / _WorkingRate);
	}
	else
	if (s == "300") {
	   bandwidthSlider		-> setValue (300 * 1000 / _WorkingRate);
	   set_bandWidthSelect		(300 * 1000 / _WorkingRate);
	   bandoffsetSlider		-> setValue (0 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(0 * 1000 / _WorkingRate);
	}
	else
	if (s == "600") {
	   bandwidthSlider		-> setValue (600 * 1000 / _WorkingRate);
	   set_bandWidthSelect		(600 * 1000 / _WorkingRate);
	   bandoffsetSlider		-> setValue (0 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(0 * 1000 / _WorkingRate);
	}
	else
	if (s == "1500") {
	   bandwidthSlider		-> setValue (1500 * 1000 / _WorkingRate);
	   set_bandWidthSelect		(1500 * 1000 / _WorkingRate);
	   bandoffsetSlider		-> setValue (0 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(0 * 1000 / _WorkingRate);
	}
	else
	if (s == "2000") {
	   bandwidthSlider		-> setValue (2000 * 1000 / _WorkingRate);
	   set_bandWidthSelect		(2000 * 1000 / _WorkingRate);
	   bandoffsetSlider		-> setValue (0 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(0 * 1000 / _WorkingRate);
	}
	else
	if (s == "2500") {
	   bandwidthSlider		-> setValue (2500 * 1000 / _WorkingRate);
	   set_bandWidthSelect		(2500 * 1000 / _WorkingRate);
	   bandoffsetSlider		-> setValue (0 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(0 * 1000 / _WorkingRate);
	}
	else {				/* mode = small	*/
	   bandwidthSlider		-> setValue (500 * 1000 / _WorkingRate);
	   set_bandWidthSelect		(500 * 1000 / _WorkingRate);
	   bandoffsetSlider		-> setValue (0 * 1000 / _WorkingRate);
	   set_bandWidthOffset		(0 * 1000 / _WorkingRate);
	}
}

void	RadioInterface::setrigControlScreen (int8_t n) {
int8_t	index = 0;

	switch (n) {
	   case DABSTICK:
	   case DABSTICK_DLL:
	      index = 4;
	      break;
	      
	   case EXTIO:
	      index = 3;
	      break;

	   case NO_RIG:
	   default:
	      index = 2;
	      break;

	   case ELEKTOR:
	      index = 1;
	      break;

	   case PMSDR:
	      index = 0;
	      break;
	}

	rigControl -> setCurrentIndex ((int)index);
}

void	RadioInterface::setDetectorScreen (int8_t n) {
int8_t	index	= 0;

	switch (n) {
	   case ANALOG_DETECTOR:
	      index = 1;
	      break;

	   case PSK_DETECTOR:
	      index = 2;
	      break;

	   case RTTY_DETECTOR:
	      index = 3;
	      break;

	   case CW_DETECTOR:
	      index = 4;
	      break;

	   case MFSK_DETECTOR:
	      index = 5;
	      break;

	   case AMTOR_DETECTOR:
	      index = 6;	
	      break;

	   default:
	   case SDR_DECODER:
	      index = 7;
	      break;

	   case FAX_DETECTOR:
	      index = 8;
	      break;

	   case DOMINO_DETECTOR:
	      index = 9;
	      break;

	   case OLIVIA_DETECTOR:
	      index = 10;
	      break;
	}

	decoderDisplays -> setCurrentIndex ((int)index);
}

void	RadioInterface::updateTimeDisplay () {
QDateTime	currentTime = QDateTime::currentDateTime ();

	timeDisplay	-> setText (currentTime. toString (QString ("dd.MM.yy:hh:mm:ss")));
}

void	RadioInterface::lcd_timeout () {
	Panel		= 0;			// throw away anything
	lcd_Frequency	-> display (CurrentFreq);
}

void	RadioInterface::addtoIQplot (double x, double y) {
	IQScope -> DisplayIQ (DSPCOMPLEX (x, y), (float)IQDisplay_Ampl / 10);
}

void	RadioInterface::IQDisplay_setAmpl (int d) {
	IQDisplay_Ampl = d;
}

void	RadioInterface::setStreamInSelector (int idx) {
	if (idx == 0)
	   return;

	inputDevice = inTable [idx];
	fprintf (stderr, "with %d going for inputDevice %d\n", idx, inputDevice);

//	if (!MySoundcardIn -> isValidDevice (inputDevice)) 
//	   return;
//
	MySoundcardOut	-> stopWriter	();
	MySoundcardIn	-> stopReader	();	// may be superfluous
	if (!MySoundcardIn -> selectDevice (inputDevice)) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Selecting  input stream failed\n"));
	   return;
	}
	qWarning () << "selected input device " <<idx << inputDevice;
}

void	RadioInterface::setStreamOutSelector (int idx) {
	if (idx == 0)
	   return;

	outputDevice = outTable [idx];
	if (!MySoundcardIn -> isValidDevice (outputDevice)) 
	   return;

	MySoundcardOut	-> stopWriter	();
	if (!MySoundcardOut -> selectDevice (outputDevice)) {
	   QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Selecting  input stream failed\n"));
	   return;
	}

	qWarning () << "selected output device " << idx << outputDevice;
}
//
//	do not forget that icnt and ocnt start with 1, due
//	to Qt list conventions
bool	RadioInterface::setupSoundOut (QComboBox	*streamOutSelector,
	                               paWriter		*MySoundcardOut,
	                               int32_t		cardRate,
	                               int16_t		*table) {
uint16_t	ocnt	= 1;
uint16_t	i;

	for (i = 0; i < MySoundcardOut -> numberofDevices (); i ++) {
	   qDebug ("Investigating Device %d\n", i);
	   const char *so =
	             MySoundcardOut -> outputChannelwithRate (i, cardRate);

	   if (so != NULL) {
	      streamOutSelector -> insertItem (ocnt, so, QVariant (i));
	      table [ocnt] = i;
	      qDebug (" (output):item %d wordt stream %d\n", ocnt , i);
	      ocnt ++;
	   }
	}

	qDebug () << "added items to combobox";
	return ocnt > 1;
}

bool	RadioInterface::setupSoundIn (QComboBox	*streamInSelector,
	                              paReader	*MySoundcardIn,
	                              int32_t	cardRate,
	                              int16_t	*table) {
int16_t		icnt	= 1;
int16_t		i;

	for (i = 0; i < MySoundcardIn ->  numberofDevices (); i ++) {
	   qDebug ("Investigating Device %d\n", i);
	   const char *si =
	             MySoundcardIn -> inputChannelwithRate (i, cardRate);

	   if (si != NULL) {
              streamInSelector -> insertItem (icnt, si, QVariant (i));	
	      table [icnt] = i;
	      qDebug (" (input): item %d wordt stream %d (%s) \n", 
	                        icnt, i, si);
	      icnt ++;
	   }
	}

	qDebug () << "added items to combobox";
	return icnt > 1;
}

void	RadioInterface::setDumpisOn (void) {
SF_INFO	* sf_info	= (SF_INFO *) alloca (sizeof (SF_INFO));
	if (DumpisOn)
	   return;

	QString file = QFileDialog::getSaveFileName (this,
	                                        tr ("open file .."),
	                                        QDir::homePath (),
	                                        tr ("Sound (*.wav)"));
	sf_info		-> samplerate	= _ICardRate;
	sf_info		-> channels	= 2;
	sf_info		-> format	= SF_FORMAT_WAV | SF_FORMAT_PCM_24;

	DumpfilePointer	= sf_open (file. toAscii (). data (),
	                                                  SFM_WRITE, sf_info);
	if (DumpfilePointer == NULL) {
	   qDebug () << "Cannot open " << dumpFilename;
	   return;
	}

	DumpisOn	= TRUE;
}

void	RadioInterface::setDumpisOff (void) {
	if (!DumpisOn)
	   return;

	if (dumpBufferpointer > 0)
	      sf_writef_float (DumpfilePointer, dumpBuffer, dumpBufferpointer / 2);
	sf_close (DumpfilePointer);
	DumpisOn	= FALSE;
}

void	RadioInterface::dump (DSPCOMPLEX *v, int32_t amount) {
int32_t i;

	for (i = 0; i < amount; i ++) {
	   dumpBuffer [dumpBufferpointer ++] 	= real (v [i]);
	   dumpBuffer [dumpBufferpointer ++]	= imag (v [i]);
	   if (dumpBufferpointer >= dumpBuffersize) {
	      sf_writef_float (DumpfilePointer, dumpBuffer, dumpBufferpointer / 2);
	      dumpBufferpointer = 0;
	   }
	}
}
/*
 *	there is a tremendous amount of signal/slot connections
 *	The local connects, knobs, sliders and displays,
 *	are connected here.
 */
void	RadioInterface::localConnects (void) {
	connect (pauseButton, SIGNAL (clicked ()),
	              this, SLOT (clickPause ()));
	connect (offsetSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (set_offsetSlider (int)));
	connect (startButton, SIGNAL (clicked ()),
	              this, SLOT (setStart ()));
	connect (streamInSelector, SIGNAL (activated (int)),
	              this, SLOT (setStreamInSelector (int)));
	connect (streamOutSelector, SIGNAL (activated (int)),
	              this, SLOT (setStreamOutSelector (int)));
	connect (QuitButton, SIGNAL (clicked ()),
	              this, SLOT (TerminateProcess ()));

	connect (symbol_shifter, SIGNAL (valueChanged (int)),
	              this, SLOT (set_symbolLeftshift (int)));

/*
 */
	connect (workingrateSelect, SIGNAL (activated(const QString&) ),
	              this, SLOT (setWorkingRate (const QString&) ) );

	connect (rigSelect, SIGNAL (activated (const QString &)),
	              this, SLOT (set_rigSelect (const QString &)));
/*
 * 	for the pmsdr, we have:
 */
	connect (pmsdr_filter1, SIGNAL (clicked (void)),
	            this, SLOT (PMSDR_setFilter1 (void)));
	connect (pmsdr_filter2, SIGNAL (clicked (void)),
	            this, SLOT (PMSDR_setFilter2 (void)));
	connect (pmsdr_filter3, SIGNAL (clicked (void)),
	            this, SLOT (PMSDR_setFilter3 (void)));
	connect (pmsdr_filter4, SIGNAL (clicked (void)),
	            this, SLOT (PMSDR_setFilter4 (void)));
	connect (pmsdr_filter5, SIGNAL (clicked (void)),
	            this, SLOT (PMSDR_setFilter5 (void)));
	connect (PMSDR_preSelectButton, SIGNAL (clicked (void)),
	            this, SLOT (PMSDR_PreSelectionMode (void)));
	connect (pmsdr_mute, SIGNAL (clicked (void)),
	            this, SLOT (PMSDR_setmute (void)));
	connect (pmsdr_ifgain, SIGNAL (valueChanged (int)),
	            this, SLOT (PMSDR_setIfgain (int)));
	connect (pmsdr_biasSlider, SIGNAL (valueChanged (int)),
	            this, SLOT (PMSDR_setBias (int)));
//
//	for the dongle
	connect (dongleRate, SIGNAL (activated (const QString &)),
	            this, SLOT (set_dongleRate (const QString &)));
	connect (dongleGain, SIGNAL (valueChanged (int)),
	            this, SLOT (set_dongleGain (int)));
/*
 *
 */
	connect (dumpON, SIGNAL (clicked ()),
	              this, SLOT (setDumpisOn (void)));
	connect (dumpOFF, SIGNAL (clicked ()),
	              this, SLOT (setDumpisOff (void)));
	connect (HFplotterView, SIGNAL (activated (const QString &)),
	              this, SLOT (set_HFplotterView (const QString &)));
	connect (LFplotterView, SIGNAL (activated (const QString &)),
	              this, SLOT (set_LFplotterView (const QString &)));

	connect (HFspectrumamplitudeSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (set_HFspectrumAmplitude (int)));
	connect (LFspectrumamplitudeSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (set_LFspectrumAmplitude (int)));
	connect (attenuationSlider, SIGNAL (valueChanged (int) ),
		      this, SLOT (setAttenuator (int) ) );
	connect (balanceSlider, SIGNAL (valueChanged (int) ),
		      this, SLOT (setBalanceSlider (int) ) );
	connect (bandwidthSlider, SIGNAL (valueChanged(int) ),
	              this, SLOT (set_bandWidthSelect(int) ) );
	connect (bandoffsetSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (set_bandWidthOffset (int)));
/*
 *	Mode setters
 */
	connect (AGC_select,      SIGNAL (activated(const QString&) ),
	              this, SLOT (setAGCMode (const QString&) ) );
	connect (agcThresholdSlider, SIGNAL (valueChanged (int)),
	              this, SLOT (set_agcThresholdSlider (int)));
	connect (inputModeSelect, SIGNAL (activated(const QString&) ),
	              this, SLOT (setInputMode (const QString&) ) );
	connect (operationSelect, SIGNAL (activated(const QString&) ),
	              this, SLOT (setOperation(const QString&) ) );
	connect (quickbandSelect, SIGNAL (activated(const QString&) ),
	              this, SLOT (setquickBand(const QString&) ) );
/*
 */
/*
 *	connections for the console, first the digits
 */
	connect (add_one,   SIGNAL (clicked() ), this, SLOT (addOne() ) );
	connect (add_two,   SIGNAL (clicked() ), this, SLOT (addTwo() ) );
	connect (add_three, SIGNAL (clicked() ), this, SLOT (addThree() ) );
	connect (add_four,  SIGNAL (clicked() ), this, SLOT (addFour() ) );
	connect (add_five,  SIGNAL (clicked() ), this, SLOT (addFive() ) );
	connect (add_six,   SIGNAL (clicked() ), this, SLOT (addSix() ) );
	connect (add_seven, SIGNAL (clicked() ), this, SLOT (addSeven() ) );
	connect (add_eight, SIGNAL (clicked() ), this, SLOT (addEight() ) );
	connect (add_nine,  SIGNAL (clicked() ), this, SLOT (addNine() ) );
	connect (add_zero,  SIGNAL (clicked() ), this, SLOT (addZero() ) );
/*
 *	function buttons
 */
	connect (HzSelector,  SIGNAL (clicked() ),
	                      this, SLOT (AcceptFreqinHz() ) );
	connect (khzSelector, SIGNAL (clicked ()),
	                      this, SLOT (AcceptFreqinKhz ()));
	connect (add_correct, SIGNAL (clicked() ), this, SLOT (addCorr() ) );
	connect (add_clear,   SIGNAL (clicked() ), this, SLOT (addClear() ) );
/*
 *	The increment and decrement buttons
 */
	connect (dec_5,	   SIGNAL (clicked() ), this, SLOT (decT5() ) );
	connect (dec_50,   SIGNAL (clicked() ), this, SLOT (decT50() ) );
	connect (dec_500,  SIGNAL (clicked() ), this, SLOT (decT500() ) );
	connect (inc_5,	   SIGNAL (clicked() ), this, SLOT (incT5() ) );
	connect (inc_50,   SIGNAL (clicked() ), this, SLOT (incT50() ) );
	connect (inc_500,  SIGNAL (clicked() ), this, SLOT (incT500() ) );

	connect (setFrequencyinMiddle,
	                   SIGNAL (clicked ()), this, SLOT (setInMiddle ()));
/*
 *	detector selection
 */
	connect (nodetectorButton, SIGNAL (toggled (bool)),
	              this, SLOT (set_analyzer (bool)));
	connect (analogDecoderButton, SIGNAL (toggled (bool)),
		      this, SLOT (set_analogDecoder (bool)));
	connect (rttyButton,       SIGNAL (toggled (bool)),
	              this, SLOT (set_rtty (bool)));
	connect (cwButton,         SIGNAL (toggled (bool)),
		      this, SLOT (set_cw (bool)));
	connect (pskButton,        SIGNAL (toggled (bool)),
		      this, SLOT (set_psk (bool)));
	connect (mfskButton,       SIGNAL (toggled (bool)),
		      this, SLOT (set_mfsk (bool)));
	connect (amtorButton,	   SIGNAL (toggled (bool)),
	              this, SLOT (set_amtor (bool)));
	connect (faxButton,	SIGNAL (toggled (bool)),
	              this, SLOT (set_fax (bool)));
	connect (dominoButton,	SIGNAL (toggled (bool)),
	              this, SLOT (set_domino (bool)));
	connect (oliviaButton, SIGNAL (toggled (bool)),
	              this, SLOT (set_olivia (bool)));
/*
 *	connect for the iqslider
 */
	connect (iqslider,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (IQDisplay_setAmpl (int)));
}
/*
 *	HandleSamples is the real processor. It is called
 *	either through a signal issued by a timer on a 1000 / MSECFORTIMER
 *	Hz rate	or through a signal from the soundsystem.
 *	For portaudio we take the soundsystem signal route,
 *	for processing input from files, however, we need
 *	the timer.  
 */
#define	HANDLE_SIZE	2048

void	RadioInterface::JansRadioThroughSignal (void) {
int32_t	amount;
DSPCOMPLEX	*p = (DSPCOMPLEX *)alloca (HANDLE_SIZE * sizeof (DSPCOMPLEX));
	if (MyRig -> RigprovidesSamples ()) {
	   amount	= MyRig -> Samples ();
	   while (amount >= HANDLE_SIZE) {
	      (void)MyRig -> getSamples (p, HANDLE_SIZE, inputMode);
	      HandleSamples (p, HANDLE_SIZE);
	      amount -= HANDLE_SIZE;
	   }
	}
	else {
	   amount	= MySoundcardIn -> Samples ();
	   while (amount >= HANDLE_SIZE) {
	      (void)MySoundcardIn -> getSamples (p, HANDLE_SIZE, inputMode);
	      HandleSamples (p, HANDLE_SIZE);
	      amount -= HANDLE_SIZE;
	   }
	}
}
/*
 *	we apply a correction of the balance between I and Q,
 *	due to different multiplication
 *	factors somewhere in the generation or ad process
 *	User selectable
 */
void	RadioInterface::HandleSamples (DSPCOMPLEX *vi, int32_t size) {
int32_t		i, j;
DSPCOMPLEX	sample, audioSample;
/*
 *	if requested for, dump the input signals to a file
 */
	   if (DumpisOn)
	      dump (vi, size);
/*
 *	apply whatever correction is required per sample
 *	and process the sample
 */
	for (i = 0; i < size; i ++) {
	   if (IandQ_switch)
	      vi [i] = DSPCOMPLEX (imag (vi [i]) * attValueL,
	                           real (vi [i]) * attValueR);
	   else
	      vi [i] = DSPCOMPLEX (real (vi [i]) * attValueL,
	                           imag (vi [i]) * attValueR);
/*
 *	quick hack to allow the shift of the left of the right channel
 *	by a limited number of samples
 */
	   Circular [cc] = vi [i];
	   if (symbol_Leftshift == 0)
	      sample = vi [i];
	   else {
	      DSPFLOAT	Re = real (Circular [(cc - symbol_Leftshift) & 0xF]);
	      sample = DSPCOMPLEX (Re, imag (vi [i]));
	   }

	   cc	= (cc + 1) & 0xF;
/*
 *	HandleSamples in the Frontend will do filtering,
 *	frequency shifting and filtering.
 */
	   if (FrontEndoperation == RADIO) { 
	      if (HFScope != NULL)	// should be an ASSERT
	         HFScope -> addElement (sample);

	      sampleSum += MyFrontEnd -> HandleSampleHighSpeed (sample);
	      if (myDecimator -> doDecimate (sampleSum, &sampleSum)) {
	         DSPFLOAT gain;
	         sampleSum = MyFrontEnd -> BandPassLowSpeed (sampleSum);
	         gain = agc -> doAgc (sampleSum);
	         sampleSum = cmul (sampleSum, gain);
	         if (++showAgcCount > _ICardRate / 10) {
	            showAgcCount = 0;
	            showAgcGain -> display (gain);
	         }
	         LFScope -> addElement (sampleSum);
	         audioSample = MyDetector -> doDecode (sampleSum);

//	      ... and make it into a nice output signal
	         MySoundcardOut -> putSample (upFilter -> Pass (audioSample));
	         for (j = 1; j < upCount; j ++)
	            MySoundcardOut -> putSample (upFilter -> Pass (DSPCOMPLEX (0, 0)));
	         sampleSum = 0;
	      }
	   }
	   else	  {	// FrontEndoperation == BACKEND
	      DSPCOMPLEX tmp;
	      if (forBackendOnly (sample, &tmp)) {
	         if (BackendScope != NULL)	// should be an ASSERT
	            BackendScope -> addElement (tmp);
	         tmp = BackendShifter -> nextValue (TunedIF) * tmp;
	         LFScope -> addElement (tmp);
	         tmp = MyDetector -> doDecode (tmp);
//	...   and make it into a nice output signal
	         MySoundcardOut -> putSample (upFilter -> Pass (tmp));
	         for (j = 1; j < upCount; j ++) 
	            MySoundcardOut -> putSample (upFilter -> Pass (DSPCOMPLEX (0, 0)));
	      }
	   }
	}
}

void	RadioInterface::showPeak (int i, double v) {
	(void)i;
	peakIndicator	-> display (v);
}

/*
 *	Functions for the analog decoder
 */
void	RadioInterface::set_AnalogVolume	(int d) {
	analogVolumeIndicator	-> display	(d);
	((analogDecoder *)MyDetector) -> Analog_setVolume (d);
}

void	RadioInterface::set_AnalogLPFraction (int d) {
	((analogDecoder *)MyDetector) ->  Analog_setLPFraction (d);
}

void	RadioInterface::set_AnalogHPFraction (int d) {
	((analogDecoder *)MyDetector) ->  Analog_setHPFraction (d);
}

void	RadioInterface::switchadaptiveFilter	() {
	adaptiveFilterisOn = !adaptiveFilterisOn;
	if (adaptiveFilterisOn)
	   adaptiveFilterButton -> setText (QApplication::
	                                    translate ("elektorSDR",
	                                               "Filter On",
	                                               0, QApplication::UnicodeUTF8));
	else
	   adaptiveFilterButton -> setText (QApplication::
	                                    translate ("elektorSDR",
	                                               "Filter off", 0,
	                                               QApplication::UnicodeUTF8));

	((analogDecoder *)MyDetector) -> Analog_setAdaptiveFilter (adaptiveFilterisOn);
}

void	RadioInterface::set_AdaptiveFiltersize (int d) {
	((analogDecoder *)MyDetector) -> Analog_setAdaptiveFiltersize (d);
}

void	RadioInterface::set_Squelch (int n)  {
	((analogDecoder *)MyDetector) -> Analog_setSquelch (n);
}

void	RadioInterface::Analog_setDecoder (const QString &s) {
	if (s == "am")
	   AnalogDecoder	= analogDecoder::AM_DECODER;
	else
	if (s == "fm")
	   AnalogDecoder	= analogDecoder::FM_DECODER;
	else
	if (s == "usb")
	   AnalogDecoder	= analogDecoder::USB_DECODER;
	else
	if (s == "lsb")
	   AnalogDecoder	= analogDecoder::LSB_DECODER;
	else
	if (s == "isb")
	   AnalogDecoder	= analogDecoder::ISB_DECODER;
	else
	if (s == "coherent")
	   AnalogDecoder	= analogDecoder::COHERENT;
	else
	   AnalogDecoder	= analogDecoder::AM_DECODER;

	((analogDecoder *)MyDetector) -> Analog_setDecoder (AnalogDecoder);
}

void	RadioInterface::Analog_setText (QString s) {
	analogFunctionLabel	-> setText (s);
}

void	RadioInterface::Analog_showStrength (float f) {
	analogSignalStrength	-> display (f);
}

void	RadioInterface::Analog_showIF (float f) {
	analog_if		-> display (f);
}
/*
 *	for the CW decoder we define
 */
void	RadioInterface::set_cwWordsperMinute (int n) {
	((cwDecoder *)MyDetector) -> cw_setWordsperMinute (n);
}

void	RadioInterface::set_cwTracking (QString const& s) {
	((cwDecoder *)MyDetector) -> cw_setTracking (s == "On Tracking");
}

void	RadioInterface::set_cwFilterDegree (int n) {
	((cwDecoder *)MyDetector) -> cw_setFilterDegree (n);
}

void	RadioInterface::set_cwSquelchValue (int n) {
	((cwDecoder *)MyDetector) -> cw_setSquelchValue (n);
}

void	RadioInterface::clr_cwText () {
	cwText = QString ("");
	cwtextBox	-> setText (cwText);
}

void	RadioInterface::add_cwText (char c) {
	if (c < ' ') c = ' ';
	cwText.append (QString (QChar (c)));
	if (cwText. length () > 120)
	   cwText. remove (0, 1);
	cwtextBox	-> setText (cwText);
}

void	RadioInterface::add_cwToken (const char *s) {
	cwCharBox	-> setText (QString (s));
}

void	RadioInterface::show_cwdotLength (int d) {
	cwdotLengthMeter	-> display (d);
}

void	RadioInterface::show_cwspaceLength (int d) {
	cwspaceLengthMeter	-> display (d);
}

void	RadioInterface::show_cwspeed (int d) {
	cwspeedMeter		-> display (d);
}

void	RadioInterface::show_agcpeak (int d) {
	agcpeakMeter		-> display (d);
}

void	RadioInterface::show_noiseLevel (int d) {
	noiseLevelMeter		-> display (d);
}
/*
 *	for handling the psk mode selection, we have
 */
void	RadioInterface::set_pskAfcon (QString const& s) {
	((pskDecoder *)MyDetector) -> psk_setAfcon (s != "afc off");
}

void	RadioInterface::set_pskReverse (QString const& s) {
	((pskDecoder *)MyDetector) -> psk_setReverse (s == "normal");
}

void	RadioInterface::set_pskSquelchLevel (int n){
	((pskDecoder *)MyDetector) -> psk_setSquelchLevel (n);
}

void	RadioInterface::set_pskMode (QString const& s) {
int	QPSKSelector;

	QPSKSelector	= (s == "psk31" ? pskDecoder::MODE_PSK31 :
	                   s == "psk63" ? pskDecoder::MODE_PSK63 :
	                   s == "qpsk31" ? pskDecoder::MODE_QPSK31 :
	                   s == "qpsk63" ? pskDecoder::MODE_QPSK63 :
	                   s == "psk125" ? pskDecoder::MODE_PSK125 :
	                   s == "qpsk125" ? pskDecoder::MODE_QPSK125 :
	                   pskDecoder::MODE_PSK31);
	((pskDecoder *)MyDetector) -> psk_setMode (QPSKSelector);
}

void	RadioInterface::set_pskFilterDegree (int n) {
	((pskDecoder *)MyDetector) -> psk_setFilterDegree (n);
}

void	RadioInterface::show_pskQuality (float k) {
	pskQualitydisplay	-> display (k);
}

void	RadioInterface::show_pskIF (float f) {
	pskIFdisplay		-> display (f);
}

void	RadioInterface::show_pskDial (float f) {
	pskPhaseDisplay		-> setValue (f * 360.0 / (2 * M_PI));
}

void	RadioInterface::clr_pskText () {
	pskText = QString ("");
	psktextBox		-> setText (pskText);
}

void	RadioInterface::add_pskText (char c) {
	if (c < ' ') c = ' ';
	pskText.append (QString (QChar (c)));
	if (pskText. length () > 120)
	   pskText. remove (0, 1);
	psktextBox	-> setText (pskText);
}
/*
 *	Now for rtty
 */
void	RadioInterface::show_rttyStrength (float f) {
	rttyStrengthMeter	-> display (f);
}

void	RadioInterface::show_rttyFreqCorrection (float f) {
	rttyFreqCorrection	-> display (f);
}

void	RadioInterface::show_rttyGuess (int f) {
	rttyGuess		-> display (f);
}

void	RadioInterface::show_rttyBaudRate (int f) {
	rttyBaudRate		-> display (f);
}

void	RadioInterface::clr_rttyText () {
	rttyText = QString ("");
	rttytextBox		-> setText (rttyText);
}

void	RadioInterface::add_rttyText (char c) {
	if (c < ' ') c = ' ';
	rttyText.append (QString (QChar (c)));
	if (rttyText. length () > 120)
	   rttyText. remove (0, 1);
	rttytextBox		-> setText (rttyText);
}

void	RadioInterface::set_rttyStopbits (const QString &s) {
int	stopbits;

	stopbits = (s == "2 stop") ? 3 : (s == "1.5 stop") ? 2 : 1;
	((rttyDecoder *)MyDetector) ->  rtty_setStopbits (stopbits);
}

void	RadioInterface::set_rttyBaudrate (const QString &s) {
	((rttyDecoder *)MyDetector) -> rtty_setBaudrate (s. toInt ());
}

void	RadioInterface::set_rttyShift (const QString &s) {
	((rttyDecoder *)MyDetector) -> rtty_setShift (s. toInt ());
}

void	RadioInterface::set_rttyParity (const QString &s) {
int	rttyParity;

	if (s == "par none")
           rttyParity = rttyDecoder::RTTY_PARITY_NONE;
	else
	if (s == "par one")
           rttyParity = rttyDecoder::RTTY_PARITY_ONE;
	else
	if (s == "par odd")
           rttyParity = rttyDecoder::RTTY_PARITY_ODD;
	else
	if (s == "par even")
           rttyParity = rttyDecoder::RTTY_PARITY_EVEN;
	else
	if (s == "par zero")
           rttyParity = rttyDecoder::RTTY_PARITY_ZERO;
	else
           rttyParity = rttyDecoder::RTTY_PARITY_NONE;

	((rttyDecoder *)MyDetector) -> rtty_setParity (rttyParity);
}

void	RadioInterface::set_rttyReverse (const QString &s) {
	((rttyDecoder *)MyDetector) -> rtty_setReverse (s == "normal");
}

void	RadioInterface::set_rttyNbits (const QString &s) {
int	nbits;

	nbits = (s == "8 bits") ? 8 : (s == "7 bits") ? 7 : 5;
	((rttyDecoder *)MyDetector) -> rtty_setNbits (nbits);
}

void	RadioInterface::set_rttyMsb (const QString &s) {
	((rttyDecoder *)MyDetector) -> rtty_setMsb (s == "msb true");
}

void	RadioInterface::set_rttyAfcon (const QString &s) {
	((rttyDecoder *)MyDetector) -> rtty_setAfcon (s == "afc on");
}
/*
 *	some functions for the mfsk decoder
 */
void	RadioInterface::set_mfskAfcon	(const QString &s) {
	((mfskDecoder *)MyDetector) -> mfsk_setAfcon (s == "afc on");
}

void	RadioInterface::set_mfskReverse (const QString &s) {
	((mfskDecoder *)MyDetector) -> mfsk_setReverse (s != "normal");
}

void	RadioInterface::set_mfskMode  (const QString &s) {
int	MfskMode;

	if (s == "mfsk 8")
	   MfskMode = mfskDecoder::MODE_MFSK_8;
	else
	if (s == "mfsk 16")
	   MfskMode = mfskDecoder::MODE_MFSK_16;
	else
	if (s == "mfsk 4")
	   MfskMode = mfskDecoder::MODE_MFSK_4;
	else
	if (s == "mfsk 11")
	   MfskMode = mfskDecoder::MODE_MFSK_11;
	else
	if (s == "mfsk 22")
	   MfskMode = mfskDecoder::MODE_MFSK_22;
	else
	if (s == "mfsk 31")
	   MfskMode = mfskDecoder::MODE_MFSK_31;
	else
	if (s == "mfsk 32")
	   MfskMode = mfskDecoder::MODE_MFSK_32;
	else
	   MfskMode = mfskDecoder::MODE_MFSK_64;

	((mfskDecoder *)MyDetector) -> mfsk_setMode (MfskMode);
}

void	RadioInterface::set_mfskSquelchLevel (int n) {
	((mfskDecoder *)MyDetector) -> mfsk_setSquelchLevel (n);
}

void	RadioInterface::set_mfskIFadjuster (int n) {
	((mfskDecoder *)MyDetector) -> mfsk_setIFadjuster (n);
}

void	RadioInterface::mfsk_clrText () {
	mfskText = QString ("");
	mfsktextBox		-> setText (mfskText);
}

void	RadioInterface::mfsk_addText (char c) {
	if (c < ' ') c = ' ';
	mfskText.append (QString (QChar (c)));
	if (mfskText. length () > 120)
	   mfskText. remove (0, 1);
	mfsktextBox		-> setText (mfskText);
}

void	RadioInterface::mfsk_showCF (float f) {
	mfskStrengthMeter	-> display (f);
}

void	RadioInterface::mfsk_addLocalScope (int index, double v) {
	mfsk_X [mfsk_count]	= index;
	mfsk_Y [mfsk_count]	= v;
	mfsk_count ++;
}

void	RadioInterface::mfsk_showLocalScope (int m) {
double	*X	= (double *)alloca ((4 * (32 + 16)) * sizeof (double));
double	*Y	= (double *)alloca ((4 * (32 + 16)) * sizeof (double));
int16_t	i, j;

	for (i = 0; i < mfsk_count; i ++)
	   for (j = 0; j < (4 * (32 + 16)) / mfsk_count; j ++) {
	      X [i * 4 * (32 + 16) / mfsk_count + j] = mfsk_X [i];
	      Y [i * 4 * (32 + 16) / mfsk_count + j] = mfsk_Y [i];
	   }

	mfsk_Scope		-> Display (X, Y, 1.0, m);
	mfsk_count		= 0;
}

void	RadioInterface::mfsk_showS2N (int n) {
	mfsk_shows2n		-> display (n);
}

void	RadioInterface::mfsk_showEstimate (float f) {
	mfsk_showestimate	-> display (f);
}

void	RadioInterface::mfsk_showTuning (int n) {
	if (n == 0) {
	   mfskCentrumindicator	-> setStyleSheet ("QLabel { background-color: green}");
	   mfskLeftindicator	-> setStyleSheet ("QLabel { background-color: yellow}");
	   mfskRightindicator	-> setStyleSheet ("QLabel { background-color: yellow}");
	}
	else
	if (n < 0) {
	   mfskLeftindicator	-> setStyleSheet ("QLabel { background-color: red}");
	   mfskCentrumindicator	-> setStyleSheet ("QLabel { background-color: yellow}");
	   mfskRightindicator	-> setStyleSheet ("QLabel { background-color: yellow}");
	}
	else
	if (n > 0) {
	   mfskRightindicator	-> setStyleSheet ("QLabel { background-color: red}");
	   mfskLeftindicator	-> setStyleSheet ("QLabel { background-color: yellow}");
	   mfskCentrumindicator	-> setStyleSheet ("QLabel { background-color: yellow}");
	}
}
/*
 *	functions for the amtor decoder
 */
void	RadioInterface::set_amtorAfcon (const QString &s) {
	((amtorDecoder *)MyDetector) -> amtor_setAfcon (s == "afc on");
}

void	RadioInterface::set_amtorReverse (const QString &s) {
	((amtorDecoder *)MyDetector) -> amtor_setReverse (s != "normal");
}

void	RadioInterface::set_amtorFecError (const QString &s) {
	((amtorDecoder *)MyDetector) -> amtor_setFecError (s == "strict fec");
}

void	RadioInterface::set_amtorMessage (const QString &s) {
	((amtorDecoder *)MyDetector) -> amtor_setMessage (s == "message");
}

void	RadioInterface::show_amtorStrength (float f) {
	amtorStrengthMeter	-> display (f);
}

void	RadioInterface::show_amtorCorrection (float f) {
	amtorFreqCorrection	-> display (f);
}

void	RadioInterface::clr_amtorText () {
	amtorText = QString ("");
	amtortextBox	-> setText (amtorText);
}

void	RadioInterface::add_amtorText (char c) {
	if (c < ' ') c = ' ';
	amtorText.append (QString (QChar (c)));
	if (amtorText. length () > 120)
	   amtorText. remove (0, 1);
	amtortextBox	-> setText (amtorText);
}
/*
 *	fax
 */
void	RadioInterface::fax_showState (const QString &s) {
	fax_state	-> setText (s);
}

void	RadioInterface::fax_displayImage (QImage image) {
QLabel	*imageLabel = new QLabel;

	imageLabel	-> setPixmap (QPixmap::fromImage (image));
	faxContainer -> setWidget (imageLabel);
	imageLabel	-> show ();
}

void	RadioInterface::fax_displayImage (QImage image, int x, int y) {
QLabel	*imageLabel = new QLabel;

	imageLabel	-> setPixmap (QPixmap::fromImage (image));
	faxContainer	-> setWidget (imageLabel);
	imageLabel	-> show ();
	faxContainer	-> ensureVisible (x, y);
}

void	RadioInterface::fax_setIOC (const QString& s) {
	((faxDecoder *)MyDetector) -> fax_setIOC (s. toInt ());
}

void	RadioInterface::fax_setMode	(const QString& s) {
int	m;

	m = s == "FM" ? faxDecoder::FAX_FM :  faxDecoder::FAX_AM;
	((faxDecoder *)MyDetector) -> fax_setMode (m);
}

void	RadioInterface::fax_setClicked (int x_coord, int y_coord) {
	((faxDecoder *)MyDetector) -> fax_setSlant (QPoint (x_coord, y_coord));
}

void	RadioInterface::fax_setDeviation (int n) {
	((faxDecoder *)MyDetector) -> fax_setDeviation (n);
}

void	RadioInterface::fax_setSkip (void) {
	faxSkip ->
	            setText (QApplication::
	                     translate ("elektorSDR",
	                                "skip", 0,
	                                QApplication::UnicodeUTF8));
	((faxDecoder *)MyDetector) -> fax_setSkip ();
}

void	RadioInterface::fax_setLPM (int l) {
	((faxDecoder *)MyDetector) -> fax_setLPM ((DSPFLOAT)l);
}

void	RadioInterface::fax_setphase (const QString &s) {
	((faxDecoder *)MyDetector) -> fax_setPhase (s == "phase" ? 0 : -1);
}

void	RadioInterface::fax_setfaxColor (const QString &s) {
	((faxDecoder *)MyDetector) ->
	     fax_setColor (s == "black white" ? faxDecoder::FAX_BLACKWHITE : 
	                     s == "gray" ? faxDecoder::FAX_GRAY :
	                                   faxDecoder::FAX_COLOR);
}

void	RadioInterface::fax_setstartFreq (int f) {
	((faxDecoder *)MyDetector) -> fax_setstartFreq (f);
}

void	RadioInterface::fax_setstopFreq (int f) {
	((faxDecoder *)MyDetector) -> fax_setstopFreq (f);
}

void	RadioInterface::fax_setCarrier (int f) {
	((faxDecoder *)MyDetector) -> fax_setCarrier (f);
}

void	RadioInterface::fax_setsaveButton (const QString &s) {
	((faxDecoder *)MyDetector) -> fax_setsavingonDone (s != "no save");
}

void	RadioInterface::fax_setdecoderSelector (const QString &s) {
	((faxDecoder *)MyDetector) -> fax_setdecoderSelector (s == "fm1" ? 1
	                                          : s == "fm2" ? 2
	                                          : s == "fm3" ? 3 : 4);
}

void	RadioInterface::fax_showaptFreq (int f) {
	faxaptFreq	-> display (f);
}

void	RadioInterface::fax_showLinenumber (int f) {
	faxLinenumber	-> display (f);
}

void	RadioInterface::fax_saveFile (QImage image) {
QFile	outFile;
QString file	= QFileDialog::getSaveFileName (this,
	                                        tr ("save file as .."),
	                                        QDir::homePath (),
	                                        tr ("Images (*.jpg)"));
	                               
	outFile. setFileName (file);
	if (!outFile. open (QIODevice::WriteOnly)) {
	   qDebug () << "Cannot open" << file;
	   return;
	}

	image. save (&outFile);
	outFile. close ();
}

void	RadioInterface::fax_slantText (const QString &s) {
	slantText	-> setText (s);
}

void	RadioInterface::domino_clrText () {
	dominoText = QString ("");
	dominoText2 = QString ("");
	dominotextBox	-> setText (dominoText);
	dominotextBox2	-> setText (dominoText2);
}

void	RadioInterface::domino_addText (char c) {
	if (c < ' ') c = ' ';
	dominoText.append (QString (QChar (c)));
	if (dominoText. length () > 120)
	   dominoText. remove (0, 1);
	dominotextBox	-> setText (dominoText);
}

void	RadioInterface::domino_addText2 (char c) {
	if (c < ' ') c = ' ';
	dominoText2.append (QString (QChar (c)));
	if (dominoText2. length () > 96)
	   dominoText2. remove (0, 1);
	dominotextBox2	-> setText (dominoText2);
}

void	RadioInterface::domino_showS2N (int n) {
	domino_shows2n	-> display (n);
}

void	RadioInterface::set_dominoMode (const QString &s) {
int	dominoMode;

	if (s == "domino5")
	   dominoMode = dominoDecoder::MODE_DOMINOEX5;
	else
	if (s == "domino11")
	   dominoMode = dominoDecoder::MODE_DOMINOEX11;
	else
	if (s == "domino22")
	   dominoMode = dominoDecoder::MODE_DOMINOEX22;
	else
	if (s == "domino4")
	   dominoMode = dominoDecoder::MODE_DOMINOEX4;
	else
	if (s == "domino16")
	   dominoMode = dominoDecoder::MODE_DOMINOEX16;
	else
	   dominoMode = dominoDecoder::MODE_DOMINOEX8;
	
	((dominoDecoder *)MyDetector) -> domino_setMode (dominoMode);
}

void	RadioInterface::set_dominoSquelch (int s) {
	((dominoDecoder *)MyDetector) -> domino_setSquelch (s);
}

void	RadioInterface::set_dominoFec (const QString &s) {
	((dominoDecoder *)MyDetector) -> domino_setFec (s == "fec");
}

void	RadioInterface::set_dominoReverse (const QString &s) {
	((dominoDecoder *)MyDetector) -> domino_setReverse (s == "reverse");
}
//
//	Olivia
void	RadioInterface::set_oliviaTones	(const QString &s) {
	((oliviaDecoder *)MyDetector) -> olivia_setTones (s. toInt ());
}

void	RadioInterface::set_oliviaBW (const QString &s) {
	((oliviaDecoder *)MyDetector) -> olivia_setBW (s. toInt ());
}

void	RadioInterface::set_oliviaSyncMargin (int n) {
	((oliviaDecoder *)MyDetector) -> olivia_setSyncMargin (n);
}

void	RadioInterface::set_oliviaSyncIntegLen (int n) {
	((oliviaDecoder *)MyDetector) -> olivia_setSyncIntegLen (n);
}

void	RadioInterface::set_oliviaSquelch (const QString &s) {
	((oliviaDecoder *)MyDetector) -> olivia_setSquelch (s == "squelch on");
}

void	RadioInterface::set_oliviaSquelchvalue (int n) {
	((oliviaDecoder *)MyDetector) -> olivia_setSquelchvalue (n);
}

void	RadioInterface::set_oliviaReverse (const QString &s) {
	((oliviaDecoder *)MyDetector) -> olivia_setReverse (s == "reverse");
}
//
//	and the slots
void	RadioInterface::olivia_clrText	(void) {
	oliviaText = QString ("");
	oliviatextBox	-> setText (oliviaText);
}

void	RadioInterface::olivia_addChar	(char c) {
	if (c < ' ') c = ' ';
	oliviaText.append (QString (QChar (c)));
	if (oliviaText. length () > 120)
	   oliviaText. remove (0, 1);
	oliviatextBox	-> setText (oliviaText);
}

void	RadioInterface::olivia_put_Status1	(char *s) {
	olivia_status1_textBox -> setText (QString (s));
}

void	RadioInterface::olivia_put_Status2	(char *s) {
	olivia_status2_textBox -> setText (QString (s));
}

/*
 * 	for the analog decoder, we only set the volume
 * 	and the subdecoder text
 */
void	RadioInterface::setAnalogSliders (void) {
	connect (analogDecoderSelect,
	         SIGNAL (activated (const QString &)),
	         this,
	         SLOT (Analog_setDecoder (const QString &)));
	connect (analogDecoderVolume,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (set_AnalogVolume (int)));
	connect (adaptiveFilterButton,
	         SIGNAL (clicked ()),
	         this,
	         SLOT (switchadaptiveFilter ()));
	connect (adaptiveFiltersize,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (set_AdaptiveFiltersize (int)));
	connect (analogDecoderLowPass,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (set_AnalogLPFraction (int)));	
	connect (analogDecoderHighPass,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (set_AnalogHPFraction (int)));	
	connect (squelchSlider,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (set_Squelch (int)));
	Analog_setDecoder	(analogDecoderSelect	-> currentText  ());
	set_AnalogVolume	(analogDecoderVolume	-> value 	());
	set_AnalogLPFraction	(analogDecoderLowPass	-> value	());
	set_AnalogHPFraction	(analogDecoderHighPass	-> value	());
	set_Squelch		(squelchSlider		-> value	());
}

void	RadioInterface::unsetAnalogSliders (void) {
	disconnect (analogDecoderSelect,
	            SIGNAL (activated (const QString &)),
	            this,
	            SLOT (Analog_setDecoder (const QString &)));
	disconnect (analogDecoderVolume,
	            SIGNAL (valueChanged (int)),
	            this,
	            SLOT (set_AnalogVolume (int)));
	disconnect (adaptiveFilterButton,
	            SIGNAL (clicked ()),
	            this,
	            SLOT (switchadaptiveFilter ()));
	disconnect (adaptiveFiltersize,
	            SIGNAL (valueChanged (int)),
	            this,
	            SLOT (set_AdaptiveFiltersize (int)));
	disconnect (analogDecoderLowPass,
	            SIGNAL (valueChanged (int)),
	            this,
	            SLOT (set_AnalogLPFraction (int)));	
	disconnect (analogDecoderHighPass,
	            SIGNAL (valueChanged (int)),
	            this,
	            SLOT (set_AnalogHPFraction (int)));	
	disconnect (squelchSlider,
	            SIGNAL (valueChanged (int)),
	            this,
	            SLOT (set_Squelch (int)));
}

/*
 * 	cw
 */
void	RadioInterface::setCwSliders	(void) {
	connect (cwTracker, SIGNAL (activated (const QString &)),
	         this, SLOT (set_cwTracking (const QString &)));
	connect (cwwpm, SIGNAL (valueChanged (int)),
	         this, SLOT (set_cwWordsperMinute (int)));
	connect (cwSquelchLevel, SIGNAL (valueChanged (int)),
	         this, SLOT (set_cwSquelchValue (int)));
	connect (cwFilterDegree, SIGNAL (valueChanged (int)),
	         this, SLOT (set_cwFilterDegree (int)));
	set_cwWordsperMinute	(cwwpm			-> value ());
	set_cwTracking		(cwTracker		-> currentText ());
	set_cwFilterDegree	(cwFilterDegree		-> value ());
	set_cwSquelchValue	(cwSquelchLevel		-> value ());
	cwText = QString ();
}

void	RadioInterface::unsetCwSliders	(void) {
	disconnect (cwTracker, SIGNAL (activated (const QString &)),
	            this, SLOT (set_cwTracking (const QString &)));
	disconnect (cwwpm, SIGNAL (valueChanged (int)),
	            this, SLOT (set_cwWordsperMinute (int)));
	disconnect (cwSquelchLevel, SIGNAL (valueChanged (int)),
	            this, SLOT (set_cwSquelchValue (int)));
	disconnect (cwFilterDegree, SIGNAL (valueChanged (int)),
	            this, SLOT (set_cwFilterDegree (int)));
}
/*
 * 	for rtty we set some state values by reading out the
 * 	corresponding gui elements.
 */
void	RadioInterface::setRttySliders	(void) {
	connect (rttyBaudrateSelect, SIGNAL (activated (const QString &)),
	         this, SLOT (set_rttyBaudrate (const QString &)));
	connect (rttyWidthSelect, SIGNAL (activated (const QString &)),
	         this, SLOT (set_rttyShift (const QString &)));
	connect (rttyParity, SIGNAL (activated (const QString &)),
		 this, SLOT (set_rttyParity (const QString &)));
	connect (rttyMsb,    SIGNAL (activated (const QString &)),
		 this, SLOT (set_rttyMsb   (const QString &)));
	connect (rttyNbits,  SIGNAL (activated (const QString &)),
		 this, SLOT (set_rttyNbits (const QString &)));
	connect (rttyStopbits, SIGNAL (activated (const QString &)),
		 this, SLOT (set_rttyStopbits (const QString &)));
	connect (rttyReverse, SIGNAL (activated (const QString &)),
		 this, SLOT (set_rttyReverse (const QString &)));
	connect (rttyAfcon, SIGNAL (activated (const QString &)),
		 this, SLOT (set_rttyAfcon (const QString &)));
	set_rttyBaudrate	(rttyBaudrateSelect	-> currentText ());
	set_rttyShift		(rttyWidthSelect	-> currentText ());
	set_rttyParity		(rttyParity		-> currentText ());
	set_rttyMsb		(rttyMsb		-> currentText ());
	set_rttyNbits		(rttyNbits		-> currentText ());
	set_rttyStopbits	(rttyStopbits		-> currentText ());
	set_rttyReverse		(rttyReverse		-> currentText ());
	set_rttyAfcon		(rttyAfcon		-> currentText ());
	rttyText		= QString ();
}

void	RadioInterface::unsetRttySliders	(void) {
	disconnect (rttyBaudrateSelect, SIGNAL (activated (const QString &)),
	            this, SLOT (set_rttyBaudrate (const QString &)));
	disconnect (rttyWidthSelect, SIGNAL (activated (const QString &)),
	            this, SLOT (set_rttyShift (const QString &)));
	disconnect (rttyParity, SIGNAL (activated (const QString &)),
		    this, SLOT (set_rttyParity (const QString &)));
	disconnect (rttyMsb,    SIGNAL (activated (const QString &)),
		    this, SLOT (set_rttyMsb   (const QString &)));
	disconnect (rttyNbits,  SIGNAL (activated (const QString &)),
		    this, SLOT (set_rttyNbits (const QString &)));
	disconnect (rttyStopbits, SIGNAL (activated (const QString &)),
		    this, SLOT (set_rttyStopbits (const QString &)));
	disconnect (rttyReverse, SIGNAL (activated (const QString &)),
		    this, SLOT (set_rttyReverse (const QString &)));
	disconnect (rttyAfcon, SIGNAL (activated (const QString &)),
		    this, SLOT (set_rttyAfcon (const QString &)));
}
/*
 * 	for psk we set some state values by reading out the
 * 	corresponding gui elements
 */
void	RadioInterface::setPskSliders	(void) {
	connect (pskAfcon, SIGNAL (activated (const QString &)),
	         this, SLOT (set_pskAfcon (const QString &)));
	connect (pskReverse, SIGNAL (activated (const QString &)),
	         this, SLOT (set_pskReverse (const QString &)));
	connect (qpskSelector, SIGNAL (activated (const QString &)),
	         this, SLOT (set_pskMode (const QString &)));
	connect (pskSquelchLevel, SIGNAL (valueChanged (int)),
	         this, SLOT (set_pskSquelchLevel (int)));
	connect (pskFilterDegree, SIGNAL (valueChanged (int)),
	         this, SLOT (set_pskFilterDegree (int)));
	set_pskAfcon	 	(pskAfcon 		-> currentText ());
	set_pskReverse		(pskReverse		-> currentText ());
	set_pskSquelchLevel	(pskSquelchLevel	-> value ());
	set_pskMode		(qpskSelector		-> currentText ());
	set_pskFilterDegree	(pskFilterDegree	-> value ());
	pskText			= QString ();
}

void	RadioInterface::unsetPskSliders	(void) {
	disconnect (pskAfcon, SIGNAL (activated (const QString &)),
	            this, SLOT (set_pskAfcon (const QString &)));
	disconnect (pskReverse, SIGNAL (activated (const QString &)),
	            this, SLOT (set_pskReverse (const QString &)));
	disconnect (qpskSelector, SIGNAL (activated (const QString &)),
	            this, SLOT (set_pskMode (const QString &)));
	disconnect (pskSquelchLevel, SIGNAL (valueChanged (int)),
	            this, SLOT (set_pskSquelchLevel (int)));
	disconnect (pskFilterDegree, SIGNAL (valueChanged (int)),
	            this, SLOT (set_pskFilterDegree (int)));
}
/*
 * 	for mfsk we set the state according to the sliders and switches
 */
void	RadioInterface::setMfskSliders	(void) {
	connect (mfskMode, SIGNAL (activated (const QString &)),
		 this, SLOT (set_mfskMode (const QString &)));
	connect (mfskReverse, SIGNAL (activated (const QString &)),
		 this, SLOT (set_mfskReverse (const QString &)));
	connect (mfskAfcon, SIGNAL (activated (const QString &)),
		 this, SLOT (set_mfskAfcon (const QString &)));
	connect (mfskIFadjuster, SIGNAL (valueChanged (int)),
		 this, SLOT (set_mfskIFadjuster (int)));
	set_mfskMode 		(mfskMode		-> currentText ());
	set_mfskReverse		(mfskReverse		-> currentText ());
	set_mfskAfcon		(mfskAfcon		-> currentText ());
	set_mfskSquelchLevel	(mfskSquelch		-> value ());
	set_mfskIFadjuster	(mfskIFadjuster		-> value ());
	mfskText		= QString ();
}

void	RadioInterface::unsetMfskSliders	(void) {
	disconnect (mfskMode, SIGNAL (activated (const QString &)),
		    this, SLOT (set_mfskMode (const QString &)));
	disconnect (mfskReverse, SIGNAL (activated (const QString &)),
		    this, SLOT (set_mfskReverse (const QString &)));
	disconnect (mfskAfcon, SIGNAL (activated (const QString &)),
		    this, SLOT (set_mfskAfcon (const QString &)));
	disconnect (mfskIFadjuster, SIGNAL (valueChanged (int)),
		    this, SLOT (set_mfskIFadjuster (int)));
}

/*
 * 	for amtor (navtex) we set some state elements according to the
 * 	read out of corresponding gui elements.
 */
void	RadioInterface::setAmtorSliders	(void) {
	connect (amtorAfcon, SIGNAL (activated (const QString &)),
	         this, SLOT (set_amtorAfcon (const QString &)));
	connect (amtorReverse, SIGNAL (activated (const QString &)),
	         this, SLOT (set_amtorReverse (const QString &)));
	connect (amtorFecError, SIGNAL (activated (const QString &)),
	         this, SLOT (set_amtorFecError (const QString &)));
	connect (amtorMessage, SIGNAL (activated (const QString &)),
	         this, SLOT (set_amtorMessage (const QString &)));
	set_amtorAfcon		(amtorAfcon		-> currentText ());
	set_amtorReverse	(amtorReverse		-> currentText ());
	set_amtorFecError	(amtorFecError		-> currentText ());
	set_amtorMessage	(amtorMessage		-> currentText ());
	amtorText		= QString ();
}

void	RadioInterface::unsetAmtorSliders	(void) {
	disconnect (amtorAfcon, SIGNAL (activated (const QString &)),
	            this, SLOT (set_amtorAfcon (const QString &)));
	disconnect (amtorReverse, SIGNAL (activated (const QString &)),
	            this, SLOT (set_amtorReverse (const QString &)));
	disconnect (amtorFecError, SIGNAL (activated (const QString &)),
	            this, SLOT (set_amtorFecError (const QString &)));
	disconnect (amtorMessage, SIGNAL (activated (const QString &)),
	            this, SLOT (set_amtorMessage (const QString &)));
}
/*
 * 	for fax we set some state elements according to the
 * 	read out of corresponding gui elements.
 */
void	RadioInterface::setFaxSliders	(void) {
	connect (faxContainer, SIGNAL (fax_Clicked (int, int)),
	         this, SLOT (fax_setClicked (int, int)));
	connect	(faxModeSelector, SIGNAL (activated (const QString &)),
	         this, SLOT (fax_setMode (const QString &)));
	connect (faxIOCSelector, SIGNAL (activated (const QString &)),
	         this, SLOT (fax_setIOC (const QString &)));
	connect (faxDeviationSelector, SIGNAL (valueChanged (int)),
	         this, SLOT (fax_setDeviation (int)));
	connect (faxSkip, SIGNAL (clicked ()),
	         this, SLOT (fax_setSkip ()));
	connect (LPMBox, SIGNAL (valueChanged (int)),
	         this, SLOT (fax_setLPM (int)));
	connect (phaseSelector, SIGNAL (activated (const QString &)),
	         this, SLOT (fax_setphase (const QString &)));
	connect (faxColorSelector, SIGNAL (activated (const QString &)),
	         this, SLOT (fax_setfaxColor (const QString &)));
	connect (faxstartFrequency, SIGNAL (valueChanged (int)),
	         this, SLOT (fax_setstartFreq (int)));
	connect (faxstopFrequency, SIGNAL (valueChanged (int)),
	         this, SLOT (fax_setstopFreq (int)));
	connect (faxCarrier, SIGNAL (valueChanged (int)),
	         this, SLOT (fax_setCarrier (int)));
	connect (fax_saveButton, SIGNAL (activated (const QString &)),
	         this, SLOT (fax_setsaveButton (const QString &)));
	connect (faxdecoderSelector, SIGNAL (activated (const QString &)),
	         this, SLOT (fax_setdecoderSelector (const QString &)));
	fax_setIOC		(faxIOCSelector		-> currentText ());
	fax_setMode		(faxModeSelector	-> currentText ());
	fax_setDeviation	(faxDeviationSelector	-> value ());
	fax_setLPM		(LPMBox			-> value ());
	fax_setphase		(phaseSelector		-> currentText ());
	fax_setfaxColor		(faxColorSelector	-> currentText ());
	fax_setstartFreq	(faxstartFrequency	-> value ());
	fax_setstopFreq		(faxstopFrequency	-> value ());
	fax_setCarrier		(faxCarrier		-> value ());
	fax_setdecoderSelector	(faxdecoderSelector	-> currentText ());
}

void	RadioInterface::unsetFaxSliders	(void) {
	disconnect (faxContainer, SIGNAL (fax_Clicked (int, int)),
	            this, SLOT (fax_setClicked (int, int)));
	disconnect (faxModeSelector, SIGNAL (activated (const QString &)),
	            this, SLOT (fax_setMode (const QString &)));
	disconnect (faxIOCSelector, SIGNAL (activated (const QString &)),
	            this, SLOT (fax_setIOC (const QString &)));
	disconnect (faxDeviationSelector, SIGNAL (valueChanged (int)),
	            this, SLOT (fax_setDeviation (int)));
	disconnect (faxSkip, SIGNAL (clicked ()),
	            this, SLOT (fax_setSkip ()));
	disconnect (LPMBox, SIGNAL (valueChanged (int)),
	            this, SLOT (fax_setLPM (int)));
	disconnect (phaseSelector, SIGNAL (activated (const QString &)),
	            this, SLOT (fax_setphase (const QString &)));
	disconnect (faxColorSelector, SIGNAL (activated (const QString &)),
	            this, SLOT (fax_setfaxColor (const QString &)));
	disconnect (faxstartFrequency, SIGNAL (valueChanged (int)),
	            this, SLOT (fax_setstartFreq (int)));
	disconnect (faxstopFrequency, SIGNAL (valueChanged (int)),
	            this, SLOT (fax_setstopFreq (int)));
	disconnect (faxCarrier, SIGNAL (valueChanged (int)),
	            this, SLOT (fax_setCarrier (int)));
	disconnect (fax_saveButton, SIGNAL (activated (const QString &)),
	            this, SLOT (fax_setsaveButton (const QString &)));
	disconnect (faxdecoderSelector, SIGNAL (activated (const QString &)),
	            this, SLOT (fax_setdecoderSelector (const QString &)));
}
/*
 * 	for domino we set some state elements according to the
 * 	read out of corresponding gui elements.
 */
void	RadioInterface::setDominoSliders	(void) {
	connect (dominoMode, SIGNAL (activated (const QString &)),
	         this, SLOT (set_dominoMode (const QString &)));
	connect (dominoFec, SIGNAL (activated (const QString &)),
	         this, SLOT (set_dominoFec (const QString &)));
	connect (dominoReverse, SIGNAL (activated (const QString &)),
	         this, SLOT (set_dominoReverse (const QString &)));
	connect (dominoSquelch, SIGNAL (valueChanged (int)),
	         this, SLOT (set_dominoSquelch (int)));
	set_dominoSquelch	(dominoSquelch		-> value ());
	set_dominoFec		(dominoFec		-> currentText ());
	set_dominoReverse	(dominoReverse		-> currentText ());
	dominoText		= QString ();
	dominoText2		= QString ();
}

void	RadioInterface::unsetDominoSliders	(void) {
	disconnect (dominoMode, SIGNAL (activated (const QString &)),
	            this, SLOT (set_dominoMode (const QString &)));
	disconnect (dominoFec, SIGNAL (activated (const QString &)),
	            this, SLOT (set_dominoFec (const QString &)));
	disconnect (dominoReverse, SIGNAL (activated (const QString &)),
	            this, SLOT (set_dominoReverse (const QString &)));
	disconnect (dominoSquelch, SIGNAL (valueChanged (int)),
	            this, SLOT (set_dominoSquelch (int)));
}
//
void	RadioInterface::setOliviaSliders	(void) {
	connect (oliviaTones, SIGNAL (activated (const QString &)),
	         this, SLOT (set_oliviaTones (const QString &)));
	connect (oliviaBW, SIGNAL (activated (const QString &)),
	         this, SLOT (set_oliviaBW (const QString &)));
	connect (oliviaSyncMargin,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (set_oliviaSyncMargin (int)));
	connect (oliviaSyncIntegLen,
	         SIGNAL (valueChanged (int)),
	         this,
	         SLOT (set_oliviaSyncIntegLen (int)));
	connect (oliviaSquelch, SIGNAL (activated (const QString &)),
	         this, SLOT (set_oliviaSquelch (const QString &)));
	connect (oliviaSquelchvalue, SIGNAL (valueChanged (int)),
	         this, SLOT (set_oliviaSquelchvalue (int)));
	connect (oliviaReverse, SIGNAL (activated (const QString &)),
	         this, SLOT (set_oliviaReverse (const QString &)));
	set_oliviaTones		(oliviaTones		-> currentText ());
	set_oliviaBW		(oliviaBW		-> currentText ());
	set_oliviaSyncMargin	(oliviaSyncMargin	-> value ());
	set_oliviaSyncIntegLen	(oliviaSyncIntegLen	-> value ());
	set_oliviaSquelch	(oliviaSquelch		-> currentText ());
	set_oliviaSquelchvalue	(oliviaSquelchvalue	-> value ());
	set_oliviaReverse	(oliviaReverse		-> currentText ());
}

void	RadioInterface::unsetOliviaSliders	(void) {
	disconnect (oliviaTones, SIGNAL (activated (const QString &)),
	            this, SLOT (set_oliviaTones (const QString &)));
	disconnect (oliviaBW, SIGNAL (activated (const QString &)),
	            this, SLOT (set_oliviaBW (const QString &)));
	disconnect (oliviaSyncMargin, SIGNAL (valueChanged (int)),
	            this, SLOT (set_oliviaSyncMargin (int)));
	disconnect (oliviaSyncIntegLen, SIGNAL (valueChanged (int)),
	            this, SLOT (set_oliviaSyncIntegLen (int)));
	disconnect (oliviaSquelch, SIGNAL (activated (const QString &)),
	            this, SLOT (set_oliviaSquelch (const QString &)));
	disconnect (oliviaSquelchvalue, SIGNAL (valueChanged (int)),
	            this, SLOT (set_oliviaSquelchvalue (int)));
	disconnect (oliviaReverse, SIGNAL (activated (const QString &)),
	            this, SLOT (set_oliviaReverse (const QString &)));
}

void	RadioInterface::createpskPhaseDisplay () {
QwtDialSimpleNeedle *needle;

	pskPhaseDisplay		-> setWrapping	(FALSE);
	pskPhaseDisplay		-> setRange	(0.0, 360.0);
	pskPhaseDisplay		-> setOrigin	(0.0);
	pskPhaseDisplay		-> setScale	(-1, 2, 90);
	pskPhaseDisplay		-> setScaleArc	(-180.0, 180.0);
	pskPhaseDisplay		-> scaleDraw () -> setSpacing (5);
	needle			= new QwtDialSimpleNeedle (
	                              QwtDialSimpleNeedle::Arrow, true, Qt::red,
	                                 QColor (Qt::yellow). light(130));
	pskPhaseDisplay		-> setNeedle (needle);
	pskPhaseDisplay		-> setScaleOptions (QwtDial::ScaleTicks |
					            QwtDial::ScaleLabel);
	pskPhaseDisplay		-> setScaleTicks   (0, 4, 8);
}
/*
 * 	In case we do the backend only, we sample at the
 * 	original speed but we assume that we only have a
 * 	signal in the real domain, that we convert into
 * 	a signal in the complex domain.
 */
bool	RadioInterface::forBackendOnly (DSPCOMPLEX v, DSPCOMPLEX *result) {
DSPCOMPLEX	tmp = DSPCOMPLEX (0, 0);

	switch (BackendState) {
	   case 0:
	      tmp = BackendOnlyFilter -> Pass (DSPCOMPLEX (real (v), 0));
	      break;

	   case 1:
	      tmp = BackendOnlyFilter -> Pass (DSPCOMPLEX (0, real (v)));
	      break;

	   case 2:
	      tmp = BackendOnlyFilter -> Pass (DSPCOMPLEX (- real (v), 0));
	      break;

	   case 3:
	      tmp = BackendOnlyFilter -> Pass (DSPCOMPLEX (0, -real (v)));
	      break;
	}

	BackendState = (BackendState + 1) & 03;
	if (myDecimator	-> doDecimate (tmp, result)) {
	   return true;	
	}

	return false;
}
//
//	The rigs
//
void	RadioInterface::set_rigSelect (const QString &s) {
int16_t	newRig;
bool	success;
QString	file;
//
//	first, find out the rig to be selected
	 newRig = s == "pmsdr" ? PMSDR :
	          s == "replay" ? REPLAY :
	          s == "dabstick" ? DABSTICK :
	          s == "dabstick (dll)" ? DABSTICK_DLL :
	          s == "extio"  ? EXTIO : NO_RIG;

	if ((CurrentRig != NO_RIG) && (CurrentRig == newRig))
	   return;
//
//	ensure that the previous rig goes down in a controlled way.
//	Stop some timers and register that we are IDLE
	runMode	= IDLE;		// just stop activities
	lcd_timer -> stop ();
//	if the current rig is a dll, we need to stop the 
//	polling clock and hide the gui
	if (CurrentRig == EXTIO) { 
	   dll_timer	-> stop ();
	   disconnect (MyRig,
	               SIGNAL (setDLLRequest (int)),
	               this,
	               SLOT (setDLLRequest (int)));
	}
//
//	ok, first we stop the samplestream  and we
//	disconnect the signal for the input stream
	if (MyRig -> RigprovidesSamples ()) {
	   disconnect (MyRig,
	               SIGNAL (samplesAvailable ()),
	               this,
	               SLOT (JansRadioThroughSignal ()));
	   MyRig -> stopReader ();
	}
	else {
	   disconnect (MySoundcardIn,
	               SIGNAL (samplesAvailable ()),
	               this,
	               SLOT (JansRadioThroughSignal ()));
	   MySoundcardIn -> stopReader ();
	}
//	then the samplestream out
	MySoundcardOut	-> stopWriter ();
//	and finally, delete the device
	usleep (1000);
	delete MyRig;

//	Ok, the old rig is shut down, set a default
	CurrentRig	= NO_RIG;	// default
	switch (newRig) {
	   case REPLAY: 
	      file = QFileDialog::getOpenFileName (this,
	                                           tr ("open file .."),
	                                           QDir::homePath (),
	                                           tr ("Sound (*.wav)"));
	      file	= QDir::toNativeSeparators (file);
	      MyRig		= new replayRig (_ICardRate,
	                                         file. toAscii (). data (),
	                                         &success);
	      if (success) {
	         CurrentRig	= REPLAY;
	      }
	      else {
	         QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Could not open file"));
	         delete MyRig;
	         MyRig		= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	      }
	      break;

	   case EXTIO:
#ifdef __MINGW32__
	      file = QFileDialog::getOpenFileName (this,
	                                           tr ("open file .."),
	                                           QDir::currentPath (),
	                                           tr ("dll (Extio*.dll)"));
#else
	      file = QFileDialog::getOpenFileName (this,
	                                           tr ("open file ..."),
	                                           QDir::currentPath (),
	                                           tr ("so (Extio*.so)"));
#endif
	      file	= QDir::toNativeSeparators (file);
	      MyRig	= new dll_handler (file. toAscii (). data (),
	                                          &success, _ICardRate);
	      if (success) {
	         CurrentRig = EXTIO;
	         dll_label -> setText (MyRig -> nameofRig ());
	         connect (MyRig,
	                  SIGNAL (setDLLRequest (int)),
	                  this,
	                  SLOT (setDLLRequest (int)));
	         dll_timer	-> start (100);
	      } 
	      else {
	         QMessageBox::warning (this, tr ("sdr"),
	                                     tr ("error in opening"));
	         delete MyRig;
	         MyRig		= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	      }
	      break;

	   case DABSTICK:
	      MyRig	= new dabstick (_ICardRate, 1152000, &success);
	      if (success) {
	         CurrentRig	= DABSTICK;
	      }
	      else {
	         QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Could not open dongle"));
	         delete (dabstick *)MyRig;
	         MyRig		= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	      }
	      break;

	   case DABSTICK_DLL:
	      MyRig	= new dabstick_dll (_ICardRate, 1152000, &success);
	      if (success) {
	         CurrentRig	= DABSTICK_DLL;
	      }
	      else {
	         QMessageBox::warning (this, tr ("sdr"),
	                               tr ("Could not open dongle"));
	         delete (dabstick *)MyRig;
	         MyRig		= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	      }
	      break;

	   case PMSDR:
	      MyRig		= new pmSDR (this, &success);
	      if (success) {
	         CurrentRig	= PMSDR;
	      }
	      else {
	         QMessageBox::warning(this, tr ("sdr"),
	                                    tr ("Could not open pmsdr"));
	         delete MyRig;
//	         CurrentRig	= NO_RIG;
	         MyRig		= new no_rig ();
	         rigSelect	-> setCurrentIndex (0);
	         break;
	      }
	      break;
	      
	   default:			// should not happen
	   case NO_RIG:
	      CurrentRig	= NO_RIG;
	      MyRig		= new no_rig ();
	      rigSelect		-> setCurrentIndex (0);
	      break;
	}
	setrigControlScreen	(CurrentRig);
	MyRig			-> setOffset (offsetSlider -> value ());
	Set_Tuner 		(MyRig -> defaultFrequency (),
	                                   &CurrentFreq, &TunedIF);
	setPreSelector		(CurrentFreq);
	lcd_Frequency 		-> display (CurrentFreq);
	MyRig			-> displayVFO ();
}
/*
 * 	Specifics for the pmsdr kit
 */
void	RadioInterface::PMSDR_setFilter1	(void) {
	if (PMSDR_preSelectorMode == MANUAL) {
	   ((pmSDR *)MyRig) -> setFilter (4);
	   PMSDR_bandlabel -> setText (((pmSDR *)MyRig) -> getFilterRange (4));
	}
}

void	RadioInterface::PMSDR_setFilter2	(void) {
	if (PMSDR_preSelectorMode == MANUAL) {
	   ((pmSDR *)MyRig) -> setFilter (1);
	   PMSDR_bandlabel -> setText (((pmSDR *)MyRig) -> getFilterRange (1));
	}
}

void	RadioInterface::PMSDR_setFilter3	(void) {
	if (PMSDR_preSelectorMode == MANUAL) {
	   ((pmSDR *)MyRig) -> setFilter (2);
	   PMSDR_bandlabel -> setText (((pmSDR *)MyRig) -> getFilterRange (2));
	}
}

void	RadioInterface::PMSDR_setFilter4	(void) {
	if (PMSDR_preSelectorMode == MANUAL) {
	   ((pmSDR *)MyRig) -> setFilter (3);
	   PMSDR_bandlabel -> setText (((pmSDR *)MyRig) -> getFilterRange (3));
	}
}

void	RadioInterface::PMSDR_setFilter5	(void) {
	if (PMSDR_preSelectorMode == MANUAL) {
	   ((pmSDR *)MyRig) -> setFilter (0);
	   PMSDR_bandlabel -> setText (((pmSDR *)MyRig) -> getFilterRange (0));
	}
}

void	RadioInterface::PMSDR_setmute		(void) {
	PMSDR_mute = !PMSDR_mute;
	((pmSDR *)MyRig) -> setMute	(PMSDR_mute);
}
/*
 *	ifgain expects a value 0 .. 4
 */
void	RadioInterface::PMSDR_setIfgain		(int v) {
	((pmSDR *)MyRig)	-> setGain	(v);
	pmsdr_gain		-> display (v);
}
/*
 *	setbias expects a value -50 .. 50
 */
void	RadioInterface::PMSDR_setBias		(int v) {
	((pmSDR *)MyRig)	-> setBias	(v);
	pmsdr_bias		-> display (v);
}

void	RadioInterface::PMSDR_PreSelectorMode (const char *s) {
	PMSDR_preSelectButton -> 
	               setText (QApplication::
	                        translate ("elektorSDR", s, 0,
	                                   QApplication::UnicodeUTF8));
}

void	RadioInterface::PMSDR_PreSelectionMode (void) {

	PMSDR_preSelectorMode	=
	          (PMSDR_preSelectorMode != MANUAL) ? MANUAL : AUTOMATIC; 

	if (PMSDR_preSelectorMode == MANUAL) 
	   PMSDR_PreSelectorMode ("manual");
	else {
	   PMSDR_PreSelectorMode ("automatic");
	   setPreSelector (CurrentFreq);
	}
}

void	RadioInterface::set_symbolLeftshift	(int s) {
	symbol_Leftshift	= s;
}

void	RadioInterface::HandleDeviceError	(int e) {

	fprintf (stderr, "PMSDR does not function %d\n", e);
	QMessageBox::warning (this, tr ("sdr"),
	                            tr ("PMSDR is not connected anymore")); 
	
	delete MyRig;
	if (lcd_timer -> isActive ())
	   lcd_timer -> stop ();
	CurrentRig	= NO_RIG;
	MyRig		= new no_rig ();
	CurrentRig	= NO_RIG;
	rigSelect	-> setCurrentIndex (0);
	setrigControlScreen (CurrentRig);
	Set_Tuner (MyRig -> defaultFrequency (), &CurrentFreq, &TunedIF);
	lcd_Frequency	-> display (CurrentFreq);
}

//
//	These functions currently only make sense when
//	a dll is controling a device
//	we queue the requests and once every 10 msec we try to
//	process the requests
void	RadioInterface::setDLLRequest	(int r) {
	requestQueue. enqueue (r);
}

void	RadioInterface::handleDLLRequest (void) {
int32_t	myRequest;
dll_handler	*myHandler = (dll_handler *)MyRig;

	if ((runMode == RUNNING) && MyRig -> RigprovidesSamples ())
	   JansRadioThroughSignal ();

	while (!requestQueue. isEmpty ()) {
	   myRequest = requestQueue. dequeue (); 
	   switch (myRequest) {
	      case 100:			// sampling frequency changed
	      {  int32_t newRate = myHandler -> GetHWSR ();
	         fprintf (stderr, "External rate set to %d\n", newRate);
	         myHandler -> setExternalRate (newRate);
	      }
	         break;

	      case 101:			// change LO
	      {  int32_t vfo	= myHandler -> GetHWLO ();
                 TunedIF	= _ICardRate / 4;
	         CurrentFreq	= vfo + TunedIF;
	         HFScope	-> setZero (vfo);
	         HFScope	-> setNeedle (TunedIF);
	         MyFrontEnd	-> setSelectedFrequencyto (TunedIF,
	                                           bandLowend, bandHighend);
	         lcd_Frequency	-> display (CurrentFreq);
	      }
	      break;

	      case 102:			// block changes in frequency
	         FrequencyChangeAllowed	= false;
	         break;

	      case 103:			// unblock
	         FrequencyChangeAllowed = true;
	         break;

	      case 104:			// not implemented
	         break;

	      case 105:			// Tuned frequency changed
	      {  int32_t newFreq	= myHandler -> GetTune ();
                 int32_t vfo	= newFreq - _ICardRate / 4;
                 int32_t TunedIF	= _ICardRate / 4;
	         HFScope		-> setZero (vfo);
	         HFScope		-> setNeedle (_ICardRate / 4);
	         MyFrontEnd	-> setSelectedFrequencyto (TunedIF,
	                                           bandLowend, bandHighend);
	         lcd_Frequency	-> display (vfo + TunedIF);
	      } 
	      break;

	      case 106:			// changed demodulation mode
	      {  uint8_t r = myHandler -> GetMode ();
	         const char *selector;
	         int16_t k;
	         fprintf (stderr, "Mode received %c\n", r);
	         switch (r) {
	            case 'A':
	               selector = "am";
	               break;

	            case 'L':
	               selector = "lsb";
	               break;

	            case 'U':
	               selector = "usb";
	               break;

	            case 'F':
	            case 'E':
	               selector = "am";
	               break;

	            case 'C':
	               selector = "usb";
	               break;

	            default:
	               selector = "am";
	               break;
	         }
	         k	= quickbandSelect  -> findText (selector);
	         if (k != -1) {
	            quickbandSelect -> setCurrentIndex (k);
	            setquickBand (selector);
	         }
	         break;
	      }

	      case 107:			// behave as if pressed start
	         if (runMode == IDLE)
	            setStart ();
	         break;

	      case 108:			// behave as if pressed stop
	         if (runMode == RUNNING)
	            clickPause ();
	         break;

	      case 109:			// change passband limits
	      {  int32_t lo, hi, pitch;
	         myHandler -> GetFilters (lo, hi, pitch);
//	         fprintf (stderr, "filter %d -> %d, %d\n", lo, hi, pitch);
	         set_bandWidthSelect ((hi - lo) * 1000 / _WorkingRate);
	         set_bandWidthOffset (pitch * 1000 / _WorkingRate);
	         bandwidthSlider -> setValue ((hi - lo) * 1000 / _WorkingRate);
	         bandoffsetSlider -> setValue (pitch * 1000 / _WorkingRate);
	      }
	      break;  

	      default:
	         break;
	   }
	}
}

void	RadioInterface::set_dongleGain (int t) {
	if (CurrentRig == DABSTICK) {
	   t = ((dabstick *)MyRig) ->  setExternalGain (t);
	   dongleGainDisplay -> display (t);
	}
	else
	if (CurrentRig == DABSTICK_DLL) {
	   t = ((dabstick_dll *)MyRig) -> setExternalGain (t);
	   dongleGainDisplay -> display (t);
	}
}

void	RadioInterface::set_dongleRate (const QString &s) {
int32_t	t = s. toInt ();

	if (CurrentRig == DABSTICK) {
	   t = ((dabstick *)MyRig) -> setExternalRate (t);
	   dongleRateDisplay -> display (t);
	}
	else
	if (CurrentRig == DABSTICK_DLL) {
	   t = ((dabstick_dll *)MyRig) -> setExternalRate (t);
	   dongleRateDisplay -> display (t);
	}
}

void	RadioInterface::wheelEvent (QWheelEvent *e) {
	if (e -> delta () > 0)
	   Inc_Tuner (100);
	else
	   Inc_Tuner (-100);
}

