#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in ESDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ESDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef _GUI
#define _GUI

#include	<QDialog>
#include	"ui_sdrgui.h"
#include	<qwt.h>
#include	<qwt_slider.h>
#include	<qwt_plot.h>
#include	<qwt_plot_curve.h>
#include	<qwt_plot_marker.h>
#include	<qwt_plot_grid.h>
#include	<qwt_dial.h>
#include	<qwt_dial_needle.h>
#include	<qwt_plot_spectrogram.h>
#include	<QTimer>
#include	<QQueue>
#include	<QWheelEvent>
#include	"jff-include.h"
#include	"scope.h"
#include	"iqdisplay.h"
#include	"virtualrig.h"
#include	"pa_writer.h"
#include	"pa_reader.h"
#include	<sndfile.h>

class	LowPassFIR;
class	LowPassIIR;
class	BandPassFIR;
class	Oscillator;
class	spectrumManager;
class	QSettings;
class	Filtering;
class	agcHandler;
class	fft_scope;
class	virtualDecoder;
class	downDecimator;
/*
 *	GThe main gui object. It inherits from
 *	QDialog and the generated form
 */
class RadioInterface: public QDialog,
		      private Ui_SDR {
Q_OBJECT
public:
		RadioInterface		(QSettings *,
	                                 bool ,
	                                 int32_t,	// Icardrate
	                                 int32_t,	// Ocardrate
	                                 int16_t,	// displaysize
	                                 int16_t,	// rastersize
	                                 DSPCOMPLEX	*, // sin table
	                                 QWidget *parent = NULL);
		~RadioInterface		();

private:
	enum Decoders {
	   ANALOG_DETECTOR =	0101,
	   CW_DETECTOR	   =	0102,
	   RTTY_DETECTOR =	0103,
	   AMTOR_DETECTOR =	0104,
	   PSK_DETECTOR	=	0105,
	   MFSK_DETECTOR =	0106,
	   SDR_DECODER =	0107,
	   OLIVIA_DETECTOR =	0110,
	   FAX_DETECTOR	=	0111,
	   DOMINO_DETECTOR =	0112
	};

	enum Modi {
	   RADIO	=	01,
	   BACKEND	=	02
	};

	enum	PreSelector {
	   MANUAL	= 01,
	   AUTOMATIC	= 02
	};

	QSettings	*ISettings;
	bool		doInit;
	paWriter	*MySoundcardOut;
	paReader	*MySoundcardIn;
	bool		setupSoundOut		(QComboBox *,
	                                         paWriter	*,
	                                         int32_t, int16_t *);
	bool		setupSoundIn		(QComboBox	*,
	                                         paReader	*,
	                                         int32_t, int16_t *);
	int32_t		_ICardRate;
	int32_t		_OCardRate;
	int32_t		_WorkingRate;
	int16_t		upCount;
	int16_t		downCount;
	int8_t		runMode;
	void		setInitialControlState	(void);
	void		setControlState		(QSettings *);
	void		dumpControlState	(QSettings *);
	int16_t		displaywidth;
	int16_t		rastersize;
	DSPCOMPLEX	*Table;

	int32_t		showAgcCount;
	int16_t		*inTable;
	int16_t		*outTable;
	int16_t		numberofDevices;
	bool		createStreamSelectors	(void);
	int16_t		inputDevice;
	int16_t		outputDevice;
	char		*inputFilename;

	void		localConnects		(void);
	void		setAnalogSliders	(void);
	void		unsetAnalogSliders	(void);
	void		setCwSliders		(void);
	void		unsetCwSliders		(void);
	void		setRttySliders		(void);
	void		unsetRttySliders	(void);
	void		setPskSliders		(void);
	void		unsetPskSliders		(void);
	void		setMfskSliders		(void);
	void		unsetMfskSliders	(void);
	void		setAmtorSliders		(void);
	void		unsetAmtorSliders	(void);
	void		setFaxSliders		(void);
	void		unsetFaxSliders		(void);
	void		setDominoSliders	(void);
	void		unsetDominoSliders	(void);
	void		setOliviaSliders	(void);
	void		unsetOliviaSliders	(void);

	void		Set_Tuner		(int32_t, int32_t *, int32_t *);
	void		Set_TunerThirdHarmonic	(int32_t, int32_t *, int32_t *);
	void		Tuneto			(int32_t,
	                                         int32_t, int32_t, int32_t);
	bool		forBackendOnly		(DSPCOMPLEX , DSPCOMPLEX *);
	bool		RigwithThirdHarmonic	(virtualRig *);
	bool		IandQ_switch;

	QQueue<int>	requestQueue;
	LowPassFIR	*BackendOnlyFilter;
	uint16_t	BackendState;
	Oscillator	*BackendShifter;

	downDecimator	*myDecimator;
	DSPCOMPLEX	sampleSum;
	uint16_t	cc;
	DSPCOMPLEX	*Circular;
	int16_t		symbol_Leftshift;

	uint8_t		CurrentRig;
	Filtering	*MyFrontEnd;
	virtualDecoder	*MyDetector;

	spectrumManager	*LFSpectrum;
	fft_scope	*HFScope;
	fft_scope	*BackendScope;

	Scope		*mfsk_Scope;
	int		*mfsk_X;
	double		*mfsk_Y;
	int16_t		mfsk_count;

	uint8_t		LFViewMode;
	fft_scope	*LFScope;

	IQDisplay	*IQScope;
	int		IQDisplay_Ampl;

	LowPassIIR	*upFilter;
	BandPassFIR	*secondFilter;
	virtualRig	*MyRig;

	uint8_t		inputMode;
	uint8_t		PMSDR_preSelectorMode;
	uint8_t		PMSDR_preSelectorBand;
	uint8_t		PMSDR_mute;

	uint8_t		FrontEndoperation;
	int32_t		TunedIF;
	int32_t		CurrentFreq;
	int32_t		NeedlePosition;
	int32_t		DetectorOffset;

	uint16_t	currAttSliderValue;
	DSPFLOAT	attValueL;
	DSPFLOAT	attValueR;
	DSPFLOAT	Bl;
	DSPFLOAT	Br;
	agcHandler	*agc;

	uint8_t		HFViewMode;
	void		createpskPhaseDisplay	(void);

	void		ClearPanel		(void);
	void		AddtoPanel		(int);
	int32_t		getPanel		(void);
	void		CorrectPanel		(void);

	int32_t		bandOffset;
	int32_t		bandLowend;
	int32_t		bandHighend;
	uint8_t		FitsInWindow		(int32_t);
	void		Inc_Tuner		(int32_t);
	void		IncrementFrequencyforBackend	(int32_t);

	void		setPreSelector		(int32_t);

	int32_t		lcd_allocator;
	QTimer		*lcd_timer;
	int32_t		Panel;
	QTimer		*dll_timer;
/*
 */
	bool		FrequencyChangeAllowed;
	uint8_t		AnalogDecoder;
	QTimer		*display_timer;
	QTimer		*scope_timer;
	uint8_t		adaptiveFilterisOn;
	QString		pskText;
	QString		cwText;
	QString		rttyText;
	QString		mfskText;
	QString		dominoText;
	QString		dominoText2;
	QString		oliviaText;
	QString		amtorText;
/*
 *	dumping
 */
	int		DumpisOn;
	void		dump		(DSPCOMPLEX *, int32_t);
	int32_t		dumpBufferpointer;
	int32_t		dumpBuffersize;
	float		*dumpBuffer;
	char		*dumpFilename;
	SNDFILE		*DumpfilePointer;
/*
 * 	setting the selected rig
 */

	void	setDetectorScreen	(int8_t);
	void	setrigControlScreen	(int8_t);
/*
 *	The UI acts as a kind of server, the public slots are the entries
 *	for the radio elements, signals (below) are the triggers
 *	for the radio elements.
 */
private slots:
	void		setDLLRequest		(int);
	void		handleDLLRequest	(void);
	void	clickPause		(void);
	void	set_offsetSlider	(int);
	void	setStart		(void);
	void	AdjustFrequencywithKHz	(int);
	void	IQDisplay_setAmpl	(int);

	void	set_HFplotterView	(const QString &);
	void	set_LFplotterView	(const QString &);
	void	lcd_timeout		(void);
	void	updateTimeDisplay	(void);
	void	setDetectorMarker	(int);

	void	setInMiddle		(void);
	void	set_HFspectrumAmplitude	(int);
	void	set_LFspectrumAmplitude	(int);
//
//	decoder stuff, reacting on gui actions

	void	set_symbolLeftshift	(int);
	void	set_AnalogVolume	(int);
	void	set_AdaptiveFiltersize	(int);
	void	set_AnalogLPFraction	(int);
	void	set_AnalogHPFraction	(int);
	void	switchadaptiveFilter	(void);
	void	set_Squelch		(int);

	void	set_pskAfcon		(const QString &);
	void	set_pskReverse		(const QString &);
	void	set_pskSquelchLevel	(int);
	void	set_pskMode		(const QString &);
	void	set_pskFilterDegree	(int);

	void	set_cwWordsperMinute	(int);
	void	set_cwTracking		(const QString &);
	void	set_cwSquelchValue	(int);
	void	set_cwFilterDegree	(int);
	void	set_rttyAfcon		(const QString &);
	void	set_rttyReverse		(const QString &);
	void	set_rttyShift		(const QString &);
	void	set_rttyBaudrate	(const QString &);
	void	set_rttyParity		(const QString &);
	void	set_rttyMsb		(const QString &);
	void	set_rttyNbits		(const QString &);
	void	set_rttyStopbits	(const QString &);

	void	set_mfskAfcon		(const QString &);
	void	set_mfskReverse		(const QString &);
	void	set_mfskMode		(const QString &);
	void	set_mfskSquelchLevel	(int);
	void	set_mfskIFadjuster	(int);

	void	set_amtorAfcon		(const QString &);
	void	set_amtorReverse	(const QString &);
	void	set_amtorFecError	(const QString &);
	void	set_amtorMessage	(const QString &);

	void	set_dominoMode		(const QString &);
	void	set_dominoSquelch	(int);
	void	set_dominoFec		(const QString &);
	void	set_dominoReverse	(const QString &);

	void	set_oliviaTones		(const QString &);
	void	set_oliviaBW		(const QString &);
	void	set_oliviaSyncMargin	(int);
	void	set_oliviaSyncIntegLen	(int);
	void	set_oliviaSquelch	(const QString &);
	void	set_oliviaSquelchvalue	(int);
	void	set_oliviaReverse	(const QString &);

	void	set_analogDecoder	(bool);
	void	set_cw			(bool);
	void	set_rtty		(bool);
	void	set_psk			(bool);
	void	set_mfsk		(bool);
	void	set_amtor		(bool);
	void	set_analyzer		(bool);
	void	set_fax			(bool);
	void	set_domino		(bool);
	void	set_olivia		(bool);

	void	set_bandWidthSelect	(int);
	void	set_bandWidthOffset	(int);
	void	setquickBand		(const QString &);
	void	setInputMode		(const QString &);
	void	setOperation		(const QString &);
	void	setAGCMode		(const QString &);
	void	addOne			();
	void	addTwo			();
	void	addThree		();
	void	addFour			();
	void	addFive			();
	void	addSix			();
	void	addSeven		();
	void	addEight		();
	void	addNine			();
	void	addZero			();
	void	addClear		();
	void	AcceptFreqinHz		();
	void	AcceptFreqinKhz		();
	void	addCorr			();
	void	setAttenuator		(int);
	void	setBalanceSlider	(int);

	void	decT5			();
	void	decT50			();
	void	decT500			();
	void	decT5000		();
	void	incT5			();
	void	incT50			();
	void	incT500			();
	void	incT5000		();

	void	PMSDR_PreSelectionMode	(void);
	void	PMSDR_setFilter1	(void);
	void	PMSDR_setFilter2	(void);
	void	PMSDR_setFilter3	(void);
	void	PMSDR_setFilter4	(void);
	void	PMSDR_setFilter5	(void);
	void	PMSDR_setmute		(void);
	void	PMSDR_setIfgain		(int);
	void	PMSDR_setBias		(int);
	void	PMSDR_PreSelectorMode	(const char *);

	void	set_dongleRate		(const QString &);
	void	set_dongleGain		(int);
	void	setStreamInSelector	(int);
	void	setStreamOutSelector	(int);
	void	abortSystem		(int);
	void	TerminateProcess	();

	void	setWorkingRate		(const QString &);
	void	set_rigSelect		(const QString &);
	void	setDumpisOn		(void);
	void	setDumpisOff		(void);
	void	HandleSamples		(DSPCOMPLEX *, int32_t);

	void	set_agcThresholdSlider	(int);
public slots:
	void	wheelEvent		(QWheelEvent *);
	void	JansRadioThroughSignal	(void);
	void	AdjustFrequency		(int);
	void	addtoIQplot		(double, double);
	void	showPeak		(int, double);

	void	fax_setClicked		(int, int);
	void	HandleDeviceError	(int);
//
//	decoder stuff
	void	Analog_setText		(QString);
	void	Analog_showStrength	(float);
	void	Analog_showIF		(float);
	void	Analog_setDecoder	(const QString &);

	void	add_pskText		(char);
	void	clr_pskText		();
	void	show_pskDial		(float);
	void	show_pskQuality		(float);
	void	show_pskIF		(float);

	void	add_cwText		(char);
	void	clr_cwText		();
	void	add_cwToken		(const char *);
	void	show_cwdotLength	(int);
	void	show_cwspaceLength	(int);
	void	show_cwspeed		(int);
	void	show_agcpeak		(int);
	void	show_noiseLevel		(int);

	void	clr_rttyText		();
	void	add_rttyText		(char);
	void	show_rttyStrength	(float);
	void	show_rttyFreqCorrection	(float);
	void	show_rttyBaudRate	(int);
	void	show_rttyGuess		(int);

	void	mfsk_clrText		();
	void	mfsk_addText		(char);
	void	mfsk_showCF		(float);
	void	mfsk_addLocalScope	(int, double);
	void	mfsk_showLocalScope	(int);
	void	mfsk_showS2N		(int);
	void	mfsk_showTuning		(int);
	void	mfsk_showEstimate	(float);

	void	clr_amtorText		();
	void	add_amtorText		(char);
	void	show_amtorStrength	(float);
	void	show_amtorCorrection	(float);

	void	fax_showState		(const QString &);
	void	fax_showLinenumber	(int);
	void	fax_showaptFreq		(int);
	void	fax_displayImage	(QImage);
	void	fax_displayImage	(QImage, int, int);
	void	fax_setIOC		(const QString &);
	void	fax_setMode		(const QString &);
	void	fax_setDeviation	(int);
	void	fax_setSkip		(void);
	void	fax_setLPM		(int);
	void	fax_setphase		(const QString &);
	void	fax_setfaxColor		(const QString &);
	void	fax_setstartFreq	(int);
	void	fax_setstopFreq		(int);
	void	fax_setCarrier		(int);
	void	fax_setsaveButton	(const QString &);
	void	fax_saveFile		(QImage);
	void	fax_setdecoderSelector	(const QString &);
	void	fax_slantText		(const QString &);

	void	domino_clrText		(void);
	void	domino_addText		(char c);
	void	domino_addText2		(char c);
	void	domino_showS2N		(int);

	void	olivia_clrText			(void);
	void	olivia_addChar			(char);
	void	olivia_put_Status1		(char *);
	void	olivia_put_Status2		(char *);
};

#endif
