#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ESDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 *	Main program
 */
#include	<QApplication>
#include	<QSettings>
#include	<QDir>
#include	"jff-include.h"
#include	"utilities.h"
#include	"gui.h"
#include	"sw-constants.h"

void	fullPathfor (const char *v, char *out) {
int16_t	i;
QString	homeDir;

	if (v == NULL) {
	   sprintf (out, "%s", MISERY);
	   return;	// should not happen
	}

	if (v [0] == '/') {		// full path specified
	   sprintf (out, "%s", v);
	   return;
	}

	homeDir	= QDir::homePath ();
	homeDir. append ("/");
	homeDir. append (v);
	homeDir	= QDir::toNativeSeparators (homeDir);
	sprintf (out, "%s", homeDir. toAscii (). data ());
	fprintf (stderr, "ini file = %s\n", out);

	for (i = 0; out [i] != 0; i ++);
	if (out [i - 4] != '.' ||
	    out [i - 3] != 'i' ||
	    out [i - 2] != 'n' ||
	    out [i - 1] != 'i') {
	    out [i] = '.';
	    out [i + 1] = 'i';
	    out [i + 2] = 'n';
	    out [i + 3] = 'i';
	    out [i + 4] = 0;
	}
}

bool	fileExists (char *v) {
FILE *f;

	f = fopen (v, "r");
	if (f == NULL)
	   return false;
	fclose (f);
	return true;
}

int	main (int argc, char **argv) {
int16_t		i;
int32_t		tmp;
int32_t		opt;

int16_t		displaysize		= FE_DISPLAYSIZE;
int16_t		rastersize		= RASTER;

QSettings	*ISettings;			/* persistent settings	*/
int32_t		CardRate	= 96000;	/* settable through cmdline */
int32_t		OutputRate	= 48000;	/* for now: fixed	*/
RadioInterface	*MyRadioInterface;
bool		showGlobals	= true;
DSPCOMPLEX	*Table;
bool		doInit		= true;
char		*initFilename	= NULL;
char		*defaultInit	= (char *)alloca (1024 * sizeof (char));

	fullPathfor (DEFAULT_INI, defaultInit);

	while ((opt = getopt (argc, argv, "SsRVdneTA:i:o:C:P:")) != -1) {
	   switch (opt) {
	      case 'A':	tmp  = atoi (optarg);
	                displaysize = nearestTwoPower (tmp);
	                break;

	      case 'C':	rastersize = atoi (optarg);
	                break;

	      case 'V':	showGlobals = TRUE;
	                break;

	      case 'S':	CardRate	= 48000;
	                OutputRate	= 24000;
	                break;

	      case 'R':	CardRate	= 96000;
	                OutputRate	= 48000;
	                break;

	      case 'T':	CardRate	= 192000;
	                OutputRate	= 48000;
	                break;

	      case 'i':	initFilename	=
	                            (char *) alloca (512 * sizeof (char));
	                fullPathfor (optarg, initFilename);
	                i ++;
	                break;

	      case 'd':	doInit		= false;
	                break;

	      default:
	                break;
	      }
	}

	if (showGlobals) {
	   qDebug ("fe_displaysize (option A) = %d\n", displaysize);
	   qDebug ("fe_raster   (option C)    = %d\n", rastersize);
	}
/*
 *	... and the settings of the "environment"
 */
	if (initFilename == NULL) 	// -n flag
	   initFilename = defaultInit;
	ISettings	= new QSettings (initFilename, QSettings::IniFormat);

	Table		= (DSPCOMPLEX *)alloca (CardRate * sizeof (DSPCOMPLEX));
	for (i = 0; i < CardRate; i ++)
	   Table [i] = DSPCOMPLEX (cos (2 * M_PI * i / CardRate),
	                           sin (2 * M_PI * i / CardRate));
/*
 *	Before we connect control to the gui, we have to
 *	instantiate
 */
	QApplication a (argc, argv);
	MyRadioInterface = new RadioInterface (ISettings,
	                                       doInit,
	                                       CardRate,
	                                       OutputRate,
			                       displaysize,
	                                       rastersize,
	                                       Table);

	MyRadioInterface -> show ();
	a. exec ();
/*
 *	done:
 */
	fflush (stdout);
	fflush (stderr);
	qDebug ("It is done\n");

	if (ISettings != NULL)
	   ISettings	-> ~QSettings ();
	exit (1);
}

