#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy 
 *
 *    This file is part of the JFF SDR (JSDR).
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ESDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	The decoder for the analog modes
 */
#
#include		"gui.h"
#include		"analogdecoder.h"
#include		"oscillator.h"
#include		"pllC.h"
#include		"squelchClass.h"

extern	DSPCOMPLEX	ComplexZero;
#define	ANALOG_IF	0
#define	FBANDWIDTH	10000


	analogDecoder::analogDecoder (int32_t workingR, RadioInterface	*RI) {
int16_t squelchFreq;

	_WorkingRate		= workingR;
	MyRadioInterface	= RI;

	analog_IF		= ANALOG_IF;
	analog_LowPass		= 99;
	LPM_Filter = new LowPassIIR (6,
	                             analog_LowPass * _WorkingRate / 200,
	                             _WorkingRate, S_BUTTERWORTH);

	analog_HighPass		= (100 - 99);
	HPM_Filter = new HighPassIIR (6,
	                              analog_HighPass * _WorkingRate / 200,
	                              _WorkingRate, S_BUTTERWORTH);
	adaptiveFiltersize	= 15;		/* the default	*/
	adaptive		= new adaptiveFilter	(adaptiveFiltersize);

	analog_Volume		= 30;
/*
 *	for the FM decoder we need:
 */
	Imin1			= 1;
	Imin2			= 1;
	Qmin1			= 1;
	Qmin2			= 1;

	detectorType		= AM_DECODER;
	SSB_Filter		= new HilbertFilter (31, 0.25, _WorkingRate);

	am_pll			= new pllC (_WorkingRate,
	                                   0,		// center frequency
	                                   - 500,	// low offset
	                                   + 500,	// high offset
	                                   4000);	// bandwidth

	if (_WorkingRate <= 8000)
	   squelchFreq = _WorkingRate / 3;
	else
	   squelchFreq = 3000;
	mySquelch		= new squelch (1, squelchFreq,
	                                       _WorkingRate / 20, _WorkingRate);
	currLock		= 0.5;
	prevLock		= 1.0;
	amDc			= 0;
	amSmooth		= 0;
	connect (this,
	         SIGNAL (Analog_showStrength (float)),
	         MyRadioInterface,
	         SLOT (Analog_showStrength (float)));
	connect (this,
	         SIGNAL (Analog_showIF (float)),
	         MyRadioInterface,
	         SLOT (Analog_showIF (float)));
	connect (this,
	         SIGNAL (Analog_setText (QString)),
	         MyRadioInterface,
	         SLOT (Analog_setText (QString)));
	connect (this,
	         SIGNAL (Analog_shiftFrequency (int)),
	         MyRadioInterface,
	         SLOT (AdjustFrequency (int)));
	connect (this,
	         SIGNAL (setDetectorMarker (int)),
	         MyRadioInterface,
	         SLOT (setDetectorMarker (int)));

        showDetectorType ();
	adaptive	-> adaptFilter (0.1);
	adaptiveFiltering = FALSE;
	CycleCount	= 0;
	setDetectorMarker	(analog_IF);
}

	analogDecoder::~analogDecoder () {
	disconnect (this,
	            SIGNAL (Analog_showStrength (float)),
	            MyRadioInterface,
	            SLOT (Analog_showStrength (float)));
	disconnect (this,
	            SIGNAL (Analog_setText (QString)),
	            MyRadioInterface,
	            SLOT (Analog_setText (QString)));


	delete	LPM_Filter;
	delete	HPM_Filter;
	delete	SSB_Filter;
	delete	am_pll;
	delete	adaptive;
}
/*
 *	if the workingrate is changed, we have to adapt
 *	the workingrate-dependent stuff
 */
void	analogDecoder::setWorkingRate	(int32_t workingRate) {
int16_t	squelchFreq;

	if (_WorkingRate == workingRate)
	   return;

	delete	LPM_Filter;
	LPM_Filter = new LowPassIIR (6,
	                             analog_LowPass * _WorkingRate / 200,
	                             _WorkingRate, S_BUTTERWORTH);

	delete HPM_Filter;
	HPM_Filter = new HighPassIIR (6,
	                              analog_HighPass * _WorkingRate / 200,
	                              _WorkingRate, S_BUTTERWORTH);

	delete	adaptive;
	adaptive	= new adaptiveFilter	(adaptiveFiltersize);

	delete	SSB_Filter;
	SSB_Filter		= new HilbertFilter (31, 0.25, _WorkingRate);

	delete	am_pll;
	am_pll			= new pllC (_WorkingRate,
	                                   0,
	                                   - 500,
	                                   + 500,
	                                   0.9 * _WorkingRate);

	squelchON		= true;
	squelchFreq		= _WorkingRate <= 8000 ?
	                                       _WorkingRate / 3 : 3000;
	mySquelch		= new squelch (0.1,
	                                       squelchFreq, 512, _WorkingRate);
	CycleCount		= _WorkingRate / 10;
}

void	analogDecoder::Analog_setDecoder (int d) {
	detectorType	= d;
	this -> showDetectorType ();
	if (d == COHERENT)
	   analog_IF = ANALOG_IF;
	else
	   analog_IF = 0;
	setDetectorMarker (analog_IF);
}

void	analogDecoder::Analog_setVolume (int d) {
	analog_Volume	= d;
}

void	analogDecoder::Analog_setLPFraction (int d) {
	analog_LowPass	= d;
	delete LPM_Filter;
	LPM_Filter = new LowPassIIR (6,
	                             analog_LowPass * _WorkingRate / 200,
	                             _WorkingRate, S_BUTTERWORTH);
}

void	analogDecoder::Analog_setHPFraction (int d) {
	analog_HighPass	= (101 - d);
	delete HPM_Filter;
	HPM_Filter = new HighPassIIR (6,
	                              analog_HighPass * _WorkingRate / 200,
	                              _WorkingRate, S_BUTTERWORTH);
}

void	analogDecoder::Analog_setAdaptiveFilter (int d) {
	delete	adaptive;
	adaptive	= new adaptiveFilter	(adaptiveFiltersize);
}

void	analogDecoder::Analog_setAdaptiveFiltersize (int d) {
	adaptiveFiltersize	= d;
	delete	adaptive;
	adaptive	= new adaptiveFilter	(adaptiveFiltersize);
}

void	analogDecoder::showDetectorType () {
	switch (detectorType) {
	   case AM_DECODER:
	      Analog_setText (QString("Decoding AM"));
	      break;

	   case FM_DECODER:
	      Analog_setText (QString ("Decoding FM"));
	      break;

	   case USB_DECODER:
	      Analog_setText (QString ("Decoding USB"));
	      break;

	   case LSB_DECODER:
	      Analog_setText (QString ("Decoding LSB"));
	      break;

	   case ISB_DECODER:
	      Analog_setText (QString ("Decoding ISB"));
	      break;

	   case COHERENT:
	      Analog_setText (QString ("Decoding AM with pll"));
	      break;

	   default:
	      Analog_setText (QString ("Do not know"));
	      break;
	}
}

DSPCOMPLEX	analogDecoder::doDecode (DSPCOMPLEX z) {
double	power	=  abs (z);
double	analogStrength;
DSPCOMPLEX	lf;

	if (++CycleCount > _WorkingRate / 10) {
	   CycleCount = 0;

	   if (power > 0) {
	      analogStrength = (int)get_db (power);
	   }
	   else
	      analogStrength = 0;

	   Analog_showStrength	(analogStrength);
	   if (detectorType == COHERENT)
	      Analog_showIF	(currLock);
	   else
	      Analog_showIF	(0);
	}

	switch (detectorType) {
	   case AM_DECODER:
	      lf = amDecoder (z, (float)analog_Volume / 10.0);
	      break;

	   case FM_DECODER:
	      lf = fmDecoder (z, (float)analog_Volume / 10.0);
	      break;

	   case USB_DECODER:
	      lf = usbDecoder (z, (float)analog_Volume / 10.0);
	      break;

	   case LSB_DECODER:
	      lf = lsbDecoder (z, (float)analog_Volume / 10.0);
	      break;

	   case ISB_DECODER:
	      lf = isbDecoder (z, (float)analog_Volume / 10.0);
	      break;

	   case COHERENT:
	      lf = syncDecoder (z, (float)analog_Volume / 10.0);
	      break;

	   default:
	      return z;
	}

	lf	= LPM_Filter	-> Pass (lf);
	lf	= HPM_Filter	-> Pass (lf);
	if (adaptiveFiltering)
	   lf	= adaptive	-> Pass (lf);
	return lf;
}

/*
 */
DSPCOMPLEX	analogDecoder::fmDecoder (DSPCOMPLEX z, double Vol) {
DSPFLOAT	res;
DSPFLOAT	mag;

	double	I	= real (z);
	double	Q	= imag (z);
	res		= Imin1 * (Q - Qmin2) - Qmin1 * (I - Imin2);
	res		/= Imin1 * Imin1 + Qmin1 * Qmin1 ;
	Imin2		= Imin1;
	Qmin2		= Qmin1;
	Imin1		= I;
	Qmin1		= Q;
	res		= mySquelch -> do_squelch (res);
	return 	DSPCOMPLEX (Vol * res, Vol * res);
}

DSPCOMPLEX	analogDecoder::usbDecoder (DSPCOMPLEX z, double Vol) {
DSPCOMPLEX H;
	z	= SSB_Filter	-> Pass (z);
	H	= DSPCOMPLEX (Vol * (real (z) - imag (z)),
		           Vol * (real (z) - imag (z)));
	return H;
}

DSPCOMPLEX	analogDecoder::lsbDecoder (DSPCOMPLEX z, double Vol) {
DSPCOMPLEX H;
	z	= SSB_Filter	-> Pass (z);
	H	= DSPCOMPLEX (Vol * (real (z) + imag (z)),
			   Vol * (real (z) + imag (z)));
	return H;
}

DSPCOMPLEX	analogDecoder::isbDecoder (DSPCOMPLEX z, double Vol) {
DSPCOMPLEX H;
	z	= SSB_Filter -> Pass (z);
	H	= DSPCOMPLEX (Vol * (real (z) - imag (z)),
	                   Vol * (real (z) + imag (z)));
	return H;
}

#define	pllBeta	0.5
#define	pllGain	0.3
#define	NcoFreqHLimit	 0.8
#define	NcoFreqLLimit	 - 0.8
#define	DC_ALPHA	0.85

DSPCOMPLEX	analogDecoder::amDecoder (DSPCOMPLEX z, double Vol) {
	currLock	= Vol * abs (z);
	amDc		= 0.9999f * amDc + 0.0001f * currLock;
	amSmooth	= 0.5 * amSmooth + 0.5 * (currLock - amDc);
	return DSPCOMPLEX (amSmooth, amSmooth);
}
//
//	
DSPCOMPLEX	analogDecoder::syncDecoder (DSPCOMPLEX z, double Vol) {
DSPFLOAT	res;

	am_pll		-> do_pll (z);
	currLock	= 0.999 * currLock +
	                  0.001 * fabs (imag (am_pll -> getDelay ()));
	prevLock	= currLock;
	amDc		= 0.9999 * amDc + 0.0001 * real (am_pll -> getDelay ());
	res		= Vol * (real (am_pll -> getDelay ()) - amDc);
	return DSPCOMPLEX (res, res);
}

void		analogDecoder::Analog_setSquelch (int n) {
	mySquelch	-> setSquelchLevel (n);
}

