#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef ANALOG_DECODER
#define ANALOG_DECODER
#include	"jff-include.h"
#include	"virtualdecoder.h"
#include	<QObject>
#include	"iir-filters.h"
#include	"fir-filters.h"
#include	"utilities.h"
/*
 *	The analog decoder
 */
class	RadioInterface;
class	pllC;
class	squelch;

class analogDecoder: public virtualDecoder {
Q_OBJECT
public:
		analogDecoder		(int32_t, RadioInterface *);
		~analogDecoder		(void);
	void	setWorkingRate		(int32_t);
	DSPCOMPLEX	doDecode	(DSPCOMPLEX);

	enum Decoder {
	   AM_DECODER =		0121,
	   FM_DECODER =		0122,
	   USB_DECODER =	0123,
	   LSB_DECODER =	0124,
	   ISB_DECODER =	0125,
	   COHERENT =		0126
	};

	void	Analog_setDecoder		(int);
	void	Analog_setVolume		(int);
	void	Analog_setLPFraction		(int);
	void	Analog_setHPFraction		(int);
	void	Analog_setAdaptiveFilter	(int);
	void	Analog_setAdaptiveFiltersize	(int);	
	void	Analog_setSquelch		(int);
private:
	void	showDetectorType	();
	DSPCOMPLEX	fmDecoder		(DSPCOMPLEX, double);
	DSPCOMPLEX	usbDecoder		(DSPCOMPLEX, double);
	DSPCOMPLEX	lsbDecoder		(DSPCOMPLEX, double);
	DSPCOMPLEX	isbDecoder		(DSPCOMPLEX, double);
	DSPCOMPLEX	amDecoder		(DSPCOMPLEX, double);
	DSPCOMPLEX	syncDecoder		(DSPCOMPLEX, double);

	DSPFLOAT		analog_IF;
	int32_t			_WorkingRate;
	RadioInterface		*MyRadioInterface;
	uint16_t		CycleCount;

	DSPFLOAT		NcoPhase;
	DSPFLOAT		NcoPhaseIncr;
	DSPFLOAT		Imin1;
	DSPFLOAT		Imin2;
	DSPFLOAT		Qmin1;
	DSPFLOAT		Qmin2;

	LowPassIIR		*LPM_Filter;
	HighPassIIR		*HPM_Filter;
	adaptiveFilter		*adaptive;
	int16_t			adaptiveFiltersize;
	int16_t			old_adaptiveFiltersize;
	bool			adaptiveFiltering;
	HilbertFilter		*SSB_Filter;
	int16_t			detectorType;
	int16_t			analog_Volume;
	int16_t			analog_LowPass;
	int16_t			analog_HighPass;

	pllC			*am_pll;
	DSPFLOAT		currLock;
	DSPFLOAT		prevLock;
	DSPFLOAT		amDc;
	DSPFLOAT		amSmooth;
	bool			squelchON;
	squelch			*mySquelch;
signals:
	void	Analog_setText		(QString);
	void	Analog_showStrength	(float);
	void	Analog_shiftFrequency	(int);
	void	Analog_showIF		(float);
	void	setDetectorMarker	(int);

};

#endif
