#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    The initial code was more or less based on the code from Jesus Arias,
 *    the thing implemented as state machine. 
 *    An improved version - actually working with signals of sufficient
 *    strength - is based on the code of fldigi, especially with the
 *    estimation of the dot and dash lengths. Any copyright that might
 *    be applicable for lines of code inspired by code from others
 *    is gratefully acknowledged
 *
 */
#
#include	"gui.h"
#include	"./cwdecoder.h"
#include	"oscillator.h"

#define	USECS_PER_SEC		1000000
#define	MODE_IDLE		0100
#define	MODE_IN_TONE		0200
#define	MODE_AFTER_TONE		0300
#define	MODE_END_OF_LETTER	0400

#define	CW_DOT_REPRESENTATION	'.'
#define	CW_DASH_REPRESENTATION	'_'
#define	CW_RECEIVE_CAPACITY	020

#define	CW_IF			800
/*
 * 	The standard morse wordlength (i.e.
 * 	PARIS) is 50 bits, then for a wpm of 1,
 * 	one bit takes 1.2 second or 1.2 x 10^6	microseconds
 */
#define	DOT_MAGIC		1200000

extern const char *const codetable[];
/*
 *	The implementation is derived from the implementation of fldigi,
 */
	cwDecoder::cwDecoder (int32_t workingRate, RadioInterface *RI) {
	_WorkingRate		= workingRate;
	MyRadioInterface	= RI;

	LocalOscillator		= new Oscillator (_WorkingRate);
	CycleCount		= _WorkingRate / 10;
	cw_IF			= CW_IF;

	SmoothenSamples		= new average (4);
	thresholdFilter		= new average (8);
	spaceFilter		= new average (16);

	cwFilterDegree		= 8;		/* default	*/
	cwTracking		= TRUE;		/* default	*/
	SquelchValue		= 2;		/* default	*/
	cwDefaultSpeed		= 15;		/* default	*/
/*
 *	the filter is also a default
 */
	cw_LowPassFilter	= new LowPassIIR (cwFilterDegree,
	                                          75,
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);
	rx_rep_buf		= new char [CW_RECEIVE_CAPACITY];

/*
 *	There is no real need to set the following variables here,
 *	they will get a value anyhow when restarting
 */
	cwCurrent		= 0;
	agc_peak		= 0;
	noiseLevel		= 0;
	cwState			= MODE_IDLE;
	cwPreviousState		= MODE_IDLE;

	connect (this,
	         SIGNAL (cw_showdotLength (int)),
	         MyRadioInterface,
	         SLOT (show_cwdotLength (int)));
	connect (this,
	         SIGNAL (cw_showspaceLength (int)),
	         MyRadioInterface,
	         SLOT (show_cwspaceLength (int)));
	connect (this,
	         SIGNAL (cw_showspeed (int)),
	         MyRadioInterface,
	         SLOT (show_cwspeed (int)));
	connect (this,
	         SIGNAL (cw_showagcpeak (int)),
	         MyRadioInterface,
	         SLOT (show_agcpeak (int)));
	connect (this,
	         SIGNAL (cw_shownoiseLevel (int)),
	         MyRadioInterface,
	         SLOT (show_noiseLevel (int)));
	connect (this,
	         SIGNAL (cw_addToken (const char *)),
	         MyRadioInterface,
	         SLOT (add_cwToken (const char *)));
	connect (this,
	         SIGNAL (cw_addText (char)),
		 MyRadioInterface,
	         SLOT (add_cwText (char)));
	connect (this,
	         SIGNAL (cw_clrText ()),
		 MyRadioInterface,
	         SLOT (clr_cwText ()));
	connect (this,
	         SIGNAL (setDetectorMarker (int)),
	         MyRadioInterface,
	         SLOT (setDetectorMarker (int)));

//
//	and start up
	CycleCount		= _WorkingRate / 10;
//	SquelchValue		= 2;
//	cwDefaultSpeed		= 15;
//	cwTracking		= TRUE;
//	cwFilterDegree		= 8;
//	delete cw_LowPassFilter;
//	cw_LowPassFilter	= createIIRLowPass (cwFilterDegree,
//	                                            75,
//	                                            _WorkingRate,
//	                                            S_BUTTERWORTH);
	
	CWrange			= cwDefaultSpeed / 2;
	cwSpeed			= cwDefaultSpeed;
	lowerBoundWPM		= cwDefaultSpeed - CWrange;
	upperBoundWPM		= cwDefaultSpeed + CWrange;
	cwSpeed			= cwDefaultSpeed;	/* initial	*/

	cwDefaultDotLength	= DOT_MAGIC / cwDefaultSpeed;
	cwDotLength		= cwDefaultDotLength;
	cwSpaceLength		= cwDefaultDotLength;
	cwDotLengthMax		= DOT_MAGIC / lowerBoundWPM;
	cwDotLengthMin		= DOT_MAGIC / upperBoundWPM;

	cwDashLength		= 3 * cwDotLength;
	cwDashLengthMax		= 3 * cwDotLengthMax;
	cwDashLengthMin		= 3 * cwDotLengthMin;

	cw_adaptive_threshold	= 2 * cwDefaultDotLength;
	cwNoiseSpike		= cwDotLength / 3;

	cwStartTimestamp	= 0;
	cwEndTimestamp		= 0;

	cwCurrent		= 0;
	agc_peak		= 0;
	noiseLevel		= 0;
	
	memset			(rx_rep_buf, 0, sizeof (rx_rep_buf));
	thresholdFilter		-> clear (2 * cwDotLength);
	spaceFilter		-> clear (cwDotLength);
	cw_clrText		();
	cw_addToken		((const char *)" ");
}

	cwDecoder::~cwDecoder () {
	disconnect (this,
	            SIGNAL (cw_showdotLength (int)),
	            MyRadioInterface,
	            SLOT (show_cwdotLength (int)));
	disconnect (this,
	            SIGNAL (cw_showspaceLength (int)),
	            MyRadioInterface,
	            SLOT (show_cwspaceLength (int)));
	disconnect (this,
	            SIGNAL (cw_showspeed (int)),
	            MyRadioInterface,
	            SLOT (show_cwspeed (int)));
	disconnect (this,
	            SIGNAL (cw_showagcpeak (int)),
	            MyRadioInterface,
	            SLOT (show_agcpeak (int)));
	disconnect (this,
	            SIGNAL (cw_shownoiseLevel (int)),
	            MyRadioInterface,
	            SLOT (show_noiseLevel (int)));
	disconnect (this,
	            SIGNAL (cw_addToken (const char *)),
	            MyRadioInterface,
	            SLOT (add_cwToken (const char *)));
	disconnect (this,
	            SIGNAL (cw_addText (char)),
		    MyRadioInterface,
	            SLOT (add_cwText (char)));
	disconnect (this,
	            SIGNAL (cw_clrText ()),
		    MyRadioInterface,
	            SLOT (clr_cwText ()));
	delete		cw_LowPassFilter;
	delete		SmoothenSamples;
	delete		thresholdFilter;
	delete		spaceFilter;	
	delete[]	rx_rep_buf;
	delete		LocalOscillator;
}

void	cwDecoder::setWorkingRate	(int32_t workingRate) {
	if (_WorkingRate == workingRate)
	   return;

	this	-> _WorkingRate	= _WorkingRate;
	delete	LocalOscillator;
	LocalOscillator		= new Oscillator (_WorkingRate);
	delete	cw_LowPassFilter;
	cw_LowPassFilter	= new LowPassIIR (cwFilterDegree,
	                                          75,
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);
	CycleCount		= _WorkingRate / 10;
}

void	cwDecoder::cw_setWordsperMinute (int16_t n) {
	cwDefaultSpeed	= n;
	speedAdjust ();
}

void	cwDecoder::cw_setTracking (bool f) {
	if (cwTracking == f)
	   return;
	cwTracking	= f;
	speedAdjust ();
}

void	cwDecoder::cw_setSquelchValue (int16_t s) {
	SquelchValue	= s;
}

void	cwDecoder::cw_setFilterDegree (int16_t n) {
	cwFilterDegree	= n;
	delete cw_LowPassFilter;
	cw_LowPassFilter	= new LowPassIIR (cwFilterDegree,
	                                          75,
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);
}

void	cwDecoder::speedAdjust() {
	cwSpeed			= cwDefaultSpeed;	/* init	*/
	CWrange			= cwSpeed / 2;
	lowerBoundWPM		= cwDefaultSpeed - CWrange;
	upperBoundWPM		= cwDefaultSpeed + CWrange;

	cwDefaultDotLength	= DOT_MAGIC / cwDefaultSpeed;
	cwDotLength		= cwDefaultDotLength;
	cw_adaptive_threshold	= 2 * cwDefaultDotLength;

	cwDotLengthMax		= DOT_MAGIC / lowerBoundWPM;
	cwDotLengthMin		= DOT_MAGIC / upperBoundWPM;

	cwDashLength		= 3 * cwDotLength;
	cwDashLengthMax		= 3 * cwDotLengthMax;
	cwDashLengthMin		= 3 * cwDotLengthMin;

	cwNoiseSpike		= cwDotLength / 3;
	cwStartTimestamp	= 0;
	cwEndTimestamp		= 0;

	cwState			= MODE_IDLE;
	cwCurrent		= 0;
	memset			(rx_rep_buf, 0, sizeof (rx_rep_buf));
	thresholdFilter		-> clear (2 * cwDotLength);
	cw_clrText	();
	cw_addToken	((const char *)" ");
}

DSPCOMPLEX	cwDecoder::doDecode (DSPCOMPLEX z) {
DSPCOMPLEX	ret;
int32_t	lengthOfTone;
int32_t	lengthOfSilence;
int32_t	t;
char	buffer [4];
DSPCOMPLEX	s;

	if (++CycleCount > _WorkingRate / 10){
	   CycleCount	= 0;
	   cw_showdotLength	(cwDotLength);
	   cw_showspaceLength	(cwSpaceLength);
	   cw_showspeed		(cwSpeed);
	   cw_showagcpeak	(clamp (agc_peak * 1000.0, 0.0, 100.0));
	   cw_shownoiseLevel	(clamp (noiseLevel * 1000.0, 0.0, 100.0));
	   setDetectorMarker	(cw_IF);
	}

	s	= LocalOscillator	-> nextValue (cw_IF) * z;
	s	= cw_LowPassFilter	-> Pass (s);
	value	=  abs (s);

	if (value > agc_peak)
	   agc_peak = decayingAverage (agc_peak, value, 50.0);
	else
	   agc_peak = decayingAverage (agc_peak, value, 500.0);

	currentTime += USECS_PER_SEC / _WorkingRate;
	switch (cwState) {
	   case MODE_IDLE:
	      if ((value > 0.67 * agc_peak) &&
	          (value > SquelchValue * noiseLevel)) { 
/*
 *	we seem to start a new tone
 */
	         cwState		= MODE_IN_TONE;
	         currentTime		= 0;
	         cwCurrent		= 0;
	         cwStartTimestamp	= currentTime;
	         cwPreviousState	= cwState;
	      }
	      else 
	         noiseLevel		= decayingAverage (noiseLevel,
	                                                   value, 500.0);
	      break;				/* just wait	*/

	   case MODE_IN_TONE:
/*
 *	we are/were receiving a tone, either continue
 *	or stop it, depending on some threshold value.
 */
	      if (value > 0.33 * agc_peak)
	         break;			/* just go on	*/
/*
 *	if we are here, the tone has ended
 */
	      cwEndTimestamp	= currentTime;
	      lengthOfTone	= currentTime - cwStartTimestamp;

	      if (lengthOfTone < cwNoiseSpike) {
	         cwState = cwPreviousState;
	         break;
	      }

	      noiseLevel	= decayingAverage (noiseLevel, value, 500.0);

	      if (lengthOfTone <= cw_adaptive_threshold)
	         rx_rep_buf [cwCurrent ++] = CW_DOT_REPRESENTATION;
	      else
	         rx_rep_buf [cwCurrent ++] = CW_DASH_REPRESENTATION;

/*
 *	if we gathered too many dots and dashes, we end up
 *	with garbage.
 */
	      if (cwCurrent >= CW_RECEIVE_CAPACITY) {
	         cwCurrent = 0;
	         cwState = MODE_IDLE;
	         break;
	      }

	      rx_rep_buf [cwCurrent] = 0;
	      cw_addToken (rx_rep_buf);
	      cwState = MODE_AFTER_TONE;

	      if (cwTracking) {
	         t = newThreshold (lengthofPreviousTone, lengthOfTone);
	         if (t > 0) {
	            cw_adaptive_threshold = thresholdFilter -> filter (t);
	            cwDotLength		= cw_adaptive_threshold / 2;
	            cwDashLength	= 3 * cwDotLength;
	            cwSpeed		= DOT_MAGIC / cwDotLength;
	            cwNoiseSpike	= cwDotLength / 3;
	         }
	      }

	      lengthofPreviousTone = lengthOfTone;
	      break;

	   case MODE_AFTER_TONE:
/*
 *	following the end of the tone, we might go back for the
 *	next dash or dot, or we might decide that we reached
 *	the end of the letter
 */
	      if ((value > 0.67 * agc_peak) &&
	          (value > SquelchValue * noiseLevel)) {
	         int t = currentTime - cwEndTimestamp;
	         cwSpaceLength		= spaceFilter -> filter (t);
	         cwState		= MODE_IN_TONE;
	         cwStartTimestamp	= currentTime;
	         cwPreviousState	= cwState;
	         break;
	      }	
// 	no tone, adjust noiselevel and handle silence
	      noiseLevel	= decayingAverage (noiseLevel, value, 500.0);
	      lengthOfSilence	= currentTime - cwEndTimestamp;

	      if ((lengthOfSilence >= 2 * (cwDotLength + cwSpaceLength) / 2)) {
	         rx_lookup (rx_rep_buf, buffer);
	         cwCurrent = 0;
	         cwState = MODE_END_OF_LETTER;
	         cw_addText (buffer [0]);
	      }
//	otherwise, silence too short, do nothing as yet
	      break;
/*
 * 	end_of_letter may be followed by another letter or an end
 * 	of word indication
 */
	   case MODE_END_OF_LETTER:	/* almost MODE_IDLE	*/
	      if ((value > 0.67 * agc_peak) &&
	          (value > SquelchValue * noiseLevel)) {
	         cwState		= MODE_IN_TONE;
	         cwStartTimestamp	= currentTime;
	         cwPreviousState	= cwState;
	      }
	      else {
/*
 *	still silence, look what to do
 */
	         noiseLevel		= decayingAverage (noiseLevel,
	                                                   value, 500.0);
	         lengthOfSilence = currentTime - cwEndTimestamp;
	         if (lengthOfSilence > 4.0 * (cwSpaceLength + cwDotLength) / 2) {
	            cw_addText (' ');	/* word space	*/
	            cwState = MODE_IDLE;
	         }
	      }
	      break;

	   default:
	      break;
	   }
	return z;
}
/*
 * 	check for dot/dash or dash/dot sequence to adapt the
 * 	threshold.
 */
int32_t	cwDecoder::newThreshold (int32_t prev_element, int32_t curr_element) {
	      if ((prev_element > 0) && (curr_element > 0)) {
	         if ((curr_element > 2 * prev_element) &&
	             (curr_element < 4 * prev_element))
	            return  getMeanofDotDash (prev_element, curr_element);
/*
 * 	check for dash dot sequence
 */
	         if ((prev_element > 2 * curr_element) &&
	             (prev_element < 4 * curr_element))
	            return getMeanofDotDash (curr_element, prev_element);
	      }

	      return -1;
}
/*
 * 	if we have a dot/dash sequence, we might adapt
 * 	the adaptive threshold if both dot and dash length
 * 	are reasonable.
 */
int32_t	cwDecoder::getMeanofDotDash (int32_t idot, int32_t idash) {
int32_t	dot, dash;

	if (idot > cwDotLengthMin &&  idot < cwDotLengthMax)
	   dot = idot;
	else
	   dot = cwDotLength;

	if (idash > cwDashLengthMin && idash < cwDashLengthMax)
	   dash = idash;
	else
	   dash = cwDashLength;

	return (dash + dot) / 2;
}

void 	cwDecoder::printChar (char a, char er) {
	if ((a & 0x7f) < 32) {
	   switch (a) {
	      case '\n':		break;
	      case '\r':		return;
	      case 8:			break;
	      case 9:			break;
	      default:			a = ' ';
	   }
	}

	switch (er) {
	   case 0:	printf("%c",a);break;
	   case 1:	printf("\033[01;42m%c\033[m",a); break;
	   case 2:	printf("\033[01;41m%c\033[m",a); break;
	   case 3:	printf("\033[01;43m%c\033[m",a); break;
           case 4:
           case 5:
           case 6:
           case 7:	printf("\033[2J\033[H<BRK>\n"); break;
	   default:
			break;
        }
}

const char * const codetable[] = {
	"A._",
	"B_...",
	"C_._.",
	"D_..",
	"E.",
	"F.._.",
	"G__.",
	"H....",
	"I..",
	"J.___",
	"K_._",
	"L._..",
	"M__",
	"N_.",
	"O___",
	"P.__.",
	"Q__._",
	"R._.",
	"S...",
	"T_",
	"U.._",
	"V..._",
	"W.__",
	"X_.._",
	"Y_.__",
	"Z__..",
	"0_____",
	"1.____",
	"2..___",
	"3...__",
	"4...._",
	"5.....",
	"6_....",
	"7__...",
	"8___..",
	"9____.",
	".._._._",
	",__..__",
        "?..__..",
	"~"
	};

void	cwDecoder::rx_lookup (char *in, char *out) {
int	i;

	for (i = 0; codetable [i][0] != '~'; i ++) {
	   if (strcmp (&codetable [i][1], in) == 0) {
	      out [0] = codetable [i][0];
	      return;
	   }
	}
	out [0] = '*';
}

