#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _CW_DECODER
#define	_CW_DECODER
#
/* CW  -- receiver classs
 */

#include	"jff-include.h"
#include	"virtualdecoder.h"
#include	"iir-filters.h"
#include	"fir-filters.h"
#include	"utilities.h"

class	Detector;
class	RadioInterface;
class	Oscillator;
/*
 *	Simple class for the functionality of the cw detector.
 *	Ideas borrowed from Jesus Arias and his rtty 2.1, however
 *	the thing was implemented as a state machine, and later
 *	improved (yes, really) by copying ideas for guessing the
 *	length of a dot from fldigi (David Freese). Obviously, all
 *	copyrights are gratefully acknowledged.
 *
 */
#define	CWError		1000
#define	CWNewLetter	1001
#define	CWCarrierUp	1002
#define	CWCarrierDown	1005
#define	CWStroke	1003
#define	CWDot		1004
#define	CWendofword	1006
#define	CWFailing	1007

class cwDecoder: public virtualDecoder {
Q_OBJECT
public:
		cwDecoder	(int32_t, 
	                         Detector *,
	                         RadioInterface *);
		cwDecoder	(int32_t, 
	                         RadioInterface *);
		~cwDecoder		(void);
	void	setWorkingRate		(int32_t);
	DSPCOMPLEX	doDecode	(DSPCOMPLEX);
	void	cw_setWordsperMinute	(int16_t);
	void	cw_setTracking		(bool);
	void	cw_setFilterDegree	(int16_t);
	void	cw_setSquelchValue	(int16_t);

private:
	void	speedAdjust		(void);
	void	printChar		(char, char);
	void	addtoPlotter		(DSPCOMPLEX);
	void	addtoStream		(double);
	void	add_cwTokens		(char *);
	int32_t	newThreshold		(int32_t, int32_t);
	int32_t	getMeanofDotDash	(int32_t, int32_t);
	void	rx_lookup		(char *, char *);

	int32_t			_WorkingRate;
	Detector		*MyDetector;
	Oscillator		*LocalOscillator;
	RadioInterface		*MyRadioInterface;
	int32_t			cwPhaseAcc;

	int32_t			CycleCount;
	int16_t			cwFilterDegree;
	average			*SmoothenSamples;
	average			*thresholdFilter;
	average			*spaceFilter;

	float			agc_peak;
	float			noiseLevel;
	float			value;
	float			metric;
	double			cw_IF;
	LowPassIIR		*cw_LowPassFilter;
	int32_t			cwDotLength;
	int32_t			cwSpaceLength;
	int32_t			cwDashLength;
	int16_t			SquelchValue;
	int16_t			cwState;
	int16_t			cwPreviousState;
	int32_t			cw_adaptive_threshold;
	int32_t			currentTime;
	int32_t			cwCurrent;
	int16_t			cwSpeed;
	int16_t			cwDefaultSpeed;
	char			*rx_rep_buf;
	int32_t			CWrange;
	int32_t			cwDefaultDotLength;
	int32_t			lowerBoundWPM;
	int32_t			upperBoundWPM;
	int32_t			cwDotLengthMin;
	int32_t			cwDotLengthMax;
	int32_t			cwDashLengthMin;
	int32_t			cwDashLengthMax;
	int32_t			lengthofPreviousTone;
	int32_t			cwNoiseSpike;
	int32_t			cwStartTimestamp;
	int32_t			cwEndTimestamp;
	bool			cwTracking;

signals:
	void			cw_clrText		();
	void			cw_addText		(char);
	void			cw_addToken		(const char *);
	void			cw_showdotLength	(int);
	void			cw_showspaceLength	(int);
	void			cw_showspeed		(int);
	void			cw_showagcpeak		(int);
	void			cw_shownoiseLevel	(int);

	void			setDetectorMarker	(int);
};
#endif

