#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *    The fax implementation is a rewrite of hamfax
 *    Copyright (c) 2001, 2002
 *	Christof Schmitt, DH1CS <cschmitt@users.sourceforge.net>
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include	"demodulator.h"
#include	"utilities.h"
#include	"faxdecoder.h"
	Demodulator::Demodulator	(int8_t mode,
	                                 int32_t rate,
	                                 int32_t dev) {
int16_t	i;

	this	-> mode	= mode;
	Rate		= rate;
	deviation	= dev;
	ArcsineSize	= 512;
	Arcsine		= new double [ArcsineSize];
	for (i = 0; i < ArcsineSize; i ++)
	   Arcsine [i] = asin (2.0 * i / ArcsineSize - 1.0) / 2.0 / M_PI;
	Table		= new DSPCOMPLEX [Rate];
	for (i = 0; i < Rate; i ++)
	   Table [i] = DSPCOMPLEX (cos (2 * M_PI * i / Rate),
	                           sin (2 * M_PI * i / Rate));
	pll_Lowpass	= new LowPassIIR (2,
	                                  500,
	                                  Rate,
	                                  S_CHEBYSHEV);
	Qmin1		= 0;
	Qmin2		= 0;
	Imin1		= 0;
	Imin2		= 0;
	fax_oldz	= 0;
	selector	= 2;
}

	Demodulator::~Demodulator	() {
	delete[]	Arcsine;
	delete		pll_Lowpass;
}

void	Demodulator::setMode		(int8_t m) {
	mode = m;
}

void	Demodulator::setDeviation	(int16_t dev) {
	deviation	= dev;
}
/*
 *	FM demodulation essentially is looking at
 *	variations in phase for the subsequent samples
 *	The IF is 0, so the phase variations are
 *	the signal
 *
 *	select 1 for the traditional FM decoder
 */
int16_t	Demodulator::demodulate_1 (DSPCOMPLEX z) {
DSPFLOAT	res;
DSPFLOAT	I;
DSPFLOAT	Q;

	z		= cdiv (z, abs (z));
	if (mode == faxDecoder::FAX_AM)
	   return abs (z) * 255.0;

	I		= real (z);
	Q		= imag (z);
	res		= Imin1 * (Q - Qmin2) - Qmin1 * (I - Imin2);
	res		/= Imin1 * Imin1 + Qmin1 * Qmin1;
	res		*= Rate / (2 * M_PI);
	res		= clamp (res, - deviation, + deviation);
	Imin2		= Imin1;
	Qmin2		= Qmin1;
	Imin1		= I;
	Qmin1		= Q;

	fax_oldz	= z;
	return  (int16_t)(res / deviation * 128 + 127);
}
/*
 *	select 2 for the alternative traditional FM demodulator
 */
int16_t	Demodulator::demodulate_2 (DSPCOMPLEX z) {
DSPFLOAT	res;

	z		= cdiv (z, abs (z));
	if (mode == faxDecoder::FAX_AM)
	   return abs (z) * 255.0;

	res		= arg (conj (fax_oldz) * z) / (2 * M_PI) * Rate;
	res		= clamp (res, - deviation, + deviation);
	fax_oldz	= z;
	return (int16_t)(res / deviation * 128 + 127);
}
/*
 *	for the original demodulator from Schitt, select 3
 */
int16_t	Demodulator::demodulate_3	(DSPCOMPLEX z) {
DSPFLOAT	res;
DSPFLOAT	I, Iold;
DSPFLOAT	Q, Qold;

	z	= cdiv (z, abs (z));

	if (mode == faxDecoder::FAX_AM) 
	   return abs (z) * 255.0;

	I	= real (z);
	Q	= imag (z);
	Iold	= real (fax_oldz);
	Qold	= imag (fax_oldz);
	if (abs (z) > 0.001) {
	   res = (Iold * Q - Qold * I + 1.0) / 2.0;
	   res = Arcsine [(int)(res * ArcsineSize)];
	   res = res * Rate / deviation;
	   fax_oldz	= z;
	   return (int16_t) clamp (res * 128 + 127, 0, 255);
	}
	else {
	   fax_oldz	= z;
	   return 0;
	}
}

int16_t	Demodulator::demodulate_4 (DSPCOMPLEX z) {
int32_t	IndexforRefValue	= pll_integrator / (2 * M_PI) * Rate;
DSPFLOAT	res;

	res	= imag (z * Table [IndexforRefValue]) * 0.5;
	pll_integrator	= PI_Constrain (pll_integrator + res);
	res	= pll_Lowpass -> Pass (res);
	return (int16_t)(res / deviation * 128 + 127);
}

int16_t	Demodulator::demodulate (DSPCOMPLEX z) {
	if (selector == 1)
	   return demodulate_1 (z);
	else
	if (selector == 2)
	   return demodulate_2 (z);
	else
	   return demodulate_3 (z);
}

void	Demodulator::select (int8_t s) {
	selector	= s;
}

