#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *	This faxdecoder is a partial rewrite of the hamfax decoder
 *	by Gerhard Schmitt
 *	All rights are acknowledged
 *      Copyright (c) 2001, 2002
 *	Christof Schmitt, DH1CS <cschmitt@users.sourceforge.net>
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	"faximage.h"
#include	"oscillator.h"
#include	"demodulator.h"
#include	"faxdecoder.h"
#include	"gui.h"
#include	<QFile>

#define		FAX_IF		0

	faxDecoder::faxDecoder (int32_t	Rate, RadioInterface *ri) {
	_WorkingRate		= Rate;
	MyRadioInterface	= ri;
	
	LocalOscillator		= new Oscillator (_WorkingRate);
	savingFlag		= false;
	phaseInvers		= false;
	faxMode			= FAX_FM;
	faxColor		= FAX_BLACKWHITE;
	lpm			= 120;
	lastCol			= 0;
	f_lpm			= 120;
	txLPM			= 120;
	aptStartFreq		= 300;		
	aptStopFreq		= 450;		
	deviation		= 400;
	fax_IOC			= 288;
	carrier			= FAX_IF;
	faxLowPass		= new LowPassIIR (21,
	                                          deviation + 50,
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);
	faxAverager		= new average (20);
	MyDemodulator		= new Demodulator (true,
	                                           _WorkingRate,
	                                           deviation);
/*
 *	we build the fax image in:
 */
	numberofColums		= M_PI * 288;
	lastRow			= 140;
	theImage		= new faxImage	(numberofColums, lastRow);

	connect (this, SIGNAL (showState (const QString &)),
	         MyRadioInterface, SLOT (fax_showState (const QString &)));
	connect (this, SIGNAL (displayImage (QImage)),
	         MyRadioInterface, SLOT (fax_displayImage (QImage)));
	connect (this, SIGNAL (displayImage (QImage, int, int)),
	         MyRadioInterface, SLOT (fax_displayImage (QImage, int, int)));
	connect (this, SIGNAL (showLinenumber (int)),
	         MyRadioInterface, SLOT (fax_showLinenumber (int)));
	connect (this, SIGNAL (showaptFreq (int)),
	         MyRadioInterface, SLOT (fax_showaptFreq (int)));
	connect (this, SIGNAL (saveFile (QImage)),
	         MyRadioInterface, SLOT (fax_saveFile (QImage)));
	connect (this, SIGNAL (setDetectorMarker (int)),
	         MyRadioInterface, SLOT (setDetectorMarker (int)));
	connect (this, SIGNAL (slantText (const QString &)),
	         MyRadioInterface, SLOT (fax_slantText (const QString &)));
//
//	and set everything
//
	pointNeeded		= false;
	faxState		= APTSTART;
	aptCount		= 0;
	aptTrans		= 0;
	aptStop			= false;
	aptHigh			= false;
	pixelValue		= 0;
	pixelSamples		= 0;
	imageSample		= 0;
	emit	showState	(QString ("APTSTART"));
	theImage	-> clear	();
	emit	displayImage	(theImage -> getImage ());
	rawData. resize (1024 * 1024 * 8);
	setDetectorMarker	(carrier);
}


	faxDecoder::~faxDecoder () {
	disconnect (this, SIGNAL (showState (const QString &)),
	            MyRadioInterface, SLOT (fax_showState (const QString &)));
	disconnect (this, SIGNAL (displayImage (QImage)),
	            MyRadioInterface, SLOT (fax_displayImage (QImage)));
	disconnect (this, SIGNAL (displayImage (QImage, int, int)),
	            MyRadioInterface, SLOT (fax_displayImage (QImage, int, int)));
	disconnect (this, SIGNAL (showLinenumber (int)),
	            MyRadioInterface, SLOT (fax_showLinenumber (int)));
	disconnect (this, SIGNAL (showaptFreq (int)),
	            MyRadioInterface, SLOT (fax_showaptFreq (int)));
	disconnect (this, SIGNAL (saveFile (QImage)),
	            MyRadioInterface, SLOT (fax_saveFile (QImage)));
	disconnect (this, SIGNAL (setDetectorMarker (int)),
	            MyRadioInterface, SLOT (setDetectorMarker (int)));

	delete	LocalOscillator;
	delete	MyDemodulator;
	delete	theImage;
	delete	faxLowPass;
}

void	faxDecoder::setWorkingRate	(int32_t workingRate) {
	delete	LocalOscillator;
	LocalOscillator		= new Oscillator	(_WorkingRate);
	delete	MyDemodulator;
	MyDemodulator		= new Demodulator (true,
	                                           _WorkingRate,
	                                           deviation);
	delete	faxLowPass;
	faxLowPass		= new LowPassIIR (21,
	                                          deviation + 50,
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);
}
/*
 *	as always, we "doDecode" one sample at the time.
 */
int	inRange (int x, int y) {
	return y - 1 <= x && x <= y + 1;
}

DSPCOMPLEX	faxDecoder::doDecode	(DSPCOMPLEX z) {
int16_t	sampleValue;

	if (faxState == DONE)
	   return z;

	z		= z * LocalOscillator -> nextValue (carrier);
	z		= faxLowPass	-> Pass (z);
	sampleValue	= MyDemodulator -> demodulate (z);

	if (faxColor == FAX_BLACKWHITE)
	   sampleValue = sampleValue < 128 ? 0 : 255;
	fax_oldz	= z;
	currentValue	= sampleValue;
/*
 *	The number of transitions between black and white is
 *	counted. Each 0.5 second, the frequency is measured.
 *	If it matches the aptStart frequency, the state skips to
 *	the detection of phasing lines.
 *	if it matches the aptStop frequency two times, the
 *	reception is ended.
 */
	if (!aptHigh && currentValue > 229) {
	   aptHigh = true;
	   aptTrans ++;
	}
	else
	if (aptHigh && currentValue < 25)
	   aptHigh = false;

	if (++aptCount >= _WorkingRate / 2) {
	   int freq	= 2 * aptTrans;
	   showaptFreq (freq);

	   aptCount	= 0;
	   aptTrans	= 0;

	   if ((faxState == APTSTART) && inRange (freq, aptStartFreq)) {
	      faxState		= PHASING;
	      lpm		= 0;
	      f_lpm		= 0;
	      lpmSum		= 0;
	      phaseHigh		= currentValue >= 128 ? true : false;
	      currPhaseLength	= 0;
	      currPhaseHigh	= 0;
	      phaseLines	= 0;
	      noPhasingLines	= 0;
	      emit	showState (QString ("Phasing"));
	      theImage	-> clear	();
	   }
	   else
	   if (!aptStop && inRange (freq, aptStopFreq))
	         aptStop	= true;
	   else
	   if (aptStop && inRange (freq, aptStopFreq)) {
	      rawData. resize (imageSample);
	      faxState		= DONE;	
	      aptStop		= false;
	      showState (QString ("Done"));
	      displayImage (theImage	-> getImage (),
	                    0,
	                    faxColor == FAX_COLOR ? lastRow / 3 : lastRow);
	   }
	}

	if (faxState == PHASING)
	   decodePhasing (sampleValue);

	if ((faxState == PHASING || faxState == IMAGE) && lpm > 0) 
	   decodeImage (sampleValue);

	return z;
}
/*
 *	Phasing lines consists of 2.5% white at the beginning,
 *	95 % black and again 2.5 % white at the end (or inverted).
 *	In normal cases we try to count the length between the
 *	white-black transitions. If the line has a reasonable
 *	amount of black (4.8 % - 5.2 %) and the length fits in
 *	the range of 60 -- 360 lpm (plus some tolerance) it is considered
 *	a valid phasing line. Then the start of a line and the
 *	lpm is calculated
 */
bool	faxDecoder::weFoundaValidPhaseline (void) {
double bound1 = (phaseInvers ? 0.948 : 0.048) * currPhaseLength;
double bound2 = (phaseInvers ? 0.952 : 0.052) * currPhaseLength;
	return ((currPhaseHigh >= bound1) && (currPhaseHigh <= bound2) &&
	        ((double)currPhaseLength >= 0.15 * _WorkingRate) &&
	        ((double)currPhaseLength <= 1.1 * _WorkingRate));
}

void	faxDecoder::decodePhasing (int16_t x) {
	x	= faxAverager	-> filter (x);
	currPhaseLength ++;
	if (x > 128)
	   currPhaseHigh ++;

	if ((!phaseInvers && x > 229 && !phaseHigh)) {
	   phaseHigh = true;
	   return;
	}

	if ((phaseInvers && x < 25 && phaseHigh)) {
	   phaseHigh = false;
	   return;
	}

	if ((!phaseInvers && x < 25 && phaseHigh) ||
	    (phaseInvers && x > 229 && !phaseHigh)) {
	   phaseHigh = phaseInvers ? true :false;
/*
 *	we had a change to !phaseHigh (with !phaseInvers), 
 *	so potentially a phaseline
 */
	   if (weFoundaValidPhaseline ()) {
	      double l		= 60.0 * _WorkingRate / currPhaseLength;
	      lpmSum		+= l;
	      phaseLines ++;
	      lpm		= lpmSum / phaseLines;
	      f_lpm		= lpmSum / phaseLines;
	      fprintf (stderr, " lpm = %d, f_lpm = %f\n", lpm, f_lpm);
	      imageSample	= (int)(1.025 * 60.0 / lpm * _WorkingRate);
	      noPhasingLines	= 0;
	   }
	   else			// keep current line and start
	   if (phaseLines > 0 && ++ noPhasingLines >= 5) {
	      if (lpm == 0)
	         lpm = txLPM;
	      if (115 <= lpm && lpm <= 125) lpm = 120; 
	      fprintf (stderr, "Starting with an image, lpm = %d\n", lpm);
	      faxState		= IMAGE;
	      emit showState (QString ("Image"));
	      double pos	= fmod ((double)imageSample,
	                                 (double)_WorkingRate * 60 / lpm);
	      pos		/= _WorkingRate * 60.0 / lpm;
	      lastCol		= (int)(pos * numberofColums);
	      pixelValue	= 0;
	      lastRow		= 140;
	      phaseLines	= 0;
	      noPhasingLines	= 0;
	   }
	   else			// check for garbage
	   if (currPhaseLength > 5 * _WorkingRate) {
	      currPhaseLength	= 0;
	   }

	   currPhaseLength	= 0;
	   currPhaseHigh	= 0;
	}
}
/*
 *	we add the demodulated value (x) to the
 *	current pixel, so first we find out
 *	the position of the current pixel
 *
 *	Number of samples per line =
 *		_WorkingRate  * 60.0 / lpm
 *	samplenumber in currentline =
 *		fmod (imageSample, _WorkingRate * 60.0 / lpm)
 *	position of imagesample in current column =
 *	       samplenumber in current line / number of samples per Line * x
 */
void	faxDecoder::decodeImage (int32_t x) {
int32_t	SamplesperLine	= _WorkingRate * 60 / lpm;
int32_t	samplenum	= imageSample % SamplesperLine;
DSPFLOAT	Position	= (DSPFLOAT)samplenum / SamplesperLine;
int32_t	col		= floor (Position * numberofColums);
int32_t	currRow		= (int32_t) (imageSample * lpm / 60.0 / _WorkingRate);

	if ((int32_t) (rawData. size ()) <= imageSample) 
	   rawData. resize (rawData. size () + 1024 * 1024);

	rawData [imageSample]	= x;
	if (col == lastCol) {
	   pixelValue += x;
	   pixelSamples ++;
	}
	else {
	   if (pixelSamples > 0) {
	      pixelValue /= pixelSamples;
	      theImage -> setPixel (lastCol, 
	                            faxColor == FAX_COLOR ? currRow / 3 : currRow,
	                            pixelValue,
	                            faxColor == FAX_COLOR ? currRow % 3 : 3);
	      if (lastRow != currRow) {
	         showLinenumber (faxColor == FAX_COLOR ? currRow / 3 : currRow);
	         displayImage (theImage	-> getImage (),
	                       0,
	                       faxColor == FAX_COLOR ? currRow / 3 : currRow);
	         lastRow	= currRow;
	      }
	   }
	   lastCol		= col;
	   pixelValue		= x;
	   pixelSamples		= 1;
	}

	imageSample ++;
}
void	faxDecoder::setAptStopFreq (int16_t f) {
	aptStopFreq	= f;
}

void	faxDecoder::setAptStartFreq (int16_t f) {
	aptStartFreq	= f;
}
/*
 */
void	faxDecoder::correctWidth	(int16_t w) {
	theImage	-> correctWidth (w);
}

void	faxDecoder::adjustNext	(void) {
}

void	faxDecoder::fax_setIOC	(int16_t ioc) {
int	numberofSamples;

	if (ioc == fax_IOC) 
	   return;

	fax_IOC			= ioc;
	numberofColums		= M_PI * ioc;
	lastRow			= 140;
	pixelValue		= 0;
	imageSample		= 0;

	if (faxState != DONE) {
	   delete theImage;
	   theImage		= new faxImage	(numberofColums, lastRow);
	   rawData. resize (1024 * 1024 * 8);
	   theImage	-> clear	();
	   emit	displayImage	(theImage	-> getImage ());
	   return;
	}
/*
 *	apply the changes to the current image
 */
	numberofSamples	= rawData. size ();
	for (int i = 0; i < numberofSamples; i ++) {
	   decodeImage (rawData [i]);
	}
}

void	faxDecoder::fax_setMode	(int16_t m) {
	faxMode	= m;
	MyDemodulator	-> setMode	(faxMode);
}

void	faxDecoder::fax_setDeviation (int16_t m) {
	deviation	= m;
	delete	faxLowPass;
	faxLowPass		= new LowPassIIR (21,
	                                          deviation + 50,
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);
	MyDemodulator	-> setDeviation	(deviation);
}
/*
 *	settingskip is only called through a signal
 *	from the main gui
 */
void	faxDecoder::fax_setSkip	(void) {
	if (faxState == APTSTART) {
	   faxState		= PHASING;
//	   lpm			= 0;
	   lpmSum		= 0;
	   phaseHigh		= currentValue >= 128 ? true : false;
	   currPhaseLength	= 0;
	   currPhaseHigh	= 0;
	   showState (QString ("Phasing"));
	   theImage	-> clear	();
	}
	else
	if (faxState == PHASING) {
	   faxState		= IMAGE;
	   lpm			= txLPM;
	   double pos		= fmod ((double)imageSample,
	                                    (double)_WorkingRate * 60 / lpm);
	   pos			/= _WorkingRate * 60.0 / lpm;
	   lastCol		= (int) pos * numberofColums;
	   pixelValue		= 0;
	   pixelSamples		= 0;
	   imageSample		= 0;
	   lastRow		= 140;
	   showState (QString ("Imaging"));
	}
	else
	if (faxState == IMAGE) {
	   rawData. resize (imageSample);
	   faxState		= DONE;	
	   aptStop		= false;
	   showState (QString ("Done"));
	   displayImage (theImage	-> getImage (),
	                 0,
	                 faxColor == FAX_COLOR ? lastRow / 3 : lastRow);
	}
	else
	if (faxState == DONE) {
	   faxState 		= APTSTART;
	   aptCount		= 0;
	   aptTrans		= 0;
	   aptStop		= false;
	   aptHigh		= false;
	   pixelValue		= 0;
	   pixelSamples		= 0;
	   imageSample		= 0;
	   rawData. resize (1024 * 1024 * 8);
	   showState	(QString ("APTSTART"));
	}
}

void	faxDecoder::fax_setLPM (DSPFLOAT f) {
int64_t	numberofSamples;

	lpm		= f;
	txLPM		= f;
	if (faxState != DONE) {
	   pixelValue	= 0;
	   pixelSamples	= 0;
	   return;
	}

	pixelValue	= 0;
	pixelSamples	= 0;
	imageSample	= 0;
	lastRow		= 140;
	lastCol		= 0;
	numberofSamples	= rawData. size ();
	for (int i = 0; i < numberofSamples; i ++) {
	   decodeImage (rawData [i]);
	}
}

void	faxDecoder::fax_setPhase (int16_t n) {
	phaseInvers	= n;
}

void	faxDecoder::fax_setColor (int16_t n) {
	faxColor	= n;
}

void	faxDecoder::fax_setstartFreq (int16_t m) {
	aptStartFreq	= m;
}

void	faxDecoder::fax_setstopFreq (int16_t m) {
	aptStopFreq	= m;
}

void	faxDecoder::fax_setCarrier (int16_t m) {
	carrier			= m;
	delete	faxLowPass;
	faxLowPass		= new LowPassIIR (21,
	                                          deviation + 50,
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);
	setDetectorMarker	(carrier);
}

void	faxDecoder::fax_setsavingonDone (bool f) {
        (void)f;
	if (faxState != DONE)
	   return;
	saveFile (theImage -> getImage ());
}

void	faxDecoder::fax_setdecoderSelector (int16_t s) {
	decoderSelector	= s;
	MyDemodulator	-> select (s);
}

void	faxDecoder::fax_setSlant (QPoint x) {
DSPFLOAT	lpma;
DSPFLOAT	adjustment;
	if (pointNeeded) {
	   pointNeeded = false;
	   slant2	= x;
	   lpma = ((float)(slant2. x() - slant1. x())) / (slant1. y () - slant2. y ());
	   lpma	= lpma	/ theImage -> getCols ();
	   adjustment = 1.0 + (faxColor != FAX_BLACKWHITE ? lpma / 3 : lpma);
//	   fprintf (stderr, "adjusting lpm with %f\n", adjustment); 
	   fax_setLPM  (adjustment);
	   slantText	(QString (""));
	   return;
	}

	pointNeeded	= true;
	slant1		= x;
	slantText	(QString ("mark a second point"));
}

