#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *    The fax implementation is a rewrite of hamfax
 *    Copyright (c) 2001, 2002
 *	Christof Schmitt, DH1CS <cschmitt@users.sourceforge.net>
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef	__FAX_DECODER
#define	__FAX_DECODER

#include	<QObject>
#include	"jff-include.h"
#include	"virtualdecoder.h"
#include	"fir-filters.h"
#include	"iir-filters.h"
#include	<QByteArray>
#include	<QImage>
#include	"utilities.h"
#include	<QPoint>

class	RadioInterface;
class	Oscillator;
class	Demodulator;
class	faxImage;
/*
 *	states:
 */
#define	APTSTART	0100
#define	PHASING		0101
#define	IMAGE		0102
#define	DONE		0103

class	faxDecoder: public virtualDecoder {
Q_OBJECT
public:
		faxDecoder			(int32_t,
	                                         RadioInterface	*);
	        ~faxDecoder ();
	void	setWorkingRate			(int32_t);
	DSPCOMPLEX	doDecode		(DSPCOMPLEX);

	void	adjustNext			(void);
	void	setAptStartFreq			(int16_t);
	void	setAptStopFreq			(int16_t);

	enum Teint {
	   FAX_COLOR	= 1,
	   FAX_GRAY	= 2,
	   FAX_BLACKWHITE	= 3
	};

	enum Mode {
	   FAX_AM	= 100,
	   FAX_FM	= 101
	};

	void		fax_setIOC		(int16_t);
	void		fax_setMode		(int16_t);
	void		fax_setDeviation	(int16_t);
	void		fax_setSkip		(void);
	void		fax_setLPM		(DSPFLOAT);
	void		fax_setPhase		(int16_t);
	void		fax_setColor		(int16_t);
	void		fax_setstartFreq	(int16_t);
	void		fax_setstopFreq		(int16_t);
	void		fax_setCarrier		(int16_t);
	void		fax_setsavingonDone	(bool);
	void		fax_setdecoderSelector	(int16_t);
	void		fax_setSlant		(QPoint);
private:
	RadioInterface	*MyRadioInterface;
	void		correctWidth		(int16_t);
	void		decodePhasing		(int16_t);
	void		decodeImage		(int32_t);
	void		endReception		(void);
	bool		weFoundaValidPhaseline	(void);
	void		correctLPM		(DSPFLOAT);
	int32_t		_WorkingRate;
	Oscillator	*LocalOscillator;

	faxImage	*theImage;
	Demodulator	*MyDemodulator;
	LowPassIIR	*faxLowPass;
	average		*faxAverager;
	QByteArray	rawData;
	uint8_t		faxState;
	int32_t		imageSample;
	DSPCOMPLEX	fax_oldz;
	int16_t		fax_IOC;
	DSPFLOAT	lpm;
	DSPFLOAT	f_lpm;
	int16_t		deviation;
	int16_t		aptCount;
	int16_t		aptTrans;
	int16_t		aptStop;
	int16_t		aptHigh;
	int16_t		aptStartFreq;
	int16_t		aptStopFreq;
	int16_t		txLPM;
	bool		phaseInvers;
	uint8_t		faxColor;
	int16_t		lpmSum;
	bool		phaseHigh;
	int16_t		currPhaseLength;
	int16_t		currPhaseHigh;
	int16_t		phaseLines;
	int16_t		noPhasingLines;
	int16_t		startLength;
	int16_t		stopLength;
	int16_t		carrier;
	uint8_t		faxMode;

	int16_t		numberofColums;
	int16_t		lastCol;
	int16_t		lastRow;
	int32_t		pixelValue;
	int16_t		pixelSamples;
	int16_t		currentValue;
	bool		savingFlag;
	int16_t		decoderSelector;
//	for adjusting, we need
	QPoint		slant1;
	QPoint		slant2;
	bool		pointNeeded;

signals:
	void		showState		(const QString &);
	void		displayImage		(QImage);
	void		displayImage		(QImage, int, int);
	void		showLinenumber		(int);
	void		showaptFreq		(int);
	void		saveFile		(QImage);
	void		setDetectorMarker	(int);
	void		slantText		(const QString &);
};

#endif

