#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in ESDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _MFSK_DECODER
#define	_MFSK_DECODER
#
/*		
 *	Mfsk  -- receiver classs
 */
#include	"jff-include.h"
#include	"slidingfft.h"
#include	"viterbi.h"
#include	"interleave.h"
#include	"virtualdecoder.h"
#include	"fft.h"
#include	"fir-filters.h"
#include	"iir-filters.h"
#include	"fft-filters.h"
#include	"utilities.h"

class	RadioInterface;
class	Oscillator;

class Cache {
public:
		Cache		(int, int);
		~Cache		();
DSPCOMPLEX	*CacheLine	(int a);
private:
	int	rowBound;
	int	columnBound;
	DSPCOMPLEX	**data;
};

class mfskDecoder: public virtualDecoder {
Q_OBJECT
public:
	mfskDecoder			(int32_t, RadioInterface *);
	~mfskDecoder			(void);
	void	AdjustParameters	(void);
	void	setWorkingRate		(int32_t);
	DSPCOMPLEX	doDecode	(DSPCOMPLEX);

	enum decoders {
	   MODE_MFSK_4 =	0100,
	   MODE_MFSK_8 =	0101,
	   MODE_MFSK_11	=	0102,
	   MODE_MFSK_16 =	0103,
	   MODE_MFSK_22	=	0104,
	   MODE_MFSK_31	=	0105,
	   MODE_MFSK_32	=	0106,
	   MODE_MFSK_64 =	0107
	};
	void		mfsk_setAfcon		(bool);
	void		mfsk_setReverse		(bool);
	void		mfsk_setMode		(int16_t);
	void		mfsk_setSquelchLevel	(int16_t);
	void		mfsk_setIFadjuster	(int16_t);
private:
	void		MapTonetoBits 	(DSPCOMPLEX *, int16_t, unsigned char *);
	void		setupInterfaceforMode	(uint8_t);
	int16_t		IndextoFullestBin	(DSPCOMPLEX *);
	void		handle_mfsk_bits	(uint8_t);
	void		shiftBit		(uint8_t);
	int16_t		vari_Decode		(uint16_t symbol);

	int32_t		_WorkingRate;
	Oscillator	*LocalOscillator;
	RadioInterface	*MyRadioInterface;

	double		mfskIF;
	bool		mfskAfcon;
	uint8_t		mfskMode;
	bool		mfskReverse;
	int16_t		mfskSquelchLevel;
	int16_t		mfskSamplesperSymbol;
	int16_t		mfskNumBits;
	int16_t		mfskBaseTone;	/* 1500 Hz		*/
	int16_t		mfskNumTones;
	int16_t		mfskBorder;
	double		mfskBaudRate;
	double		mfskToneSpacing;

	float		mfskBw;

	unsigned char	mfskDibitpair [2];
	fftFilter	*mfskbandFilter;

	viterbi		*ViterbiDecodera;
	viterbi		*ViterbiDecoderb;
	Interleaver	*mfskInterleaver;
	Cache		*mfskSpectrumCache;
	uint16_t	mfskCacheLineP;

	slidingFFT	*mfskSlidingfft;
	uint16_t	mfskDatashreg;

	DSPCOMPLEX	mfskPrev1Vector;
	DSPCOMPLEX	mfskPrev2Vector;

	uint16_t	mfskPrev1Index;
	uint16_t	mfskPrev2Index;
	int16_t		showCount;

	float		mfskMet1;
	float		mfskMet2;
	float		mfskSignalMetric;
	uint8_t		mfskbitCounter;
	int		mfskSyncCounter;

	float		mfskbaudrateEstimate;


signals:
	void		mfsk_clrText	();
	void		mfsk_addText	(char);
	void		mfsk_showLocalScope	(int);
	void		mfsk_addLocalScope	(int, double);
	void		mfsk_showCF	(float);
	void		mfsk_showS2N	(int);
	void		mfsk_showTuning	(int);
	void		mfsk_showEstimate	(float);
	void		setDetectorMarker	(int);
};

#endif


