/*
 *
 *	Copyright (C) 2008, 2009, 2010
 *	Jan van Katwijk (J.vanKatwijk@gmail.com)
 *	JFF Consultancy
 *
 *	This file is part of jsdr.
 *	The code is adapted from code contained in gmfsk source code 
 *	distribution. Copyrights gratefully acknowledged
 *	Copyright (C) 2005
 *	Tomi Manninen (oh2bns@sral.fi)
 *
 *	The software is essentially a wrapper around the olivia decoder
 *	made by Pawel Jalocha (Copyright 2004)
 *
 *    This file is part of the JFF SDR (JSDR).
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include	"gui.h"
#include	"oliviadecoder.h"
#include	"utilities.h"
#define	BUFSIZE	512
#define	IF	1500

/*
 */
	oliviaDecoder::oliviaDecoder (int32_t SampleRate, RadioInterface *RI) {
	this	-> SampleRate	= SampleRate;
	this	-> MyRadioInterface	= RI;
	oliviaBuffer		= new DSPFLOAT [BUFSIZE];
	inp			= 0;
	baseFrequency		= IF;
	frequency		= baseFrequency;
	snr			= 1e-3;
	msg1			= new char [20];
	msg2			= new char [20];
//
//	The default values for the relevant parameters
	Tones			= 32;
	Bandwidth		= 1000;
	SyncMargin		= 8;
	SyncIntegLen		= 4;
	SyncThreshold		= 3.2;
	SquelchOn		= false;
	squelchValue		= 5;
	reverse			= false;

//	just instantiate a new receiver
	Receiver		= new MFSK_Receiver<DSPFLOAT>;
	Receiver -> Tones	= 32;	// number of tones
	Receiver -> Bandwidth	= 1000;	// Bandwidth (Hz)
	Receiver -> SyncMargin	= 8;	//synchronizer tune margin
	Receiver -> SyncIntegLen	= 4;	// synchronizer integration period
	Receiver -> SyncThreshold	= 3.2;	// S/N threshold for printing
	Receiver -> SampleRate	= 8000;
	Receiver -> InputSampleRate	= SampleRate;
	Receiver -> Reverse	= false;
	if (Receiver -> Preset () < 0)
	   fprintf (stderr, "Insufficient RAM or another problem\n");

	fprintf (stderr, "\nOlivia Rx parameters:\n%s",
	                   Receiver -> PrintParameters ());
	lastFreq		= 0;
	escape			= false;
	connect (this, SIGNAL (clrText (void)),
	         MyRadioInterface, SLOT (olivia_clrText (void)));
	connect (this, SIGNAL (addChar (char)),
	         MyRadioInterface, SLOT (olivia_addChar (char)));
	connect (this, SIGNAL (put_Status1 (char *)),
	         MyRadioInterface, SLOT (olivia_put_Status1 (char *)));
	connect (this, SIGNAL (put_Status2 (char *)),
	         MyRadioInterface, SLOT (olivia_put_Status2 (char *)));
	connect (this, SIGNAL (setDetectorMarker (int)),
	         MyRadioInterface, SLOT (setDetectorMarker (int)));

	clrText ();
	put_Status1 (" ");
	put_Status2 (" ");
	setDetectorMarker	(IF);
}

	oliviaDecoder::~oliviaDecoder (void) {
	if (Receiver)
	   delete Receiver;
	delete[]	oliviaBuffer;
	delete[]	msg1;
	delete[]	msg2;
}

void	oliviaDecoder::setWorkingRate (int32_t workingRate) {
	if (SampleRate == workingRate)
	   return;

	SampleRate	= workingRate;
	Receiver	-> SampleRate		= 8000;
	Receiver	-> InputSampleRate	= SampleRate;
	restart ();
	setDetectorMarker	(IF);
}
	
void	oliviaDecoder::olivia_setTones (int16_t n) {
	Tones	= n;
	restart	();
}

void	oliviaDecoder::olivia_setBW (int16_t n) {
	bw	= n;
	Bandwidth = n;
	restart ();
}

void	oliviaDecoder::olivia_setSyncMargin (int16_t n) {
	SyncMargin	= n;
	restart	();
}

void	oliviaDecoder::olivia_setSyncIntegLen (int16_t n) {
	SyncIntegLen	= n;
	restart	();
}

void	oliviaDecoder::olivia_setSquelch (bool b) {
	SquelchOn	= b;
}

void	oliviaDecoder::olivia_setSquelchvalue (int16_t n) {
	squelchValue = n;
}

void	oliviaDecoder::olivia_setReverse (bool b) {
	reverse	= b;
}

DSPCOMPLEX	oliviaDecoder::doDecode (DSPCOMPLEX v) {
int c;
unsigned char ch = 0;
int fc_offset = Bandwidth * (1.0 - 0.5 / Tones) / 2.0;

	oliviaBuffer [inp] = real (v);
	if (++inp < BUFSIZE)
	   return v;

	inp	= 0;
	if ((lastFreq != frequency || Receiver -> Reverse) && !reverse) {
	   Receiver -> FirstCarrierMultiplier =
	                         (frequency - fc_offset)/500.0; 
	   Receiver -> Reverse = 0;
	   lastFreq = frequency;
	   Receiver -> Preset();
	}
	else if ((lastFreq != frequency || !Receiver -> Reverse) && reverse) {
	   Receiver -> FirstCarrierMultiplier =
	                              (frequency + fc_offset)/500.0; 
	   Receiver -> Reverse = 1;
	   lastFreq = frequency;
	   Receiver -> Preset();
	}

	Receiver -> SyncThreshold = SquelchOn ? 
		clamp(squelchValue / 5.0 + 3.0, 0, 90.0) : 0.0;

	Receiver -> Process(oliviaBuffer, BUFSIZE);
//
//	We ould aks here for the s/n
//
	bool gotchar = false;
	while (Receiver -> GetChar (ch) > 0) {
	   if ((c = unescape(ch)) != -1 && c > 7) {
	      addChar (c);
	      gotchar = true;
	   }
	}
	if (gotchar) {
	   snprintf(msg1, sizeof(msg1), "s/n: %4.1f dB", 10*log10(snr) - 20);
	   put_Status1 (msg1);
	   DSPFLOAT t = Receiver -> FrequencyOffset ();
	   snprintf(msg2, sizeof(msg2), "f/o %+4.1f Hz", t);
	   put_Status2 (msg2);
	}

	return v;
}

void oliviaDecoder::restart() {
int32_t	fc_offset;

	Receiver -> Tones		= Tones;
	Receiver -> Bandwidth		= Bandwidth;
	Receiver -> SyncMargin		= SyncMargin;
	Receiver -> SyncIntegLen	= SyncIntegLen;

fprintf (stderr, "tones = %d, bandwidth = %d, margin = %d, integLen = %d\n",
	 Tones, Bandwidth, SyncMargin, SyncIntegLen);

	Receiver -> SyncThreshold	= SquelchOn ? 
		        clamp(squelchValue / 5.0 + 3.0, 0, 90.0) : 0.0;

	fc_offset = Receiver -> Bandwidth * (1.0 - 0.5 / Receiver -> Tones) / 2.0;
	if (reverse) { 
	   Receiver -> FirstCarrierMultiplier =
	                               (frequency + fc_offset)/500.0; 
	   Receiver -> Reverse = 1; 
	} else { 
	   Receiver -> FirstCarrierMultiplier =
	                               (frequency - fc_offset)/500.0; 
	   Receiver -> Reverse = 0; 
	}

	if (Receiver -> Preset() < 0) {
		fprintf (stderr, "olivia: receiver preset failed!");
		return;
	}

	fprintf (stderr, "\nOlivia Rx parameters:\n%s",
	                   Receiver -> PrintParameters ());

	clrText	();
}

int16_t oliviaDecoder::unescape (int16_t c) {

	if (escape) {
	   escape = false;
	   return c + 128;
	}

	if (c == 127) {
	   escape = 1;
	   return -1;
	}

	return c;
}

