#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    The psk is strongly influenced by code
 *    of Tomi Manninen in his gmfks0.6 code
 *    Copyright 2006, 2007 Tomi Manninen
 *
 *    This file is part of the JFF SDR (JSDR).
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#
#include	"gui.h"
#include	"pskdecoder.h"
#include	"oscillator.h"
#include	"utilities.h"
#define	pskK			5
#define	pskPOLY1		0x17
#define	pskPOLY2		0x19
#define	PSK31_SPEED		31.25
#define	PSK63_SPEED		62.50
#define	PSK125_SPEED		125.0

#define	PSKIF			800
#define	PSKRATE			2000

/*
 *	the psk decoder is a rather straightforward implementation
 *	we filter the incoming signal on Workingrate,
 *	then down decimate to PSKRATE, after which we 
 *	decimate to a rate that we end up with 16 samples per bit
 */
	pskDecoder::pskDecoder (int32_t	workingR, RadioInterface *RI) {
	_WorkingRate		= workingR;
	MyRadioInterface	= RI;
/*
 *	some defaults, to be overrun by settings
 */
	pskFilterDegree		= 12;		/* default	*/
	pskAfcon		= FALSE;
	pskMode			= MODE_PSK31;
	pskSquelchLevel		= 5;
	pskReverse		= FALSE;

	LocalOscillator		= new Oscillator (_WorkingRate);
	pskDecimator		= new downDecimator (_WorkingRate, PSKRATE);
	pskBuffer		= new double [16];
	memset (pskBuffer, 0, 16 * sizeof (double));
	pskBitclk		= 0;
	pskDecimatorCount	= 0;
	LPM_Filter		= new LowPassIIR (pskFilterDegree,
	                                          (int)(PSK31_SPEED),
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);

	viterbiDecoder		= new viterbi (pskK, pskPOLY1, pskPOLY2);

	connect (this,
	         SIGNAL (psk_clrText ()),
	         MyRadioInterface,
	         SLOT (clr_pskText ()));
	connect (this,
	         SIGNAL (psk_addText (char)),
		 MyRadioInterface,
	         SLOT (add_pskText (char)));
	connect (this,
	         SIGNAL (psk_showDial (float)),
		 MyRadioInterface,
	         SLOT (show_pskDial (float)));
	connect (this,
	         SIGNAL (psk_showMetrics (float)),
	         MyRadioInterface,
	         SLOT (show_pskQuality (float)));
	connect (this,
	         SIGNAL (psk_showIF (float)),
		 MyRadioInterface,
	         SLOT (show_pskIF (float)));
	connect (this,
	         SIGNAL (setDetectorMarker (int)),
	         MyRadioInterface,
	         SLOT (setDetectorMarker (int)));


	pskSettings ();
	psk_clrText ();
}

	pskDecoder::~pskDecoder () {
	disconnect (this,
	            SIGNAL (psk_clrText ()),
	            MyRadioInterface,
	            SLOT (clr_pskText ()));
	disconnect (this,
	            SIGNAL (psk_addText (char)),
		    MyRadioInterface,
	            SLOT (add_pskText (char)));
	disconnect (this,
	            SIGNAL (psk_showDial (float)),
		    MyRadioInterface,
	            SLOT (show_pskDial (float)));
	disconnect (this,
	            SIGNAL (psk_showMetrics (float)),
	            MyRadioInterface,
	            SLOT (show_pskQuality (float)));
	disconnect (this,
	            SIGNAL (psk_showIF (float)),
		    MyRadioInterface,
	            SLOT (show_pskIF (float)));

	delete		pskDecimator;
	delete		LocalOscillator;
	delete		viterbiDecoder;
	delete		LPM_Filter;
	delete[]	pskBuffer;
}

void	pskDecoder::setWorkingRate	(int32_t workingRate) {
	if (_WorkingRate == workingRate)
	   return;
	delete	LocalOscillator;
	LocalOscillator	= new Oscillator	(_WorkingRate);
	pskSettings	();
	psk_clrText	();
}
	
void	pskDecoder::pskSettings () {

	if (!pskAfcon) {
	   psk_IF	= PSKIF;
	}

//	pskReverse
//	pskSquelchLevel
//	pskMode
//	pskFilterDegree

	pskCycleCount		= 0;
        pskPhaseAcc		= 0;
	pskOldz			= 0;
	psk_phasedifference	= 0.0;
	pskShiftReg		= 0;
	pskDecodeShifter	= 0;
	pskSyncCounter		= 0;
	pskAfcmetrics		= 0;
	pskShiftReg		= 0;
	pskQuality		= 0;
	pskBitclk		= 0;
	pskDecimatorCount	= 0;

	delete	LPM_Filter;
	LPM_Filter		= new LowPassIIR (pskFilterDegree,
	                                          (int)(speedofPskMode ()),
	                                          _WorkingRate,
	                                          S_BUTTERWORTH);
}

uint8_t	pskDecoder::isBinarypskMode () {
	if ((pskMode == MODE_PSK31) ||
	    (pskMode == MODE_PSK63) ||
	    (pskMode == MODE_PSK125)) return -1;
	return 0;
}

uint8_t	pskDecoder::isQuadpskMode () {

	if ((pskMode == MODE_QPSK31) ||
	    (pskMode == MODE_QPSK63) ||
	    (pskMode == MODE_QPSK125)) return -1;
	return 0;
}

float	pskDecoder::speedofPskMode (void) {
	switch (pskMode) {
	   default:
	   case MODE_PSK31:
	   case MODE_QPSK31:
	      return PSK31_SPEED;

	   case MODE_PSK63:
	   case MODE_QPSK63:
	      return PSK63_SPEED;

	   case MODE_PSK125:
	   case MODE_QPSK125:
	      return PSK125_SPEED;
	}
}

void	pskDecoder::psk_setAfcon (bool n) {
	pskAfcon = n;
	pskSettings ();
}

void	pskDecoder::psk_setReverse (bool n) {
	pskReverse = n;
	pskSettings ();
}

void	pskDecoder::psk_setSquelchLevel (int16_t n) {
	pskSquelchLevel = n;
}

void	pskDecoder::psk_setMode (int16_t n) {
	pskMode = n;
	pskSettings ();
}

void	pskDecoder::psk_setFilterDegree (int16_t n) {
	pskFilterDegree = n;
	pskSettings ();
}

uint8_t	pskDecoder::getIntPhase (DSPCOMPLEX z) {
double phase = arg (z);

	if (phase < 0)
	   phase += 2 * M_PI;
	if (this -> isQuadpskMode ())	/* phases 0 .. 3	*/
	   return ((int)(phase / (M_PI / 2) + 0.5)) & 03;
	else				/* just 0 or 1		*/
	   return (((int) (phase / M_PI + 0.5)) & 1);
}

int16_t	pskDecoder::DecimatingCountforpskMode (void) {
	switch (pskMode) {
	   default:
	   case MODE_PSK31:
	   case MODE_QPSK31:
	      return 4;
	   case MODE_PSK63:
	   case MODE_QPSK63:
	      return 2;
	   case MODE_PSK125:
	   case MODE_QPSK125:
	      return 1;
	}
}

DSPCOMPLEX	pskDecoder::doDecode (DSPCOMPLEX z) {
DSPCOMPLEX	old_z;
uint16_t	pskBufferp;
float		sum	= 0;
float		ampsum	= 0;
uint16_t	i;

	if (++pskCycleCount > _WorkingRate / 10) {
	   pskCycleCount = 0;
	   psk_showMetrics (pskAfcmetrics);
	   psk_showIF (psk_IF);
	   setDetectorMarker	(psk_IF);
	   psk_showDial (psk_phasedifference);
	}
/*
 *	we down-mix the signal to a zero-centered one,
 */
	old_z	= z;
	z	= z * LocalOscillator -> nextValue (psk_IF);
	z	= LPM_Filter -> Pass (z);

	if (!pskDecimator -> doDecimate (z, &z))
	   return old_z;
/*
 *	Now we are on PSKRATE 
 */
	if (++pskDecimatorCount < this -> DecimatingCountforpskMode ()) 
	   return old_z;
/*
 *	now we are finally back on 16 samples per symbol
 *	to apply the "standard" algorithm
 *	Note: experimenting with other algorithms worked
 *	as well, but this seems to be the most readable
 *	one.
 */
	pskDecimatorCount			= 0;
	pskBufferp				= (int) pskBitclk;
	pskBuffer [pskBufferp % 16]	= abs (z);
	for (i = 0; i < 16 / 2; i ++) {
	   sum += pskBuffer [i] - pskBuffer [16 / 2 + i];
	   ampsum += pskBuffer [i] + pskBuffer [16 / 2 + i];
	}

	sum = (ampsum == 0 ? 0 : sum / ampsum);
	pskBitclk -= sum / 5.0;
	pskBitclk += 1;
	if (pskBitclk < 0)
	   pskBitclk += 16;

	if (pskBitclk >= 16) {
	   uint16_t bits;
	   pskBitclk -= 16;
	   psk_phasedifference	= arg (pskOldz) - arg (z);
	   if (psk_phasedifference < 0)
	      psk_phasedifference += 2 * M_PI;

	   if (this -> isQuadpskMode ()) {
	      bits = ((int)(psk_phasedifference / (M_PI / 2) + 0.5)) & 03;
	      if (decodingIsOn (4, psk_phasedifference, bits, &pskQuality))
	         qpsk_bit (bits);
	   }
	   else  //if (isBinarypskMode (pskMode)) 
	   {
	      bits = (((uint16_t)(psk_phasedifference / M_PI + 0.5)) & 1) << 1;
	      if (decodingIsOn (2, psk_phasedifference, bits, &pskQuality))
	         bpsk_bit  (!bits);
	   }

	   pskAfcmetrics	= decayingAverage (pskAfcmetrics,
	                                   100 * norm (pskQuality), 50);
	   pskOldz	= z;
	   if (pskAfcon)
	      psk_IF += psk_IFadjust (psk_phasedifference, bits);
	}

	return old_z;
}

/*
 * 	Here we look at the data stream to see whether
 * 	or not we are getting something useful
 */
uint8_t	pskDecoder::decodingIsOn (int n, double phase,
	                        int bits, DSPCOMPLEX *pskQuality) {
	pskDecodeShifter	= (pskDecodeShifter << 2) | bits;

	switch (pskDecodeShifter) {
	   case 0xAAAAAAAA:	/* "1010101010..." preAmble	*/
	      *pskQuality	= DSPCOMPLEX (1.0, 0.0);
	      return TRUE;

	   case 0x0:		/* postamble or garbage		*/
	      *pskQuality	= 0;
	      return FALSE;

	   default:
	      *pskQuality	 =  DSPCOMPLEX (0.05 * cos (n * phase) +
		                             0.95 * real (*pskQuality),
		                             0.05 * sin (n * phase) +
		                             0.95 * imag (*pskQuality));
	      return
		 ((100.0 * norm (*pskQuality)) > (pskSquelchLevel));
	}
}
/*
 *	in case of complete sync, phase = bits * PI,
 *	otherwise, find a basis for correcting the IF
 *	phase and bits are related
 */
double	pskDecoder::psk_IFadjust (double phasedifference, uint16_t bits) {
double	error;

	if (pskAfcmetrics < 0.1)
	   return 0.0;

	error = phasedifference - (double)bits * (M_PI / 2);
	if (error < - M_PI / 2)
	   error += 2 * M_PI;
	if (error > M_PI /2)
	   error -= 2 * M_PI;

	error *= (double)_WorkingRate / (16 * 2 * M_PI);
	return  - error / 256;
}

void	pskDecoder::bpsk_bit (uint16_t bit) {
int16_t	c;

	pskShiftReg = (pskShiftReg << 1) | !!bit;
	if ((pskShiftReg & 03) == 0) {
	   c = pskVaridecode (pskShiftReg >> 2);
	   if (c != -1) 
	      psk_addText ((char)c);
	   pskShiftReg = 0;
	}
}

void	pskDecoder::qpsk_bit (uint16_t bits) {
unsigned char sym [2];
int	c;

	if (pskReverse)
	   bits = (4 - bits) & 03;

	sym [0] = (bits & 01) ? 255 : 0;
	sym [1] = (bits & 02) ? 0 : 255;

	c = viterbiDecoder -> viterbi_decode (sym, NULL);

	if (c != -1) {
	   bpsk_bit (c & 0x80);
	   bpsk_bit (c & 0x40);
	   bpsk_bit (c & 0x20);
	   bpsk_bit (c & 0x10);
	   bpsk_bit (c & 0x08);
	   bpsk_bit (c & 0x04);
	   bpsk_bit (c & 0x02);
	   bpsk_bit (c & 0x01);
	}
}
/*
 * The code table in a format suitable for decoding.
 */
static unsigned int varicodetab2[] = {
	0x2AB, 0x2DB, 0x2ED, 0x377, 0x2EB, 0x35F, 0x2EF, 0x2FD, 
	0x2FF, 0x0EF, 0x01D, 0x36F, 0x2DD, 0x01F, 0x375, 0x3AB, 
	0x2F7, 0x2F5, 0x3AD, 0x3AF, 0x35B, 0x36B, 0x36D, 0x357, 
	0x37B, 0x37D, 0x3B7, 0x355, 0x35D, 0x3BB, 0x2FB, 0x37F, 
	0x001, 0x1FF, 0x15F, 0x1F5, 0x1DB, 0x2D5, 0x2BB, 0x17F, 
	0x0FB, 0x0F7, 0x16F, 0x1DF, 0x075, 0x035, 0x057, 0x1AF, 
	0x0B7, 0x0BD, 0x0ED, 0x0FF, 0x177, 0x15B, 0x16B, 0x1AD, 
	0x1AB, 0x1B7, 0x0F5, 0x1BD, 0x1ED, 0x055, 0x1D7, 0x2AF, 
	0x2BD, 0x07D, 0x0EB, 0x0AD, 0x0B5, 0x077, 0x0DB, 0x0FD, 
	0x155, 0x07F, 0x1FD, 0x17D, 0x0D7, 0x0BB, 0x0DD, 0x0AB, 
	0x0D5, 0x1DD, 0x0AF, 0x06F, 0x06D, 0x157, 0x1B5, 0x15D, 
	0x175, 0x17B, 0x2AD, 0x1F7, 0x1EF, 0x1FB, 0x2BF, 0x16D, 
	0x2DF, 0x00B, 0x05F, 0x02F, 0x02D, 0x003, 0x03D, 0x05B, 
	0x02B, 0x00D, 0x1EB, 0x0BF, 0x01B, 0x03B, 0x00F, 0x007, 
	0x03F, 0x1BF, 0x015, 0x017, 0x005, 0x037, 0x07B, 0x06B, 
	0x0DF, 0x05D, 0x1D5, 0x2B7, 0x1BB, 0x2B5, 0x2D7, 0x3B5, 
	0x3BD, 0x3BF, 0x3D5, 0x3D7, 0x3DB, 0x3DD, 0x3DF, 0x3EB, 
	0x3ED, 0x3EF, 0x3F5, 0x3F7, 0x3FB, 0x3FD, 0x3FF, 0x555, 
	0x557, 0x55B, 0x55D, 0x55F, 0x56B, 0x56D, 0x56F, 0x575, 
	0x577, 0x57B, 0x57D, 0x57F, 0x5AB, 0x5AD, 0x5AF, 0x5B5, 
	0x5B7, 0x5BB, 0x5BD, 0x5BF, 0x5D5, 0x5D7, 0x5DB, 0x5DD, 
	0x5DF, 0x5EB, 0x5ED, 0x5EF, 0x5F5, 0x5F7, 0x5FB, 0x5FD, 
	0x5FF, 0x6AB, 0x6AD, 0x6AF, 0x6B5, 0x6B7, 0x6BB, 0x6BD, 
	0x6BF, 0x6D5, 0x6D7, 0x6DB, 0x6DD, 0x6DF, 0x6EB, 0x6ED, 
	0x6EF, 0x6F5, 0x6F7, 0x6FB, 0x6FD, 0x6FF, 0x755, 0x757, 
	0x75B, 0x75D, 0x75F, 0x76B, 0x76D, 0x76F, 0x775, 0x777, 
	0x77B, 0x77D, 0x77F, 0x7AB, 0x7AD, 0x7AF, 0x7B5, 0x7B7, 
	0x7BB, 0x7BD, 0x7BF, 0x7D5, 0x7D7, 0x7DB, 0x7DD, 0x7DF, 
	0x7EB, 0x7ED, 0x7EF, 0x7F5, 0x7F7, 0x7FB, 0x7FD, 0x7FF, 
	0xAAB, 0xAAD, 0xAAF, 0xAB5, 0xAB7, 0xABB, 0xABD, 0xABF, 
	0xAD5, 0xAD7, 0xADB, 0xADD, 0xADF, 0xAEB, 0xAED, 0xAEF, 
	0xAF5, 0xAF7, 0xAFB, 0xAFD, 0xAFF, 0xB55, 0xB57, 0xB5B
};


int16_t	pskDecoder::pskVaridecode (uint16_t symbol) {
int i;

	for (i = 0; i < 256; i++)
	   if (symbol == varicodetab2[i])
	      return i;

	return -1;
}

