#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in ESDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _PSK_DECODER
#define	_PSK_DECODER
#

#include	"jff-include.h"
#include	"viterbi.h"
#include	"virtualdecoder.h"
#include	"iir-filters.h"
#include	"fir-filters.h"
#include	"decimator.h"

class	RadioInterface;
class	Oscillator;
/*
 * PSK31  -- receiver classs
 */
class pskDecoder: public virtualDecoder {
Q_OBJECT
public:
		pskDecoder		(int32_t, RadioInterface *);
		~pskDecoder		(void);
	void	setWorkingRate		(int32_t);
	DSPCOMPLEX	doDecode	(DSPCOMPLEX);
	enum PskMode {
	   MODE_PSK31 =		0100,
	   MODE_PSK63 =		0101,
	   MODE_PSK125 =	0102,
	   MODE_QPSK31 =	0110,
	   MODE_QPSK63 =	0111,
	   MODE_QPSK125 =	0112
	};
	void	psk_setAfcon		(bool);
	void	psk_setReverse		(bool);
	void	psk_setSquelchLevel	(int16_t);
	void	psk_setMode		(int16_t);
	void	psk_setFilterDegree	(int16_t);

private:
	void	pskSettings		();
	float	speedofPskMode		(void);
	uint8_t	isBinarypskMode		(void);
	uint8_t	isQuadpskMode		(void);	
	uint8_t	decodingIsOn		(int, double, int, DSPCOMPLEX *);
	void	bpsk_bit		(uint16_t);
	void	qpsk_bit		(uint16_t);
	int16_t	pskVaridecode		(uint16_t);
	double	psk_IFadjust		(double, uint16_t);
	uint8_t	getIntPhase		(DSPCOMPLEX);
	int16_t	DecimatingCountforpskMode	(void);

	int32_t		_WorkingRate;
	Oscillator	*LocalOscillator;
	RadioInterface	*MyRadioInterface;

	downDecimator	*pskDecimator;
	int16_t		pskCycleCount;
	LowPassIIR	*LPM_Filter;
	viterbi		*viterbiDecoder;
	double		psk_IF;
	int16_t		pskDecimatorCount;
	unsigned int 	pskShiftReg;
	int16_t		pskPhaseAcc;
	uint32_t 	pskDecodeShifter;
	DSPCOMPLEX	pskQuality;
	double		pskAfcmetrics;
	bool		pskAfcon;
	bool		pskReverse;
	int16_t		pskSquelchLevel;
	uint8_t		pskMode;
	int16_t		pskFilterDegree;
	DSPCOMPLEX	pskOldz;
	double		psk_phasedifference;

	int16_t		pskPrev1Phase;
	int16_t		pskPrev2Phase;
	int16_t		pskCurrPhase;
	int16_t		pskSyncCounter;
	float		pskBitclk;
	double		*pskBuffer;

signals:
	void		psk_showDial		(float);
	void		psk_showMetrics		(float);
	void		psk_showIF		(float);
	void		psk_addText		(char);
	void		psk_clrText		();
	void		setDetectorMarker	(int);
};

#endif


