#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    Code is influenced by code in
 *    rtty decoder from Jesus Arias, however written as a state machine
 *    rather than the implied control is his code,
 *    and by gmfsk0.6 source code distribution
 *    gmfsk Copyright (C) 2001, 2002, 2003
 *    Tomi Manninen (oh2bns@sral.fi)
 *
 *    This file is part of the JFF SDR (ESDR).
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	"rttydecoder.h"
#include	"gui.h"
#include	<math.h>
#include	"oscillator.h"


#define	RTTY_IF		800

#define	StaysInBand(x)	(((RTTY_IF - rttyShift / 10) < (x)) && ((x) < RTTY_IF + rttyShift / 10))
/*
 */
	rttyDecoder::rttyDecoder (int32_t WorkingR, RadioInterface *RI) {
	_WorkingRate		= WorkingR;
	MyRadioInterface	= RI;

	LocalOscillator		= new Oscillator (_WorkingRate);
	rttyPrevsample		= 0;
	rttyCycleCount		= 0;
	rttyIF			= RTTY_IF;
	rttyStopbits		= 1;
	rttyBaudrate		= 45;
	rttyShift		= 170;
        rttyParity		= RTTY_PARITY_NONE;
	rttySymbolLength	= (int)(_WorkingRate / rttyBaudrate + 0.5);
	rttyReversed		= FALSE;
	rttyNbits		= 5;
	rttyAfcon		= FALSE;
	rttyMsb			= FALSE;
	rttyFilterDegree	= 10;
	filterLowend		= 0.1 * _WorkingRate;
	filterHighend		= 0.2 * _WorkingRate;
	LPM_Filter		= new BandPassIIR (rttyFilterDegree,
	                                           0.1 * _WorkingRate,
	                                           0.2 * _WorkingRate,
	                                           _WorkingRate,
	                                           S_BUTTERWORTH);
/*
 *	we only consider baudrates > 40, so OutputRate / 40
 *	is a safe upperbound for the length of the filter
 */
	rttyAverager	= new average (rttySymbolLength / 3);
	connect (this,
	         SIGNAL (rtty_clrText ()),
	         MyRadioInterface,
	         SLOT (clr_rttyText ()));
	connect (this,
	         SIGNAL (rtty_addText (char)),
		 MyRadioInterface,
	         SLOT (add_rttyText (char)));
	connect (this,
	         SIGNAL (rtty_showStrength (float)),
	         MyRadioInterface,
	         SLOT (show_rttyStrength (float)));
	connect (this,
	         SIGNAL (rtty_showCorrection (float)),
		 MyRadioInterface,
	         SLOT (show_rttyFreqCorrection (float)));
	connect (this,
	         SIGNAL (rtty_showBaudRate (int)),
		 MyRadioInterface,
	         SLOT (show_rttyBaudRate (int)));
	connect (this,
	         SIGNAL (rtty_showGuess (int)),
	         MyRadioInterface,
	         SLOT (show_rttyGuess (int)));
	connect (this,
	         SIGNAL (setDetectorMarker (int)),
	         MyRadioInterface,
	         SLOT (setDetectorMarker (int)));

        rtty_clrText	();
	rttySettings ();
}

	rttyDecoder::~rttyDecoder () {
	disconnect (this,
	            SIGNAL (rtty_clrText ()),
	            MyRadioInterface,
	            SLOT (clr_rttyText ()));
	disconnect (this,
	            SIGNAL (rtty_addText (char)),
		    MyRadioInterface,
	            SLOT (add_rttyText (char)));
	disconnect (this,
	            SIGNAL (rtty_showStrength (float)),
	            MyRadioInterface,
	            SLOT (show_rttyStrength (float)));
	disconnect (this,
	            SIGNAL (rtty_showCorrection (float)),
		    MyRadioInterface,
	            SLOT (show_rttyFreqCorrection (float)));
	disconnect (this,
	            SIGNAL (rtty_showBaudRate (int)),
		    MyRadioInterface,
	            SLOT (show_rttyBaudRate (int)));
	disconnect (this,
	            SIGNAL (rtty_showGuess (int)),
	            MyRadioInterface,
	            SLOT (show_rttyGuess (int)));

	delete	rttyAverager;
	delete	LPM_Filter;
	delete	LocalOscillator;
}

void	rttyDecoder::setWorkingRate	(int32_t workingRate) {
	if (workingRate == _WorkingRate)
	   return;
	_WorkingRate = workingRate;

	delete	LocalOscillator;
	LocalOscillator		= new Oscillator (_WorkingRate);
	rttySettings ();
	rtty_clrText	();
	setDetectorMarker	(rttyIF);
}

void	rttyDecoder::rttySettings (void) {

	if (rttyStopbits == 1)
	   rttyStoplen	= (int)(_WorkingRate / rttyBaudrate + 0.5);
	else
	if (rttyStopbits == 2)
	   rttyStoplen	= (int)(1.5 * _WorkingRate / rttyBaudrate + 0.5);
	else
	   rttyStoplen	= (int)(2.0 * _WorkingRate / rttyBaudrate + 0.5);

	if (!rttyAfcon) {
	   rttyIF = RTTY_IF;
	}

	if (rttyShift / 2 <= rttyBaudrate) {
	   rttyShift	= 170;
	   rttyBaudrate = 45;
	}

	rttyIF		= RTTY_IF;

        filterLowend	= (int)(rttyShift / 2 - 3 * rttyBaudrate / 4);
        filterHighend	= (int)(rttyShift / 2 + 3 * rttyBaudrate / 4);

	delete		LPM_Filter;
	LPM_Filter	= new  BandPassIIR (rttyFilterDegree,
	                                    filterLowend, 
	                                    filterHighend,
	                                    _WorkingRate,
	                                    S_BUTTERWORTH);
	rttySymbolLength	= (int)(_WorkingRate / rttyBaudrate + 0.5);
	delete		rttyAverager;
	rttyAverager		= new average (rttySymbolLength / 3);
	rttyState		= RTTY_RX_STATE_IDLE;
        rttyPhaseacc		= 0;
	rttyPluscnt		= 0;
	rttyMincnt		= 0;
	rttyPrevfragment	= 0;
	rttySignalPower		= 0;
	rttyNoisePower		= 0;

	rttyPoscnt		= 0;
	rttyNegcnt		= 0;
	rttyPosFreq		= 0.0;
	rttyNegFreq		= 0.0;
	rttyFreqError		= 0.0;
}

void	rttyDecoder::rtty_setStopbits (int16_t n) {
	rttyStopbits = n;
	rttySettings ();
}

void	rttyDecoder::rtty_setBaudrate (int16_t n) {
	rttyBaudrate = n;
	rttySettings ();
}

void	rttyDecoder::rtty_setShift (int16_t n) {
	rttyShift = n;
	rttySettings ();
}

void	rttyDecoder::rtty_setParity (int16_t n) {
	rttyParity = n;
	rttySettings ();
}

void	rttyDecoder::rtty_setReverse (bool n) {
	rttyReversed = n;
	rttySettings ();
}

void	rttyDecoder::rtty_setNbits (int16_t n) {
	rttyNbits = n;
	rttySettings ();
}

void	rttyDecoder::rtty_setMsb (bool n) {
	rttyMsb = n;
	rttySettings ();
}

void	rttyDecoder::rtty_setAfcon (bool n) {
	rttyAfcon = n;
	rttySettings ();
}

int	rttyDecoder::reverseBits (int in, int n) {
	if (n <= 1)
	   return in;
	return ((in & 01) << (n - 1)) | reverseBits (in >> 1, n - 1);
}

int rttyDecoder::pickup_char() {
int	data_mask	= (1 << rttyNbits) - 1;
int	parbit 		= (rttyRxdata >> rttyNbits) & 01;
int	data;

        if (rttyParity != RTTY_PARITY_NONE) {
	   if (parbit != check_parity (rttyRxdata, rttyNbits, rttyParity)) {
//	      fprintf (stderr, "Parity error");
	      return 0;
	   }
	}

	data	= (int)(rttyRxdata & data_mask);
	if (rttyMsb)
	   data = reverseBits (data, rttyNbits);

	if (rttyNbits == 5)
	   return BaudottoASCII (data);

	return data;
}
/*
 *	feed the sample here
 */
DSPCOMPLEX	rttyDecoder::doDecode (DSPCOMPLEX z) {
DSPCOMPLEX	local_copy;
double		FreqOffset;
int	res;
/*
 *	keep a local copy of the sample
 */
	local_copy	= z;
/*
 *	processing is straightforward, we down-mix
 *	the signal with the - possible adjusted IF
 */
	z	= z * LocalOscillator -> nextValue (rttyIF);
	z	= LPM_Filter -> Pass (z);
/*
 * 	then we apply slicing to end up with omega and
 *	we compute the offset frequency
 */
	FreqOffset	= arg (conj (rttyPrevsample) * z) *
	                            _WorkingRate / (2 * M_PI);
	rttyPrevsample	= z;

	if (FreqOffset > 0) {
	   rttyPoscnt ++;
	   rttyPosFreq += FreqOffset;
	}

	if (FreqOffset < 0) {
	   rttyNegcnt ++;
	   rttyNegFreq += FreqOffset;
	}

	FreqOffset	= clamp (FreqOffset, - rttyShift, + rttyShift);
	FreqOffset	= rttyAverager -> filter (FreqOffset);
/*
 */
	rttyBitFragment	= (rttyReversed) ?
	                     (FreqOffset > 0.0) : (FreqOffset < 0.0);
/*
 *	Note: we were experimenting here with the approach to
 *	compute the "average values" of the mark and space
 *	frequencies
 *	Not very successful so far
 */
	res	= this -> addrttyBitFragment ();
	if (res && rttyAfcon) {
	   float FreqAdjust = this -> adjust_IF (FreqOffset);
	   if (StaysInBand (rttyIF + FreqAdjust)) 
	      rttyIF += FreqAdjust;
	}
	else 
	if (res) 
	   if ((rttyPoscnt > 100) && (rttyNegcnt > 100)) {
	      double ferr = - (rttyPosFreq / rttyPoscnt +
	                             rttyNegFreq / rttyNegcnt) / 2;
	      rttyFreqError = decayingAverage (rttyFreqError, ferr, 10);
	      rtty_showCorrection ((float)rttyFreqError);
	      rttyPoscnt = 0; rttyPosFreq = 0.0;
	      rttyNegcnt = 0; rttyNegFreq = 0.0;
	   }

	if (++rttyCycleCount > _WorkingRate / 10) {
	   rttyCycleCount	= 0;
//	   rtty_showStrength   (rttyS2N ());
	   rtty_showStrength   ((float)rttyIF);
//	   rtty_showCorrection ((float)rttyIF);
	   setDetectorMarker	(rttyIF);
	   rtty_showGuess      ((int)(2.0 * FreqOffset));
	}

	return local_copy;
}
/*
 *	frequency is "equal" to (+ or -) rttyShift / 2 when
 *	rttyIF is precisely in the middle between the mark and the space.
 *	Adjust slowly if needed
 */
double	rttyDecoder::adjust_IF (double f) {

	if (f > 0.0) {
	   f = f - rttyShift / 2;
	}
	else {
	   f = f + rttyShift / 2;
	}

	if (fabs (f) < rttyShift / 2)
	   return f / 256;

	return 0.0;
}

int	rttyDecoder::addrttyBitFragment (void) {
unsigned char c;
int	res	= 0;

	switch (rttyState) {
	   case RTTY_RX_STATE_IDLE:
	      if (rttyBitFragment == 0) {
	         rttyState = RTTY_RX_STATE_START;
	         rttyCounter = rttySymbolLength / 2;
	      }
	      break;

	   case RTTY_RX_STATE_START:
	      if (--rttyCounter == 0) {
	         if (rttyBitFragment == 0) {	/* read the data	*/
	            rttyCounter	= rttySymbolLength;
	            rttyBitcntr	= 0;
	            rttyRxdata		= 0;
	            res			= 1;
	            rttyState		= RTTY_RX_STATE_DATA;
	         }
	         else
	            rttyState = RTTY_RX_STATE_IDLE;
	      }
	      break;

	   case RTTY_RX_STATE_DATA:
	      if (--rttyCounter <= 0) {
	         rttyRxdata	|= rttyBitFragment << rttyBitcntr;
	         rttyBitcntr ++;
	         rttyCounter	= rttySymbolLength;
	         res		= 1;
	      }

	      if (rttyBitFragment == 1)
	         rttyPluscnt ++;
	      else
	         rttyMincnt ++;
	      if (rttyBitFragment != rttyPrevfragment) {
	         check_baudrate (rttyMincnt, rttyPluscnt);
	         rttyMincnt		= 0;
	         rttyPluscnt		= 0;
	      }

	      rttyPrevfragment = rttyBitFragment;
	      if (rttyBitcntr >= rttyNbits) {
	         res = 1;
                 if (rttyParity == RTTY_PARITY_NONE)
	            rttyState = RTTY_RX_STATE_STOP;
	         else
	            rttyState = RTTY_RX_STATE_PARITY;
	      }
	      break;

	   case RTTY_RX_STATE_PARITY:
	      if (--rttyCounter == 0) {
	         rttyState = RTTY_RX_STATE_STOP;
	         rttyRxdata |= rttyBitFragment << rttyBitcntr++;
	         rttyCounter = rttyStoplen;
	      }
	      break;

	   case RTTY_RX_STATE_STOP:
	      if (--rttyCounter == 0) {
	         if (rttyBitFragment == 1) {
	            c = pickup_char();
		    rtty_addText (c);
	         }
	         else
	            c = 'F';
	         rttyState = RTTY_RX_STATE_STOP2;
	         rttyCounter = rttySymbolLength / 2;
	      }
	      break;

	   case RTTY_RX_STATE_STOP2:
	      if (--rttyCounter == 0)
	         rttyState = RTTY_RX_STATE_IDLE;
	      break;
	}

	return res;
}

static unsigned char letters[32] = {
	'\0',	'E',	'\n',	'A',	' ',	'S',	'I',	'U',
	'\r',	'D',	'R',	'J',	'N',	'F',	'C',	'K',
	'T',	'Z',	'L',	'W',	'H',	'Y',	'P',	'Q',
	'O',	'B',	'G',	'',	'M',	'X',	'V',	''
};

///*
// * ITA-2 version of the figures case.
// */
//static unsigned char figures[32] = {
//	'\0',	'3',	'\n',	'-',	' ',	'\'',	'8',	'7',
//	'\r',	'',	'4',	'\a',	',',	'',	':',	'(',
//	'5',	'+',	')',	'2',	'',	'6',	'0',	'1',
//	'9',	'?',	'',	'',	'.',	'/',	'=',	''
//};
///*
// * U.S. version of the figures case.
// */
//static unsigned char figures[32] = {
//	'\0',	'3',	'\n',	'-',	' ',	'\a',	'8',	'7',
//	'\r',	'$',	'4',	'\'',	',',	'!',	':',	'(',
//	'5',	'"',	')',	'2',	'#',	'6',	'0',	'1',
//	'9',	'?',	'&',	'',	'.',	'/',	';',	''
//};
/*
 * A mix of the two. This is what seems to be what people actually use.
 */
static unsigned char figures[32] = {
	'\0',	'3',	'\n',	'-',	' ',	'\'',	'8',	'7',
	'\r',	'$',	'4',	'\a',	',',	'!',	':',	'(',
	'5',	'+',	')',	'2',	'H',	'6',	'0',	'1',
	'9',	'?',	'&',	'',	'.',	'/',	'=',	''
};


char	rttyDecoder::BaudottoASCII(unsigned char data) {
int out = 0;

	switch (data) {
	   case 0x1F:		/* letters */
	      rttyRxmode = BAUDOT_LETS;
	      break;

	   case 0x1B:		/* figures */
	      rttyRxmode = BAUDOT_FIGS;
	      break;

	   case 0x04:		/* unshift-on-space */
	      rttyRxmode = BAUDOT_LETS;
	      return ' ';
	      break;

	   default:
	      if (rttyRxmode == BAUDOT_LETS)
	         out = letters [data];
	      else
	         out = figures [data];
	      break;
	}

	return out;
}
/*
 * Parity function. Return one if `w' has odd number of ones, zero otherwise.
 */
int rttyDecoder::odd (unsigned int w) {
int	i;
int	b = 0;
	for (i = 0; w != 0; i ++) {
	   b += (w & 01);
	   w >>= +1;
	}
	return b & 01;
}

int	rttyDecoder::check_parity (unsigned int c,
				   unsigned int nbits, int par) {
	c &= (1 << nbits) - 1;

	switch (par) {
	default:
        case RTTY_PARITY_NONE:
		return 1;

        case RTTY_PARITY_ODD:
		return odd (c);

        case RTTY_PARITY_EVEN:
		return odd (c) == 0;

        case RTTY_PARITY_ZERO:
		return 0;

        case RTTY_PARITY_ONE:
		return 1;
	}
}
/*
 *	very experimental. While composing the bits of the character
 *	we count the number of samples with a given sign
 *	Obviously, this stands (if anything) for one or more
 *	bits, we'll count a while.
 */

int	inrange (int t, int l, int h) {
	return (l <= t && t <= h);
}

void	rttyDecoder::check_baudrate (int mincnt, int pluscnt) {
int	baudrate;

	mincnt = mincnt > pluscnt ? mincnt : pluscnt;
	if (mincnt < _WorkingRate / 1000) /* arbitrary value */
	   return;

	baudrate = _WorkingRate / mincnt;

	if (inrange (baudrate, 40, 47))
	   baudrate = 45;
	else
	if (inrange (baudrate, 47, 55))
	   baudrate = 50;
	else
	if (inrange (baudrate, 65, 85))
	   baudrate = 75;
	else
	if (inrange (baudrate, 90, 110))
	   baudrate = 100;
	else
	   baudrate = 0;

	if (baudrate != 0)
	   rtty_showBaudRate (baudrate);
}

double	rttyDecoder::rttyS2N (void) {
double	sPower, nPower;
double	signal2noise;
int	delta	= rttyShift / 2;
int	rttyMetric;

//	sPower		= MyDetector -> getSpectrumPower (rttyIF - rttyShift / 2, delta) +
//	                 MyDetector -> getSpectrumPower (rttyIF + rttyShift / 2, delta);
//	sPower		= sPower / delta + 10e-8;
//
//	nPower		= MyDetector -> getSpectrumPower (rttyIF + 3 * rttyShift / 2, delta) / delta + 10e-8;
//
//	rttySignalPower	= decayingAverage (rttySignalPower, sPower, 8);
//	rttyNoisePower	= decayingAverage (rttyNoisePower, nPower, 8);
//	signal2noise	= 10 * log10 (rttySignalPower / (rttyNoisePower * (1500 / delta)));
////	signal2noise	= 20 * log10 (rttySignalPower / rttyNoisePower);
//	rttyMetric	= (int)clamp (rttySignalPower / rttyNoisePower, 0.0, 100.0);
//	return rttyMetric;
	return 0;
}

