#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _RTTY_DECODER
#define	_RTTY_DECODER
#
#include	"jff-include.h"
#include	"../virtualdecoder.h"
#include	"fir-filters.h"
#include	"iir-filters.h"
#include	"utilities.h"

class	RadioInterface;
class	Oscillator;
/*
 *	defines, local for the rttyDetector stuff
 */
#define	RTTY_RX_STATE_IDLE 	0
#define	RTTY_RX_STATE_START	1
#define	RTTY_RX_STATE_DATA	2
#define	RTTY_RX_STATE_PARITY	3
#define	RTTY_RX_STATE_STOP	4
#define	RTTY_RX_STATE_STOP2	5

#define	BAUDOT_LETS		0
#define	BAUDOT_FIGS		1

class rttyDecoder: public virtualDecoder {
Q_OBJECT
public:
		rttyDecoder (int, RadioInterface *);
		~rttyDecoder (); 
	void	setWorkingRate		(int32_t);
	DSPCOMPLEX	doDecode	(DSPCOMPLEX);

	enum Parities {
	   RTTY_PARITY_NONE =	0,
	   RTTY_PARITY_EVEN =	1,
	   RTTY_PARITY_ODD =	2,
	   RTTY_PARITY_ZERO =	3,
	   RTTY_PARITY_ONE =	4
	};
	void		rtty_setStopbits	(int16_t);
	void		rtty_setBaudrate	(int16_t);
	void		rtty_setShift		(int16_t);
	void		rtty_setParity		(int16_t);
	void		rtty_setReverse		(bool);
	void		rtty_setNbits		(int16_t);
	void		rtty_setAfcon		(bool);
	void		rtty_setMsb		(bool);

private:
	void		rttySettings		(void);
	int		addrttyBitFragment	(void);
	int		pickup_char		(void);
	int		reverseBits		(int, int);
	char		BaudottoASCII		(unsigned char);
	void		printChar		(char, char);
	int		check_parity		(unsigned int,
	                                         unsigned int, int);
	int		odd			(unsigned int);
	void		check_baudrate		(int, int);
	double		adjust_IF		(double);
	double		rttyS2N			(void);

	average		*rttyAverager;
	int32_t		_WorkingRate;
	double		filterLowend;
	double		filterHighend;
	Oscillator	*LocalOscillator;
	RadioInterface	*MyRadioInterface;
	int16_t		rttyCycleCount;
	uint8_t		rttyBitFragment;
	int16_t		rttyPoscnt;
	int16_t		rttyNegcnt;
	double		rttyPosFreq;
	double		rttyNegFreq;
	double		rttyFreqError;
	DSPCOMPLEX	rttyPrevsample;
	BandPassIIR	*LPM_Filter;
	uint8_t		rttyAfcon;
	double		rttyShift;
	double		rttyIF;
	int16_t		rttySymbolLength;
	int16_t		rttyNbits;
	uint8_t		rttyParity;
	int16_t		rttyStopbits;
	bool		rttyMsb;
	int16_t		rttyStoplen;
	bool		rttyReversed;
	int16_t		rttyFilterDegree;
	int16_t		rttyPhaseacc;
	int16_t		rttyState;
	int16_t		rttyCounter;
	int16_t		rttyBitcntr;
	int16_t		rttyRxdata;
	uint8_t		rttyRxmode;
	double		rttyBaudrate;
	int16_t		rttyPluscnt;
	int16_t		rttyMincnt;
	int16_t		rttyPrevfragment;

	double		rttySignalPower;
	double		rttyNoisePower;
signals:
	void		rtty_showStrength	(float);
	void		rtty_showCorrection	(float);
	void		rtty_showBaudRate	(int);
	void		rtty_showGuess		(int);
	void		rtty_clrText		();
	void		rtty_addText		(char);
	void		doDisplay		(double *, double *, double *, int);
	void		setDetectorMarker	(int);
};

#endif

