#
/*
 *
 *    Copyright (C) 2008, 2009, 2010
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    JFF Consultancy
 *
 *    This file is part of the JFF SDR (JSDR).
 *    Many of the ideas as implemented in JSDR are derived from
 *    other work, made available through the GNU general Public License. 
 *    All copyrights of the original authors are recognized.
 *
 *    JSDR is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JSDR is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JSDR; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include	"spectrummanager.h"

	spectrumManager::spectrumManager (uint16_t	fillLimit,
	                                  uint16_t	displaySize,
	                                  uint32_t	maxFrequency,
	                                  uint8_t	ModeofOperation) {
uint16_t	i;

	cnt			= 0;
	if ((displaySize & (displaySize - 1)) != 0)
	   displaySize = 2048;
	this	-> displaySize		= displaySize;
	this	-> ModeofOperation	= ModeofOperation;
	this	-> MaxFrequency		= maxFrequency;

	spectrumFillpoint	= fillLimit;
	spectrumSize		= getSpectrumSizefor (spectrumFillpoint);
	spectrumBuffer		= spectrum_fft -> getVector ();
	spectrum_fft		= new common_fft (spectrumSize);
	displayBuffer		= new double [displaySize];
	Window			= new double [spectrumFillpoint];
	binWidth		= (2 * maxFrequency) / spectrumSize;
	X_axis			= new double [displaySize];
/*
 *	a blackman window for the display
 */
	for (i = 0; i < spectrumFillpoint; i ++)
	   Window [i] = 0.42 - 0.5 * cos ((2.0 * M_PI * i) / (spectrumFillpoint - 1)) +
	                        0.08 * cos ((4.0 * M_PI * i) / (spectrumFillpoint - 1));
	if (ModeofOperation == FFT_POSITIVE)
	   for (i = 0; i < displaySize; i ++)
	      X_axis [i] = (double)((i) * ((double)MaxFrequency / displaySize));
	else
	if (ModeofOperation == FFT_TWOSIDES)
	   for (i = -displaySize / 2; i < displaySize / 2; i ++)
	      X_axis [displaySize / 2 + i] =
	         (double)(i) * ((double) 2 * MaxFrequency / displaySize);
	else
	   for (i = 0; i < displaySize; i ++)
	      X_axis [i] = - MaxFrequency +
	                     i * (double)MaxFrequency / displaySize;

}

	spectrumManager::~spectrumManager () {
	delete		spectrum_fft;
	delete[]	displayBuffer;
	delete[]	Window;
	delete[]	X_axis;
}

void	spectrumManager::setModeofOperation (uint8_t m) {
uint16_t	i;

	if (m == ModeofOperation)
	   return;

	ModeofOperation = m;

	if (ModeofOperation == FFT_POSITIVE)
	   for (i = 0; i < displaySize; i ++)
	      X_axis [i] = (double)((i) * ((double)MaxFrequency / displaySize));
	else
	if (ModeofOperation == FFT_NEGATIVE)
	   for (i = 0; i < displaySize; i ++)
	      X_axis [i] = - MaxFrequency +
	                        i * ((double)MaxFrequency / displaySize);
	else
	   for (i = -displaySize / 2; i < displaySize / 2; i ++)
	      X_axis [displaySize / 2 + i] =
	         (double)(i) * ((double) (2 * MaxFrequency / displaySize));
}

void	spectrumManager::addElement (DSPCOMPLEX z) {
int	i, j;
int	Incr;

	z = cmul (z, Window [cnt]);
	spectrumBuffer [cnt] = DSPCOMPLEX (real (z), imag (z));
	if (++cnt < spectrumFillpoint) 
	   return;

	cnt = 0;
	for (i = spectrumFillpoint; i < spectrumSize; i ++) 
	   spectrumBuffer [i] = 0;
	spectrum_fft	-> do_FFT ();

	switch (ModeofOperation) {
	case FFT_TWOSIDES:
	   Incr = spectrumSize / displaySize;
	   for (i = 0; i < displaySize / 2; i ++) {
	      displayBuffer [displaySize / 2 + i] = 0;
	      for (j = 0; j < Incr; j ++) 
	         displayBuffer [displaySize / 2 + i] +=
	                       abs (spectrumBuffer [i * Incr + j]) / binWidth;

	      displayBuffer [displaySize / 2 + i] /= Incr;
	   }

	   for (i = 0; i < displaySize / 2; i ++) {
	      displayBuffer [i] = 0;
	      for (j = 0; j < Incr; j ++) 
	         displayBuffer [i] +=
	                      abs (spectrumBuffer [spectrumSize / 2 + i * Incr + j]) / binWidth;

	      displayBuffer [i] /= Incr;
	   }
	   break;

	case FFT_POSITIVE:
	   Incr = spectrumSize / (2 * displaySize);
	   for (i = 0; i < displaySize; i ++) {
	      displayBuffer [i] = 0.0;
	      for (j = 0; j < Incr; j ++)
	         displayBuffer [i] += abs (spectrumBuffer [i * Incr + j]) / binWidth;
	      displayBuffer [i] /= Incr;
	   }
	   break;

	case FFT_NEGATIVE:
	   Incr = spectrumSize / (2 * displaySize);
	   for (i = 0; i < displaySize; i ++) {
	      displayBuffer [i] = 0.0;
	      for (j = 0; j < Incr; j ++)
	         displayBuffer [i] += abs (spectrumBuffer [spectrumSize / 2 + i * Incr + j]) / binWidth;
	      displayBuffer [i] /= Incr;
	   }
	   break;

	default:
	   break;		// cannot happen
	}

	ReadyforDisplay (X_axis, displayBuffer, displaySize);
}

void	spectrumManager::addElements (DSPCOMPLEX *v, int n) {
int	i;

	for (i = 0; i < n; i ++)
	   this -> addElement (v [i]);
}

/*
 * this spectrummanager understands negative frequencies, which complicates
 * the thing
 */
double	spectrumManager::getSpectrumPower (int frequency, int bandwidth) {
double sum	= 0;
int	i;
double	width = Binwidth ();
int	indexofLowestFrequency	= Binfor (frequency - bandwidth / 2);
int	indexofHighestFrequency	= Binfor (frequency + bandwidth / 2);
double fracofFirst = ((frequency - bandwidth / 2) -
	                     indexofLowestFrequency * width) / width;
double	fracofLast = ((frequency - bandwidth / 2) -
	                     indexofHighestFrequency * width) / width;

	sum = fracofFirst * abs (spectrumBuffer [indexofLowestFrequency]);
	sum += fracofLast * abs (spectrumBuffer [indexofHighestFrequency]);
	
	if (indexofLowestFrequency > indexofHighestFrequency) {
	   for (i = indexofLowestFrequency + 1; i < spectrumSize - 1; i ++)
	      sum += abs (spectrumBuffer [i]);
	   for (i = 0; i < indexofHighestFrequency; i ++)
	      sum += abs (spectrumBuffer [i]);
	}
	else
	   for (i = indexofLowestFrequency + 1;
	        i < indexofHighestFrequency - 1;
	        i ++)
	      sum += abs (spectrumBuffer [i]);

	sum /= bandwidth;
	return sum;
}

int	spectrumManager::Binfor (int frequency) {
	if (frequency > 0)
	   return (int)(frequency / Binwidth ());
	else
	   return (int)(frequency / Binwidth ()) + spectrumSize;
}

double	spectrumManager::Binwidth (void) {
	return 2 * (double)MaxFrequency / spectrumSize;
}

double	spectrumManager::getDisplayYelement (int index) {
	return displayBuffer [index];
}

double	spectrumManager::getDisplayXelement (int index) {
	return X_axis [index];
}

int	spectrumManager::getSpectrumSizefor (int fillpoint) {
/*
 *	just ensure that  res >= 2 * fillpoint
 */
int	twopower	= 1;
	if (fillpoint < 0)
	   fillpoint = 8192 - 1;

	if (fillpoint >= 2 * 8192)
	   fillpoint = 2 * 8192 - 1;

	while (fillpoint != 0) {
	   fillpoint = fillpoint >> 1;
	   twopower = twopower << 1;
	}
//	printf ("spectrumsize %d for fillpoint %d\n", of, 2 * twopower);
	return 2 * twopower;
}

void	spectrumManager::Restart (int Mode) {
int	i;

	ModeofOperation = Mode;

	if (ModeofOperation == FFT_POSITIVE)
	   for (i = 0; i < displaySize; i ++)
	      X_axis [i] = (double)((i) * ((double)MaxFrequency / displaySize));
	else
	   for (i = -displaySize / 2; i < displaySize / 2; i ++)
	      X_axis [displaySize / 2 + i] =
	         (double)(i) * ((double) (2 * MaxFrequency / displaySize));

	cnt	= 0;
}

